/*
 * Created on Apr 3, 2005
 */
package org.exoplatform.velocity.directive;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import javax.faces.context.FacesContext;
import javax.jcr.Property;

import org.apache.commons.lang.StringUtils;
import org.apache.velocity.context.InternalContextAdapter;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.RuntimeServices;
import org.apache.velocity.runtime.directive.Directive;
import org.apache.velocity.runtime.parser.node.Node;
import org.exoplatform.commons.utils.ISO8601;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.cms.CMSDialogComponent;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.cms.SelectorHandlerImpl;
import org.exoplatform.faces.cms.listener.SelectorActionClassMaps;
import org.exoplatform.faces.cms.listener.SelectorActionListener;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UIDateInput;
import org.exoplatform.faces.core.component.UIDateTimeInput;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.UIHtmlTextArea;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.cms.scripts.CmsScript;
import org.exoplatform.services.cms.scripts.ScriptService;

/**
 * @author benjaminmestrallet
 */
public class JSFDialogFormField extends JSFFormField {

  private static final String SEPARATOR = "=";

  private static final String INPUT_NAME = "inputName" + SEPARATOR;
  private static final String JCR_PATH = "jcrPath" + SEPARATOR;
  private static final String WIDGET = "widget" + SEPARATOR;
  private static final String EDITABLE = "editable" + SEPARATOR;
  private static final String OPTIONS = "options" + SEPARATOR;
  private static final String PARAMETER = "parameter" + SEPARATOR;
  private static final String VISIBLE = "visible" + SEPARATOR;
  private static final String NODETYPE = "nodetype" + SEPARATOR;
  private static final String SELECTOR_ACTION = "selectorAction" + SEPARATOR;
  private static final String SELECTOR_CLASS = "selectorClass" + SEPARATOR;
  private static final String SELECTOR_LABEL = "selectorLabel" + SEPARATOR;
  private static final String SELECTOR_PARAMS = "selectorParams" + SEPARATOR;
  private static final String SELECTOR_SCRIPT = "selectorScript" + SEPARATOR;
  private static final String SCRIPT = "script" + SEPARATOR;
  private static final String MULTI_VALUES = "multiValues" + SEPARATOR;

  private static final String TEXT = "text";
  private static final String TEXTAREA = "textarea";
  private static final String HTMLTEXTAREA = "htmltextarea";
  private static final String DATE = "date";
  private static final String DATETIME = "datetime";
  private static final String UPLOAD = "upload";
  private static final String SELECTBOX = "selectbox";

  public String getName() {
    return "jsfDialogFormField";
  }

  public int getType() {
    return LINE;
  }

  public void init(RuntimeServices rs, InternalContextAdapter context, Node node)
      throws Exception {
    super.init(rs, context, node);
  }

  public boolean render(InternalContextAdapter context, Writer w, Node node)
      throws IOException, MethodInvocationException, ResourceNotFoundException,
      ParseErrorException {
    UISimpleForm uiForm = (UISimpleForm) context
        .get(UIExoComponent.UICOMPONENT);
    String name = null;
    String jcrPath = null;
    String widget = "text";
    String editable = "true";
    String defaultValue = "";
    String options = null;
    String parameter = null;
    String visible = "true";
    String nodetype = null;
    String selectorAction = null;
    String selectorClass = null;
    String selectorLabel = null;
    String selectorParams = null;
    String selectorScript = null;
    String script = null;
    String multiValues = null;
    int size = node.jjtGetNumChildren();
    for (int i = 0; i < size; i++) {
      String argument = (String) node.jjtGetChild(i).value(context);
      if (argument.startsWith(INPUT_NAME)) {
        name = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(JCR_PATH)) {
        jcrPath = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(WIDGET)) {
        widget = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(EDITABLE)) {
        editable = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(OPTIONS)) {
        options = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(PARAMETER)) {
        parameter = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(VISIBLE)) {
        visible = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(NODETYPE)) {
        nodetype = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(SELECTOR_ACTION)) {
        selectorAction = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(SELECTOR_CLASS)) {
        selectorClass = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(SELECTOR_LABEL)) {
        selectorLabel = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(SELECTOR_PARAMS)) {
        selectorParams = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(SELECTOR_SCRIPT)) {
        selectorScript = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(SCRIPT)) {
        script = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else if (argument.startsWith(MULTI_VALUES)) {
        multiValues = argument.substring(argument.indexOf(SEPARATOR) + 1);
      } else {
        // default value
        defaultValue = argument;
      }
    }
    javax.jcr.Node jcrNode = null;
    try {
      jcrNode = ((CMSDialogComponent) uiForm).getNode();
    } catch (Exception e) {
    }

    UIInput uiInput = uiForm.getUIInput(name);
    if (uiInput == null && widget != null) {
      uiInput = getComponent(name, widget, defaultValue, options, parameter,
          script);
      JcrInputProperty inputProperty = new JcrInputProperty();
      inputProperty.setJcrPath(jcrPath);
      if (nodetype != null) {
        inputProperty.setType(JcrInputProperty.NODE);
        inputProperty.setNodetype(nodetype);
      }
      uiInput.setProperty(CMSUtils.JCR_INFO, inputProperty);
      uiForm.add(uiInput);
      try {
        if (jcrNode != null && nodetype == null) {
          if ("/node".equals(jcrPath)) {
            uiInput.setValue(jcrNode.getName());
          } else {
            int index = jcrPath.indexOf("node/") + "node/".length();
            String relativePath = jcrPath.substring(index);
            Property prop = jcrNode.getProperty(relativePath);
            if (prop == null)
              uiInput.setValue("");
            else
              uiInput.setValue(prop.getString());
          }
        }
      } catch (Exception e) {
      }
    }

    if (uiInput != null) {
      if (editable != null && "false".equals(editable)) uiInput.setEditable(false) ;
      else uiInput.setEditable(true) ;
      if (visible != null) {
        if ("true".equals(visible)
            || ("if-not-null".equals(visible) && uiInput.getValue() != null && !""
                .equals(uiInput.getValue()))) {
          if (selectorAction != null) {
            if (selectorLabel.startsWith("@")) {
              selectorLabel = selectorLabel.substring(1);
              ResourceBundle res = (ResourceBundle) context.get("res");
              selectorLabel = res.getString(selectorLabel);
            }
            String button = generateSelectorButton(uiForm.getFormName(),
                selectorAction, selectorLabel);
            uiInput.addSelector(selectorAction, button);
            SelectorActionListener listener = new SelectorActionListener(
                uiInput);
            listener.setActionToListen(selectorAction);
            uiForm.addActionListener(listener);
            Class clazz = getClass(selectorAction, selectorClass);
            try {
              UIExoComponent uiComponent = (UIExoComponent) uiForm.getParent();
              UIExoComponent exoComp = null;
              if (!uiComponent.hasChild(clazz))
                exoComp = uiComponent.addChild(clazz);
              else
                exoComp = (UIExoComponent) uiComponent
                    .getChildComponentOfType(clazz);
              exoComp.setRendered(false);
              if (selectorScript != null) {
                executeScript(selectorScript, exoComp);
              }
            } catch (Exception e) {
              throw new MethodInvocationException(
                  "Can not add child component", e, "");
            }
            SelectorActionClassMaps maps = getMaps();
            SelectorHandler selectorHandler = new SelectorHandlerImpl(uiInput,
                uiForm.getClass());
            maps.handlers.put(uiInput.getName(), selectorHandler);
            if (selectorParams != null && selectorScript == null) {
              String[] nonResolvedParams = StringUtils.split(selectorParams,
                  ",");
              maps.parameters.put(uiInput.getName(), nonResolvedParams);
            }
          }
          if ("true".equals(multiValues)) {
            manageMultiValue(context, uiForm, w, name);
          } else
            render(name, uiInput, w);
        }
      }
    } else {
      w.write("#jsfFormField( Unknow Field Name: " + name + ")");
    }
    return true;
  }

  private static SelectorActionClassMaps getMaps() {
    SessionContainer sC = SessionContainer.getInstance();
    SelectorActionClassMaps maps = (SelectorActionClassMaps) sC
        .getComponentInstanceOfType(SelectorActionClassMaps.class);
    if (maps == null) {
      sC.registerComponentInstance(SelectorActionClassMaps.class,
          new SelectorActionClassMaps());
      maps = (SelectorActionClassMaps) sC
          .getComponentInstanceOfType(SelectorActionClassMaps.class);
    }
    return maps;
  }

  private static Class getClass(String selectorAction, String className) {
    SelectorActionClassMaps maps = getMaps();
    Class mappedClass = (Class) maps.mappings.get(selectorAction);
    if (mappedClass != null)
      return mappedClass;
    ClassLoader cl = Thread.currentThread().getContextClassLoader();
    Class clazz = null;
    try {
      clazz = Class.forName(className, true, cl);
    } catch (ClassNotFoundException e) {
      return null;
    }
    maps.mappings.put(selectorAction, clazz);
    return clazz;
  }

  private static UIInput getComponent(String key, String component,
      String defaultValue, String options, String parameter, String script) {
    if (component == null || TEXT.equals(component)) {
      if (defaultValue != null && !"".equals(defaultValue))
        return new UIStringInput(key, defaultValue);
      else
        return new UIStringInput(key, "");
    } else if (TEXTAREA.equals(component)) {
      if (defaultValue != null && !"".equals(defaultValue))
        return new UITextArea(key, defaultValue);
      else
        return new UITextArea(key, "");
    } else if (HTMLTEXTAREA.equals(component)) {
      if (defaultValue != null && !"".equals(defaultValue))
        return new UIHtmlTextArea(key, defaultValue, "100%", "300px");
      else
        return new UIHtmlTextArea(key, "", "100%", "300px");
    } else if (DATE.equals(component)) {
      if (defaultValue != null && !"".equals(defaultValue))
        return new UIDateInput(key, ISO8601.parse(defaultValue).getTime());
      else
        return new UIDateInput(key, new Date());
    } else if (DATETIME.equals(component)) {
      if (defaultValue != null && !"".equals(defaultValue)) {
        if (options != null && !"".equals(options)) {
          if (parameter != null && "true".equals(parameter))
            return new UIDateTimeInput(key, ISO8601.parse(defaultValue)
                .getTime(), Integer.parseInt(options))
                .setUpdateOnChangeAction(true);
          else
            return new UIDateTimeInput(key, ISO8601.parse(defaultValue)
                .getTime(), Integer.parseInt(options));
        } else {
          if (parameter != null && "true".equals(parameter))
            return new UIDateTimeInput(key, ISO8601.parse(defaultValue)
                .getTime(), Integer.parseInt(options))
                .setUpdateOnChangeAction(true);
          else
            return new UIDateTimeInput(key, ISO8601.parse(defaultValue)
                .getTime(), Integer.parseInt(options));
        }
      } else {
        GregorianCalendar gCal = new GregorianCalendar();
        gCal.set(GregorianCalendar.SECOND, 0);
        gCal.set(GregorianCalendar.MILLISECOND, 0);
        if (options != null && !"".equals(options)) {
          if (parameter != null && "true".equals(parameter))
            return new UIDateTimeInput(key, gCal.getTime(), Integer
                .parseInt(options)).setUpdateOnChangeAction(true);
          else
            return new UIDateTimeInput(key, gCal.getTime(), Integer
                .parseInt(options));
        } else {
          if (parameter != null && "true".equals(parameter))
            return new UIDateTimeInput(key, gCal.getTime())
                .setUpdateOnChangeAction(true);
          else
            return new UIDateTimeInput(key, gCal.getTime());
        }
      }
    } else if (UPLOAD.equals(component)) {
      return new UIUploadInput(key);
    } else if (SELECTBOX.equals(component)) {
      List<SelectItem> optionsList = null;
      UISelectBox selectBox = new UISelectBox(key, "", null);
      if (script != null) {
        executeScript(script, selectBox);
      } else if (options != null && !"".equals(options)) {
        StringTokenizer optionsTk = new StringTokenizer(options, ",");
        optionsList = new ArrayList<SelectItem>(3);
        while (optionsTk.hasMoreTokens()) {
          String value = optionsTk.nextToken();
          optionsList.add(new SelectItem(value, value));
        }
        selectBox.setOptions(optionsList);
      }
      if (defaultValue != null && !"".equals(defaultValue)) {
        selectBox.setDefaultValue(defaultValue);
        selectBox.reset();
      }
      return selectBox;
    }
    return null;
  }

  private static void executeScript(String script, Object o) {
    PortalContainer pC = PortalContainer.getInstance();
    ScriptService scriptService = (ScriptService) pC
        .getComponentInstanceOfType(ScriptService.class);
    try {
      CmsScript dialogScript = scriptService.getScript(script);
      dialogScript.execute(o);
    } catch (Exception e) {
      // TODO use log instead
      e.printStackTrace();
    }
  }

}
