/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.portlets.content;

import java.util.Locale;

import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.portal.session.ExoPortal;
import org.exoplatform.portal.session.PortalResources;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.portal.community.CommunityConfigService;
import org.exoplatform.services.portal.community.CommunityNavigation;
import org.exoplatform.services.portletcontainer.ExoPortletRequest;

/**
 * @author Benjamin Mestrallet
 * @since Jul 5, 2004 1:35:46 PM
 */
public class ContentUtil {

  public static final String EDIT_STATE = "ACTION_EDIT";

  public static final String MODIFY_STATE = "ACTION_MODIFY";

  /*#####################################*/
  /*         SOME PORTAL PROPERTIES      */
  /*#################################### */
  public static Locale getLocale() {
    PortalResources appres = (PortalResources) SessionContainer
        .getComponent(PortalResources.class);
    return appres.getLocale();
  }    
  
  public static String resolveLocalisedViewPath(String uri) throws Exception{
    return getLocalisedAbsoluteSitePath() + uri;       
  }    
  
  public static String getLocalisedAbsoluteSitePath() throws Exception{
    return getUserHomeAbsolutePath() + getLocalisedRelativeSitePath();
  }
  
  public static String getLocalisedRelativeSitePath() {
    return "/site/" + ContentUtil.getLocale();
  }
  
  public static String getUserHomeAbsolutePath() throws Exception{
    return getUsersPath() + "/" + getPageOwner();
  }
  
  public static String getGroupId() {
    ExoPortal portal = (ExoPortal) SessionContainer
        .getComponent(ExoPortal.class);
    org.exoplatform.services.portal.model.Node currentNode = portal
        .getSelectedNode();
    if (currentNode.isShare()) {
      return currentNode.getGroupId();
    }
    return null;
  }

  public static String getPageOwner() throws Exception {
    ExoPortal portal = (ExoPortal) SessionContainer
        .getComponent(ExoPortal.class);
    org.exoplatform.services.portal.model.Node currentNode = portal
        .getSelectedNode();
    if (currentNode.isShare()) {
      String groupId = currentNode.getGroupId();
      PortalContainer manager = PortalContainer.getInstance();
      OrganizationService orgService = (OrganizationService) manager
          .getComponentInstanceOfType(OrganizationService.class);
      CommunityConfigService communityConfigService = (CommunityConfigService) manager
          .getComponentInstanceOfType(CommunityConfigService.class);
      Group group = orgService.getGroupHandler().findGroupById(groupId);
      CommunityNavigation cn = communityConfigService.getCommunityNavigation(group.getId());
      return cn.getNavigation();
    }
    return getPortalOwner();
  }

  public static final String getPortalOwner() {
    return SessionContainer.getInstance().getOwner();
  }

  private static String getUsersPath() {
    PortalContainer manager = PortalContainer.getInstance();
    CmsConfigurationService cmsConfigurationService = (CmsConfigurationService) manager
        .getComponentInstanceOfType(CmsConfigurationService.class);
    return cmsConfigurationService.getJcrPath(BasePath.CMS_USERS_PATH);
  }
  
  public static String getPortalUri() {
    ExoPortletRequest request = (ExoPortletRequest) FacesContext
        .getCurrentInstance().getExternalContext().getRequest();
    ExoPortal portal = (ExoPortal) SessionContainer
        .getComponent(ExoPortal.class);
    org.exoplatform.services.portal.model.Node selectedNode = portal
        .getSelectedNode();
    String uri = selectedNode.getUri();
    if ("/".equals(uri) && selectedNode.getName() != null && !selectedNode.getName().equals("")) {
      uri += selectedNode.getName() + "/";
    }
    String uniqueID = request.getWindowID().getUniqueID();
    return uri += uniqueID;
  }  

  public static String getTemplatesPath() {
    PortalContainer manager = PortalContainer.getInstance();
    CmsConfigurationService cmsConfigurationService = (CmsConfigurationService) manager
        .getComponentInstanceOfType(CmsConfigurationService.class);
    return cmsConfigurationService.getJcrPath(BasePath.CMS_TEMPLATES_PATH);
  }

  /*#####################################*/
  /*         SOME JCR/CMS ACTIONS        */
  /*#################################### */
  public static Node createNodeFromNode(Node node, String uri)
      throws RepositoryException {
    String[] splittedName = StringUtils.split(uri, "/");
    for (int i = 0; i < splittedName.length; i++) {
      try {
        node = node.getNode(splittedName[i]);
      } catch (PathNotFoundException exc) {
        node = node.addNode(splittedName[i], "nt:folder");
      }
    }
    return node;
  }

  public static Node getHomeNode(Session session) throws Exception {
    String homeDir = getUsersPath() + "/" + getPageOwner();
    Node node = null;
    try {
      node = (Node) session.getItem(homeDir);
    } catch (PathNotFoundException ex) {
      node = createNodeFromNode(session.getRootNode(), homeDir);
      session.save();
    }
    return node;
  }

}
