package org.exoplatform.portlets.content.categorybrowser.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.TreeMap;

import javax.jcr.AccessDeniedException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDelegate;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.AlphaNodeComparator;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.JCRUtil;
import org.exoplatform.portlets.content.jcr.TypeNodeComparator;
import org.exoplatform.portlets.content.jcr.component.Utils;
import org.exoplatform.portlets.content.jcr.component.model.ClipboardCommand;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.security.SecurityService;

public class UICategoryList extends UIExoCommand implements ChangeNodeListener {

  public static final String[] NON_EDITABLE_NODETYPES = { "nt:unstructured",
      "nt:folder", "nt:resource" };
  public static final String[] PASTE_TYPES = { "nt:unstructured", "nt:folder" };
  final static public String ACTION_NAME = "actionName".intern();
  
  static Parameter[] CHANGE_NODE_PARAMS = { new Parameter(ACTION, 
                                                UICategoryExplorer.CHANGE_NODE_ACTION) };
  
  public static final int NB_PER_ROW = 4;
  public static final int MAC_VIEW_NB_COLUMNS = 3;

  protected Node node_;
  private CMSDelegate delegate;
  protected ActionServiceContainer actionService_;
  private SecurityService securityService_;
  private TemplateService templateService_;

  public UICategoryList(TemplateService templateService, SecurityService securityService, 
      ActionServiceContainer actionService) {
    setId("UICategoryList");
    setRendererType("VelocityRenderer");

    templateService_ = templateService;
    securityService_ = securityService;
    actionService_ = actionService;

    addActionListener(ChangeNodeActionListener.class, UICategoryExplorer.CHANGE_NODE_ACTION);
    
  }

  public void onChange(UICategoryExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
    if (delegate == null)
      delegate = new CMSDelegate(node);
    else
      delegate.setNode(node);
    setRendered(true);
  }

  public boolean isJcrViewEnable() {
    UICategoryExplorer explorer = (UICategoryExplorer) getAncestorOfType(UICategoryExplorer.class);
    Preference pref = explorer.getPreference();
    return pref.isJcrEnable();
  }

  public boolean isReadAuthorized(ExtendedNode node) throws RepositoryException {
    UICategoryExplorer explorer = (UICategoryExplorer) getAncestorOfType(UICategoryExplorer.class);
    return explorer.isReadAuthorized(node);
  }

  public boolean isReadAuthorized(String name) throws RepositoryException {
    UICategoryExplorer explorer = (UICategoryExplorer) getAncestorOfType(UICategoryExplorer.class);
    ExtendedNode node = (ExtendedNode) node_.getNode(name);
    return explorer.isReadAuthorized(node);
  }

  public String getNodeOwner() {
    return ((ExtendedNode) node_).getChildrenACL().getOwner();
  }

  public boolean isListView() {
    UICategoryExplorer explorer = (UICategoryExplorer) getAncestorOfType(UICategoryExplorer.class);
    Preference pref = explorer.getPreference();
    String displayType = pref.getDisplayType();
    if (Preference.LIST_TYPE.equals(displayType))
      return true;
    return false;
  }

  public boolean hasChildren() {
    return hasChildren(null);
  }

  public boolean hasChildren(Node pNode) {
    try {
      if (pNode == null)
        return node_.hasNodes();
      else
        return pNode.hasNodes();
    } catch (RepositoryException e) {
      return false;
    }
  }

  public Parameter[] getChangeNodeParams() {
    return CHANGE_NODE_PARAMS;
  }

  public String getViewToParse() throws Exception {
    String nodetype;
    try {
      nodetype = getNodeType();
    } catch (Exception e) {
      return "";
    }
    UICategoryExplorer explorer = (UICategoryExplorer) getAncestorOfType(UICategoryExplorer.class);
    Preference pref = explorer.getPreference();
    String displayType = pref.getDisplayType();

    if (isJcrViewEnable()) {
      return new StringBuffer(
          "org/exoplatform/portlets/content/jcr/renderer/html/").append(
          displayType).append("View.vm").toString();
    } else {
      if (isNodeTypeSupported()) {
        return "jcr:" + getTemplatePath();
      } else {
        return new StringBuffer(
            "org/exoplatform/portlets/content/jcr/renderer/html/").append(
            displayType).append("View.vm").toString();
      }
    }
  }

  public Collection getNodeIteratorList() throws Exception {
    UICategoryExplorer uiExplorer = (UICategoryExplorer) getAncestorOfType(UICategoryExplorer.class);
    Preference pref = uiExplorer.getPreference();
    String displayType = pref.getDisplayType();
    Node currentNode = uiExplorer.getSelectNode();
    Node selectedNode;
    int nbColumns = 1;
    int start;
    int columnMax;
    boolean rootFirst = false;
    List iteratorList;
    String currentPath = currentNode.getPath();

    if (Preference.MAC_TYPE.equals(displayType)) {
      nbColumns = MAC_VIEW_NB_COLUMNS;
    }
    iteratorList = new ArrayList(nbColumns);
    columnMax = nbColumns;

    if (hasChildren(currentNode)) {
      columnMax--;
    }
    selectedNode = currentNode;
    for (int i = 1; i < columnMax; i++) {
      try {
        selectedNode = selectedNode.getParent();
      } catch (Exception e) {
        rootFirst = true;
        break;
      }
    }
    if ("/".equals(selectedNode.getPath())) {
      rootFirst = true;
    }
    if (rootFirst) {
      Map m = new TreeMap();
      m.put("/", selectedNode);
      iteratorList.add(m.values().iterator());
      start = 2;
    } else {
      start = 1;
      try {
        selectedNode = selectedNode.getParent();
      } catch (Exception e) {
        return null;
      }
    }
    if ("/".equals(currentNode.getPath()))
      nbColumns = 2;
    for (int i = start; i <= nbColumns; i++) {
      if (hasChildren(selectedNode)) {
        Map nodeMap = getChildrenMap(selectedNode);
        iteratorList.add(nodeMap.values().iterator());
        if (i < nbColumns) {
          Iterator iter = nodeMap.values().iterator();
          while (iter.hasNext()) {
            Node node = (Node) iter.next();
            String nodePath = node.getPath();
            if (currentPath.indexOf(nodePath) > -1) {
              selectedNode = node;
              break;
            }
          }
        }
      } else
        break;
    }

    return iteratorList;
  }

  public Map getChildrenMap(Node pNode) throws Exception {
    UICategoryExplorer explorer = (UICategoryExplorer) getAncestorOfType(UICategoryExplorer.class);
    Preference pref = explorer.getPreference();
    String order = pref.getOrder();
    String sort = pref.getSort();
    NodeIterator i;
    if (pNode == null)
      i = node_.getNodes();
    else
      i = pNode.getNodes();
    Map nodesMap;
    int j = 0;
    String nodeType = "";
    if (Preference.ALPHABETICAL_SORT.equals(sort)) {
      nodesMap = new TreeMap(new AlphaNodeComparator(order));
      while (i.hasNext()) {
        Node childNode = i.nextNode();
        try {
          nodeType = childNode.getPrimaryNodeType().getName();
          // unstructured node considered as folder node for sorting
          if ("nt:unstructured".equals(nodeType))
            nodeType = "nt:folder";

        } catch (Exception ex) {
        }
        if ("nt:folder".equals(nodeType))
          nodeType = "1";
        else
          nodeType = "2";
        String key = new StringBuffer(nodeType).append(childNode.getName())
            .append(j).toString().toLowerCase();
        nodesMap.put(key, childNode);
        j++;
      }
    } else {
      // Sort by node type
      nodesMap = new TreeMap(new TypeNodeComparator(order));
      String contentType = "";
      String mimeType = "";
      Node jcrContentNode;
      StringTokenizer strTk;
      while (i.hasNext()) {
        Node childNode = i.nextNode();
        jcrContentNode = null;
        nodeType = "default";
        contentType = "default";
        mimeType = "default";
        try {
          nodeType = childNode.getPrimaryNodeType().getName();
          // unstructured node considered as folder node for sorting
          if ("nt:unstructured".equals(nodeType))
            nodeType = "nt:folder";
          if ("nt:file".equals(nodeType))
            jcrContentNode = childNode.getNode("jcr:content");
          else if ("nt:resource".equals(nodeType))
            jcrContentNode = childNode;
          if (jcrContentNode != null) {
            if (jcrContentNode.hasProperty("jcr:mimeType"))
              mimeType = jcrContentNode.getProperty("jcr:mimeType").getString();
          }
        } catch (Exception ex) {
        }
        // 2 node types for sorting : folder or file
        if ("nt:folder".equals(nodeType))
          nodeType = "folder";
        else
          nodeType = "file";
        // mime type if available. Ex : pdf for application/pdf
        if (!"default".equals(mimeType)) {
          strTk = new StringTokenizer(mimeType, "/");
          if (strTk.countTokens() == 2) {
            strTk.nextToken();
            mimeType = strTk.nextToken();
          }
        }
        String key = new StringBuffer(nodeType).append("//").append(mimeType)
            .append("//").append(childNode.getName()).append(j).toString()
            .toLowerCase();
        nodesMap.put(key, childNode);
        j++;
      }
    }

    return nodesMap;
  }

  public Iterator getChildrenIterator() throws Exception {
    return getChildrenIterator(null);
  }

  public Iterator getChildrenIterator(Node pNode) throws Exception {

    return getChildrenMap(pNode).values().iterator();
  }

  public Collection getChildrenPerRow() throws Exception {
    return Utils.getChildrenPerRow(NB_PER_ROW, getChildrenIterator());
  }

  public String getThumbnailName(Node node) throws RepositoryException {
    String icon = getLargeNodeTypeIcon(node);
    return icon + "<br clear=all/>" + node.getName();
  }

  public String getNodeFlag(Node node) {
    return JCRUtil.getNodeFlag(node);
  }

  public String getSmallNodeTypeIcon(Node node) {
    return JCRUtil.getSmallNodeTypeIcon(node);
  }

  public String getLargeNodeTypeIcon(Node node) {
    return JCRUtil.getLargeNodeTypeIcon(node);
  }

  public boolean isEditPossible(String name) throws Exception {
    boolean isEdit = true;
    Node childNode = node_.getNode(name);

    String nodeType = childNode.getPrimaryNodeType().getName();
    for (int i = 0; i < NON_EDITABLE_NODETYPES.length; i++) {
      String nonEditableType = NON_EDITABLE_NODETYPES[i];
      if (nonEditableType.equals(nodeType))
        return false;
    }
    return isEdit;
  }

  public Node getNode() throws Exception {
    return delegate.getNode();
  }

  public void setNode(Node node) {
    delegate.setNode(node);
  }

  public String getPortalName() {
    return delegate.getPortalName();
  }

  public String getWorkspaceName() throws Exception {
    return delegate.getWorkspaceName();
  }

  public boolean isNodeTypeSupported() {
    return delegate.isNodeTypeSupported();
  }

  public WikiContext getWikiContext() {
    return delegate.getWikiContext();
  }

  public WikiEngineService getWikiService() {
    return delegate.getWikiService();
  }
  
  public String getNodeType() throws Exception {
    return delegate.getNodeType();
  }
  
  public String getTemplatePath() throws Exception {
    return delegate.getTemplatePath();
  }  

  public boolean isVersionableOrAncestor(Node node) throws RepositoryException {
    if (isVersionable(node))
      return true;
    if (isAncestorVersionable(node))
      return true;
    return false;
  }

  public boolean isVersionable(Node node) throws RepositoryException {
    return node.isNodeType("mix:versionable")
        && !node.isNodeType("nt:frozenNode");
  }

  public boolean isAncestorVersionable(Node node) throws RepositoryException {
    int depth = node.getDepth();
    int i = depth - 1;
    while (i != 0) {
      Node parent = (Node) node.getAncestor(i);
      if (parent.isNodeType("mix:versionable"))
        return true;
      i--;
    }
    return false;
  }
  
  public String getVersionNumber(Node node) throws RepositoryException {
    if(!isVersionable(node))
      return "-";
    return node.getBaseVersion().getName();
  }
  
  public List<Node> getCustomActions(Node node) throws Exception {
    List<Node> safeActions = new ArrayList<Node>();
    
    try {
      String user = SessionContainer.getInstance().getRemoteUser();
      List<Node> unsafeActions = actionService_.getActions(node, ActionServiceContainer.READ_PHASE);
      for (Iterator iter = unsafeActions.iterator(); iter.hasNext();) {
        Node actionNode = (Node) iter.next();
        Value[] roles = actionNode.getProperty("exo:roles").getValues();
        for (int i = 0; i < roles.length; i++) {
          String role = roles[i].getString();
          if(securityService_.hasMembershipInGroup(user, role))
            safeActions.add(actionNode);
        }
      }      
    }
    catch(AccessDeniedException ignore) {
    }

    return safeActions;
  }

  static public class ChangeNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICategoryList uicomp = (UICategoryList) event.getComponent();
      UICategoryExplorer uiExplorer = (UICategoryExplorer) uicomp
          .getAncestorOfType(UICategoryExplorer.class);
      String uri = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      try {
        if ("../".equals(uri)) {
          if (!"/".equals(uiExplorer.getSelectNode().getPath())) {
            uiExplorer.setSelectNode(uiExplorer.getSelectNode().getParent());
          }
        }
        uiExplorer.setSelectNode(uri);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
}
