/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.categorybrowser.component;

import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.queries.QueryService;

/**
 * @author Nguyen Quang Hung (nguyenkequanghung@yahoo.com)
 * @since Aug 27, 2006
 * @version $Id$
 */
public class UISimpleSearch extends UISimpleForm {

  private Node node_;
  protected UIStringInput inputText_;
  protected QueryService queryService_;
  protected String query_;

  public static final String SEARCH_ACTION = "searchAction";
  
  private static final String ROOT_SQL_QUERY = "select * from nt:base where contains(*, '$1')";
  private static final String SQL_QUERY = "select * from nt:base where jcr:path like '$0/%' where contains(*, '$1')";

  public UISimpleSearch(QueryService queryService) throws Exception {
    super("UISimpleSearch", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    queryService_ = queryService;
    
    inputText_ = new UIStringInput("text", "");
    add(inputText_);
    
    addActionListener(SearchActionListener.class, SEARCH_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }

  public String getSearchAction() {
    return SEARCH_ACTION;
  }  

  public void updateNode(Node node) throws Exception {
    node_ = node;
    String queryText = null;
    if ("/".equals(node_.getPath()))
      queryText = ROOT_SQL_QUERY;
    else
      queryText = StringUtils.replace(SQL_QUERY, "$0", node_.getPath());
    query_ = queryText;
  }
  
  static public class SearchActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISimpleSearch uiForm = (UISimpleSearch) event.getSource();
      String text = uiForm.inputText_.getValue();
      UISearch uiComp = (UISearch)event.getComponent().getParent();
      UICategoryExplorer explorer = (UICategoryExplorer) uiComp.getParent();
      Session session = explorer.getSession();
      QueryManager queryManager = session.getWorkspace().getQueryManager();
      Information iprovider = findInformationProvider(uiForm);
      Query query = null;
      try {
        String statement = StringUtils.replace(uiForm.query_, "$1", text);
        query = queryManager.createQuery(statement, Query.SQL);        
        long beforeTime = System.currentTimeMillis();
        QueryResult queryResult = query.execute();
        long duration = System.currentTimeMillis() - beforeTime;
        UIResults results = (UIResults) uiForm.getSibling(UIResults.class);
        results.setQueryResults(duration, queryResult);
        uiForm.setRenderedSibling(UIResults.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  } 
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UISimpleSearch uicomp = (UISimpleSearch) event.getComponent() ;
      UISearch uiSearch = (UISearch) uicomp.getParent() ;
      UICategoryExplorer uiExplorer = 
        (UICategoryExplorer)uicomp.getAncestorOfType(UICategoryExplorer.class) ;      
      Node node = uiExplorer.getSelectNode();
      uiExplorer.broadcastOnChange(node);
      uiSearch.setRenderedSibling(UIBrowserController.class);
    }
  }
}