package org.exoplatform.portlets.content.categorybrowser.component;

import java.util.ResourceBundle;

import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.JCRUtil;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;

public class UIWorkspaceBrowser extends UIExoCommand{
  private static String SELECT_WORKSPACE_ACTION = "selectWorkspace".intern() ;
  
  static Parameter[] SELECT_WORKSPACE_PARAMS = { new Parameter(ACTION , SELECT_WORKSPACE_ACTION)} ;
  
  public static final int NB_PER_ROW = 4;
  
  private ManageableRepository repository;

  public UIWorkspaceBrowser(RepositoryService repositoryService) throws Exception {
    setRendererType("VelocityRenderer") ;
    setId("UIWorkspaceBrowser");
    repository = repositoryService.getRepository();    
    
    addActionListener(SelectWorkspaceActionListener.class, SELECT_WORKSPACE_ACTION);
  }
  
  public Parameter[] getSelectWorkspaceParams() { return SELECT_WORKSPACE_PARAMS ; }


  public String[] getWorkspaces() {
    return repository.getWorkspaceNames();
  }
  
  
  public boolean hasWorkspaces(){
    String[] workspaces = repository.getWorkspaceNames();
    if(workspaces == null || workspaces.length == 0){
      return false;
    }
    return true;
  }
  
  public String getActionIcon(String name) throws RepositoryException {
    ResourceBundle res = JCRUtil.getBundle();
    String icon = res.getString("UIWorkspaceBrowser.icon.workspace."+name);
    return icon;
  }
  
  static public class SelectWorkspaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event)  throws Exception {
      UIWorkspaceBrowser uicomp = (UIWorkspaceBrowser) event.getComponent() ;
      Information iprovider = findInformationProvider(uicomp) ;
      String workspaceName = event.getParameter(OBJECTID) ;      
      Session session = null;
      try {
        session = uicomp.repository.login(workspaceName);
        UICategoryExplorer explorer = (UICategoryExplorer) uicomp.getSibling(UICategoryExplorer.class);
        explorer.setSession(session);
        uicomp.setRenderedSibling(UICategoryExplorer.class) ;
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);                
      }   
    }
  }  
  
}
