/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.security.AccessControlException;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Stack;
import java.util.TreeMap;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.Button;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.AlphaNodeComparator;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.JCRUtil;
import org.exoplatform.portlets.content.jcr.component.model.ClipboardCommand;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedNode;

/**
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIExplorer.java,v 1.2 2004/07/31 15:07:15 tuan08 Exp $
 */
public class UIJCRExplorer extends UIExoCommand {
  final static public String SELECT_WORKSPACE_ACTION = "selectWorkspace".intern();
  final static public String CHANGE_NODE_ACTION = "changeNode".intern();
  final static public String EDIT_NODE_CONTENT = "editContent".intern();
  final static public String VIEW_NODE_CONTENT = "viewNodeContent".intern();
  final static public String UPLOAD_FILE = "uploadFile".intern();
  final static public String SAVE_SESSION = "saveSession".intern();
  final static public String PREFERENCES_ACTION = "preferences".intern();
  final static public String REFRESH_SESSION_ACTION = "refreshSession".intern();
  
  public static Parameter[] SELECT_WORKSPACES_PARAMS = { new Parameter(ACTION,  SELECT_WORKSPACE_ACTION) };
  public static Parameter[] SAVE_SESSION_PARAMS = { new Parameter(ACTION, SAVE_SESSION) };
  public static Parameter[] CHANGE_NODE = { new Parameter(ACTION, CHANGE_NODE_ACTION) };
  public static Parameter[] PREFERENCES_PARAMS = { new Parameter(ACTION, PREFERENCES_ACTION) };
  public static Parameter[] REFRESH_SESSION_PARAMS = { new Parameter(ACTION,  REFRESH_SESSION_ACTION) };
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION,  BACK_ACTION) };
  
  private ClipboardCommand clipboard;
  private Stack<Node> nodesHistory;
  
  private Session session;
  protected Node currentNode_;
  protected Node previousNode_;
  private Preference pref;

  public UIJCRExplorer() throws Exception {
    setId("UIJCRExplorer");
    setRendererType("VelocityRenderer");

    UIToolbar toolbar = new UIToolbar("MainToolbar");
    getChildren().add(toolbar);
    toolbar.addLeftButton(new Button("#{UIJCRExplorer.button.preferences}", PREFERENCES_PARAMS));
    toolbar.addLeftButton(new Button("#{UIJCRExplorer.button.back}",  BACK_PARAMS));
    toolbar.addRightButton(new Button("#{UIJCRExplorer.button.save-session}", SAVE_SESSION_PARAMS));
    toolbar.addRightButton(new Button("#{UIJCRExplorer.button.refresh-session}", REFRESH_SESSION_PARAMS));
    toolbar.addRightButton(new Button("#{UIJCRExplorer.button.select-workspace}", SELECT_WORKSPACES_PARAMS));

    addChild(UIViewController.class);
    pref = new Preference();
    clipboard = new ClipboardCommand();
    nodesHistory = new Stack<Node>();

    addActionListener(ChangeNodeActionListener.class, CHANGE_NODE_ACTION);
    addActionListener(SaveSessionActionListener.class, SAVE_SESSION);
    addActionListener(RefreshSessionActionListener.class, REFRESH_SESSION_ACTION);
    addActionListener(SavePreferencesActionListener.class, PREFERENCES_ACTION);
    addActionListener(SelectWorkspaceActionListener.class, SELECT_WORKSPACE_ACTION);
    addActionListener(BackActionListener.class, BACK_ACTION);
  }
  
  public Parameter[] getChangeNodeParams() { return CHANGE_NODE ; }

  public Session getSession() { return session; }

  public void setSession(Session session) throws Exception {
    this.session = session;
    setSelectNode("/");
  }

  public ClipboardCommand getClipBoard() { return clipboard; }

  public Node getSelectNode() {  return currentNode_; }

  public String[] getSelectedNodePath() throws Exception {  
    String path = currentNode_.getPath() ;
    String[] temp =  StringUtils.split(path, "/") ;
    return temp ;
  }
  
  public void setSelectNode(Node node) throws Exception {
    currentNode_ = node;
    record(node);
    broadcastOnChange(node);
  }

  public void setSelectNode(String uri) throws Exception {
    if (uri == null){
      //uri = "/";
      previousNode_ = null ;
      currentNode_ = (Node) session.getItem("/");
    } else{
      previousNode_ = currentNode_ ;
      currentNode_ = (Node) session.getItem(uri);
    }
    if(previousNode_ != null ) record(previousNode_) ;
    broadcastOnChange(currentNode_);
    //setSelectNode(currentNode_);
  }
  
  public boolean isReadAuthorized(ExtendedNode node) throws RepositoryException {
    try {
      node.checkChildrenPermission(PermissionType.READ);
      return true;
    } catch(AccessControlException e) {
      return false;
    }    
  }  

  public Preference getPreference() { return pref; }

  public void broadcastOnChange(Node node) throws Exception {    
    List list = findDescendantsOfType(ChangeNodeListener.class);
    for (int i = 0; i < list.size(); i++) {
      ChangeNodeListener listener = (ChangeNodeListener) list.get(i);
      listener.onChange(this, node);
    }        
  }

  public void record(Node node) { nodesHistory.push(node); }
  
  public Node rewind() { return nodesHistory.pop(); }
  
  public Collection getSiblingNodes(Node node)  throws Exception {
    String order = pref.getOrder();
    Node parent = node.getParent();
    NodeIterator i = parent.getNodes();
    Map nodesMap = new TreeMap(new AlphaNodeComparator(order));
    int j = 0;
    String nodeType = "";
    while (i.hasNext()) {
      Node childNode = i.nextNode();
      try {
        if (childNode.hasProperty("jcr:primaryType")) {
          nodeType = childNode.getProperty("jcr:primaryType").getString();
          // unstructured node considered as folder node for sorting
          if ("nt:unstructured".equals(nodeType))
            nodeType = "nt:folder";
        }
      } catch (Exception ex) {
      }
      if ("nt:folder".equals(nodeType))
        nodeType = "1";
      else
        nodeType = "2";
      String key = 
        new StringBuffer(nodeType).append(childNode.getName()).append(j).toString().toLowerCase();
      nodesMap.put(key, childNode);
      j++;
    }
    return nodesMap.values() ;
  }

  public String getNodeIcon(Node node)  throws Exception {
    return JCRUtil.getSmallNodeTypeIcon(node) ;
  }
  
  static public class SavePreferencesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      uicomp.setRenderedSibling(UIPreferencesForm.class);
    }
  }

  static public class ChangeNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uiExplorer = (UIJCRExplorer) event.getComponent();
      String uri = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uiExplorer);
      try {
        if ("../".equals(uri)) {
          if (!"/".equals(uiExplorer.currentNode_.getPath())) {
            uiExplorer.setSelectNode(uiExplorer.getSelectNode().getParent());
          }
        } else {
          uiExplorer.setSelectNode(uri);
        }
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class SelectWorkspaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIJCRExplorer uiExplorer = (UIJCRExplorer) event.getComponent();
      uiExplorer.setRenderedSibling(UIWorkspaceBrowser.class);
    }
  }

  static public class SaveSessionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.getSession().save();
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIJCRExplorer.msg.save-session-success")));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }
  
  static public class RefreshSessionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.getSession().refresh(false);
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIJCRExplorer.msg.refresh-session-success")));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }  

  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRExplorer uicomp = (UIJCRExplorer) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node previousNode = uicomp.rewind();
        uicomp.currentNode_ = previousNode;
        uicomp.broadcastOnChange(previousNode);
      } catch (Exception e) {
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIJCRExplorer.msg.no-node-history")));
      }
    }
  }
}