package org.exoplatform.portlets.content.jcr.component;

import java.io.ByteArrayInputStream;
import java.util.GregorianCalendar;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.ValueFormatException;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.version.VersionException;

import org.exoplatform.commons.utils.MimeTypeResolver;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;

public class UIUploadForm extends UISimpleMulltipartForm implements
    ChangeNodeListener {

  private Node node_;
  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  public UIUploadForm() {
    super("uploadForm", "post", null);

    setRendererType("SimpleFormVelocityRenderer");

    add(new UIUploadInput("upload"));

    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public Parameter[] getCancelParams() { return CANCEL_PARAMS ; }
  public void onChange(UIJCRExplorer explorer, Node node) {
    setRendered(false);
    node_ = node;
  }
  
  public Node getNode() {
    return node_;
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIUploadForm uiForm = (UIUploadForm) event.getSource();

      UIUploadInput input = (UIUploadInput) uiForm.getUIInput("upload");
      byte[] content = input.getContent();
      String fileName = input.getFileName();

      MimeTypeResolver resolver = new MimeTypeResolver();
      String mimeType = resolver.getMimeType(fileName);

      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm
          .getAncestorOfType(UIJCRExplorer.class);
      Node selectedNode = uiExplorer.getSelectNode();

      Information iprovider = findInformationProvider(uiForm);
      try {
        selectedNode.getSession().checkPermission(
            selectedNode.getPath(), "add_node,set_property");
        Node node = selectedNode.addNode(fileName, "nt:file");
        Node contentNode = node.addNode("jcr:content", "nt:resource");
        contentNode.setProperty("jcr:data", new ByteArrayInputStream(content));
        contentNode.setProperty("jcr:mimeType", mimeType);
        contentNode.setProperty("jcr:lastModified", new GregorianCalendar());
        uiExplorer.broadcastOnChange(selectedNode);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

}
