/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.jcr.Node;
import javax.jcr.Workspace;
import javax.jcr.nodetype.NodeType;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.component.JcrBrowser;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UICommandNode;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UINode;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.Button;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.actions.component.UIActionFormController;
import org.exoplatform.portlets.content.actions.component.UIActionList;
import org.exoplatform.portlets.content.categories.component.UICategoriesList;
import org.exoplatform.portlets.content.categories.component.UIReferencesList;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.model.ClipboardCommand;
import org.exoplatform.portlets.content.search.component.UISearch;
import org.exoplatform.portlets.content.versions.component.UIDiff;
import org.exoplatform.portlets.content.versions.component.UILabelForm;
import org.exoplatform.portlets.content.versions.component.UIRemoveLabelForm;
import org.exoplatform.portlets.content.versions.component.UIVersionInfo;

/**
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIExplorer.java,v 1.2 2004/07/31 15:07:15 tuan08 Exp $
 */
public class UIViewController extends UINode implements
    ChangeNodeListener {
  final static public String SEARCH = "search".intern();
  final static public String VIEW_PERMISSIONS = "viewPermissions".intern();
  final static public String VIEW_PROPERTIES = "viewProperties".intern();
  final static public String VIEW_NODETYPE = "viewNodeType".intern();
  final static public String IMPORT_NODE = "importNode".intern();
  final static public String EXPORT_NODE = "exportNode".intern();
  final static public String ADD_DOCUMENT = "addDocument".intern();
  final static public String UPLOAD = "upload".intern();
  final static public String ADD_FOLDER = "addFolder".intern();
  public static final String PASTE_ACTION = "paste".intern();
  public static final String MANAGE_ACTIONS_ACTION = "manageActions".intern();
  public static final String MANAGE_CATEGORIES_ACTION = "manageCategories".intern();
  public static final String VIEW_REFERENCES_ACTION = "viewReferences".intern();
  public static final String MANAGE_VERSIONS_ACTION = "manageVersions".intern();

  public static final String[] PASTE_TYPES = { "nt:unstructured", "nt:folder" };

  public static Parameter[] SEARCH_PARAMS = { new Parameter(ACTION, SEARCH) };
  public static Parameter[] VIEW_PERMISSIONS_PARAMS = { new Parameter(ACTION, VIEW_PERMISSIONS) };
  public static Parameter[] VIEW_PROPS_PARAMS = { new Parameter(ACTION, VIEW_PROPERTIES) };
  public static Parameter[] VIEW_NODETYPE_PARAMS = { new Parameter(ACTION,VIEW_NODETYPE) };
  public static Parameter[] ADD_DOCUMENT_PARAMS = { new Parameter(ACTION, ADD_DOCUMENT) };
  public static Parameter[] UPLOAD_PARAMS = { new Parameter(ACTION, UPLOAD) };
  public static Parameter[] IMPORT_NODE_PARAMS = { new Parameter(ACTION, IMPORT_NODE) };
  public static Parameter[] EXPORT_NODE_PARAMS = { new Parameter(ACTION, EXPORT_NODE) };
  public static Parameter[] ADD_FOLDER_PARAMS = { new Parameter(ACTION, ADD_FOLDER) };
  public static Parameter[] PASTE_NODE_PARAMS = { new Parameter(ACTION, PASTE_ACTION) };
  public static Parameter[] MANAGE_ACTIONS_PARAMS = { new Parameter(ACTION, MANAGE_ACTIONS_ACTION) };
  public static Parameter[] MANAGE_CATEGORIES_PARAMS = { new Parameter(ACTION, MANAGE_CATEGORIES_ACTION) };
  public static Parameter[] VIEW_REFERENCES_PARAMS = { new Parameter(ACTION, VIEW_REFERENCES_ACTION) };
  public static Parameter[] MANAGE_VERSIONS_PARAMS = { new Parameter(ACTION, MANAGE_VERSIONS_ACTION) };
  
  private Button pasteButton_;
  private UIToolbar uitoolbar;
  private Node node_;

  public UIViewController() throws Exception {
    //setRendererType("ChildrenRenderer");
    //UINode tabMenu = (UINode)addChild(UINode.class) ;
    setRendererType("VelocityNodeMenuRenderer") ;
    setTemplatePath("org/exoplatform/faces/core/renderer/html/UISecondTabLevel.vm") ;
    
    UICommandNode adminAction = (UICommandNode)addChild(UICommandNode.class) ;
    adminAction.setId("UIAdminAction") ;
    uitoolbar = new UIToolbar("UIToolbar");
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.manage-versions}", MANAGE_VERSIONS_PARAMS));
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.manage-categories}", MANAGE_CATEGORIES_PARAMS));    
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.manage-actions}", MANAGE_ACTIONS_PARAMS));
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.export-node}", EXPORT_NODE_PARAMS));
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.import-node}", IMPORT_NODE_PARAMS));
    adminAction.setRendererType("ChildrenRenderer") ;
    adminAction.setRendered(false) ;
    adminAction.getChildren().add(uitoolbar) ;
    adminAction.addChild(UIDocumentInfo.class);
    adminAction.addChild(UIVersionInfo.class).setRendered(false);
    adminAction.addChild(UIActionList.class).setRendered(false);
    adminAction.addChild(UICategoriesList.class).setRendered(false);
    adminAction.addChild(UIExport.class).setRendered(false);
    adminAction.addChild(UIImport.class).setRendered(false);
    adminAction.addChild(UIResourceForm.class).setRendered(false);
    adminAction.addChild(UIPermissionForm.class).setRendered(false);
    adminAction.addChild(UIExoPermissionBrowser.class).setRendered(false);
    adminAction.addChild(UIPropertiesInfo.class).setRendered(false);
    adminAction.addChild(UIPropertyForm.class).setRendered(false);
    adminAction.addChild(UIActionFormController.class).setRendered(false);
    adminAction.addChild(JcrBrowser.class).setRendered(false);
    adminAction.addChild(UILabelForm.class).setRendered(false);    
    adminAction.addChild(UIDiff.class).setRendered(false);
    adminAction.addChild(UIRemoveLabelForm.class).setRendered(false);
    
    
    adminAction.addActionListener(ManageVersionsActionListener.class, MANAGE_VERSIONS_ACTION);
    adminAction.addActionListener(ManageActionsActionListener.class, MANAGE_ACTIONS_ACTION);
    adminAction.addActionListener(ManageCategoriesActionListener.class, MANAGE_CATEGORIES_ACTION);
    
    adminAction.addActionListener(ExportNodeActionListener.class, EXPORT_NODE);
    adminAction.addActionListener(ImportNodeActionListener.class, IMPORT_NODE);
    adminAction.addActionListener(PasteActionListener.class, PASTE_ACTION);
    
    UICommandNode infoAction = (UICommandNode)addChild(UICommandNode.class) ;
    infoAction.setId("UIInfoAction") ;
    uitoolbar = new UIToolbar("UIToolbar");
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.view-references}", VIEW_REFERENCES_PARAMS));
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.view-nodetype}", VIEW_NODETYPE_PARAMS));
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.view-permissions}", VIEW_PERMISSIONS_PARAMS));
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.view-properties}", VIEW_PROPS_PARAMS));
    
    infoAction.setRendererType("ChildrenRenderer") ;
    infoAction.setRendered(false) ;
    infoAction.getChildren().add(uitoolbar) ;
    infoAction.addChild(UIDocumentInfo.class);
    infoAction.addChild(UIReferencesList.class).setRendered(false);
    infoAction.addChild(UINodeTypeInfo.class).setRendered(false);
    infoAction.addChild(UIPermissionsInfo.class).setRendered(false);
    infoAction.addChild(UIResourceForm.class).setRendered(false);
    infoAction.addChild(UIPermissionForm.class).setRendered(false);
    infoAction.addChild(UIExoPermissionBrowser.class).setRendered(false);
    infoAction.addChild(UIPropertiesInfo.class).setRendered(false);
    infoAction.addChild(UIPropertyForm.class).setRendered(false);
    infoAction.addChild(UIActionFormController.class).setRendered(false);
    infoAction.addChild(JcrBrowser.class).setRendered(false);
    infoAction.addChild(UILabelForm.class).setRendered(false);    
    infoAction.addChild(UIDiff.class).setRendered(false);
    infoAction.addChild(UIRemoveLabelForm.class).setRendered(false);
    
    infoAction.addActionListener(ViewReferencesActionListener.class, VIEW_REFERENCES_ACTION);
    infoAction.addActionListener(ViewNodeTypeActionListener.class, VIEW_NODETYPE);
    infoAction.addActionListener(ViewPermissionsActionListener.class, VIEW_PERMISSIONS);
    infoAction.addActionListener(ViewPropertiesActionListener.class, VIEW_PROPERTIES);
    
    
    UICommandNode defaultAction = (UICommandNode)addChild(UICommandNode.class) ;
    defaultAction.setId("UIDefaultAction") ;
    uitoolbar = new UIToolbar("UIToolbar");
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.add-folder}", ADD_FOLDER_PARAMS));
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.add-document}", ADD_DOCUMENT_PARAMS));    
    uitoolbar.addLeftButton(new Button("#{UIViewController.button.upload}", UPLOAD_PARAMS));
    pasteButton_ = new Button("#{UIViewController.button.paste}", PASTE_NODE_PARAMS);
    uitoolbar.addRightButton(pasteButton_);
    uitoolbar.addRightButton(new Button("#{UIViewController.button.search}", SEARCH_PARAMS));
    defaultAction.setRendererType("ChildrenRenderer") ;
    defaultAction.setRendered(true) ;
    defaultAction.getChildren().add(uitoolbar) ;
    defaultAction.addChild(UIDocumentInfo.class);
    defaultAction.addChild(UIDocumentFormController.class).setRendered(false);
    defaultAction.addChild(UIFolderForm.class).setRendered(false);
    defaultAction.addChild(UIUploadForm.class).setRendered(false);
    defaultAction.addChild(UIResourceForm.class).setRendered(false);
    defaultAction.addChild(UIExoPermissionBrowser.class).setRendered(false);
    defaultAction.addChild(UIPropertiesInfo.class).setRendered(false);
    defaultAction.addChild(UIPropertyForm.class).setRendered(false);
    defaultAction.addChild(UIActionFormController.class).setRendered(false);
    defaultAction.addChild(JcrBrowser.class).setRendered(false);
    defaultAction.addChild(UILabelForm.class).setRendered(false);    
    defaultAction.addChild(UIDiff.class).setRendered(false);
    defaultAction.addChild(UIRemoveLabelForm.class).setRendered(false);
    defaultAction.addChild(UISearch.class).setRendered(false);    
    
    defaultAction.addActionListener(AddDocumentActionListener.class, ADD_DOCUMENT) ;    
    defaultAction.addActionListener(AddFolderActionListener.class, ADD_FOLDER);
    defaultAction.addActionListener(UploadActionListener.class, UPLOAD);
    defaultAction.addActionListener(SearchActionListener.class, SEARCH);
    defaultAction.addActionListener(PasteActionListener.class, PASTE_ACTION);
     
    
    //addChild(UIDocumentFormController.class).setRendered(false);
  }   

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
    if (isPastePossible(node)) {
      List rightButton = uitoolbar.getRightButtons();
      if (rightButton.size() == 0) rightButton.add(pasteButton_);
    } else {
      List rightButton = uitoolbar.getRightButtons();
      if (rightButton.size() == 1)  rightButton.clear();
    }
  }

  public boolean isPastePossible(Node node) throws Exception {
    boolean isPaste = false;
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    if (uiExplorer.getClipBoard() == null) return false;
    String nodeType = node.getPrimaryNodeType().getName();
    for (int i = 0; i < PASTE_TYPES.length; i++) {
      String pasteType = PASTE_TYPES[i];
      if (pasteType.equals(nodeType)) return true;
    }
    return isPaste;
  }

  static public class AddFolderActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIFolderForm.class);
    }
  }

  static public class AddDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      UIDocumentFormController uiNewDocumentController = 
        (UIDocumentFormController) uicomp.getChildComponentOfType(UIDocumentFormController.class);
      uiNewDocumentController.showNewDocumentForm();
      uicomp.setRenderedComponent(UIDocumentFormController.class);
    }
  }

  static public class UploadActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIUploadForm.class);
    }
  }

  static public class PasteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIViewController uicomp = (UIViewController) event.getComponent().getParent();
      Information iprovider = findInformationProvider(uicomp);
      UIJCRExplorer uiExplorer = 
        (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node node = uiExplorer.getSelectNode();
      String type = uiExplorer.getClipBoard().getType();
      String srcPath = uiExplorer.getClipBoard().getSrcPath();
      if(srcPath == null){
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIViewController.msg.no-node-selected")));
        return ;
      }
      String nodePath = node.getPath();
      String destPath = nodePath + srcPath.substring(nodePath.lastIndexOf("/"));
      try {
        Workspace workspace = uiExplorer.getSession().getWorkspace();
        if (ClipboardCommand.COPY.equals(type)) {
          workspace.copy(srcPath, destPath);
        } else {
          workspace.move(srcPath, destPath);
        }
        Object[] args = { srcPath, destPath };
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIViewController.msg.node-pasted"), args));
        uiExplorer.broadcastOnChange(node);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class ViewPermissionsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIPermissionsInfo.class);
    }
  }

  static public class ViewPropertiesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIPropertiesInfo.class);
    }
  }

  static public class ViewNodeTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uiForm = (UICommandNode)event.getComponent() ;
      UIViewController uiViewControl = (UIViewController) uiForm.getParent();
      
      UINodeTypeInfo uiInfo = 
        (UINodeTypeInfo) uiForm.getChildComponentOfType(UINodeTypeInfo.class);
      Information iprovider = findInformationProvider(uiViewControl); 
      try {
        NodeType nodetype = uiViewControl.node_.getPrimaryNodeType();
        Collection types = new ArrayList();
        types.add(nodetype);
        NodeType[] mixins = uiViewControl.node_.getMixinNodeTypes();
        if (mixins != null) {
          List list = Arrays.asList(mixins);
          types.addAll(list);
        }
        uiInfo.setNodeTypes(types);
        uiForm.setRenderedComponent(UINodeTypeInfo.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class ImportNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIImport.class);
    }
  }

  static public class ExportNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIExport.class);
    }
  }

  static public class SearchActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UISearch.class);
    }
  }

  static public class ManageActionsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIActionList.class);
    }
  }

  static public class ManageCategoriesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UICategoriesList.class);
    }
  }

  static public class ViewReferencesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIReferencesList.class);
    }
  }  

  static public class ManageVersionsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIVersionInfo.class);
    }
  }
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINode uiForm = (UINode) event.getComponent().getParent().getParent() ;
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm
          .getAncestorOfType(UIJCRExplorer.class);
      Node node = uiExplorer.getSelectNode();
      uiExplorer.broadcastOnChange(node);      
    }
  }
}
