/*
 * Created on Apr 8, 2005
 */
package org.exoplatform.portlets.content.templates.component;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.PermissionValidator;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.templates.TemplateService;

/**
 * @author benjaminmestrallet
 */
public class UITemplateForm extends UISimpleForm implements SelectorHandler{

  static String TEMPLATE_NAME = "template".intern(), CONTENT = "content"
      .intern(), VIEW = "view";
  
  public static final String SEARCH_GROUP = "searchGroup";

  protected UIStringInput templateNameInput_;
  protected UITextArea templateContentInput_;
  protected TemplateService templateService_;
  protected String nodeType_;

  protected String templateType_;
  protected String templateName_;

  private boolean isDialog_;

  protected UIExoPermissionSelector permSelector_;

  public UITemplateForm(TemplateService templateService) throws Exception {
    super("templateForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    templateService_ = templateService;

    templateNameInput_ = new UIStringInput(TEMPLATE_NAME, "");
    templateContentInput_ = new UITextArea(CONTENT, "");
    templateContentInput_.setClazz("large");    
    permSelector_ = new UIExoPermissionSelector(VIEW, null) ;    
    permSelector_.addValidator(PermissionValidator.class) ;
    add(templateNameInput_);
    add(permSelector_) ;
    add(templateContentInput_);

    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public String getSearchGroup() { return SEARCH_GROUP; }

  public void editTemplate(boolean isDialog, String nodeType, String templateName) throws Exception{
    isDialog_ = isDialog;
    nodeType_ = nodeType;
    templateName_ = templateName;

    templateNameInput_.setText(templateName).setEditable(false);

    String templateText = templateService_.getTemplate(isDialog, nodeType_,
        templateName_);
    templateContentInput_.setText(templateText);
  }

  public void addTemplate(boolean isDialog, String nodeType) {
    isDialog_ = isDialog;
    nodeType_ = nodeType;
    templateName_ = null;

    templateNameInput_.setText("").setEditable(true);
    templateContentInput_.setText("");
  }
  
  public void cancel(Selector selector) {
    selector.setRenderedSibling(UITemplateForm.class);
  }

  public void select(Selector selector) throws Exception {
    UIExoPermissionSelector editPrefPerm = (UIExoPermissionSelector) getUIInput(VIEW);
    editPrefPerm.setPermission(selector.getInputValue());
    selector.setRenderedSibling(UITemplateForm.class);
  }  

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITemplateForm uiForm = (UITemplateForm) event.getComponent();
      String content = uiForm.getUIStringInput(CONTENT).getValue();
      UIStringInput inputString = uiForm.getUIStringInput(TEMPLATE_NAME);
      Information iprovider = findInformationProvider(uiForm);      
      String userOrGroup = uiForm.permSelector_.getValue();
      if (userOrGroup == null || userOrGroup.length() == 0) {
        iprovider.addMessage(new ExoFacesMessage("#{UITemplateForm.msg.userOrGroup-is-null}"));
        return;
      }      
      
      try {
        String name = null;
        if (inputString.isEditable())
          name = inputString.getText();
        else
          name = uiForm.templateName_;
        
        String[] roles = { userOrGroup };
        uiForm.templateService_.addTemplate(uiForm.isDialog_, uiForm.nodeType_,
            name, roles, content);
        uiForm.setRenderedSibling(UIListsManager.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITemplateForm uiForm = (UITemplateForm) event.getComponent();
      uiForm.setRenderedSibling(UIListsManager.class);
    }
  }
  
  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UITemplateForm uiForm = (UITemplateForm) event.getComponent();
      UIExoPermissionBrowser uiGroupBrowser = 
        (UIExoPermissionBrowser) uiForm.getSibling(UIExoPermissionBrowser.class);
      uiGroupBrowser.setHandler(uiForm);
      uiForm.setRenderedSibling(UIExoPermissionBrowser.class);
    }
  }  

}
