package org.exoplatform.portlets.content.versions.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.version.Version;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.document.DocumentReaderService;
import org.exoplatform.services.document.diff.AddDelta;
import org.exoplatform.services.document.diff.ChangeDelta;
import org.exoplatform.services.document.diff.DeleteDelta;
import org.exoplatform.services.document.diff.Delta;
import org.exoplatform.services.document.diff.DiffService;
import org.exoplatform.services.document.diff.Revision;

public class UIDiff extends UIExoCommand implements ChangeNodeListener {

  private Node baseNode_;
  private Node previousNode_;
  private DocumentReaderService readerService_;
  private DiffService diffService_;

  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION, CANCEL_ACTION) };

  public UIDiff(DocumentReaderService readerService, DiffService diffService) {
    setId("UIDiff");
    setRendererType("VelocityRenderer");

    readerService_ = readerService;
    diffService_ = diffService;

    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }

  public Parameter[] getCancelParams() {
    return CANCEL_PARAMS;
  }

  public void setVersions(Version baseVersion, Version version)
      throws RepositoryException {
    baseNode_ = baseVersion.getNode("jcr:frozenNode");
    previousNode_ = version.getNode("jcr:frozenNode");
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    setRendered(false);
  }

  public String getText(Node node) throws Exception {
    Node content = node.getNode("jcr:content");
    Property mime = content.getProperty("jcr:mimeType");
    if (mime.getString().startsWith("text"))
      return content.getProperty("jcr:data").getString();
    else {
      return readerService_.getContentAsText(mime.getString(), content
          .getProperty("jcr:data").getStream());
    }
  }

  public List<Delta> getDeltas() throws Exception {
    String previousText = getText(previousNode_);
    String currentText = getText(baseNode_);
    
    String lineSeparator = DiffService.NL;
    
    Object[] orig = StringUtils.split(previousText, lineSeparator);
    Object[] rev = StringUtils.split(currentText, lineSeparator);

    Revision revision = diffService_.diff(orig, rev);
    List<Delta> deltas = new ArrayList<Delta>();
    for (int i = 0; i < revision.size(); i++) {
      deltas.add(revision.getDelta(i));
    }
    return deltas;
  }

  public boolean isDeleteDelta(Delta delta) {
    if (delta instanceof DeleteDelta)
      return true;
    return false;
  }

  public boolean isAddDelta(Delta delta) {
    if (delta instanceof AddDelta)
      return true;
    return false;
  }

  public boolean isChangeDelta(Delta delta) {
    if (delta instanceof ChangeDelta)
      return true;
    return false;
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDiff uicomp = (UIDiff) event.getSource();
      uicomp.setRenderedSibling(UIVersionInfo.class);
    }
  }
}
