package org.exoplatform.services.cms.scripts;

import groovy.lang.GroovyClassLoader;

import java.io.File;
import java.io.InputStream;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.Session;
import javax.jcr.observation.Event;
import javax.jcr.observation.EventIterator;
import javax.jcr.observation.EventListener;
import javax.jcr.observation.ObservationManager;

import org.codehaus.groovy.control.CompilationFailedException;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.configuration.ConfigurationManager;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.impl.BaseResourceLoaderService;
import org.exoplatform.services.cms.impl.ResourceConfig;
import org.exoplatform.services.jcr.RepositoryService;

public class ScriptServiceImpl extends BaseResourceLoaderService implements ScriptService, EventListener {

  private GroovyClassLoader groovyClassLoader_;

  public ScriptServiceImpl(RepositoryService repositoryService,
      CmsConfigurationService cmsConfigService, ConfigurationManager cservice,
      InitParams params) throws Exception {
    super((ResourceConfig)params.getObjectParamValues(ResourceConfig.class).get(0), 
        cservice,cmsConfigService, repositoryService);
    groovyClassLoader_ = createGroovyClassLoader();    
  }
  
  public void start() {
    super.start();
    try {
      initObserver();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  
  protected String getBasePath() {  
    return cmsConfigService_.getJcrPath(BasePath.CMS_SCRIPTS_PATH);
  }  

  private void initObserver() throws Exception {
    String scriptsPath = getBasePath();
    ObservationManager obsManager = getSystemSession(config_.getWorkspace())
        .getWorkspace().getObservationManager();
    obsManager.addEventListener(this, Event.PROPERTY_CHANGED, scriptsPath,
        true, null, null, true);
  }

 
  public String getScriptAsText(String scriptName) throws Exception {
    return getResourceAsText(scriptName);
  }

  public CmsScript getScript(String scriptName) throws Exception {
    CmsScript scriptObject = (CmsScript) localCache_.get(scriptName);
    if (scriptObject != null)
      return scriptObject;
    Node scriptsHome = getResourcesHome();
    Node scriptNode = scriptsHome.getNode(scriptName);

    InputStream iS = scriptNode.getProperty("jcr:data").getStream();
    Class scriptClass = groovyClassLoader_.parseClass(iS);
    PortalContainer pC = PortalContainer.getInstance();
    pC.registerComponentImplementation(scriptName, scriptClass);
    scriptObject = (CmsScript) pC.getComponentInstance(scriptName);
    localCache_.put(scriptName, scriptObject);
    return scriptObject;
  }

  public NodeIterator getScripts() throws Exception {
    return getResources();
  }

  public boolean hasScripts() throws Exception {
    return hasResources();
  }

  public void addScript(String name, String text) throws Exception {
    addResource(name, text);
  }

  public void removeScript(String scriptName) throws Exception {
    removeResource(scriptName);
  }

  protected void removeFromCache(String scriptName) {
    Object cachedobject = localCache_.get(scriptName);
    if (cachedobject != null) {
      groovyClassLoader_ = createGroovyClassLoader();
      localCache_.remove(scriptName);
      PortalContainer pC = PortalContainer.getInstance();
      pC.unregisterComponent(scriptName);
    }
  }

  public void onEvent(EventIterator events) {
    while (events.hasNext()) {
      Event event = events.nextEvent();
      String path = null;
      try {
        path = event.getPath();
        Session jcrSession = getSystemSession(config_.getWorkspace());
        Property property = (Property) jcrSession.getItem(path);
        if ("jcr:data".equals(property.getName())) {
          Node node = (Node) property.getParent();
          removeFromCache(node.getName());
        }
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }

  private GroovyClassLoader createGroovyClassLoader() {
    ClassLoader parentLoader = Thread.currentThread().getContextClassLoader();    
    return new GroovyClassLoader(parentLoader) {
      protected Class findClass(String className) throws ClassNotFoundException {
        String filename = className.replace('.', File.separatorChar)
            + ".groovy";

        InputStream in = null;
        try {
          Node scriptsHome = getResourcesHome();
          Node scriptNode = scriptsHome.getNode(filename);
          in = scriptNode.getProperty("jcr:data").getStream();
        } catch (Exception e) {
          throw new ClassNotFoundException("Could not read " + className + ": " + e);
        }
        try {
          return parseClass(in, filename);
        } catch (CompilationFailedException e2) {
          throw new ClassNotFoundException("Syntax error in " + className
              + ": " + e2);
        }
      }
    };
  }

}
