/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.calendar.component ;

import java.util.ArrayList ;
import java.util.Calendar ;
import java.util.GregorianCalendar ;
import java.util.List ;
import java.util.Map ;
import java.util.ResourceBundle;

import javax.faces.context.FacesContext;
import javax.faces.event.PhaseId ;
import javax.jcr.Node ;

import org.exoplatform.commons.utils.ISO8601 ;
import org.exoplatform.faces.FacesConstants ;
import org.exoplatform.faces.cms.CMSUtils ;
import org.exoplatform.faces.context.PortletExternalContext;
import org.exoplatform.faces.core.Util ;
import org.exoplatform.faces.core.component.UIDateTimeInput ;
import org.exoplatform.faces.core.component.UISimpleForm ;
import org.exoplatform.faces.core.event.ExoActionEvent ;
import org.exoplatform.faces.core.event.ExoActionListener ;
import org.exoplatform.portlets.calendar.Utils;
import org.exoplatform.services.calendar.CalendarService ;
import org.exoplatform.services.cms.JcrInputProperty ;
import org.exoplatform.services.cms.templates.TemplateService ;


public class UICalendarForm extends UISimpleForm {

	private CalendarService calendarService_ ;
	private TemplateService templateService_ ;
	private boolean displayCheckInputMsg_ ;
	private List errorMsgList_ ;
	private boolean editMode_ ;
	private Node calendarNode_ ;

	public UICalendarForm(TemplateService templateService, CalendarService service)  
						throws Exception {

		super("calendarForm", "post", null) ;
		setRendererType("SimpleFormVelocityRenderer")  ;

		this.templateService_ = templateService ;
		setId("UICalendarForm") ;
		this.calendarService_ = service ;
		this.displayCheckInputMsg_ = false ;
		this.editMode_ = true ;

		addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
		addActionListener(SaveActionListener.class, SAVE_ACTION) ;
	}

	public CalendarService getCalendarService() {
		return this.calendarService_ ;
	}
	
	public String getDialogPath() {    
		try {
			return this.templateService_.getDefaultTemplatePath(true, "exo:calendar") ;
		} catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}    

	public String getRenameDialogPath() {    
		try {
			return this.templateService_.getTemplatePath(true, "exo:calendar", "renameDialog") ;
		} catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}    

	public String getViewPath() {
		try {
			return this.templateService_.getDefaultTemplatePath(false, "exo:calendar") ;
		} catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}
	
	public void setNode(Node calendarNode) {
		this.calendarNode_ = calendarNode ;
	}
	
	public Node getNode() {
		return this.calendarNode_ ;
	}
	
	public String getEditAction() { return EDIT_ACTION  ; }

	public String getDeleteAction() { return DELETE_ACTION  ; }

	public void clean() {
		clearInputs() ;
		getChildren().clear() ;
		this.displayCheckInputMsg_ = false ;
	}
	
	public List getErrorMsgList() {
		return this.errorMsgList_ ;
	}

	public void setErrorMsgList(List errorMsgList) {
		this.errorMsgList_ = errorMsgList ;
	}

	public boolean isEditMode() {
		return this.editMode_ ;
	}

	public void setEditMode(boolean editMode) {
		this.editMode_ = editMode ;
	}
	
	public boolean isCheckInputMsgDisplayed() {
		return this.displayCheckInputMsg_ ;
	}

	public void setDisplayCheckInputMsg(boolean displayCheckInputMsg) {
		this.displayCheckInputMsg_ = displayCheckInputMsg ;
	}

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UICalendarForm uiForm = (UICalendarForm) event.getComponent() ;
			UICalendarPortlet calendarPortlet =
				(UICalendarPortlet) uiForm.getAncestorOfType(UICalendarPortlet.class) ;
			calendarPortlet.setRenderedComponent(UICalendar.class) ;
			((UIToolbarPanel) calendarPortlet.getChildComponentOfType(
					UIToolbarPanel.class)).setRendered(true) ;
		}
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
	    PortletExternalContext econtext =  
	    	(PortletExternalContext) FacesContext.getCurrentInstance().getExternalContext();
	    ResourceBundle res = econtext.getApplicationResourceBundle() ;
			FacesContext context = FacesContext.getCurrentInstance() ; 
			String user = FacesContext.getCurrentInstance().getExternalContext().getRemoteUser() ;
			if (user == null) user = "anonymous" ;

			UICalendarForm uiCalendarForm = (UICalendarForm) event.getComponent() ;
			UICalendarPortlet uiCalendarPortlet = (UICalendarPortlet) uiCalendarForm.getAncestorOfType(UICalendarPortlet.class) ;
			Map rawInputs = CMSUtils.prepareMap(uiCalendarForm.getUIInputs()) ;
			JcrInputProperty property ;
			List errorMsgList = new ArrayList(5) ;
			boolean inputOk = true ;
			// Check input
			// calendar name not null
			property = (JcrInputProperty) rawInputs.get("/node") ;
			property.getValue() ;
			if (property == null || "".equals(property.getValue())) {
				inputOk = false ;
				errorMsgList.add(res.getString("UICalendarForm.msg.calendarName-error")) ;
			}

			if (inputOk) {
				Node calendarNode = uiCalendarForm.getNode() ;
		    property  =  (JcrInputProperty) rawInputs.get("/node") ;
				String newCalendarName = (String) property.getValue() ;
		    property  =  (JcrInputProperty) rawInputs.get("/node/exo:shared") ;
		    boolean newSharedValue = Boolean.getBoolean((String) property.getValue()) ;
		    String newCalendarType ;
	    	if (newSharedValue) newCalendarType = CalendarService.PUBLIC_CALENDAR_TYPE ;
	    	else newCalendarType = CalendarService.PRIVATE_CALENDAR_TYPE ;
		    
				List l;
				if (newSharedValue) l = new ArrayList(Utils.getPublicCalendarsName(uiCalendarPortlet.getCurrentUser())) ;
				else l = new ArrayList(Utils.getPrivateCalendarsName(uiCalendarPortlet.getCurrentUser())) ;
				for (int i = 0; i < l.size(); i++) {
					if (newCalendarName.equals(l.get(i))) {
						uiCalendarForm.setDisplayCheckInputMsg(true) ;
						errorMsgList.add("Calendar name \"" + newCalendarName + "\n already exists !") ;
						uiCalendarPortlet.setRenderedComponent(UICalendarForm.class) ;
						return ;
					}
				}
				if (calendarNode != null) {
					String oldCalendarName = calendarNode.getName() ;
					if (oldCalendarName.equals(CalendarService.DEFAULT_CALENDAR)) {
						uiCalendarForm.setDisplayCheckInputMsg(true) ;
						errorMsgList.add("Unable to rename calendar to 'default' !") ;
						uiCalendarForm.setErrorMsgList(errorMsgList) ;
						uiCalendarPortlet.setRenderedComponent(UICalendarForm.class) ;
						return ;
					}
					boolean oldSharedValue = calendarNode.getProperty("exo:shared").getValue().getBoolean() ;
			    if (newCalendarName != oldCalendarName || newSharedValue != oldSharedValue) {
			    	String oldCalendarType ;
			    	if (oldSharedValue) oldCalendarType = CalendarService.PUBLIC_CALENDAR_TYPE ;
			    	else oldCalendarType = CalendarService.PRIVATE_CALENDAR_TYPE ;
			    	uiCalendarForm.calendarService_.renameCalendar(user, oldCalendarType, oldCalendarName, newCalendarType, newCalendarName) ;
			    }
				}
				else uiCalendarForm.calendarService_.createCalendar(user, newCalendarType, newCalendarName) ;

				uiCalendarPortlet.setRenderedComponent(UICalendarManagement.class) ;
			}
			else {
				uiCalendarForm.setDisplayCheckInputMsg(true) ;
				uiCalendarForm.setErrorMsgList(errorMsgList) ;
				uiCalendarPortlet.setRenderedComponent(UICalendarForm.class) ;
				return ;
			}
			uiCalendarForm.clean() ;
		}
	}

}
