/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.forum.component;

import org.exoplatform.faces.core.component.UIIntegerInput;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.EmptyFieldValidator;
import org.exoplatform.services.forum.Category;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIForumForm.java,v 1.7 2004/10/17 13:46:25 tuan08 Exp $
 */
public class UIForumForm extends UISimpleForm {
  static String FORUM_NAME = "forumName" , FORUM_DESC = "forumDesc" ,
                VIEW_FORUM_ROLE = "viewForumRole", CREATE_TOPIC_ROLE = "createTopicRole",
                REPLY_TOPIC_ROLE = "replyTopicRole", MODERATORS = "moderators" ,
                FORUM_ORDER = "order"  ;
    
  Forum forum_ ;
  String categoryId_ ;
  
  public UIForumForm(ForumService service) throws Exception {
    super("forumForm", "post", null) ;
    setId("UIForumForm") ;
    setRendererType("SimpleFormVelocityRenderer") ;
  
    add(new UIStringInput(FORUM_NAME, "").addValidator(EmptyFieldValidator.class)) ;
    add(new UITextArea(FORUM_DESC, "")) ;
    add(new UIStringInput(VIEW_FORUM_ROLE, "")) ;
    add(new UIStringInput(CREATE_TOPIC_ROLE, "")) ;
    add(new UIStringInput(REPLY_TOPIC_ROLE, "")) ;
    add(new UIStringInput(MODERATORS, "")) ;
    add(new UIIntegerInput(FORUM_ORDER, 0)) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION)  ;
    addActionListener(SaveActionListener.class, SAVE_ACTION)  ;
  }
  
  public void setForum(String categoryId, Forum forum) {
    forum_ = forum ;
    categoryId_ = categoryId ;
    if (forum == null) {
      reset() ;
    } else {
      getUIStringInput(FORUM_NAME).setValue(forum.getForumName()) ;
      getUIStringInput(FORUM_DESC).setValue(forum.getDescription()) ;
      getUIStringInput(VIEW_FORUM_ROLE).setValue(forum.getViewForumRole()) ;
      getUIStringInput(CREATE_TOPIC_ROLE).setValue(forum.getCreateTopicRole()) ;
      getUIStringInput(REPLY_TOPIC_ROLE).setValue(forum.getReplyTopicRole()) ;
      getUIStringInput(MODERATORS).setValue(forum.getModerators()) ;
      getUIIntegerInput(FORUM_ORDER).setIntValue(forum.getForumOrder()) ;
    }
  }
  
  public boolean canDecodeInvalidState()  { return categoryId_ != null; }
  
  static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIForumForm uiForm = (UIForumForm) event.getComponent() ;
			uiForm.setRenderedSibling(UIAdminViewCategories.class) ;
		}
  }
  
  static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIForumForm uiForm = (UIForumForm) event.getComponent() ;
      ForumService service = UIForumPortlet.getForumService() ;
      
			Forum forum = uiForm.forum_ ;
      boolean isNew =  false ;
      if(forum == null) {
      	forum = service.createForumInstance() ;
        isNew = true ;
      }
      forum.setForumName(uiForm.getUIStringInput(FORUM_NAME).getValue()) ;
      forum.setDescription(uiForm.getUIStringInput(FORUM_DESC).getValue()) ;
      forum.setViewForumRole(uiForm.getUIStringInput(VIEW_FORUM_ROLE).getValue()) ;
      forum.setCreateTopicRole(uiForm.getUIStringInput(CREATE_TOPIC_ROLE).getValue()) ;
      forum.setReplyTopicRole(uiForm.getUIStringInput(REPLY_TOPIC_ROLE).getValue()) ;
      forum.setModerators(uiForm.getUIStringInput(MODERATORS).getValue()) ;
      forum.setForumOrder(uiForm.getUIIntegerInput(FORUM_ORDER).getIntValue()) ;
      if(isNew) {
        Category category = service.getCategory(uiForm.categoryId_) ;
        service.addForum(category , forum) ;
      } else {
        service.updateForum(forum) ;
      }
      UIAdminViewCategories uiCategories =
       	(UIAdminViewCategories) uiForm.getSibling(UIAdminViewCategories.class) ;
      uiCategories.update() ;
			uiForm.setRenderedSibling(UIAdminViewCategories.class) ;
		}
  }
}