/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.pmanager.component;

import java.util.List;
import javax.portlet.PortletPreferences;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.pmanager.Project;
import org.exoplatform.services.pmanager.ProjectManager;
import org.exoplatform.services.pmanager.ProjectManagerService;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id$
 */
public class UIProjectManager extends UIExoCommand {
  static String NEW_PROJECT = "createProject".intern() ;  
  static String VIEW_DOCUMENTS = "viewDocuments".intern() ;  
  static String CREATE_DOCUMENT = "createDocument".intern() ;  
  static String UPLOAD_DOCUMENT = "uploadDocument".intern() ;  
  static String VIEW_ISSUES = "viewIssues".intern() ;  
  static String CREATE_ISSUE = "createIssue".intern() ; 
  static String VIEW_VERSIONS = "viewVersions".intern() ;  
  static String CREATE_VERSION = "createVersion".intern() ; 
  static String VIEW_COMPONENTS = "viewComponents".intern() ;  
  static String CREATE_COMPONENT = "createComponent".intern() ; 
  static String VIEW_PARTICIPANTS = "viewParticipants".intern() ;  
  static String CREATE_PARTICIPANT = "createPartcipant".intern() ; 
  static String EDIT_PERMISSIONS = "editPermission".intern() ; 
  
  static Parameter[] NEW_PROJECT_PARAMS = { new Parameter(ACTION , NEW_PROJECT)} ;
  static Parameter[] VIEW_DOCUMENTS_PARAMS = { new Parameter(ACTION , VIEW_DOCUMENTS)} ;
  static Parameter[] CREATE_DOCUMENT_PARAMS = { new Parameter(ACTION , CREATE_DOCUMENT)} ;
  static Parameter[] UPLOAD_DOCUMENT_PARAMS = { new Parameter(ACTION , UPLOAD_DOCUMENT)} ;
  static Parameter[] VIEW_ISSUES_PARAMS = { new Parameter(ACTION , VIEW_ISSUES)} ;
  static Parameter[] CREATE_ISSUE_PARAMS = { new Parameter(ACTION , CREATE_ISSUE)} ;
  static Parameter[] VIEW_VERSIONS_PARAMS = { new Parameter(ACTION , VIEW_VERSIONS)} ;
  static Parameter[] CREATE_VERSION_PARAMS = { new Parameter(ACTION , CREATE_VERSION)} ;
  static Parameter[] VIEW_COMPONENTS_PARAMS = { new Parameter(ACTION , VIEW_COMPONENTS)} ;
  static Parameter[] CREATE_COMPONENT_PARAMS = { new Parameter(ACTION , CREATE_COMPONENT)} ;
  static Parameter[] VIEW_PARTICIPANTS_PARAMS = { new Parameter(ACTION , VIEW_PARTICIPANTS)} ;
  static Parameter[] CREATE_PARTICIPANT_PARAMS = { new Parameter(ACTION , CREATE_PARTICIPANT)} ;
  static Parameter[] EDIT_PERMISSIONS_PARAMS = { new Parameter(ACTION , EDIT_PERMISSIONS)} ;

  ProjectManager pmanager_ ;
  List projects_ ;
  String remoteUser_ ;
  
  public UIProjectManager() throws Exception {
    setId("UIProjectManager") ;
    setRendererType("VelocityRenderer") ;
    remoteUser_ = getRemoteUser()  ;
    if(remoteUser_ == null) remoteUser_ = "guest" ;
    PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
    String path = prefs.getValue("default", null) ;
    ProjectManagerService service = 
      (ProjectManagerService) PortalContainer.getComponent(ProjectManagerService.class) ;
    pmanager_ = service.getProjectManager(null, path) ;
    projects_ = pmanager_.getProjects(remoteUser_) ;
    addActionListener(ViewDocumentsActionListener.class, VIEW_DOCUMENTS)  ;
    addActionListener(CreateDocumentActionListener.class, CREATE_DOCUMENT)  ;
    addActionListener(UploadDocumentActionListener.class, UPLOAD_DOCUMENT )  ;
    addActionListener(CreateIssueActionListener.class, CREATE_ISSUE)  ;
    addActionListener(ViewIssuesActionListener.class, VIEW_ISSUES)  ;
    addActionListener(CreateVersionActionListener.class, CREATE_VERSION)  ;
    addActionListener(ViewVersionsActionListener.class, VIEW_VERSIONS)  ;
    addActionListener(CreateComponentActionListener.class, CREATE_COMPONENT)  ;
    addActionListener(ViewComponentsActionListener.class, VIEW_COMPONENTS)  ;
    addActionListener(CreateParticipantActionListener.class, CREATE_PARTICIPANT)  ;
    addActionListener(ViewParticipantsActionListener.class, VIEW_PARTICIPANTS)  ;
    addActionListener(EditPermissionsActionListener.class, EDIT_PERMISSIONS)  ;
  }
  
  public Parameter[] getViewDocumentsParams() { return VIEW_DOCUMENTS_PARAMS ; }
  public Parameter[] getCreateDocumentParams() { return CREATE_DOCUMENT_PARAMS ; }
  public Parameter[] getUploadDocumentParams() { return UPLOAD_DOCUMENT_PARAMS ; }
  public Parameter[] getViewIssuesParams() { return VIEW_ISSUES_PARAMS ; }
  public Parameter[] getCreateIssueParams() { return CREATE_ISSUE_PARAMS ; }
  public Parameter[] getViewVersionsParams() { return VIEW_VERSIONS_PARAMS ; }
  public Parameter[] getCreateVersionParams() { return CREATE_VERSION_PARAMS ; }
  public Parameter[] getViewComponentsParams() { return VIEW_COMPONENTS_PARAMS ; }
  public Parameter[] getCreateComponentParams() { return CREATE_COMPONENT_PARAMS ; }
  public Parameter[] getViewParticipantsParams() { return VIEW_PARTICIPANTS_PARAMS ; }
  public Parameter[] getAddParticipantParams() { return CREATE_PARTICIPANT_PARAMS ; }
  public Parameter[] getEditPermissionsParams() { return EDIT_PERMISSIONS_PARAMS ; }
  
  public ProjectManager getProjectManager() { return pmanager_ ; }
  public void setProjectManager(ProjectManager pmanager) throws Exception { 
    pmanager_ = pmanager ;
    projects_ = pmanager_.getProjects(remoteUser_) ;
  }
  
  public List getProjects() throws Exception { return projects_  ; }
  
  public void update() throws Exception { 
    projects_ = pmanager_.getProjects(remoteUser_) ;
  }
  
  Project findProject(String name) throws Exception {
    for(int i = 0 ; i < projects_.size(); i++) {
      Project project = (Project) projects_.get(i) ;
      if(name.equals(project.getName())) return project ;
    }
    return null ;
  }

  static public class ViewDocumentsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      UIDocumentSearcher uiSearcher = 
        (UIDocumentSearcher) uicomp.getSibling(UIDocumentSearcher.class);
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      uiSearcher.setData(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIDocumentSearcher.class) ;
    }
  }
  
  static public class CreateDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      UIDocumentForm uiForm = 
        (UIDocumentForm) uicomp.getSibling(UIDocumentForm.class);
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      uiForm.setData(project.getDocuments(), UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIDocumentForm.class) ;
    }
  }  
  
  static public class UploadDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      UIUploadForm uiForm = (UIUploadForm) uicomp.getSibling(UIUploadForm.class);
      uiForm.setData(UIUploadForm.UPLOAD_DOCUMENT, project.getDocuments(), UIProjectManager.class);
      uicomp.setRenderedSibling(UIUploadForm.class) ;
    }
  }  
  
  static public class CreateIssueActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      UIIssueForm uiForm = (UIIssueForm) uicomp.getSibling(UIIssueForm.class);
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      uiForm.addNewIssue(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIIssueForm.class) ;
    }
  }  
  
  static public class ViewIssuesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      UIIssueSearcher uiSearcher = (UIIssueSearcher) uicomp.getSibling(UIIssueSearcher.class);
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      uiSearcher.setProject(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIIssueSearcher.class) ;
    }
  } 
  
  static public class ViewVersionsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      UIProjectVersions uiVersions = 
        (UIProjectVersions) uicomp.getSibling(UIProjectVersions.class);
      uiVersions.setData(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIProjectVersions.class) ;
    }
  }
  
  static public class CreateVersionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      UIProjectVersionForm uiForm = 
        (UIProjectVersionForm) uicomp.getSibling(UIProjectVersionForm.class);
      uiForm.setData(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIProjectVersionForm.class) ;
    }
  }  
  
  static public class CreateComponentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      UIProjectComponentForm uiForm = 
        (UIProjectComponentForm) uicomp.getSibling(UIProjectComponentForm.class);
      uiForm.setData(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIProjectComponentForm.class) ;
    }
  }  
  
  static public class ViewComponentsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      UIProjectComponents uiComponents = 
        (UIProjectComponents) uicomp.getSibling(UIProjectComponents.class);
      uiComponents.setData(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIProjectComponents.class) ;
    }
  }
  
  static public class ViewParticipantsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      UIProjectParticipants uiParticipants = 
        (UIProjectParticipants) uicomp.getSibling(UIProjectParticipants.class);
      uiParticipants.setData(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIProjectParticipants.class) ;
    }
  }
  
  static public class CreateParticipantActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      UIProjectParticipantForm uiForm = 
        (UIProjectParticipantForm) uicomp.getSibling(UIProjectParticipantForm.class);
      uiForm.setData(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIProjectParticipantForm.class) ;
    }
  }
  
  static public class EditPermissionsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIProjectManager uicomp = (UIProjectManager) event.getComponent() ;
      String projectName = event.getParameter(OBJECTID) ;
      Project project = uicomp.findProject(projectName) ;
      UIProjectPermissionsForm uiForm = 
        (UIProjectPermissionsForm) uicomp.getSibling(UIProjectPermissionsForm.class);
      uiForm.setData(project, UIProjectManager.class) ;
      uicomp.setRenderedSibling(UIProjectPermissionsForm.class) ;
    }
  }  
}