/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.calendar.impl;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.version.OnParentVersionAction;

import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;

public class CalendarEventType extends NodeTypeImpl {

  public CalendarEventType(NodeTypeManager manager) throws NoSuchNodeTypeException, RepositoryException {
    super(manager);
    this.name = "exo:calendarEvent";
    this.mixin = false;
    this.orderableChild = false;
    this.primaryItemName = null;
    this.declaredSupertypes = new NodeType[2];
    this.declaredSupertypes[0] = manager.getNodeType("nt:base");
    this.declaredSupertypes[1] = manager.getNodeType("exo:categorized");
    
    this.declaredPropertyDefinitions = new PropertyDefinition[8];
    
    // summary : subject of event
    this.declaredPropertyDefinitions[0] = 
      new PropertyDefinitionImpl("exo:summary",          //name
                            this,                        //declaringNodeType
                            PropertyType.STRING,         //requiredType
                            null,                        //valueConstraints 
                            null,                        //defaultValues
                            false,                       //autoCreate       
                            true,                        //mandatory        
                            OnParentVersionAction.IGNORE,//onVersion        
                            false,                       //readOnly         
                            false                        //multiple         
      );
    // description : short description
    this.declaredPropertyDefinitions[1] = 
        new PropertyDefinitionImpl("exo:description",	 //name
                            this,						 //declaringNodeType
                            PropertyType.STRING,		 //requiredType
                            null,                        //valueConstraints 
                            null,                        //defaultValues
                            false,                       //autoCreate       
                            false,                       //mandatory        
                            OnParentVersionAction.IGNORE,//onVersion        
                            false,                       //readOnly         
                            false                        //multiple         
        );
    // comment : notes or comments
    this.declaredPropertyDefinitions[2] = 
        new PropertyDefinitionImpl("exo:comment",        //name
                            this,                        //declaringNodeType
                            PropertyType.STRING,         //requiredType
                            null,                        //valueConstraints 
                            null,                        //defaultValues
                            false,                       //autoCreate       
                            false,                       //mandatory        
                            OnParentVersionAction.IGNORE,//onVersion        
                            false,                       //readOnly         
                            false                        //multiple         
        );
    // location : where event takes place
    this.declaredPropertyDefinitions[3] = 
        new PropertyDefinitionImpl("exo:location",       //name
                            this,                        //declaringNodeType
                            PropertyType.STRING,         //requiredType
                            null,                        //valueConstraints 
                            null,                        //defaultValues
                            false,                       //autoCreate       
                            true,                        //mandatory        
                            OnParentVersionAction.IGNORE,//onVersion        
                            false,                       //readOnly         
                            false                        //multiple         
        );
    // dtstart : date/time start
    this.declaredPropertyDefinitions[4] = 
        new PropertyDefinitionImpl("exo:dtstart",        //name
                            this,                        //declaringNodeType
                            PropertyType.DATE,           //requiredType
                            null,                        //valueConstraints 
                            null,                        //defaultValues
                            false,                       //autoCreate       
                            true,                        //mandatory        
                            OnParentVersionAction.IGNORE,//onVersion        
                            false,                       //readOnly         
                            false                        //multiple         
        );
    // dtend : date/time end
    this.declaredPropertyDefinitions[5] = 
        new PropertyDefinitionImpl("exo:dtend",          //name
                            this,                        //declaringNodeType
                            PropertyType.DATE,           //requiredType
                            null,                        //valueConstraints 
                            null,                        //defaultValues
                            false,                       //autoCreate       
                            false,                       //mandatory        
                            OnParentVersionAction.IGNORE,//onVersion        
                            false,                       //readOnly         
                            false                        //multiple         
      ); 
    // exo:category : category (list of categories in ical)
    // see CategoryMixinType
     
    // fbtype : free/busy type
    this.declaredPropertyDefinitions[6] = 
         new PropertyDefinitionImpl("exo:fbtype",        //name
                            this,                        //declaringNodeType
                            PropertyType.STRING,         //requiredType
                            null,                        //valueConstraints 
                            null,                        //defaultValues
                            false,                       //autoCreate       
                            true,                        //mandatory        
                            OnParentVersionAction.IGNORE,//onVersion        
                            false,                       //readOnly         
                            false                        //multiple         
        );
    // sharetype : share type (private / detail / freebusy)
    this.declaredPropertyDefinitions[7] = 
         new PropertyDefinitionImpl("exo:sharetype",     //name
                            this,                        //declaringNodeType
                            PropertyType.STRING,         //requiredType
                            null,                        //valueConstraints 
                            null,                        //defaultValues
                            false,                       //autoCreate       
                            true,                        //mandatory        
                            OnParentVersionAction.IGNORE,//onVersion        
                            false,                       //readOnly         
                            false                        //multiple         
        ); 
       
    this.declaredChildNodeDefinitions = new NodeDefinition[0];
  }
}
