/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.       *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.forum.test;

import java.util.Comparator;
import java.util.Date;

import org.exoplatform.services.forum.Category;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumService;
import org.exoplatform.services.forum.ForumServiceContainer;
import org.exoplatform.services.forum.Post;
import org.exoplatform.services.forum.Topic;
import org.exoplatform.services.forum.Watcher;
import org.exoplatform.test.BasicTestCase;

/**
 * Thu, May 15, 2003 @   
 * @author: Tuan Nguyen
 * @version: $Id: ForumServiceTestCase.java,v 1.8 2004/07/29 14:09:47 tuan08 Exp $
 * @since: 0.0
 * @email: tuan08@yahoo.com
 */
abstract public class ForumServiceTestCase extends BasicTestCase {
  static protected ForumService service_ ;
  static protected ForumServiceContainer container_ ;
  
  public ForumServiceTestCase(String name) {
    super(name);
  }
  
  public void tearDown() throws Exception {
  }
  
  protected Category createCategory(String name, String desc) throws Exception {
    Category category = service_.createCategoryInstance() ;    
    category.setCategoryName(name) ;
    category.setDescription(desc) ;  
    category = service_.addCategory(category) ;    
    return category ;
  }
  
  protected Forum createForum(Category cat, String name, String desc) throws Exception {
    Forum forum = service_.createForumInstance() ; 
    forum.setForumName(name) ;
    forum.setDescription(desc) ;
    forum = service_.addForum(cat, forum) ;    
    return forum ;
  }
  
  protected Topic createTopic(Forum forum, String name, String desc) throws Exception {
    Topic topic = service_.createTopicInstance() ; 
    topic.setOwner("exo") ;
    topic.setModifiedBy("exo") ;
    topic.setTopic(name) ;
    topic.setDescription(desc) ;
    topic = service_.addTopic(forum, topic) ;
    return topic ;
  }
  
  protected Post createPost(Topic topic, String subject, String message,String owner,Date d) throws Exception {
    Post post = service_.createPostInstance() ; 
    post.setOwner(owner) ;
    post.setModifiedBy("namph") ;
    post.setSubject(subject) ;
    post.setMessage(message) ;
    post.setCreatedDate(d) ;
    post.setModifiedDate(d) ;
    post = service_.addPost(topic, post) ;    
    return post ;
  }
  
  public static class CategoryComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Category c1 = (Category)o1 ;
      Category c2 = (Category)o2 ;
      if((c1.getCategoryName().equals(c2.getCategoryName()))&&
          (c1.getId().equals(c2.getId()))&&
          (c1.getDescription()).equals(c2.getDescription())) {return 0;}
      return -1 ;
    }    
  }
  
  public static class ForumComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Forum f1 = (Forum)o1 ;
      Forum f2 = (Forum)o2 ;
      if((f1.getCategoryId().equals(f2.getCategoryId()))&&
          (f1.getId().equals(f2.getId()))&&
          (f1.getForumName().equals(f2.getForumName()))) { return 0; }
      return -1 ;
    }    
  }
  
  public static class TopicComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Topic t1 = (Topic)o1 ;
      Topic t2 = (Topic)o2 ;
      if((t1.getId().equals(t2.getId()))&&
          t1.getTopic().equals(t2.getTopic())&&
          t1.getDescription().equals(t2.getDescription())) {return 0;}
      return -1 ;
    }   
  }
  
  public static class PostComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Post p1 = (Post)o1 ;
      Post p2 = (Post)o2 ;
      if((p1.getId().equals(p2.getId()))&&
          (p1.getSubject().equals(p2.getSubject()))&&
          (p1.getMessage().equals(p2.getMessage()))) {return 0; }
      return -1;
    }    
  }
  
  public static class WatcherComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Watcher w1 = (Watcher)o1 ;
      Watcher w2 = (Watcher)o2 ;
      if((w1.getUserName().equals(w2.getUserName()))&&
          (w1.getAddress().equals(w2.getAddress()))&&
          (w1.getMessageProtocol().equals(w2.getMessageProtocol()))) {return 0; }
      return -1;
    }    
  }
  
  protected String getDescription() {
    return "Forum Service Test Case" ;
  }
}

