/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.forum.test;

import java.util.Date;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.forum.Category;
import org.exoplatform.services.forum.Forum;
import org.exoplatform.services.forum.ForumServiceContainer;
import org.exoplatform.services.forum.ForumServiceException;
import org.exoplatform.services.forum.Post;
import org.exoplatform.services.forum.Topic;
import org.exoplatform.services.forum.Watcher;
import org.exoplatform.services.forum.hibernate.WatcherNotifyListener;

/**
 * Created by The eXo Platform SARL Author : Tuan Nguyen
 * tuan08@users.sourceforge.net Dec 15, 2005
 */
public class TestForumService extends ForumServiceTestCase {  
  
  private WatcherNotifyListener watcherListener_ ;
  
  public TestForumService(String name) {
    super(name);
  }

  public void setUp() throws Exception {
    if (service_ == null) {
      PortalContainer manager = PortalContainer.getInstance();
      container_ = 
        (ForumServiceContainer) manager.getComponentInstanceOfType(ForumServiceContainer.class);
      service_ = container_.findForumService("exo");
    }
  }

  public void tearDown() throws Exception {
  }

  public void testForumService() throws Exception {    
    Date date = new Date() ;
    Category cat = createCategory("category", "this is a test") ;
    assertTrue("expect found 1 category",service_.getCategories().size()==1) ;    
    Category catDB = (Category)service_.getCategories().get(0) ;    
    assertObject(cat,catDB, new CategoryComparator()) ;      
    /*---Update category---*/
    cat.setDescription("category is updated") ;
    service_.updateCategory(cat) ;            
    catDB = (Category)service_.getCategories().get(0) ;
    assertEquals("expect category is updated","category is updated",catDB.getDescription()) ;
    
    Forum forum = createForum(catDB, "forum", "this is a test") ;
    assertTrue("expect found 1 forum in 'cat'",service_.getForums(catDB.getId()).size() == 1) ;    
    Forum forumDB = (Forum)service_.getForums(catDB.getId()).get(0) ;
    assertObject(forum,forumDB, new ForumComparator()) ;
    /*-------update forum-----*/
    forum.setDescription("forum is updated") ;
    service_.updateForum(forum) ;
    forumDB = (Forum)service_.getForums(catDB.getId()).get(0) ;
    assertEquals("expect forum is updated","forum is updated",forumDB.getDescription()) ;
        
    Watcher forumWatcher = service_.createWatcher(forum) ;
    forumWatcher.setUserName("guest") ;
    forumWatcher.setMessageProtocol("smtp") ;
    forumWatcher.setAddress("exo.platform@laposte.net") ;
    service_.saveWatcher(forumWatcher) ;
    Watcher forumWatcherDB = service_.getWatcher(forum, "guest") ;
    assertTrue("expect forum watcher is found", forumWatcherDB != null) ;
    System.out.println("forumWatcher:"+forumWatcher.hashCode()+"\nforumWatcherDB:"+forumWatcherDB.hashCode()) ;
    
    
    Topic topic = createTopic(forum, "topic", "this is a test") ;
    assertEquals("check topic : ", "topic", topic.getTopic()) ;
    assertEquals("check topic desc: ", "this is a test", topic.getDescription()) ;
    Watcher topicWatcher = service_.createWatcher(topic) ;
    topicWatcher.setUserName("guest") ;
    topicWatcher.setMessageProtocol("smtp") ;
    topicWatcher.setAddress("exo.platform@laposte.net") ;
    service_.saveWatcher(topicWatcher) ;
    topicWatcher = service_.getWatcher(topic, "guest") ;
    assertTrue("expect topic watcher is found", topicWatcher != null) ;
            
    Post post = createPost(topic, "post", "this is a test","exo",date) ;
    assertEquals("check post subject : ", "post", post.getSubject()) ;
    assertEquals("check post message : ", "this is a test", post.getMessage()) ;
    forum = service_.getForum(forum.getId()) ;// with no cache forum has been updated in db
    topic = service_.getTopic(topic.getId()) ;// with no cache topic has been updated in db
    assertEquals("check forum topic count: ", 1 , forum.getTopicCount()) ;
    assertEquals("check forum post count: ", 1 , forum.getPostCount()) ;
    assertEquals("check topic post  count: ", 1 , topic.getPostCount()) ;
    
    
    Post post2 = createPost(topic, "post2", "this is a test","exo",date) ;
    assertEquals("check post subject : ", "post2", post2.getSubject()) ;
    assertEquals("check post message : ", "this is a test", post.getMessage()) ;
    
    topic = service_.getTopic(topic.getId()) ;// with no cache topic has been updated in db
    forum = service_.getForum(forum.getId()) ;// with no cache forum has been updated in db
    assertEquals("check topic post  count: ", 2 , topic.getPostCount()) ;
    assertEquals("check forum topic count: ", 1 , forum.getTopicCount()) ;
    assertEquals("check forum post count: ", 2 , forum.getPostCount()) ;
    
    
    cat = createCategory("2nd category", "this is a test") ;
    List postList = service_.getPosts(topic.getId()).getAll() ;
    assertEquals("check number of post in topic: ", 2 , postList.size()) ;
    for (int i = 0; i < postList.size(); i++) {
      Post test = (Post) postList.get(i) ;
    }
          
    service_.removePost(post2.getId()) ;
    forum = service_.getForum(forum.getId()) ;// with no cache forum has been updated in db
    topic = service_.getTopic(topic.getId()) ;// with no cache topic has been updated in db
    assertEquals("check forum post count: ", 1 , forum.getPostCount()) ;
    assertEquals("check forum topic count: ", 1 , forum.getTopicCount()) ;
    assertEquals("check topic post  count: ", 1 , topic.getPostCount()) ;
    try {
      service_.removeTopic(topic.getId()) ;
      topic = service_.getTopic(topic.getId()) ;
    } catch (ForumServiceException ex) {
      assertEquals("check topic not found exception ", ex.TOPIC_NOT_FOUND, ex.getErrorCode()) ;
    }
    
    try {
      service_.removeForum(forum.getId()) ;
      forum = service_.getForum(forum.getId()) ;
    } catch (ForumServiceException ex) {
      assertEquals("check forum not found exception ", ex.FORUM_NOT_FOUND, ex.getErrorCode()) ;
    }
    
    forum = createForum(cat, "forum", "this is a test") ;
    for (int i = 0; i < 25; i++) {
      createTopic(forum, "topic" + i, "this is a test") ;
    }
    List topicList = service_.getTopics(forum.getId()).getAll() ;
    assertEquals("check number of topics in forum: ", 25 , topicList.size()) ;
    PageList pages = service_.getTopics(forum.getId()) ;
    pages.setPageSize(10) ;
    topicList = pages.getPage(1) ;
    assertEquals("check number of topics in forum: ", 10 , topicList.size()) ;
    
    try {
      service_.removeCategory(cat.getId()) ;
      cat = service_.getCategory(cat.getId()) ;
    } catch (ForumServiceException ex) {
      assertEquals("check category not found exception ", ForumServiceException.CATEGORY_NOT_FOUND, ex.getErrorCode()) ;
    }      
  }
  protected String getDescription() {
    return "Test Forum Service";
  }
  
}
