/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.pmanager;

import java.util.GregorianCalendar;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.idgenerator.IDGeneratorService;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Mar 23, 2005
 * @version $Id$
 */
public class Project {
  private Node node_ ;
  private ProjectACL acl_ ;
  
  public Project(Node node) {
    node_ = node ;
  } 
  
  public ProjectACL getProjectACL() { return acl_ ; }
  public void  setProjectACL(ProjectACL acl) { acl_ = acl ; }
  
  public String getName() throws Exception { return node_.getName() ; }
  
  public String getOwner() throws Exception {
    return node_.getProperty("exo:owner").getString() ;
  }
  
  public String getDesciption() throws Exception {
    if(node_.hasProperty("exo:description")) {
      return node_.getProperty("exo:description").getString() ;
    }
    return "N/A"  ;
  }
  
  public Node getProjectNode() { return node_ ; }
  
  public Node getDocuments() throws Exception {
    return node_.getNode("documents") ;
  }
  
  public Node createDocument(String owner, String name, String mimeType) throws Exception {
    Node documents = node_.getNode("documents") ;
    Node document = DocumentUtil.createDocument(documents, owner, name, mimeType) ;
    return document ;
  }
  
  public Node getDocument(String name) throws Exception {
    Node documents = node_.getNode("documents") ;
    if(!documents.hasNode(name)) return null ;
    return documents.getNode(name) ;
  }
  
  public Node removeDocument(String name) throws Exception {
    Node documents = node_.getNode("documents") ;
    if(!documents.hasNode(name)) return null ;
    Node document = documents.getNode(name) ;
    document.remove() ;
    documents.save() ;
    return document ;
  }
  
  public NodeIterator getDocumentIterator() throws Exception {
    return  node_.getNode("documents").getNodes() ;
  }
  
  
  public Node getIssuesNode() throws Exception {
    return node_.getNode("issues") ;
  }
  
  public Node createIssue(String creator, String type) throws Exception {
    Node issues = node_.getNode("issues") ;
    IDGeneratorService service = 
      (IDGeneratorService) PortalContainer.getComponent( IDGeneratorService.class);
    String id = service.generateStringID(issues) ;
    Node issue = issues.addNode(id, "exo:issue");
    issue.setProperty("exo:type", type);
    issue.setProperty("exo:creator", creator);
    GregorianCalendar date = new GregorianCalendar() ;
    issue.setProperty("exo:createdDate", date);
    issue.setProperty("exo:modifiedDate", date);
    issue.addNode("comments", "exo:container");
    issue.addNode("tasks", "exo:container");
    return issue ;
  }
  
  public Node getIssue(String id) throws Exception {
    Node issues = node_.getNode("issues") ;
    if(!issues.hasNode(id)) return null ;
    return issues.getNode(id) ;
  }
  
  public Node removeIssue(String id) throws Exception {
    Node issues = node_.getNode("issues") ;
    if(!issues.hasNode(id)) return null ;
    Node issue = issues.getNode(id) ;
    issue.remove() ;
    return issue ;
  }
  
  public NodeIterator getIssueIterator() throws Exception {
    return node_.getNode("issues").getNodes() ;
  }
  
  public NodeIterator getVersions() throws Exception {
    return  node_.getNode("versions").getNodes() ;
  }
  
  public NodeIterator getComponents() throws Exception {
    return  node_.getNode("components").getNodes() ;
  }
  
  public NodeIterator getParticipants() throws Exception {
    return  node_.getNode("participants").getNodes() ;
  }
  
  public Node getPermissions() throws Exception {
    return  node_.getNode("permissions");
  }
  
  public void save() throws Exception {  node_.save() ; }
}