/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.pmanager;

import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import javax.jcr.Node;
import javax.jcr.NodeIterator ;
import org.exoplatform.container.PortalContainer ;
import org.exoplatform.services.organization.OrganizationService ;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Mar 23, 2005
 * @version $Id$
 */
public class ProjectManager {
  private Node node_ ;
  
  public ProjectManager(Node node) {
    node_ = node ;
  } 
  
  public Node getProjectNode() { return node_ ; }
  
  public String getName() throws Exception {
    return node_.getName() ;
  }
  
  public String getDesciption() throws Exception {
    if(node_.hasProperty("exo:description")) {
      return node_.getProperty("exo:description").getString() ;
    }
    return "N/A"  ;
  }
  
  public List getProjects(String user) throws Exception {
    OrganizationService service =
      (OrganizationService) PortalContainer.getComponent(OrganizationService.class) ;
    NodeIterator i = node_.getNodes();
    List projects = new ArrayList() ;
    while(i.hasNext()) {
      Node child = i.nextNode() ;
      Project project = new  Project(child) ;
      ProjectACL acl = new ProjectACL(user, project,  service) ;
      project.setProjectACL(acl) ;
      projects.add(project) ;
    }
    return projects;
  }
  
  public boolean hasProject(String name) throws Exception {
    return node_.hasNode(name) ;
  }
  
  public Project getProject(String name) throws Exception {
    Node projectNode = node_.getNode(name) ;
    return new Project(projectNode) ;
  }
  
  public Project createProject(String name, String description, String owner) throws Exception {
    Node projectNode  = node_.addNode(name, "exo:project") ;
    projectNode.setProperty("exo:owner", owner);
    projectNode.setProperty("exo:createdDate", new GregorianCalendar());
    projectNode.setProperty("exo:modifiedDate", new GregorianCalendar());
    if(description != null && description.length() > 0) {
      projectNode.setProperty("exo:description", description);
    }
    projectNode.addNode("documents", "exo:container");
    projectNode.addNode("issues", "exo:container");
    projectNode.addNode("components", "exo:container");
    projectNode.addNode("versions", "exo:container");
    projectNode.addNode("participants", "exo:container");
    Node permissions = projectNode.addNode("permissions", "exo:resource");
    permissions.setProperty(Constant.PROJECT_PERMISSION_VIEW_ISSUE, "*:/guest") ;
    permissions.setProperty(Constant.PROJECT_PERMISSION_CREATE_ISSUE, "member:/admin") ;
    permissions.setProperty(Constant.PROJECT_PERMISSION_ADMIN_ISSUE, "member:/admin") ;
    permissions.setProperty(Constant.PROJECT_PERMISSION_VIEW_DOCUMENT, "*:/guest") ;
    permissions.setProperty(Constant.PROJECT_PERMISSION_CREATE_DOCUMENT, "member:/admin") ;
    permissions.setProperty(Constant.PROJECT_PERMISSION_ADMIN_DOCUMENT, "member:/admin") ;
    permissions.setProperty(Constant.PROJECT_PERMISSION_ADMIN_PROJECT, "member:/admin") ;
    node_.save() ;
    Project project = new Project(projectNode) ;
    ProjectUtil.createParticipant(project, "guest", "guest/anonymous user") ;
    project.save() ;
    return project ;
  }
}