/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.pmanager.impl.indexing;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import org.apache.lucene.document.DateField;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.exoplatform.services.indexing.IndexingService;
import org.exoplatform.services.jcr.index.JCRIndexerPlugin;
import org.exoplatform.services.jcr.index.NodeDataConverter;
import org.exoplatform.services.pmanager.Constant;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Mar 31, 2005
 * @version $Id$
 */
public class ExoIssueConverter implements NodeDataConverter {
  static String[] NODE_TYPES = { "exo:issue" }  ;
  
  public String[] getCanHandleNodeType()  { return NODE_TYPES  ; }
  
  public Document createDocument(Node node, JCRIndexerPlugin plugin)  throws Exception  {
    Document doc = new Document();
    String path = node.getPath() ;
    //global field
    doc.add(Field.Keyword(IndexingService.IDENTIFIER_FIELD, path)) ;
    doc.add(Field.Keyword(IndexingService.MODULE_FIELD, plugin.getPluginIdentifier())) ;
    String prop = node.getProperty(Constant.ISSUE_CREATOR).getString();
    doc.add(Field.Keyword(IndexingService.AUTHOR_FIELD, prop)) ;
    doc.add(Field.Text(IndexingService.TITLE_FIELD, path)) ;
    prop = node.getProperty(Constant.ISSUE_SUMMARY).getString();
    doc.add(Field.Text(IndexingService.DESCRIPTION_FIELD, prop)) ;
    doc.add(Field.UnStored(IndexingService.DOCUMENT_FIELD, getIndexedText(node))) ;
    doc.add(Field.UnStored(IndexingService.DOCUMENT_ACCESS_ROLE, "public")) ;
    //customized jcr field
    doc.add(Field.Text(JCRIndexerPlugin.NAME_FIELD, node.getName())) ;
    doc.add(Field.Text(JCRIndexerPlugin.NODE_TYPE_FIELD, node.getPrimaryNodeType().getName())) ;
    doc.add(Field.Text(JCRIndexerPlugin.MIME_TYPE_FIELD, "exo/issue")) ;
    prop = DateField.dateToString(node.getProperty(Constant.ISSUE_CREATED_DATE).getDate().getTime()) ;
    doc.add(Field.Text(JCRIndexerPlugin.CREATED_DATE, prop)) ;
    prop = DateField.dateToString(node.getProperty(Constant.ISSUE_MODIFIED_DATE).getDate().getTime()) ;
    doc.add(Field.Text(JCRIndexerPlugin.MODIFIED_DATE, prop)) ;
    //customized issue field
    prop = node.getProperty(Constant.ISSUE_STATUS).getString();
    doc.add(Field.Text(Constant.ISSUE_STATUS, prop)) ;
    prop = node.getProperty(Constant.ISSUE_ASSIGNEE).getString();
    doc.add(Field.Text(Constant.ISSUE_ASSIGNEE, prop)) ;
    prop = node.getProperty(Constant.ISSUE_REPORTER).getString();
    doc.add(Field.Text(Constant.ISSUE_REPORTER, prop)) ;
    return doc ;
  }
  
  public String getNodeDataAsText(Node node) throws Exception {
    return getIndexedText(node) ;
  }
  
  private String getIndexedText(Node node) throws Exception {
    StringBuffer b = new StringBuffer() ;
    b.append(node.getProperty(Constant.ISSUE_SUMMARY).getString()).append("\n\n") ;
    b.append(node.getProperty(Constant.ISSUE_DESCRIPTION).getString()).append("\n\n") ;
    b.append(node.getProperty(Constant.ISSUE_RESOLUTION).getString()).append("\n\n") ;
    NodeIterator i = node.getNode(Constant.ISSUE_COMMENTS).getNodes() ;
    while(i.hasNext()) {
      Node comment = i.nextNode() ;
      b.append(comment.getProperty(Constant.ISSUE_COMMENT_DATA).getString()).append("\n") ;
    }
    return b.toString();
  }
}