/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.net.MalformedURLException;
import java.net.URL;

/**
 * An implementation of the import configuration url relsover that understands some of the
 * url used by the eXo platform confugration files, it does not, however, understand url
 * file:${}, or jar:
 * 
 * @author Hatim Khan
 */
public class DefaultImportConfigurationURLResolver implements IImportConfigurationURLResolver
{

  /**
   * Should return a well formatted URL given the unresolved URL. The unresolved URL is a
   * string value taken as is from the &lt;import&gt; element. For example, the value of
   * the unresloved URL could be something like file:${tobeevaluated},
   * jar:/conf/confiugration.xml, war:/conf/org-configuration.xml, file:C:/lab/file.xml.
   * However, this method will only recognize unresolved URL that starts with war: or
   * file: but not jar: nor file:${}. If this method is unable to resolve the url then
   * null will be returned.
   * 
   * @param pUnresolvedURL the unresolved URL
   * @param pInclusingFile the file in which the unresolved URL appeared
   * @return please see method description
   */
  public URL resolveImportURL(String pUnresolvedURL, URL pInclusingFile)
  {
    URL result = null;

    if (pUnresolvedURL == null) return null;
    try
    {
      if (pUnresolvedURL.startsWith("war:"))
      {
        String webinfoDir = getWebInfoDirectory(pInclusingFile);
        if (webinfoDir != null)
        {
          String importurl = webinfoDir + pUnresolvedURL.substring("war:".length());
          result = new URL(importurl);
        }
      }
      else if ((!pUnresolvedURL.startsWith("file:${")) && pUnresolvedURL.startsWith("file:"))
      {
        result = new URL(pUnresolvedURL);
      }
    }
    catch (MalformedURLException e)
    {
      //ignore
    }

    return result;
  }

  private String getWebInfoDirectory(URL pURL)
  {
    if (pURL == null) return null;

    // example
    // file:/C:/exo-workspace/exo-tomcat-1.0/webapps/portal/WEB-INF/conf/configuration.xml
    // another example
    // jar:file:/C:\exo-workspace\exo-tomcat-1.0/webapps/portal.war!/WEB-INF/conf/configuration.xml

    String url = pURL.toString();
    int index = url.lastIndexOf("/WEB-INF");

    return (index > -1) ? url.substring(0, index) + "/WEB-INF" : null;
  }
}