/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.eclipse.core.portal.model.INode;

/**
 * To be used as part of the Configuration data object. Although this class is not aware
 * of XStream, please do not change the name of the private fields since XStream is
 * dependent on them to serialize from/to xml.
 * 
 * @author Hatim Khan
 */
public class GlobalParam implements INode
{
  private List initParams = null;
  private transient Object mParent = null;

  /**
   * @return Returns a list of ValuesParam instances
   */
  public List getInitParams()
  {
    return initParams;
  }

  /**
   * @param initParams The initParams to set.
   */
  public void setInitParams(List initParams)
  {
    this.initParams = initParams;
  }

  public Object[] getChildren()
  {
    List result = new ArrayList();
    if (initParams != null)
    {
      Iterator iterator = initParams.iterator();
      while (iterator.hasNext())
      {
        Parameter parameter = (Parameter) iterator.next();
        parameter.setParent(this);
        result.add(parameter);
      }
    }
    return result.toArray();
  }

  /**
   * @return Returns the parent.
   */
  public Object getParent()
  {
    return mParent;
  }

  /**
   * @param pParent The parent to set.
   */
  public void setParent(Object pParent)
  {
    mParent = pParent;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#removeChild(java.lang.Object)
   */
  public boolean removeChild(Object pChild)
  {
    if (pChild == null) return false;
    boolean removed = false;
    if ((initParams != null) && (initParams.remove(pChild)))
    {
      removed = true;
    }

    // we disconnect the child from the parent if we are able to remove it
    if (removed)
    {
      if (pChild instanceof INode) ((INode) pChild).setParent(null);
    }

    return removed;
  }

  public List getParametersKeys()
  {
    List result = new ArrayList();
    addParameterKeys(result, initParams);
    return result;
  }

  private void addParameterKeys(List pTarget, List pSource)
  {
    if (pTarget == null) return;
    if (pSource == null) return;
    Iterator iterator = pSource.iterator();
    while (iterator.hasNext())
    {
      Parameter parameter = (Parameter) iterator.next();
      String name = parameter.getName();
      if ((name != null) && (name.length() > 0))
      {
        pTarget.add(name.trim());
      }
    }
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#addChild(java.lang.Object)
   */
  public void addChild(Object pChild)
  {
    if (pChild == null) return;
    if (pChild instanceof INode)
    {
      INode node = (INode) pChild;
      node.setParent(this);
    }

    if (pChild instanceof ValuesParam)
    {
      ValuesParam newValue = (ValuesParam) pChild;
      if (initParams == null) initParams = new ArrayList();
      initParams.add(newValue);
    }
  }

}