/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.List;

/**
 * The context from which you can obtain certain information about an installation of the
 * eXo platform. For example, if the installation in question is for an eXo Tomcat, then
 * using this context you can get the home directory of the installation, the jdk version,
 * etc.
 * 
 * @author Hatim Khan
 */
public interface IExoPlatformInstallationInfo
{
  public static final String JDK_1_5 = "1.5";
  public static final String JDK_1_4 = "1.4";

  /**
   * @return the directory name of the installation of the exo platform, for example, if
   *         the installation is for exo tomcat, then the directory might look like this
   *         'c:\exo-tomcat\'
   */
  public String getInstallationDirectory();

  /**
   * @return the version number of the jdk that the exo platform installation will be
   *         runing on. An example of the version number would be "1.4.x", "1.4.1",
   *         "1.5.x", etc.
   */
  public String getJDKVersion();

  /**
   * @return a list of Strings that points to all available library folders (full path).
   *         For example, exo tomcat installation will only have one library folder
   *         (c:\exo-tomcat\common\lib)
   */
  public List getLibraryFolders();

  /**
   * @return the portal webapp folder as a string.
   */
  public String getPortalWebappFolder();
}