/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.io.File;
import java.io.FilenameFilter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.exoplatform.eclipse.core.common.Utils;

/**
 * This class provides a few methods to scan all jar files for a given directory.
 * 
 * @author Hatim Khan
 */
public class JarsScanner
{
  private File mDirectory;
  private FilenameFilter mJarFilter = new FilenameFilter()
  {
    public boolean accept(File pDir, String pName)
    {
      return pName.endsWith(".jar");
    }
  };

  public JarsScanner(String pDirectory)
  {
    mDirectory = new File(pDirectory);
  }

  /**
   * This method scans all the jar files and looks for the given entryInJar in them
   * 
   * @param pEntryInJar the entry that you want to look for in the jar, must start with
   *          "/" for example ("/conf/configuration.xml")
   * @return a list of URL instances that can be used to directly access the entry in the
   *         jar, the URL instance will be included in the list only if the entry exists
   *         in the jar
   */
  public List scan(String pEntryInJar)
  {
    List result = new ArrayList();
    File[] files = mDirectory.listFiles(mJarFilter);
    if (files == null) return result;

    for (int i = 0, sizei = files.length; i < sizei; i++)
    {
      try
      {
        String jarURLStr = "jar:" + files[i].toURL().toString() + "!" + pEntryInJar;
        URL jarURL = new URL(jarURLStr);
        if (Utils.exists(jarURL)) result.add(jarURL);
      }
      catch (MalformedURLException e)
      {
        // simply ignore
      }
    }
    return result;
  }
}