/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.eclipse.core.common.Utils;
import org.exoplatform.eclipse.core.portal.model.INode;

/**
 * Represents an object parameter to the service configuration. Although this class is not
 * aware of XStream, please do not change the name of the private fields since XStream is
 * dependent on them to serialize from/to xml.
 * 
 * @author Hatim Khan (from Tuan Nguyen work)
 */
public class ObjectParam extends Parameter
{
  private String type;
  private List properties;

  /**
   * @return Returns a list of Property instances
   */
  public List getProperties()
  {
    return properties;
  }

  /**
   * @param properties The properties to set.
   */
  public void setProperties(List properties)
  {
    this.properties = properties;
  }

  /**
   * @return Returns the type.
   */
  public String getType()
  {
    return type;
  }

  /**
   * @param type The type to set.
   */
  public void setType(String type)
  {
    this.type = type;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#getChildren()
   */
  public Object[] getChildren()
  {
    List result = new ArrayList();
    addAfterAttachingToParent(result, properties);

    return result.toArray();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#removeChild(java.lang.Object)
   */
  public boolean removeChild(Object pChild)
  {
    if (pChild == null) return false;
    boolean removed = false;
    if ((properties != null) && (properties.remove(pChild)))
    {
      removed = true;
    }

    // we disconnect the child from the parent if we are able to remove it
    if (removed)
    {
      if (pChild instanceof INode) ((INode) pChild).setParent(null);
    }

    return removed;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#addChild(java.lang.Object)
   */
  public void addChild(Object pChild)
  {
    if (pChild == null) return;
    if (pChild instanceof INode)
    {
      INode node = (INode) pChild;
      node.setParent(this);
    }

    if (pChild instanceof Property)
    {
      Property newProperty = (Property) pChild;
      if (properties == null) properties = new ArrayList();
      properties.add(newProperty);
    }
  }

  public List getPropertiesKeys()
  {
    List keys = new ArrayList();
    if (properties != null)
    {
      for (Iterator iterator = properties.iterator(); iterator.hasNext();)
      {
        Property property = (Property) iterator.next();
        String key = property.getName();
        if ((key != null) && (key.length() > 0))
        {
          keys.add(key);
        }
      }
    }
    return keys;
  }

  public Object getCopy()
  {
    ObjectParam copy = new ObjectParam();
    populateCopy(copy);
    copy.setType(type);
    copy.setProperties(getProperitiesCopy());

    return copy;
  }

  public List getProperitiesCopy()
  {
    List copyValues = new ArrayList();
    if (properties != null)
    {
      for (Iterator iterator = properties.iterator(); iterator.hasNext();)
      {
        Property property = (Property) iterator.next();
        copyValues.add(property.getCopy());
      }
      return copyValues;
    }
    else return null;
  }

  public boolean isEquivalentProperitiesList(List p1)
  {
    if ((p1 == null) && (properties == null)) return true;
    if (p1 == null) return false;
    if (properties == null) return false;

    if (p1.size() != properties.size()) return false;
    for (int i = 0, sizei = p1.size(); i < sizei; ++i)
    {
      Property v1 = (Property) p1.get(i);
      Property v2 = (Property) properties.get(i);
      if (!(v1.isEquivalent(v2))) return false;
    }

    return true;
  }

  public boolean isEquivalent(Object pObject)
  {
    boolean paramSame = super.isEquivalent(pObject);
    if ((!paramSame) || !(pObject instanceof ObjectParam)) return false;
    
    ObjectParam param = (ObjectParam) pObject;
    boolean typeSame = Utils.isEquivalentString(this.type, param.type);
    if (!typeSame) return false;

    return isEquivalentProperitiesList(param.properties);
  }

}