/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;

/**
 * Represent an object that can hold a name and a value. Although this class is not aware
 * of XStream, please do not change the name of the private fields since XStream is
 * dependent on them to serialize from/to xml.
 * 
 * @author Hatim Khan (from Tuan Nguyen work)
 */
public class Property implements INode
{
  private String name;
  private String value;
  private transient Object mParent = null;
  private transient ValueHolder mValueHolder = null;

  /**
   * @return Returns the name.
   */
  public String getName()
  {
    return name;
  }

  /**
   * @param name The name to set.
   */
  public void setName(String name)
  {
    this.name = name;
  }

  /**
   * @return Returns the value.
   */
  public String getValue()
  {
    return value;
  }

  /**
   * @param value The value to set.
   */
  public void setValue(String value)
  {
    this.value = value;
    createValueHolder();
  }

  private void createValueHolder()
  {
    if (value == null) return;
    if (mValueHolder == null) mValueHolder = new ValueHolder();
    mValueHolder.setParent(this);
    mValueHolder.setValue(value);
  }

  public Object[] getChildren()
  {
    createValueHolder();
    List result = new ArrayList();
    if (mValueHolder != null)
    {
      result.add(mValueHolder);
    }
    return result.toArray();
  }

  public void setParent(Object pParent)
  {
    mParent = pParent;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#getParent()
   */
  public Object getParent()
  {
    return mParent;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#removeChild(java.lang.Object)
   */
  public boolean removeChild(Object pChild)
  {
    if ((pChild == null) || !(pChild instanceof ValueHolder) || (mValueHolder == null)) return false;
    boolean removed = false;

    if (mValueHolder == pChild)
    {
      mValueHolder = null;
      value = null;
      removed = true;
    }

    // we disconnect the child from the parent if we are able to remove it
    if (removed)
    {
      if (pChild instanceof INode) ((INode) pChild).setParent(null);
    }

    return removed;
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.model.INode#addChild(java.lang.Object)
   */
  public void addChild(Object pChild)
  {
    // we are not actually adding a child, we will instead replace a child
    if (pChild == null) return;

    if (pChild instanceof ValueHolder)
    {
      ValueHolder newValueHolder = (ValueHolder) pChild;
      String newValue = newValueHolder.getValue();
      if (newValue == null)
      {
        value = null;
        if (mValueHolder != null)
        {
          mValueHolder.setParent(null);
          mValueHolder = null;
        }
      }
      else
      {
        value = newValue;
        createValueHolder();
      }
    }
  }

  public Property getCopy()
  {
    Property copy = new Property();
    copy.name = this.name;
    copy.value = this.value;

    return copy;
  }

  public boolean isEquivalent(Property pTarget)
  {
    if (pTarget == null) return false;

    return isSameString(pTarget.getName(), this.name) && isSameString(pTarget.getValue(), this.value);
  }

  private boolean isSameString(String p1, String p2)
  {
    if ((p1 == null) && (p2 == null)) return true;
    if (p1 == null) return false;
    if (p2 == null) return false;

    return (p1.equals(p2));
  }
}