/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.exoservices;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;

/**
 * Inspects a given eXo platform installation for all available service configurations and
 * global parameters.
 * 
 * @author Hatim Khan
 */
public class ServicesInspector
{
  private IImportConfigurationURLResolver mResolver;
  private IExoPlatformInstallationInfo mInstallationInfo;

  public ServicesInspector(IExoPlatformInstallationInfo pInstallationInfo,
      IImportConfigurationURLResolver pResolver)
  {
    mInstallationInfo = pInstallationInfo;
    mResolver = pResolver;
  }

  public Configuration getRootServices(IProgressMonitor pMonitor) throws IOException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    ServicesMerger merger = new ServicesMerger();
    List urls = new ArrayList();
    boolean isJDK15 = IExoPlatformInstallationInfo.JDK_1_5.equalsIgnoreCase(mInstallationInfo.getJDKVersion());

    List libraryFolders = mInstallationInfo.getLibraryFolders();
    Iterator libraryFoldersIterator = libraryFolders.iterator();
    while (libraryFoldersIterator.hasNext())
    {
      JarsScanner scanner = new JarsScanner((String) libraryFoldersIterator.next());
      urls.addAll(scanner.scan("/conf/configuration.xml"));
      // there is a special case for jdk1.5, please see exo platform souce code
      // (RootContainer.getInstance())
      if (isJDK15)
      {
        urls.addAll(scanner.scan("/conf/configuration-jvm15.xml"));
      }
    }

    Iterator iterator = urls.iterator();
    pMonitor.beginTask("RootContainer ", urls.size());
    while (iterator.hasNext())
    {
      URL url = (URL) iterator.next();
      merger.add(ConfigurationParser.loadConfiguration(url, mResolver, new SubProgressMonitor(pMonitor, 1, SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK)));
    }

    pMonitor.done();
    return merger.getMergedServices();
  }

  public Configuration getCommonPortalServices(IProgressMonitor pMonitor) throws IOException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    ServicesMerger merger = new ServicesMerger();
    List urls = new ArrayList();

    List libraryFolders = mInstallationInfo.getLibraryFolders();
    Iterator libraryFoldersIterator = libraryFolders.iterator();
    while (libraryFoldersIterator.hasNext())
    {
      JarsScanner scanner = new JarsScanner((String) libraryFoldersIterator.next());
      urls.addAll(scanner.scan("/conf/portal/configuration.xml"));
    }

    Iterator iterator = urls.iterator();
    pMonitor.beginTask("PortalContainer ", urls.size());
    while (iterator.hasNext())
    {
      URL url = (URL) iterator.next();
      merger.add(ConfigurationParser.loadConfiguration(url, mResolver, new SubProgressMonitor(pMonitor, 1, SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK)));
    }

    pMonitor.done();
    return merger.getMergedServices();
  }

  public Configuration getCommonPortalSessionServices(IProgressMonitor pMonitor) throws IOException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    ServicesMerger merger = new ServicesMerger();
    List urls = new ArrayList();

    List libraryFolders = mInstallationInfo.getLibraryFolders();
    Iterator libraryFoldersIterator = libraryFolders.iterator();
    while (libraryFoldersIterator.hasNext())
    {
      JarsScanner scanner = new JarsScanner((String) libraryFoldersIterator.next());
      urls.addAll(scanner.scan("/conf/portal/session/configuration.xml"));
    }

    Iterator iterator = urls.iterator();
    pMonitor.beginTask("PortalSessionContainer ", urls.size());
    while (iterator.hasNext())
    {
      URL url = (URL) iterator.next();
      merger.add(ConfigurationParser.loadConfiguration(url, mResolver, new SubProgressMonitor(pMonitor, 1, SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK)));
    }

    pMonitor.done();
    return merger.getMergedServices();
  }

  public Configuration getPortalServices(IProgressMonitor pMonitor) throws IOException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    ServicesMerger merger = new ServicesMerger();
    pMonitor.beginTask("PortalContainer ", 1);
    URL url = new URL(mInstallationInfo.getPortalWebappFolder() + "/WEB-INF/conf/configuration.xml");
    merger.add(ConfigurationParser.loadConfiguration(url, mResolver, new SubProgressMonitor(pMonitor, 1, SubProgressMonitor.PREPEND_MAIN_LABEL_TO_SUBTASK)));

    pMonitor.done();
    return merger.getMergedServices();
  }

}