/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.filesprovider;

import java.io.InputStream;

import java.util.List;

/**
 * Interface which can provide structure and content information  for an
 * element (for example, a file system element). Copied from
 * org.eclipse.ui.wizards.datatransfer.IImportStructureProvider and got rid of
 * UI dependency.
 */
public interface IImportStructureProvider
{
  /**   
   * Returns a collection with the children of the specified structured
   * element.
   *
   * @return
   */
  List getChildren(Object element);

  /**   
   * Returns the contents of the specified structured element, or
   * <code>null</code> if there is a problem determining the element's
   * contents.
   *
   * @param element a structured element
   *
   * @return the contents of the structured element, or <code>null</code>
   */
  InputStream getContents(Object element);

  /**   
   * Returns the full path of the specified structured element.
   *
   * @param element a structured element
   *
   * @return the display label of the structured element
   */
  String getFullPath(Object element);

  /**   
   * Returns the display label of the specified structured element.
   *
   * @param element a structured element
   *
   * @return the display label of the structured element
   */
  String getLabel(Object element);

  /**   
   * Returns a boolean indicating whether the passed structured element
   * represents a container element (as opposed to a leaf element).
   *
   * @param element java.lang.Object
   *
   * @return boolean
   */
  boolean isFolder(Object element);
}
