/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching;


import org.eclipse.core.runtime.Platform;

/**
 * This class provides a static method to produce a String representation of a given
 * VMArgument. The String format is very dependent on the type of the VMArgument. Please
 * note that this is not the string used to get an instance of the VMArgumentType, instead
 * this is the string that can be given to VM. (example '-Xdebug').
 * 
 * @author Hatim Khan
 */
public class VMArgumentFormatter
{
  /**
   * This method produces a String representation of a given VMArgument. The String format
   * is very dependent on the type of the VMArgument. <br>
   * Also in cases that the values of the VMArguments are of type 'PATH', then the values
   * are separated by a semi-colon for windows and a colon for all other OSs. An example
   * of a such an argument would be '-classpath C:\bootstrap.jar;C:\tools.jar'. <br>
   * If the values are not of type 'PATH', then they are separated by a comma. An example
   * would be '-Xrunjdwp:transport=dt_socket,suspend=y,address=localhost:12593'. <br>
   * 
   * @param pVMArg an instance of VMArugment class
   * @return the String representation of the VMArgument content
   */
  public static String format(VMArgument pVMArg)
  {
    StringBuffer sb = new StringBuffer();
    if (pVMArg != null)
    {
      String key = pVMArg.getKey();
      if (key != null)
      {
        sb.append(key);

        VMArgumentType type = pVMArg.getType();
        if (type.getParameterCount() != VMArgumentType.NO_PARAMETER)
        {
          if (type.getKeyValueSeparator() == VMArgumentType.KEYVALUE_SEPARATOR_IS_ASSIGN)
          {
            sb.append("=");
          }
          else if (type.getKeyValueSeparator() == VMArgumentType.KEYVALUE_SEPARATOR_IS_ATTACH)
          {
            sb.append(":");
          }
          else if (type.getKeyValueSeparator() == VMArgumentType.KEYVALUE_SEPARATOR_IS_SPACE)
          {
            sb.append(" ");
          }

          String[] values = pVMArg.getValues();
          if (values != null)
          {
            for (int i = 0, sizei = values.length; i < sizei; ++i)
            {
              sb.append(values[i]);
              if (i < (sizei - 1))
              {
                if (type.getParameterType() == VMArgumentType.PARAMETER_TYPE_IS_PATH)
                {
                  if (Platform.getOS().equals(Platform.OS_WIN32))
                  {
                    // for windows the seperator between values should be a semi-colon
                    // while others OS the seperator should be a colon
                    sb.append(";");
                  }
                  else
                  {
                    sb.append(":");
                  }
                }
                else
                {
                  sb.append(",");
                }
              }
            }
          }
        }
      }
    }

    return sb.toString();
  }
}