/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching.delegate;


import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.exoplatform.eclipse.core.launching.ITomcatLocation;

/**
 * A way to encapsulate the information about the location of Tomcat, that can be used
 * later to determine its configuraton.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class TomcatLocation implements ITomcatLocation
{
  public static final String CLASS_VERSION = "$Id: TomcatLocation.java,v 1.1 2004/09/20 01:39:14 hatimk Exp $"; //$NON-NLS-1$
  private String mTomcatHome;
  private String mTomcatSrc;

  /**
   * The tomcat home. For example if you installed Tomcat at c:\tomcat5, then the tomcat
   * home should be c:\tomcat5.
   * 
   * @return the installation directory of Tomcat
   */
  public String getTomcatHome()
  {
    return mTomcatHome;
  }

  /**
   * The location of the main source code directory. By default, Tomcat 5, installs the
   * source code under tomcat5/src
   * 
   * @return the installation directory of the source code
   */
  public String getTomcatSrc()
  {
    return mTomcatSrc;
  }

  /**
   * Sets the installation directory of Tomcat
   * 
   * @param pTomcatHome the installation directory of Tomcat
   */
  public void setTomcatHome(String pTomcatHome)
  {
    mTomcatHome = pTomcatHome;
  }

  /**
   * Sets the installation directory of the source code of Tomcat
   * 
   * @param pTomcatSrc the installation directory of the source code of Tomcat
   */
  public void setTomcatSrc(String pTomcatSrc)
  {
    mTomcatSrc = pTomcatSrc;
  }

  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals(Object pTomcatLocation)
  {
    if ((pTomcatLocation == null)) return false;
    if (this == pTomcatLocation) return true;

    if (!(pTomcatLocation instanceof ITomcatLocation)) return false;
    ITomcatLocation location = (ITomcatLocation) pTomcatLocation;

    // now lets put all location in terms of IPath
    IPath tomcatHome1 = (mTomcatHome == null) ? Path.EMPTY : new Path(mTomcatHome);
    IPath tomcatSrc1 = (mTomcatSrc == null) ? Path.EMPTY : new Path(mTomcatSrc);

    IPath tomcatHome2 = (location.getTomcatHome() == null) ? Path.EMPTY : new Path(location.getTomcatHome());
    IPath tomcatSrc2 = (location.getTomcatSrc() == null) ? Path.EMPTY : new Path(location.getTomcatSrc());

    return (tomcatHome1.equals(tomcatHome2) && tomcatSrc1.equals(tomcatSrc2));
  }

}