/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching.provider;


import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.exoplatform.eclipse.core.launching.ExoLaunching;
import org.exoplatform.eclipse.core.launching.IExoTomcatRuntimeConfiguration;
import org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration;
import org.exoplatform.eclipse.core.launching.VMArgument;
import org.exoplatform.eclipse.core.launching.VMArgumentsContainer;

/**
 * DOCUMENT ME!
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoTomcatRuntimeConfiguration implements IExoTomcatRuntimeConfiguration
{
  public static final String CLASS_VERSION = "$Id: ExoTomcatRuntimeConfiguration.java,v 1.1 2004/09/20 01:39:14 hatimk Exp $"; //$NON-NLS-1$
  private ITomcatRuntimeConfiguration mTomcatConfig;
  private VMArgument[] mVMArguments;
  private IRuntimeClasspathEntry[] mCommonLibraries;
  private IRuntimeClasspathEntry[] mDefaultWebappsLibraries;

  /**
   * The constructor method.
   *
   * @param pTomcatRuntimeConfiguration
   * @param pExoCommonLibraries
   * @param pExoDefaultWebappsLibraries
   * @param pExoVMArgument
   */
  public ExoTomcatRuntimeConfiguration(ITomcatRuntimeConfiguration pTomcatRuntimeConfiguration,
    IRuntimeClasspathEntry[] pExoCommonLibraries,
    IRuntimeClasspathEntry[] pExoDefaultWebappsLibraries, VMArgument[] pExoVMArgument)
  {
    mTomcatConfig = pTomcatRuntimeConfiguration;

    // now lets merge the VMArgument from the tomcat config and the exo config
    VMArgument[] tomcatVMAgruments = mTomcatConfig.getVMArguments();
    if (pExoVMArgument == null)
    {
      pExoVMArgument = new VMArgument[0];
    }

    VMArgumentsContainer container = new VMArgumentsContainer();
    container.appendVMArguments(tomcatVMAgruments);
    container.appendVMArguments(pExoVMArgument);
    mVMArguments = container.getVMArguments();

    // lets combine both the tomcat and the exo common libraries
    mCommonLibraries =
      ExoLaunching.appendRuntimeClasspathEntries(mTomcatConfig.getCommonLibraries(),
        pExoCommonLibraries);

    // lets combine both the tomcat and the exo default web apps libraries
    mDefaultWebappsLibraries =
      ExoLaunching.appendRuntimeClasspathEntries(mTomcatConfig.getDefaultWebappsLibraries(),
        pExoDefaultWebappsLibraries);
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getConfigurationId()
   */
  public String getConfigurationId()
  {
    return mTomcatConfig.getConfigurationId();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getConfigurationName()
   */
  public String getConfigurationName()
  {
    return mTomcatConfig.getConfigurationName();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getMainTypeName()
   */
  public String getMainTypeName()
  {
    return mTomcatConfig.getMainTypeName();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getStartupProgramArguments()
   */
  public String[] getStartupProgramArguments()
  {
    return mTomcatConfig.getStartupProgramArguments();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getShutdownProgramArguments()
   */
  public String[] getShutdownProgramArguments()
  {
    return mTomcatConfig.getShutdownProgramArguments();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getWorkingDirectory()
   */
  public IPath getWorkingDirectory()
  {
    return mTomcatConfig.getWorkingDirectory();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getBinLibraries()
   */
  public IRuntimeClasspathEntry[] getBinLibraries()
  {
    return mTomcatConfig.getBinLibraries();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getServerLibraries()
   */
  public IRuntimeClasspathEntry[] getServerLibraries()
  {
    return mTomcatConfig.getServerLibraries();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getCommonLibraries()
   */
  public IRuntimeClasspathEntry[] getCommonLibraries()
  {
    return mCommonLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getSharedLibraries()
   */
  public IRuntimeClasspathEntry[] getSharedLibraries()
  {
    return mTomcatConfig.getSharedLibraries();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getServerWebappsLibraries()
   */
  public IRuntimeClasspathEntry[] getServerWebappsLibraries()
  {
    return mTomcatConfig.getServerWebappsLibraries();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getDefaultWebappsLibraries()
   */
  public IRuntimeClasspathEntry[] getDefaultWebappsLibraries()
  {
    return mDefaultWebappsLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getClasspathLibraries()
   */
  public IRuntimeClasspathEntry[] getClasspathLibraries()
  {
    return mTomcatConfig.getClasspathLibraries();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getEndorsedLibraries()
   */
  public IRuntimeClasspathEntry[] getEndorsedLibraries()
  {
    return mTomcatConfig.getEndorsedLibraries();
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getVMArguments()
   */
  public VMArgument[] getVMArguments()
  {
    return mVMArguments;
  }
}
