/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.launching.provider;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration;
import org.exoplatform.eclipse.core.launching.VMArgument;


/**
 * DOCUMENT ME!
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class TomcatRuntimeConfiguration implements ITomcatRuntimeConfiguration
{
  public static final String CLASS_VERSION = "$Id: TomcatRuntimeConfiguration.java,v 1.1 2004/09/20 01:39:14 hatimk Exp $"; //$NON-NLS-1$
	private String mConfigurationId = null;  
  private String mConfigurationName = null;
  private String mMainTypeName = null;
  private String[] mStartupProgramArguments = null;
  private String[] mShutdownProgramArguments = null;
  private IPath mWorkingDirectory = null;
  private IRuntimeClasspathEntry[] mBinLibraries = null;
  private IRuntimeClasspathEntry[] mServerLibraries = null;
  private IRuntimeClasspathEntry[] mCommonLibraries = null;
  private IRuntimeClasspathEntry[] mSharedLibraries = null;
  private IRuntimeClasspathEntry[] mServerWebappsLibraries = null;
  private IRuntimeClasspathEntry[] mDefaultWebappsLibraries = null;
  private IRuntimeClasspathEntry[] mClasspathLibraries = null;
  private IRuntimeClasspathEntry[] mEndorsedLibraries = null;
  private VMArgument[] mVMArguments = null;

  /**
   * The constructor method.
   *
   * @param pVersion
   * @param pMainTypeName
   * @param pStartupArguments
   * @param pShutdownArguments
   * @param pWorkingDirectory
   * @param pBinLibraries
   * @param pServerLibraries
   * @param pCommonLibraries
   * @param pSharedLibraries
   * @param pServerWebappsLibraries
   * @param pDefaultWebappsLibraries
   * @param pClasspathLibraries
   * @param pEndorsedLibraries
   * @param pVMArguments
   */
  public TomcatRuntimeConfiguration(String pConfigurationId, String pConfigurationName, String pMainTypeName,
    String[] pStartupArguments, String[] pShutdownArguments, IPath pWorkingDirectory,
    IRuntimeClasspathEntry[] pBinLibraries, IRuntimeClasspathEntry[] pServerLibraries,
    IRuntimeClasspathEntry[] pCommonLibraries, IRuntimeClasspathEntry[] pSharedLibraries,
    IRuntimeClasspathEntry[] pServerWebappsLibraries,
    IRuntimeClasspathEntry[] pDefaultWebappsLibraries,
    IRuntimeClasspathEntry[] pClasspathLibraries, IRuntimeClasspathEntry[] pEndorsedLibraries,
    VMArgument[] pVMArguments)
  {
    mConfigurationId = pConfigurationId;
    mConfigurationName = pConfigurationName;
    mMainTypeName = pMainTypeName;
    mStartupProgramArguments = pStartupArguments;
    mShutdownProgramArguments = pShutdownArguments;
    mWorkingDirectory = pWorkingDirectory;
    mBinLibraries = pBinLibraries;
    mServerLibraries = pServerLibraries;
    mCommonLibraries = pCommonLibraries;
    mSharedLibraries = pSharedLibraries;
    mServerWebappsLibraries = pServerWebappsLibraries;
    mDefaultWebappsLibraries = pDefaultWebappsLibraries;
    mClasspathLibraries = pClasspathLibraries;
    mEndorsedLibraries = pEndorsedLibraries;
    mVMArguments = pVMArguments;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getConfigurationId()
   */
  public String getConfigurationId()
  {
    return mConfigurationId;
  }

	/**   
	 * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getConfigurationName()
	 */
	public String getConfigurationName()
	{
		return mConfigurationName;
	}

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getMainTypeName()
   */
  public String getMainTypeName()
  {
    return mMainTypeName;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getStartupProgramArguments()
   */
  public String[] getStartupProgramArguments()
  {
    return mStartupProgramArguments;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getShutdownProgramArguments()
   */
  public String[] getShutdownProgramArguments()
  {
    return mShutdownProgramArguments;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getCommonLibraries()
   */
  public IRuntimeClasspathEntry[] getCommonLibraries()
  {
    return mCommonLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getBinLibraries()
   */
  public IRuntimeClasspathEntry[] getBinLibraries()
  {
    return mBinLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getSharedLibraries()
   */
  public IRuntimeClasspathEntry[] getSharedLibraries()
  {
    return mSharedLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getServerLibraries()
   */
  public IRuntimeClasspathEntry[] getServerLibraries()
  {
    return mServerLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getServerWebappsLibraries()
   */
  public IRuntimeClasspathEntry[] getServerWebappsLibraries()
  {
    return mServerWebappsLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getDefaultWebappsLibraries()
   */
  public IRuntimeClasspathEntry[] getDefaultWebappsLibraries()
  {
    return mDefaultWebappsLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getClasspathLibraries()
   */
  public IRuntimeClasspathEntry[] getClasspathLibraries()
  {
    return mClasspathLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getEndorsedLibraries()
   */
  public IRuntimeClasspathEntry[] getEndorsedLibraries()
  {
    return mEndorsedLibraries;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration#getVMArguments()
   */
  public VMArgument[] getVMArguments()
  {
    return mVMArguments;
  }

  /**   
   * @see org.exoplatform.eclipse.core.launching.ITomcatRuntimeConfiguration
   */
  public IPath getWorkingDirectory()
  {
    return mWorkingDirectory;
  }
}
