/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.library;


import org.eclipse.core.runtime.IPath;
import org.exoplatform.eclipse.internal.core.library.ExoPortletLibraryInitializer;

/**
 * This class acts as a facade to the available library services in the core
 * plugin.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoLibraryCenter
{
  public static final String CLASS_VERSION = "$Id: ExoLibraryCenter.java,v 1.1 2004/04/19 03:45:47 hatimk Exp $"; //$NON-NLS-1$
  public static final String EXO_PORTLET_LIBRARY_PATH =
    ExoPortletLibraryInitializer.EXO_PORTLET_LIBRARY_PATH;
  private static ExoPortletLibraryInitializer mExoPortletLibraryInitializer =
    new ExoPortletLibraryInitializer();

  /**   
   * Returns all available eXo portlet library containers. All library entries
   * are contributed via the extension point 'exoPortletLibraryProviders'
   *
   * @return available eXo portlet libraries
   */
  public static ExoPortletLibraryContainer[] getPortletLibraries()
  {
    return mExoPortletLibraryInitializer.getPortletLibraries();
  }

  /**   
   * Given the library path (example,
   * org.exoplatform.eclipse.core.EXO_PORTLET_LIBRARY), returns its
   * ExoPortletLibraryContainer representation. If the first segement matches,
   * but not the second one (or there was no second segment in the given path)
   * then return the latest version of the library container that is
   * available.
   *
   * @param pPortletLibraryPath the exo portlet library path
   *
   * @return an instance of ExoPortletLibraryContainer that represents the
   *         given path or null if the first segement does not match
   */
  public static ExoPortletLibraryContainer getPortletLibrary(
    IPath pPortletLibraryPath)
  {
    return mExoPortletLibraryInitializer.getPortletLibrary(pPortletLibraryPath);
  }
}
