/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.operation;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.operation.helper.ProjectConfiguration;

/**
 * @author Hatim Khan
 */
public class PortalProjectCreateOperation extends ProjectCreateOperation
{
  /**
   * The constructor method.
   *
   * @param pConfig A configuration that consists of the project name, the
   *        project path, and one or more project settings.
   * @param pProjectName the project name (can contain spaces)
   * @param pProjectPath the project path. If the path is null, then it means
   *        that the project name is to be used as the path (relative to the
   *        workspace). For example, if the given path is null, and the
   *        project name is 'test', then it should be interpreted  as a
   *        project 'test' in the workspace.
   * @param pOverwriteQuery the implementation of this interface should answer
   *        one of the prescribed return codes when asked whether to overwrite
   *        a certain path string (which could represent a resource path, a
   *        file system path, etc). The above comment was copied from the
   *        IOverwriteQuery interface comment.
   */
  public PortalProjectCreateOperation(ProjectConfiguration pConfig,
    String pProjectName, IPath pProjectPath, IOverwriteQuery pOverwriteQuery)
  {
    super(pConfig, pProjectName, pProjectPath, pOverwriteQuery);
  }

  /**   
   * @see org.eclipse.ui.actions.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("creating a portal project : ", 3000);
    super.execute(new SubProgressMonitor(pMonitor, 2500));
    upgradeToPortalProject(getProject(), new SubProgressMonitor(pMonitor, 500));
    pMonitor.done();
  }

  protected void upgradeToPortalProject(IProject pProject,
    IProgressMonitor pMonitor) throws CoreException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    pMonitor.beginTask("converting the project to portal project", 50);
    pMonitor.subTask(
      "process the project description and adds the portal nature");

    IProjectDescription description = pProject.getDescription();
    String[] natures = description.getNatureIds();
    String[] newNatures = new String[natures.length + 1];
    System.arraycopy(natures, 0, newNatures, 1, natures.length);
    newNatures[0] = ExoCorePlugin.PORTAL_NATURE_ID;
    description.setNatureIds(newNatures);
    pProject.setDescription(description, new SubProgressMonitor(pMonitor, 50));

    if (pMonitor.isCanceled())
    {
      throw new InterruptedException();
    }

    pMonitor.done();
  }
  
}
