/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.common.IExoMarker;

import java.lang.reflect.InvocationTargetException;

/**
 * An opertaion that removes all the portlet validation problem markers from
 * all the open portlet projects.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class RemovePortletValidationMarkersOperation
  extends WorkspaceModifyOperation
{
  public static final String CLASS_VERSION = "$Id: RemovePortletValidationMarkersOperation.java,v 1.1 2004/04/19 03:45:48 hatimk Exp $"; //$NON-NLS-1$

  /**   
   * @see org.exoplatform.eclipse.core.operation.WorkspaceModifyOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
   */
  protected void execute(IProgressMonitor pMonitor)
    throws CoreException, InvocationTargetException, InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    // notice that we could have simply asked the workspace root to delete all
    // markers, but unfortunately the operation does not take IProgressMonitor
    // therefore, we are going to do it the hard way. Also, we could have used
    // the visitor pattern here to visit all projects, but since we will be
    // deleting markers, it might not be appropriate to use the visitor pattern.
    IProject[] allProjects =
      ResourcesPlugin.getWorkspace().getRoot().getProjects();

    if (allProjects == null)
    {
      return;
    }

    pMonitor.beginTask("Removing portlet validation error markers : ",
      allProjects.length * 10);
    for (int i = 0, sizei = allProjects.length; i < sizei; i++)
    {
      IProject project = allProjects[i];
      if (!project.isOpen())
      {
        pMonitor.worked(10);

        continue;
      }
      if (!project.hasNature(ExoCorePlugin.PORTLET_NATURE_ID))
      {
        pMonitor.worked(10);

        continue;
      }
      if (pMonitor.isCanceled())
      {
        throw new InterruptedException();
      }
      pMonitor.subTask("working on project [" + project.getName()
        + "] to remove its portlet validation error markers");
      project.deleteMarkers(IExoMarker.PORTLET_PROBLEM, false,
        IResource.DEPTH_INFINITE);
      pMonitor.worked(10);
    }
    pMonitor.done();
  }
}
