/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.operation.helper;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Helper class for exporting resources to the file system. This is a cut and
 * paste from org.eclipse.ui.wizards.datatransfer.FileSystemExporter  I needed
 * to copy it because for some reason the class had a package visiability
 * instead of public and also to remove UI dependency.
 */
public class FileSystemExporter
{
  /**   
   * Creates the specified file system directory at
   * <code>destinationPath</code>. This creates a new file system directory.
   *
   * @param destinationPath
   */
  public void createFolder(IPath destinationPath)
  {
    new File(destinationPath.toOSString()).mkdir();
  }

  /**   
   * Writes the passed resource to the specified location recursively
   *
   * @param resource
   * @param destinationPath
   *
   * @throws CoreException
   * @throws IOException
   */
  public void write(IResource resource, IPath destinationPath)
    throws CoreException, IOException
  {
    if (resource.getType() == IResource.FILE)
    {
      writeFile((IFile) resource, destinationPath);
    }
    else
    {
      writeChildren((IContainer) resource, destinationPath);
    }
  }

  /**   
   * Exports the passed container's children
   *
   * @param folder
   * @param destinationPath
   *
   * @throws CoreException
   * @throws IOException
   */
  protected void writeChildren(IContainer folder, IPath destinationPath)
    throws CoreException, IOException
  {
    if (folder.isAccessible())
    {
      IResource[] children = folder.members();
      for (int i = 0; i < children.length; i++)
      {
        IResource child = children[i];
        writeResource(child, destinationPath.append(child.getName()));
      }
    }
  }

  /**   
   * Writes the passed file resource to the specified destination on the local
   * file system
   *
   * @param file
   * @param destinationPath
   *
   * @throws IOException
   * @throws CoreException
   */
  protected void writeFile(IFile file, IPath destinationPath)
    throws IOException, CoreException
  {
    FileOutputStream output = null;
    InputStream contentStream = null;

    try
    {
      output = new FileOutputStream(destinationPath.toOSString());
      contentStream = file.getContents(false);

      int chunkSize = contentStream.available();
      byte[] readBuffer = new byte[chunkSize];
      int n = contentStream.read(readBuffer);

      while (n > 0)
      {
        output.write(readBuffer);
        n = contentStream.read(readBuffer);
      }
    }
    finally
    {
      if (output != null)
      {
        output.close();
      }
      if (contentStream != null)
      {
        contentStream.close();
      }
    }
  }

  /**   
   * Writes the passed resource to the specified location recursively
   *
   * @param resource
   * @param destinationPath
   *
   * @throws CoreException
   * @throws IOException
   */
  protected void writeResource(IResource resource, IPath destinationPath)
    throws CoreException, IOException
  {
    if (resource.getType() == IResource.FILE)
    {
      writeFile((IFile) resource, destinationPath);
    }
    else
    {
      createFolder(destinationPath);
      writeChildren((IContainer) resource, destinationPath);
    }
  }
}
