/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.core.portal;

import java.io.File;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import org.exoplatform.eclipse.core.exoservices.IExoPlatformInstallationInfo;

/**
 * This class understands the installation structure of exo tomcat and therefore is able
 * to answer questions such as the library folders, etc. This class is usually used by
 * inspectors of the eXo platform such as the ServicesInspector.
 * 
 * @author Hatim Khan
 */
public class ExoTomcatInstallationInfo implements IExoPlatformInstallationInfo
{

  private String mInstallationDirectory;
  private String mJDKVersion;

  public ExoTomcatInstallationInfo(String pInstallationDirectory, String pJDKVersion)
  {
    mInstallationDirectory = pInstallationDirectory;
    mJDKVersion = pJDKVersion;
  }

  /**
   * @see org.exoplatform.eclipse.core.exoservices.IExoPlatformInstallationInfo#getInstallationDirectory()
   */
  public String getInstallationDirectory()
  {
    return mInstallationDirectory;
  }

  /**
   * @see org.exoplatform.eclipse.core.exoservices.IExoPlatformInstallationInfo#getJDKVersion()
   */
  public String getJDKVersion()
  {
    return mJDKVersion;
  }

  /**
   * @see org.exoplatform.eclipse.core.exoservices.IExoPlatformInstallationInfo#getLibraryFolders()
   */
  public List getLibraryFolders()
  {
    List list = new ArrayList();
    list.add(mInstallationDirectory + "/common/lib");
    return list;
  }

  /**
   * @see org.exoplatform.eclipse.core.exoservices.IExoPlatformInstallationInfo#getPortalWebappFolder()
   */
  public String getPortalWebappFolder()
  {
    String folder = mInstallationDirectory + "/webapps/portal";
    File file = new File(folder);
    try
    {
      folder = file.toURL().toString();
    }
    catch (MalformedURLException e)
    {
    }
    
    if (!file.exists())
    {
      folder = "jar:file:/" + mInstallationDirectory + "/webapps/portal.war!";
    }

    return folder;
  }

}