/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.warpackager;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IPackageFragment;

import java.util.jar.Attributes;
import java.util.jar.Manifest;

/**
 * A manifest provider creates manifest files. This is a direct adaptation of
 * the same concept from org.eclipse.jdt.ui.jarpackager.ManifestProvider. For
 * now, this class does not support sealing.
 */
public class ManifestProvider implements IManifestProvider
{
  /**   
   * Creates a manifest as defined by the <code>WarPackage</code>.
   *
   * @param pWarPackage the WAR package specification
   *
   * @return a default manifest
   *
   * @throws CoreException
   */
  public Manifest create(WarPackageData pWarPackage) throws CoreException
  {
    if (pWarPackage.isManifestGenerated())
    {
      return createGeneratedManifest(pWarPackage);
    }
    else
    {
      return null;
    }
  }

  /**   
   * Creates a default manifest.
   *
   * @param pManifestVersion the version of the manifest
   *
   * @return a default manifest
   */
  public Manifest createDefault(String pManifestVersion)
  {
    Manifest manifest = new Manifest();
    manifest.getMainAttributes().put(Attributes.Name.MANIFEST_VERSION,
      pManifestVersion);

    return manifest;
  }

  /**   
   * Hook for subclasses to add additional manifest entries.
   *
   * @param pManifest the manifest to which the entries should be added
   * @param pWarPackage the WAR package specification
   */
  protected void putAdditionalEntries(Manifest pManifest,
    WarPackageData pWarPackage)
  {
  }

  private Manifest createGeneratedManifest(WarPackageData pWarPackage)
  {
    Manifest manifest = new Manifest();
    putVersion(manifest, pWarPackage);
    putAdditionalEntries(manifest, pWarPackage);

    return manifest;
  }

  private void putVersion(Manifest manifest, WarPackageData pWarPackage)
  {
    manifest.getMainAttributes().put(Attributes.Name.MANIFEST_VERSION,
      pWarPackage.getManifestVersion());
  }

  private String getInManifestFormat(IPackageFragment pPackageFragment)
  {
    String name = pPackageFragment.getElementName();

    return name.replace('.', '/') + '/';
  }
}
