/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.core.warpackager;


import org.eclipse.core.runtime.CoreException;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;

/**
 * Utility for the War packager
 */
public class WarPackagerUtil
{
  /**   
   * Asks if it is okay to overwrite a file
   *
   * @param pOverwriteQuery an implemenation that returns the answer
   * @param pFilePath the file to overwrite
   *
   * @return true to overwrite
   */
  public static boolean askForOverwritePermission(
    final IOverwriteQuery pOverwriteQuery, String pFilePath)
  {
    String result = pOverwriteQuery.queryOverwrite(pFilePath);
    if ((result.compareTo(IOverwriteQuery.ALL) == 0)
        || (result.compareTo(IOverwriteQuery.YES) == 0))
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  /**   
   * Creates a <code>CoreException</code> with the given parameters.
   *
   * @param pMessage a string with the message
   * @param pException the exception to be wrapped or <code>null</code> if none
   *
   * @return a CoreException
   */
  public static CoreException createCoreException(String pMessage,
    Exception pException)
  {
    if (pMessage == null)
    {
      pMessage = ""; //$NON-NLS-1$
    }

    return new CoreException(ExoCorePlugin.createErrorStatus(pMessage,
        pException));
  }
}
