/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.editor;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.AbstractFormPart;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.exoplatform.eclipse.core.exoservices.Configuration;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.common.PortalConfigurationContentProvider;
import org.exoplatform.eclipse.ui.common.PortalConfigurationLabelProvider;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * This details page displays the detail parameter entry information. This page does not
 * change the model or have any field to allow the user to enter data, however, this page
 * should receive model change events and marks itself as stale. This class is an abstract
 * class; implementors must implement addValuesContent(). The implementors than can
 * provide their own values content or call one of the two convenient methods
 * addTreeValuesContent() and addTableValuesContent().
 * 
 * @author Hatim Khan
 */
public abstract class ParameterInfoDetailPart extends AbstractFormPart implements IDetailsPage,
    IPortalSettingsConfigurationChangedListener
{
  protected Parameter mParameter = null;
  protected PortalSettings mPortalSettings = null;
  protected int mViewerHeight = 0;
  protected StructuredViewer mViewer = null;
  private String mSectionTitle = "Section";
  private String mSectionDesc = "";
  private String mTypeTitle = "";
  private Text mDesc = null;
  private Text mName = null;
  private Text mAppearIn = null;
  private Label mAppearInLabel = null;

  public ParameterInfoDetailPart(PortalSettings pPortalSettings, String pTypeTitle, String pSectionTitle,
      String pSectionDesc)
  {
    mPortalSettings = pPortalSettings;
    mTypeTitle = pTypeTitle;
    mSectionTitle = pSectionTitle;
    mSectionDesc = pSectionDesc;
    mViewerHeight = 100;
  }

  protected void populateSectionTitle(Section pSection)
  {
    pSection.setText(mSectionTitle);
    pSection.setDescription(mSectionDesc);
  }

  /**
   * @see org.eclipse.ui.forms.IDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
   */
  public void createContents(Composite pParent)
  {
    GridLayout layout = new GridLayout();
    FormToolkit toolkit = getManagedForm().getToolkit();
    layout.marginHeight = 0;
    layout.marginWidth = 0;
    pParent.setLayout(layout);

    Section section = toolkit.createSection(pParent, Section.TITLE_BAR | Section.DESCRIPTION
        | Section.TWISTIE | Section.EXPANDED);
    section.clientVerticalSpacing = 4;
    section.marginHeight = 5;
    section.marginWidth = 5;
    populateSectionTitle(section);
    GridData td = new GridData(GridData.FILL_BOTH);
    section.setLayoutData(td);

    Composite client = toolkit.createComposite(section);
    layout = new GridLayout();
    layout.marginHeight = 2;
    layout.marginWidth = 2;
    layout.numColumns = 2;

    client.setLayout(layout);
    td = new GridData(GridData.FILL_BOTH);
    client.setLayoutData(td);

    populateClientContent(client, toolkit);

    toolkit.paintBordersFor(client);
    section.setClient(client);
  }

  protected void populateClientContent(Composite pClient, FormToolkit pToolkit)
  {
    //---------------------- Type title ----------------------------
    Label label = pToolkit.createLabel(pClient, mTypeTitle);
    label.setForeground(pToolkit.getColors().getColor(FormColors.TITLE));
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = 2;
    label.setLayoutData(gd);

    //---------------------- Desc ----------------------------
    label = pToolkit.createLabel(pClient, "Description:");
    label.setForeground(pToolkit.getColors().getColor(FormColors.TITLE));
    gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = 2;
    label.setLayoutData(gd);

    mDesc = pToolkit.createText(pClient, "", SWT.MULTI | SWT.WRAP | SWT.LEFT | SWT.READ_ONLY);
    gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = 2;
    mDesc.setLayoutData(gd);
    UICompositeUtil.setHeightHint(mDesc, 30);

    UICompositeUtil.createFormGridSpacer(pToolkit, pClient, 2);
    //---------------------- Name ----------------------------
    label = pToolkit.createLabel(pClient, "Name:");
    label.setForeground(pToolkit.getColors().getColor(FormColors.TITLE));

    mName = pToolkit.createText(pClient, "", SWT.MULTI | SWT.WRAP | SWT.LEFT | SWT.READ_ONLY);
    gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = 1;
    mName.setLayoutData(gd);

    addExtraContent(pClient, pToolkit);
    addValuesContent(pClient, pToolkit);
  }

  protected abstract void addValuesContent(Composite pClient, FormToolkit pToolkit);

  protected void addExtraContent(Composite pClient, FormToolkit pToolkit)
  {
  }

  protected void addTreeValuesContent(Composite pClient, FormToolkit pToolkit)
  {
    createLabel("Values:", pClient, pToolkit);
    mViewer = createTreeViewer(pClient, pToolkit);
    populateViewer(mViewer, mParameter);
  }

  protected void addTableValuesContent(Composite pClient, FormToolkit pToolkit)
  {
    createLabel("Values:", pClient, pToolkit);
    mViewer = createTableViewer(pClient, pToolkit);
    populateViewer(mViewer, mParameter);
  }

  protected void addAppearInContent(Composite pClient, FormToolkit pToolkit)
  {
    UICompositeUtil.createFormGridSpacer(pToolkit, pClient, 2);
    //---------------------- Appear In ----------------------------
    mAppearInLabel = pToolkit.createLabel(pClient, "This parameter originally appeared in: ");
    mAppearInLabel.setForeground(pToolkit.getColors().getColor(FormColors.TITLE));
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = 2;
    mAppearInLabel.setLayoutData(gd);

    mAppearIn = pToolkit.createText(pClient, "", SWT.MULTI | SWT.WRAP | SWT.LEFT | SWT.READ_ONLY);
    gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = 2;
    mAppearIn.setLayoutData(gd);
    UICompositeUtil.setHeightHint(mAppearIn, 30);

    mAppearIn.setVisible(false);
    mAppearInLabel.setVisible(false);
  }

  protected Label createLabel(String pLabel, Composite pClient, FormToolkit pToolkit)
  {
    Label label = pToolkit.createLabel(pClient, pLabel);
    label.setForeground(pToolkit.getColors().getColor(FormColors.TITLE));
    GridData gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
    gd.horizontalSpan = 1;
    label.setLayoutData(gd);

    return label;
  }

  private StructuredViewer createTreeViewer(Composite parent, FormToolkit toolkit)
  {
    int style = SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL;
    if (toolkit == null)
    {
      style |= SWT.BORDER;
    }
    else
    {
      style |= toolkit.getBorderStyle();
    }
    TreeViewer treeViewer = new TreeViewer(parent, style);
    Control tree = treeViewer.getControl();
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = 1;
    tree.setLayoutData(gridData);
    UICompositeUtil.setWidthHeight(tree, -1, mViewerHeight);

    return treeViewer;
  }

  private StructuredViewer createTableViewer(Composite parent, FormToolkit toolkit)
  {
    int style = SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL;
    if (toolkit == null)
    {
      style |= SWT.BORDER;
    }
    else
    {
      style |= toolkit.getBorderStyle();
    }
    TableViewer tableViewer = new TableViewer(parent, style);
    Control tree = tableViewer.getControl();
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = 1;
    tree.setLayoutData(gridData);
    UICompositeUtil.setWidthHeight(tree, -1, mViewerHeight);

    return tableViewer;
  }

  private void populateViewer(StructuredViewer pViewer, Object pInput)
  {
    pViewer.setContentProvider(new PortalConfigurationContentProvider());
    pViewer.setLabelProvider(new PortalConfigurationLabelProvider());
    pViewer.setInput(pInput);
  }

  /**
   * @see org.eclipse.ui.forms.IPartSelectionListener#selectionChanged(org.eclipse.ui.forms.IFormPart,
   *      org.eclipse.jface.viewers.ISelection)
   */
  public void selectionChanged(IFormPart part, ISelection pSelection)
  {
    if (pSelection instanceof IStructuredSelection)
    {
      IStructuredSelection ssel = (IStructuredSelection) pSelection;
      if (ssel.size() == 1)
      {
        Parameter newParameter = (Parameter) ssel.getFirstElement();
        if (newParameter != mParameter)
        {
          // we need to detache from the current configuration to the new one
          mPortalSettings.removeConfigurationChangedListener(this);
          Configuration config = PortalSettings.findConfiguration(newParameter);
          mPortalSettings.addConfigurationChangedListener(config, this);
        }
        mParameter = newParameter;
      }
      else
      {
        mParameter = null;
        mPortalSettings.removeConfigurationChangedListener(this);
      }
    }
    else
    {
      mParameter = null;
      mPortalSettings.removeConfigurationChangedListener(this);
    }
    update();
  }

  public void dispose()
  {
    super.dispose();
    if (mPortalSettings != null)
    {
      mPortalSettings.removeConfigurationChangedListener(this);
    }
  }

  public void refresh()
  {
    super.refresh();
    update();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementRemoved(java.lang.Object,
   *      java.lang.Object)
   */
  public void ConfigurationElementRemoved(Object pParent, Object pChild)
  {
    markStale();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementAdded(java.lang.Object)
   */
  public void ConfigurationElementAdded(Object pChild)
  {
    markStale();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementUpdated(java.lang.Object)
   */
  public void ConfigurationElementUpdated(Object pElement)
  {
    markStale();
  }

  protected void update()
  {
    if ((mParameter == null))
    {
      mDesc.setText("");
      mName.setText("");
      if (mViewer != null) mViewer.setInput(null);
      if (mAppearIn != null)
      {
        mAppearIn.setText("");
        mAppearIn.setVisible(false);
        mAppearInLabel.setVisible(false);
      }
      return;
    }
    else
    {
      mDesc.setText(UICompositeUtil.emptyIfNull(mParameter.getDescription()));
      mName.setText(UICompositeUtil.emptyIfNull(mParameter.getName()));
      if (mViewer != null)
      {
        mViewer.setInput(mParameter);
        mViewer.refresh();
      }
      if (mAppearIn != null)
      {
        String appearIn = UICompositeUtil.emptyIfNull(getAppearInValue());
        mAppearIn.setText(appearIn);
        boolean visibility = (appearIn.length() > 0);
        mAppearIn.setVisible(visibility);
        mAppearInLabel.setVisible(visibility);
      }
    }
  }

  protected String getAppearInValue()
  {
    return null;
  }

}