/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.editor;

import java.util.Iterator;
import java.util.List;

import org.exoplatform.eclipse.core.exoservices.ObjectParam;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.Property;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * Contains static methods to help with formatting all the parameters in a given service
 * configuration. The format is strictly to be used with the FormText widget only.
 * 
 * @author Hatim Khan
 */
public class ParametersDisplayHelper
{
  public static String formattedParameters(ServiceConfiguration pConfig)
  {
    if (pConfig == null) return "<form></form>";

    StringBuffer sb = new StringBuffer();
    sb.append("<form><p>");
    List objParams = pConfig.getObjectParams();
    if ((objParams != null) && (objParams.size() > 0))
    {
      Iterator iterator = objParams.iterator();
      while (iterator.hasNext())
      {
        sb.append(formattedObjectParam((ObjectParam) iterator.next()));
        sb.append("<br/></p><p><img href=\"image\"/></p><p><br/>");
        //sb.append("<br/>");
      }
    }
    sb.append("</p></form>");
    return sb.toString();
  }

  public static String formattedObjectParam(ObjectParam pParam)
  {
    if (pParam == null) return "";
    StringBuffer sb = new StringBuffer();
    sb.append(formattedParam(pParam));
    sb.append("<b>Type:</b> ");
    sb.append("Object Parameter");
    sb.append("<br/><br/> <br/>");

    List properties = pParam.getProperties();
    if ((properties != null) && (properties.size() > 0))
    {
      Iterator iterator = properties.iterator();
      while(iterator.hasNext())
      {
        sb.append(formattedProperty((Property) iterator.next()));        
      }
    }

    return sb.toString();
  }

  public static String formattedParam(Parameter pParam)
  {
    if (pParam == null) return "";
    StringBuffer sb = new StringBuffer();
    sb.append("<b>Name:</b> ");
    sb.append(UICompositeUtil.emptyIfNull(pParam.getName()));
    sb.append("<br/>");
    sb.append("<b>Description:</b> ");
    sb.append(UICompositeUtil.emptyIfNull(pParam.getDescription()));
    sb.append("<br/>");

    return sb.toString();
  }

  public static String formattedProperty(Property pParam)
  {
    if (pParam == null) return "";
    StringBuffer sb = new StringBuffer();
    sb.append("<b>Key:</b> ");
    sb.append(UICompositeUtil.emptyIfNull(pParam.getName()));
    sb.append("<br/>");
    sb.append("<b>Value:</b> ");
    sb.append(UICompositeUtil.emptyIfNull(pParam.getValue()));
    sb.append("<br/> <br/>");

    return sb.toString();
  }

}