/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.editor;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.exoplatform.eclipse.core.exoservices.ObjectParam;
import org.exoplatform.eclipse.core.exoservices.PropertiesParam;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.core.exoservices.ValueParam;
import org.exoplatform.eclipse.core.exoservices.ValuesParam;
import org.exoplatform.eclipse.core.portal.PortalSettings;

/**
 * @author Hatim Khan
 */
public class PortalServicesBlock extends MasterDetailsBlock
{

  private PortalSettings mPortalSettings;

  public PortalServicesBlock(PortalSettings pPortalSettings)
  {
    mPortalSettings = pPortalSettings;
  }

  /**
   * @see org.eclipse.ui.forms.MasterDetailsBlock#createMasterPart(org.eclipse.ui.forms.IManagedForm,
   *      org.eclipse.swt.widgets.Composite)
   */
  protected void createMasterPart(IManagedForm pManagedForm, Composite pParent)
  {
    FormToolkit toolkit = pManagedForm.getToolkit();
    Composite client = toolkit.createComposite(pParent);
    GridLayout glayout = new GridLayout();
    glayout.marginWidth = glayout.marginHeight = 0;
    glayout.numColumns = 1;
    client.setLayout(glayout);
    //client.setLayoutData(new GridData(GridData.FILL_BOTH));

    ServicesSection portalServices = new ServicesSection(
        false,
        mPortalSettings.getCustomPortalConfiguration(),
        mPortalSettings,
        "Portal Container Services",
        "The following is a collection of all service configuration entries associated with the portal container",
        400, pManagedForm, client, true);
    pManagedForm.addPart(portalServices);
    Section sc = portalServices.getSection();
    sc.marginWidth = 5;
    sc.marginHeight = 5;

    ServicesSection rootServices = new ServicesSection(
        true,
        mPortalSettings.getRootConfiguration(),
        mPortalSettings,
        "Root Container Services (Read Only)",
        "The following is a collection of all service configuration entries associated with the root container. Note: you can not add, edit, or remove any of these services.",
        100, pManagedForm, client, false);
    pManagedForm.addPart(rootServices);
    sc = rootServices.getSection();
    sc.marginWidth = 5;
    sc.marginHeight = 5;

    ServicesSection sessionServices = new ServicesSection(
        true,
        mPortalSettings.getPortalSessionConfiguration(),
        mPortalSettings,
        "Portal Session Container Services (Read Only)",
        "The following is a collection of all service configuration entries associated with the portal session container. Note: you can not add, edit, or remove any of these services.",
        100, pManagedForm, client, false);
    pManagedForm.addPart(sessionServices);
    sc = sessionServices.getSection();
    sc.marginWidth = 5;
    sc.marginHeight = 5;

  }

  /**
   * @see org.eclipse.ui.forms.MasterDetailsBlock#registerPages(org.eclipse.ui.forms.DetailsPart)
   */
  protected void registerPages(DetailsPart pDetailsPart)
  {
    pDetailsPart.registerPage(ServiceConfiguration.class, new ServiceInfoDetailPart(mPortalSettings));
    pDetailsPart.registerPage(ObjectParam.class, new ObjectParamInfoDetailPart(mPortalSettings));
    pDetailsPart.registerPage(PropertiesParam.class, new PropertiesParamInfoDetailPart(mPortalSettings));
    pDetailsPart.registerPage(ValuesParam.class, new ValuesParamInfoDetailPart(mPortalSettings));
    pDetailsPart.registerPage(ValueParam.class, new ValueParamInfoDetailPart(mPortalSettings));
  }

  /**
   * @see org.eclipse.ui.forms.MasterDetailsBlock#createToolBarActions(org.eclipse.ui.forms.IManagedForm)
   */
  protected void createToolBarActions(IManagedForm pManagedForm)
  {
  }

}