/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.editor;

import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.exoplatform.eclipse.core.exoservices.Configuration;
import org.exoplatform.eclipse.core.exoservices.GlobalParam;
import org.exoplatform.eclipse.core.exoservices.ObjectParam;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.PropertiesParam;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.core.exoservices.ValueParam;
import org.exoplatform.eclipse.core.exoservices.ValuesParam;
import org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterWizard;
import org.exoplatform.eclipse.internal.ui.wizard.EditServiceWizard;
import org.exoplatform.eclipse.ui.common.PortalConfigurationContentProvider;
import org.exoplatform.eclipse.ui.common.PortalConfigurationLabelProvider;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * This is a form part that expects to be added to a managed form. It contains a table
 * viewer that displays the content of exo Platfrom configuration. The configuration can
 * contain global param and services configurations.
 * 
 * @author Hatim Khan
 */
public class ServicesSection extends SectionPart implements IPortalSettingsConfigurationChangedListener
{
  public static final int CLIENT_VSPACING = 4;
  public static final Class[] EDIT_ACCEPT_CLASSES = new Class[] { ServiceConfiguration.class,
      ObjectParam.class, PropertiesParam.class, ValuesParam.class, ValueParam.class };
  public static final Class[] REMOVE_ACCEPT_CLASSES = new Class[] { ServiceConfiguration.class,
      ObjectParam.class, PropertiesParam.class, ValuesParam.class, ValueParam.class };

  private TreeViewer mTreeViewer;
  private Button mAddButton;
  private Button mAddGlobalButton;
  private Button mEditButton;
  private Button mRemoveButton;
  private Configuration mConfiguration;
  private PortalSettings mPortalSettings;
  private boolean mReadOnly = false;
  private int mInitHeight;

  public ServicesSection(boolean pReadOnly, Configuration pConfiguration, PortalSettings pPortalSettings,
      String pTitle, String pDescription, int pInitHeight, IManagedForm pManagedForm, Composite pParent,
      boolean pExpand)
  {
    super(pParent, pManagedForm.getToolkit(), Section.TITLE_BAR | Section.DESCRIPTION | Section.TWISTIE
        | ((pExpand) ? Section.EXPANDED : Section.TWISTIE));
    initialize(pManagedForm);
    mReadOnly = pReadOnly;
    mConfiguration = pConfiguration;
    mPortalSettings = pPortalSettings;
    if (mPortalSettings != null)
    {
      mPortalSettings.addConfigurationChangedListener(mConfiguration, this);
    }
    mInitHeight = pInitHeight;
    getSection().clientVerticalSpacing = CLIENT_VSPACING;
    getSection().setData("part", this); //$NON-NLS-1$
    getSection().setText(pTitle);
    getSection().setDescription(pDescription);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    getSection().setLayoutData(gd);

    createSectionClient(getSection(), pManagedForm.getToolkit());
  }

  private void createSectionClient(Section pSection, FormToolkit pToolkit)
  {
    Composite container = UICompositeUtil.createContainer(pSection, 2, 2, 5, pToolkit);

    mTreeViewer = createTreeViewer(container, SWT.FULL_SELECTION, pToolkit);
    createActionButtons(container, pToolkit);
    updateButtonsState(null);
    attacheListenersToButtons();

    mTreeViewer.setContentProvider(new PortalConfigurationContentProvider(true));
    mTreeViewer.setLabelProvider(new PortalConfigurationLabelProvider());
    mTreeViewer.setInput(mConfiguration);
    mTreeViewer.addSelectionChangedListener(new ISelectionChangedListener()
    {
      public void selectionChanged(SelectionChangedEvent pEvent)
      {
        getManagedForm().fireSelectionChanged(ServicesSection.this, pEvent.getSelection());
        updateButtonsState(pEvent.getSelection());
      }
    });
    mTreeViewer.addDoubleClickListener(new IDoubleClickListener()
    {
      public void doubleClick(DoubleClickEvent pEvent)
      {
        getManagedForm().fireSelectionChanged(ServicesSection.this, pEvent.getSelection());
        updateButtonsState(pEvent.getSelection());
        Object selection = UICompositeUtil.getSelection(pEvent.getSelection(), ServicesSection.EDIT_ACCEPT_CLASSES);
        editButtonPressed(selection);
      }
    });

    pToolkit.paintBordersFor(container);
    pSection.setClient(container);
  }

  private void createActionButtons(Composite pContainer, FormToolkit pToolkit)
  {
    Composite container = UICompositeUtil.createContainer(pContainer, 1, 1, 0, pToolkit);
    container.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));
    mEditButton = createButton("Edit...", container, pToolkit);
    mRemoveButton = createButton("Remove", container, pToolkit);

    Composite container2 = UICompositeUtil.createContainer(pContainer, 2, 1, 0, pToolkit);
    container2.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING));
    mAddButton = createButton("Add Service Configuration...", container2, pToolkit);
    mAddGlobalButton = createButton("Add Global Parameter...", container2, pToolkit);
  }

  private void attacheListenersToButtons()
  {
    mAddButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        addButtonPressed();
      }
    });

    mAddGlobalButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        addGlobalButtonPressed();
      }
    });

    mEditButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mTreeViewer.getSelection(), ServicesSection.EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mAddButton.getShell(), "Error while trying to edit the selection", "Either the selection is empty or you are trying to edit a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          editButtonPressed(selection);
        }
      }
    });

    mRemoveButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mTreeViewer.getSelection(), ServicesSection.REMOVE_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mAddButton.getShell(), "Error while trying to remove the selection", "Either the selection is empty or you are trying to remove a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          removeButtonPressed(selection);
        }
      }
    });
  }

  private void addButtonPressed()
  {
    if (mReadOnly) return;
    if ((mConfiguration != null) && (mConfiguration.isReadOnly())) return;

    boolean newConfiguration = false;
    Configuration config = null;

    // we have to remember that there might be not configuration object yet
    if (mConfiguration == null)
    {
      newConfiguration = true;
      config = new Configuration();
    }
    else
    {
      config = mConfiguration;
    }

    EditServiceWizard wizard = new EditServiceWizard(null, config);
    WizardDialog container = new WizardDialog(getManagedForm().getForm().getShell(), wizard);
    container.open();
    if (wizard.isDirty())
    {
      // since we did not pass the portalsettings to the wizard we need to do the events
      // firing ourselves
      if (newConfiguration)
      {
        mPortalSettings.setCustomPortalConfiguration(config);
        mPortalSettings.addConfigurationChangedListener(config, this);
        mConfiguration = config;
        mTreeViewer.setInput(mConfiguration);
      }

      mPortalSettings.markDirty();
      mPortalSettings.fireConfigurationElementUpdated(mConfiguration);
      getManagedForm().dirtyStateChanged();
    }
  }

  private void addGlobalButtonPressed()
  {
    if (mReadOnly) return;
    if ((mConfiguration != null) && (mConfiguration.isReadOnly())) return;

    boolean newGlobalParam = false;
    boolean newConfiguration = false;
    Configuration config = null;
    GlobalParam globalParam = null;

    // we have to remember that there might be not configuration object yet
    if (mConfiguration == null)
    {
      newConfiguration = true;
      config = new Configuration();
    }
    else
    {
      config = mConfiguration;
    }

    List globalParams = config.getGlobalParam();
    if ((globalParams == null) || (globalParams.size() < 1))
    {
      newGlobalParam = true;
      globalParam = new GlobalParam();
    }
    else
    {
      globalParam = (GlobalParam) globalParams.get(0);
    }

    EditServiceParameterWizard wizard = new EditServiceParameterWizard(ValuesParam.class, null, globalParam);
    WizardDialog container = new WizardDialog(getManagedForm().getForm().getShell(), wizard);
    container.open();
    if (wizard.isDirty())
    {
      // since we did not pass the portalsettings to the wizard we need to do the events
      // firing ourselves
      if (newConfiguration)
      {
        mPortalSettings.setCustomPortalConfiguration(config);
        mPortalSettings.addConfigurationChangedListener(config, this);
        mConfiguration = config;
        mTreeViewer.setInput(mConfiguration);
      }

      if (newGlobalParam)
      {
        mConfiguration.addChild(globalParam);
      }
      mPortalSettings.markDirty();
      mPortalSettings.fireConfigurationElementUpdated(globalParam);
      getManagedForm().dirtyStateChanged();
    }
  }

  private void editButtonPressed(Object pSelection)
  {
    if (mReadOnly) return;
    if ((mConfiguration != null) && (mConfiguration.isReadOnly())) return;

    if (pSelection instanceof ServiceConfiguration)
    {
      ServiceConfiguration service = (ServiceConfiguration) pSelection;
      EditServiceWizard wizard = new EditServiceWizard(mPortalSettings, mConfiguration, service);
      WizardDialog container = new WizardDialog(getManagedForm().getForm().getShell(), wizard);
      container.open();
      if (wizard.isDirty())
      {
        getManagedForm().dirtyStateChanged();
      }
    }
    if (pSelection instanceof Parameter)
    {
      Parameter parameter = (Parameter) pSelection;
      EditServiceParameterWizard wizard = new EditServiceParameterWizard(mPortalSettings, parameter);
      WizardDialog container = new WizardDialog(getManagedForm().getForm().getShell(), wizard);
      container.open();
      if (wizard.isDirty())
      {
        getManagedForm().dirtyStateChanged();
      }
    }
  }

  private void removeButtonPressed(Object pSelection)
  {
    if (mReadOnly) return;
    if ((mConfiguration != null) && (mConfiguration.isReadOnly())) return;
    if (pSelection instanceof INode)
    {
      INode node = (INode) pSelection;
      Object object = node.getParent();
      if ((object == null) || !(object instanceof INode))
      {
        MessageDialog.openError(mAddButton.getShell(), "Error while trying to remove the selection", "The selection you made does not appear to have a parent, removing it is not possible. Please make a different selection and try again.");
      }
      else
      {
        INode parent = (INode) object;
        boolean removed = parent.removeChild(node);
        if (removed)
        {
          mPortalSettings.markDirty();
          getManagedForm().dirtyStateChanged();
          mPortalSettings.fireConfigurationElementRemoved(parent, node);
          if (!(parent instanceof Configuration))
          {
            mTreeViewer.setSelection(new StructuredSelection(parent), true);
          }
          setFocus();
        }
        else
        {
          MessageDialog.openError(mAddButton.getShell(), "Error while trying to remove the selection", "There seems to be inconsistency in the model, removing the selection is not possible. Please make a different selection and try again.");
        }
      }
    }
    else
    {
      MessageDialog.openError(mAddButton.getShell(), "Error while trying to remove the selection", "Removing the selection you made is not supported. Please make a different selection and try again.");
    }
  }

  private Button createButton(String pLabel, Composite pContainer, FormToolkit pToolkit)
  {
    Button button = pToolkit.createButton(pContainer, pLabel, SWT.PUSH);
    button.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING | GridData.FILL_HORIZONTAL));
    return button;
  }

  private TreeViewer createTreeViewer(Composite parent, int style, FormToolkit toolkit)
  {
    style |= SWT.H_SCROLL | SWT.V_SCROLL;
    if (toolkit == null)
    {
      style |= SWT.BORDER;
    }
    else
    {
      style |= toolkit.getBorderStyle();
    }
    TreeViewer treeViewer = new TreeViewer(parent, style);
    Control tree = treeViewer.getControl();
    GridData gridData = new GridData(GridData.FILL_BOTH);
    tree.setLayoutData(gridData);
    UICompositeUtil.setWidthHeight(tree, -1, mInitHeight);

    return treeViewer;
  }

  private void updateButtonsState(ISelection pSelection)
  {
    Object selectedObject = null;
    if (pSelection instanceof IStructuredSelection)
    {
      IStructuredSelection ssel = (IStructuredSelection) pSelection;
      if (ssel.size() == 1)
      {
        selectedObject = ssel.getFirstElement();
      }
    }

    updateButtonsState(selectedObject);
  }

  private void updateButtonsState(Object pModelObject)
  {
    mAddButton.setEnabled(true);
    mAddGlobalButton.setEnabled(true);

    if (mReadOnly)
    {
      mAddButton.setEnabled(false);
      mAddGlobalButton.setEnabled(false);
      mEditButton.setEnabled(false);
      mRemoveButton.setEnabled(false);
      return;
    }

    // first of all if the configuration itself is read-only then we disable all buttons
    if (mConfiguration != null)
    {
      if (mConfiguration.isReadOnly())
      {
        mAddButton.setEnabled(false);
        mAddGlobalButton.setEnabled(false);
        mEditButton.setEnabled(false);
        mRemoveButton.setEnabled(false);
        return;
      }
    }

    if ((pModelObject == null) || (pModelObject instanceof Configuration))
    {
      // allow add, but not edit or remove
      mEditButton.setEnabled(false);
      mRemoveButton.setEnabled(false);
      return;
    }

    if (pModelObject instanceof ServiceConfiguration)
    {
      mEditButton.setEnabled(true);
      mRemoveButton.setEnabled(true);
      return;
    }

    if (pModelObject instanceof Parameter)
    {
      // notice that we don't allow the removable of stick parameters
      mEditButton.setEnabled(true);
      Parameter parameter = (Parameter) pModelObject;
      mRemoveButton.setEnabled(!parameter.isSticky());
      return;
    }

    if (pModelObject instanceof GlobalParam)
    {
      mEditButton.setEnabled(false);
      mRemoveButton.setEnabled(false);
      return;
    }

  }

  public void dispose()
  {
    super.dispose();
    if (mPortalSettings != null)
    {
      mPortalSettings.removeConfigurationChangedListener(this);
    }
  }

  public void refresh()
  {
    super.refresh();
    if (mTreeViewer != null)
    {
      mTreeViewer.refresh();
    }
  }

  public void setFocus()
  {
    if (mTreeViewer != null)
    {
      mTreeViewer.getControl().setFocus();
    }
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementRemoved(java.lang.Object,
   *      java.lang.Object)
   */
  public void ConfigurationElementRemoved(Object pParent, Object pChild)
  {
    markStale();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementAdded(java.lang.Object)
   */
  public void ConfigurationElementAdded(Object pChild)
  {
    markStale();
  }

  /**
   * @see org.exoplatform.eclipse.core.portal.IPortalSettingsConfigurationChangedListener#ConfigurationElementUpdated(java.lang.Object)
   */
  public void ConfigurationElementUpdated(Object pElement)
  {
    markStale();
  }
}