/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.editor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.exoplatform.eclipse.core.exoservices.ValueParam;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * This details page displays the detail value parameter entry information. This page does
 * not change the model or have any field to allow the user to enter data, however, this
 * page should receive model change events and marks itself as stale. However, since this
 * page extends from ParameterInfoDetailPart, it does not need to monitor model changes or
 * mark itself as stale since it is being done in ParameterInfoDetailPart class.
 * 
 * @author Hatim Khan
 */
public class ValueParamInfoDetailPart extends ParameterInfoDetailPart
{
  private Text mValue = null;

  public ValueParamInfoDetailPart(PortalSettings pPortalSettings)
  {
    super(pPortalSettings, "Value Parameter", "Parameter Details",
        "Detailed information about the selected parameter");
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.editor.ParameterInfoDetailPart#addValuesContent(org.eclipse.swt.widgets.Composite,
   *      org.eclipse.ui.forms.widgets.FormToolkit)
   */
  protected void addValuesContent(Composite pClient, FormToolkit pToolkit)
  {
    createLabel("Value:", pClient, pToolkit);
    mValue = pToolkit.createText(pClient, "", SWT.MULTI | SWT.WRAP | SWT.LEFT | SWT.READ_ONLY);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    gd.horizontalSpan = 1;
    mValue.setLayoutData(gd);
    UICompositeUtil.setHeightHint(mValue, 40);
  }

  protected void update()
  {
    super.update();
    if ((mParameter == null) || !(mParameter instanceof ValueParam))
    {
      mValue.setText("");
      return;
    }
    else
    {
      ValueParam valueParam = (ValueParam) mParameter;
      mValue.setText(UICompositeUtil.escapeBackslash(UICompositeUtil.emptyIfNull(valueParam.getValue())));
    }
  }

}