/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.launching;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.debug.ui.ILaunchConfigurationDialog;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.exoplatform.eclipse.core.launching.IExoLaunchingConfigurationConstants;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.exoplatform.eclipse.ui.common.OpenedPortletProjectFilter;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * This tab allows the user to select which portlet project to include in the source lookup path during debugging.
 *
 * @author Hatim Khan
 * @version $Revision: 1.2 $
 */
public class ExoSourcePortletsTab extends AbstractLaunchConfigurationTab
{
  public static final String CLASS_VERSION = "$Id: ExoSourcePortletsTab.java,v 1.2 2004/10/10 03:24:50 hatimk Exp $"; //$NON-NLS-1$
  public static final String TAB_NAME = "&Portlets";
  private static final int PROJECT_LIST_MULTIPLIER = 30;
  private Image mTabImage;
  private CheckboxTableViewer mPortletsViewer;
  private Button mSelectAllButton;
  private Button mDeselectAllButton;

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite = UICompositeUtil.createFillBothComposite(pParent, 1, 1);
    UICompositeUtil.addLabel(mainComposite, "", 1);
    createPortletsGroup(mainComposite);

    setControl(mainComposite);
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
   */
  public void setDefaults(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
   */
  public void initializeFrom(ILaunchConfiguration pConfig)
  {
    try
    {
      initializePortletsListFrom(pConfig);
    }
    catch (CoreException e)
    {
      ExoUIPlugin.logError(e);
    }
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
   */
  public void performApply(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    savePortletsListToConfig(pWorkingCopy);
  }

  private void initializePortletsListFrom(ILaunchConfiguration pConfig)
    throws CoreException
  {
    // first lets uncheck all of the projects
    mPortletsViewer.setAllChecked(false);

    // then we need to remember all the available elements by their name
    // we use a map to do that
    int count = mPortletsViewer.getTable().getItemCount();
    Map elementsMap = new HashMap();
    for (int i = 0; i < count; ++i)
    {
      Object element = mPortletsViewer.getElementAt(i);
      if (element instanceof IProject)
      {
        elementsMap.put(((IProject) element).getName(), element);
      }
    }

    List projects =
      pConfig.getAttribute(IExoLaunchingConfigurationConstants.ATTR_PORTLET_PROJECT_LIST,
        (List) null);

    if (projects == null)
    {
      elementsMap.clear();

      return;
    }

    // now we use the map to retrive the element using the name
    Iterator iterator = projects.iterator();
    while (iterator.hasNext())
    {
      Object element = elementsMap.get(iterator.next());

			// lets check if the element is there (if the project is deleted, then we might
			// get a null element).
			
			if (element != null)
			{
				mPortletsViewer.setChecked(element, true);				
			}
    }

    elementsMap.clear();
  }

  private void savePortletsListToConfig(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    List result = null;

    Object[] elements = mPortletsViewer.getCheckedElements();
    if ((elements != null) && (elements.length > 0))
    {
      // we assume that the elements are of type IProject
      result = new ArrayList();
      for (int i = 0, sizei = elements.length; i < sizei; ++i)
      {
        if (elements[i] instanceof IProject)
        {
          result.add(((IProject) elements[i]).getName());
        }
      }

      if (result.size() < 1)
      {
        result = null;
      }
    }

    pWorkingCopy.setAttribute(IExoLaunchingConfigurationConstants.ATTR_PORTLET_PROJECT_LIST, result);
  }

  private void createPortletsGroup(Composite pParent)
  {
    Font font = pParent.getFont();
    Composite group = UICompositeUtil.createGroup(pParent, "Portlets", 2, 1);
    UICompositeUtil.addLabelWithWrapping(group,
      "Select all the portlet projects that you want to appear in the source lookup path during debugging :",
      2);

    mPortletsViewer = CheckboxTableViewer.newCheckList(group, SWT.TOP | SWT.BORDER);
    mPortletsViewer.getTable().setFont(font);

    GridData data = new GridData(GridData.FILL_BOTH);

    //data.verticalSpan = 2;
    //Only set a height hint if it will not result in a cut off dialog
    if (UICompositeUtil.inRegularFontMode(group))
    {
      data.heightHint = getDefaultFontHeight(mPortletsViewer.getTable(), PROJECT_LIST_MULTIPLIER);
    }
    mPortletsViewer.getTable().setLayoutData(data);
    mPortletsViewer.getTable().setFont(font);

    mPortletsViewer.setLabelProvider(WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider());
    mPortletsViewer.setContentProvider(new WorkbenchContentProvider());
    mPortletsViewer.setInput(ResourcesPlugin.getWorkspace());
    mPortletsViewer.addFilter(new OpenedPortletProjectFilter());

    mPortletsViewer.addCheckStateListener(new ICheckStateListener()
      {
        public void checkStateChanged(CheckStateChangedEvent pEvent)
        {
          updateDialog();
        }
      });

    createButtons(group, font);
  }

  private void createButtons(Composite pParent, Font pFont)
  {
    Composite buttonsContainer = new Composite(pParent, SWT.NONE);
    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.numColumns = 1;

    GridData containerGridData = new GridData();
    containerGridData.horizontalAlignment = GridData.BEGINNING;
    containerGridData.verticalAlignment = GridData.BEGINNING;
    containerGridData.grabExcessHorizontalSpace = false;
    containerGridData.grabExcessVerticalSpace = false;
    buttonsContainer.setLayout(layout);
    buttonsContainer.setLayoutData(containerGridData);
    buttonsContainer.setFont(pFont);

    // select all button
    mSelectAllButton = new Button(buttonsContainer, SWT.PUSH);
    mSelectAllButton.setText("S&elect All");
    mSelectAllButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent pEvent)
        {
          mPortletsViewer.setAllChecked(true);
          updateDialog();
        }
      });

    mSelectAllButton.setEnabled(true);
    mSelectAllButton.setFont(pFont);

    UICompositeUtil.setButtonLayoutData(mSelectAllButton);

    // deselect all button
    mDeselectAllButton = new Button(buttonsContainer, SWT.PUSH);
    mDeselectAllButton.setText("Dese&lect All");
    mDeselectAllButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent pEvent)
        {
          mPortletsViewer.setAllChecked(false);
          updateDialog();
        }
      });

    mDeselectAllButton.setEnabled(true);
    mDeselectAllButton.setFont(pFont);

    UICompositeUtil.setButtonLayoutData(mDeselectAllButton);
  }

  /**   
   * Get the defualt widget height for the supplied control.
   *
   * @param control - the control being queried about fonts
   * @param lines - the number of lines to be shown on the table.
   *
   * @return int
   */
  private static int getDefaultFontHeight(Control control, int lines)
  {
    FontData[] viewerFontData = control.getFont().getFontData();
    int fontHeight = 10;

    //If we have no font data use our guess
    if (viewerFontData.length > 0)
    {
      fontHeight = viewerFontData[0].getHeight();
    }

    return lines * fontHeight;
  }

  private void updateDialog()
  {
    ILaunchConfigurationDialog dialog = getLaunchConfigurationDialog();
    if (dialog != null)
    {
      dialog.updateMessage();
      dialog.updateButtons();
    }
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
   */
  public String getName()
  {
    return TAB_NAME;
  }

  /**   
   * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#getImage()
   */
  public Image getImage()
  {
    if (mTabImage == null)
    {
      mTabImage = ExoUIPluginImages.DESC_OBJ_SOURCE_PORTLET_TAB.createImage();
    }

    return mTabImage;
  }

  /**   
   * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#dispose()
   */
  public void dispose()
  {
    super.dispose();

    if ((mTabImage != null) && (!mTabImage.isDisposed()))
    {
      mTabImage.dispose();
      mTabImage = null;
    }
  }
  
	public void activated(ILaunchConfigurationWorkingCopy pWorkingCopy)
	{
		// do nothing
	}

	public void deactivated(ILaunchConfigurationWorkingCopy pWorkingCopy)
	{
		// do nothing
	}
  
}
