/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.launching;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.debug.ui.ILaunchConfigurationDialog;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.IVMInstallType;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;


/**
 * A Tab that allows the user to select a JRE. The tab had a button that will open a dialog to
 * create a new JVM. However, this button is removed since it was depending on
 * IAddVMDialogRequestor which is no longer supported in Eclipse 3.0.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoTomcatJDKTab extends AbstractLaunchConfigurationTab // implements IAddVMDialogRequestor
{
  public static final String CLASS_VERSION = "$Id: ExoTomcatJDKTab.java,v 1.1 2004/09/20 01:42:56 hatimk Exp $"; //$NON-NLS-1$
  public static final String TAB_NAME = "&JDK";
  private Image mTabImage;
  private Combo mJDKCombo;

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite = UICompositeUtil.createFillBothComposite(pParent, 1, 1);
    UICompositeUtil.addLabel(mainComposite, "", 1);
    createJDKGroup(mainComposite);
    createNoteGroup(mainComposite);

    setControl(mainComposite);
  }

  /**   
   * @see ILaunchConfigurationTab#isValid(ILaunchConfiguration)
   */
  public boolean isValid(ILaunchConfiguration pConfig)
  {
    return isValid();
  }

  /**   
   * Check if the user selected a JRE
   *
   * @return true if a JRE has been selected, false otherwise
   */
  public boolean isValid()
  {
    setMessage(null);
    setErrorMessage(null);

    return validateThereIsJRE();
  }

  private boolean validateThereIsJRE()
  {
    boolean result = true;
    if ((mJDKCombo != null) && (!mJDKCombo.isDisposed()))
    {
      String text = mJDKCombo.getText();
      if ((text == null) || (text.length() < 1))
      {
        setErrorMessage("You must specify a JRE");
        result = false;
      }
    }

    return result;
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
   */
  public void setDefaults(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
   */
  public void initializeFrom(ILaunchConfiguration pConfiguration)
  {
    try
    {
      initializeJDKComboFrom(pConfiguration);
    }
    catch (CoreException e)
    {
      ExoUIPlugin.logError(e);
    }
  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
   */
  public void performApply(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    saveJDKComboToConfig(pWorkingCopy);
  }

  private void initializeJDKComboFrom(ILaunchConfiguration pConfig)
    throws CoreException
  {
    mJDKCombo.removeAll();

    IVMInstallType[] vmtypes = JavaRuntime.getVMInstallTypes();
    if (vmtypes == null)
    {
      return;
    }

    for (int i = 0, sizei = vmtypes.length; i < sizei; ++i)
    {
      IVMInstall[] vminstalls = vmtypes[i].getVMInstalls();
      if (vminstalls != null)
      {
        for (int j = 0, sizej = vminstalls.length; j < sizej; ++j)
        {
          mJDKCombo.add(vminstalls[j].getName());
        }
      }
    }

    String vmTypeID =
      pConfig.getAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_INSTALL_TYPE, (String) null);
    String vmName =
      pConfig.getAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_INSTALL_NAME, (String) null);
    if ((vmTypeID == null) || (vmName == null) || (vmName.length() < 1))
    {
      return;
    }

    mJDKCombo.setText(vmName);
  }

  private void saveJDKComboToConfig(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
    String vmTypeId = null;
    String vmName = null;

    String selectedVMName = mJDKCombo.getText();
    if ((selectedVMName != null) && (selectedVMName.length() > 0))
    {
      IVMInstallType[] vmtypes = JavaRuntime.getVMInstallTypes();
      if (vmtypes != null)
      {
outer: 
        for (int i = 0, sizei = vmtypes.length; i < sizei; ++i)
        {
          IVMInstall[] vminstalls = vmtypes[i].getVMInstalls();
          if (vminstalls != null)
          {
            for (int j = 0, sizej = vminstalls.length; j < sizej; ++j)
            {
              if (vminstalls[j].getName().equals(selectedVMName))
              {
                vmTypeId = vmtypes[i].getId();
                vmName = selectedVMName;

                break outer;
              }
            }
          }
        }
      }
    }

    pWorkingCopy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_INSTALL_TYPE, vmTypeId);
    pWorkingCopy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_INSTALL_NAME, vmName);
  }

  private void createJDKGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group = UICompositeUtil.createGroupWithNoGrapping(pParent, "Runtime JRE", 1, 1);
    mJDKCombo = new Combo(group, SWT.DROP_DOWN | SWT.READ_ONLY);

    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    mJDKCombo.setLayoutData(gridData);
    mJDKCombo.setFont(font);
    mJDKCombo.addModifyListener(new ModifyListener()
      {
        public void modifyText(ModifyEvent pEvent)
        {
          updateDialog();
        }
      });

    // installed JREs button
    //    Button newJDKButton = new Button(group, SWT.PUSH);
    //    newJDKButton.setText("Installed J&REs..");
    //    newJDKButton.addSelectionListener(new SelectionAdapter()
    //      {
    //        public void widgetSelected(SelectionEvent pEvent)
    //        {
    //          handleJREAddButtonSelected();
    //        }
    //      });
    //
    //    newJDKButton.setEnabled(true);
    //    newJDKButton.setFont(font);
    //
    //    UICompositeUtil.setButtonLayoutData(newJDKButton);
  }

  private void createNoteGroup(Composite pParent)
  {
    Font font = pParent.getFont();

    Composite group = UICompositeUtil.createGroupWithNoGrapping(pParent, "Note", 1, 1);
    UICompositeUtil.addLabelWithWrapping(group,
      "Please select a JRE that is part of a JDK installation.  Certain versions of Tomcat require the use of tools.jar which is part of a JDK installation and not a JRE installation.",
      1);
  }

  private void updateDialog()
  {
    isValid();

    ILaunchConfigurationDialog dialog = getLaunchConfigurationDialog();
    if (dialog != null)
    {
      dialog.updateMessage();
      dialog.updateButtons();
    }
  }

  //  /**   
  //   * Show a dialog that lets the user add a new JRE definition
  //   */
  //  private void handleJREAddButtonSelected()
  //  {
  //    IVMInstallType[] vmtypes = JavaRuntime.getVMInstallTypes();
  //    AddVMDialog dialog = new AddVMDialog(this, getShell(), vmtypes, null);
  //    dialog.setTitle("Select");
  //    if (dialog.open() != AddVMDialog.OK)
  //    {
  //      return;
  //    }
  //  }
  //  /**   
  //   * @see org.eclipse.jdt.internal.debug.ui.launcher.IAddVMDialogRequestor#isDuplicateName(java.lang.String)
  //   */
  //  public boolean isDuplicateName(String pName)
  //  {
  //    IVMInstallType[] vmtypes = JavaRuntime.getVMInstallTypes();
  //    if (vmtypes != null)
  //    {
  //      for (int i = 0, sizei = vmtypes.length; i < sizei; ++i)
  //      {
  //        IVMInstall[] vminstalls = vmtypes[i].getVMInstalls();
  //        if (vminstalls != null)
  //        {
  //          for (int j = 0, sizej = vminstalls.length; j < sizej; ++j)
  //          {
  //            if (vminstalls[j].getName().equals(pName))
  //            {
  //              return true;
  //            }
  //          }
  //        }
  //      }
  //    }
  //
  //    return false;
  //  }
  //  /**   
  //   * @see org.eclipse.jdt.internal.debug.ui.launcher.IAddVMDialogRequestor#vmAdded(org.eclipse.jdt.launching.IVMInstall)
  //   */
  //  public void vmAdded(IVMInstall pVMInstall)
  //  {
  //    ((VMStandin) pVMInstall).convertToRealVM();
  //    try
  //    {
  //      JavaRuntime.saveVMConfiguration();
  //    }
  //    catch (CoreException e)
  //    {
  //      ExoUIPlugin.logError(e);
  //    }
  //    mJDKCombo.add(pVMInstall.getName());
  //    mJDKCombo.setText(pVMInstall.getName());
  //  }

  /**   
   * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
   */
  public String getName()
  {
    return TAB_NAME;
  }

  /**   
   * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#getImage()
   */
  public Image getImage()
  {
    if (mTabImage == null)
    {
      mTabImage = ExoUIPluginImages.DESC_OBJ_JDK_HOME.createImage();
    }

    return mTabImage;
  }

  /**   
   * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#dispose()
   */
  public void dispose()
  {
    super.dispose();

    if ((mTabImage != null) && (!mTabImage.isDisposed()))
    {
      mTabImage.dispose();
      mTabImage = null;
    }
  }
  
  public void activated(ILaunchConfigurationWorkingCopy pWorkingCopy)
  {
  	// do nothing
  }

	public void deactivated(ILaunchConfigurationWorkingCopy pWorkingCopy)
	{
		// do nothing
	}

}
