/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.preference.helper;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.exoplatform.eclipse.core.operation.RemovePortletValidationMarkersOperation;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.action.ExoRunnableAdapter;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;
import org.exoplatform.eclipse.ui.dialog.ProblemDialog;

import java.lang.reflect.InvocationTargetException;

/**
 * This field editor is not assoicated with any preference property. Instead,
 * it just displays a message and a button that clears all portlet validation
 * errors.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PortletValidationRemoverFieldEditor extends FieldEditor
{
  public static final String CLASS_VERSION = "$Id: PortletValidationRemoverFieldEditor.java,v 1.1 2004/04/19 03:37:23 hatimk Exp $"; //$NON-NLS-1$
  private static final String GROUP_TITLE = "Portlet validation errors";
  private static final String PROBLEM_TITLE = "Portlet Validation Error Removal Problem";
  private static final String DESCRIPTION = "The above button enables you to remove all existing\n"+
  "portlet validation error markers. This button is provided for convience\n"+
  "and it only makes";
  private Composite mMainComposite;
  private Shell mShell;

  /**
   * The constructor method.
   *
   * @param pParent
   * @param pShell
   */
  public PortletValidationRemoverFieldEditor(Composite pParent, Shell pShell)
  {
    mShell = pShell;
    createControl(pParent);
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#adjustForNumColumns(int)
   */
  protected void adjustForNumColumns(int pNumColumns)
  {
    if (mMainComposite != null)
    {
      ((GridData) mMainComposite.getLayoutData()).horizontalSpan = pNumColumns;
    }
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#doFillIntoGrid(org.eclipse.swt.widgets.Composite,
   *      int)
   */
  protected void doFillIntoGrid(Composite pParent, int pNumColumns)
  {
    Group group =
      UICompositeUtil.createGroupWithNoGrapping(pParent, GROUP_TITLE, 2,
        pNumColumns);
    createErrorRemovalButton(group);
    UICompositeUtil.addLabel(group, DESCRIPTION, 2);
    
    mMainComposite = group;
  }

  private void createErrorRemovalButton(Composite pParent)
  {
    Font font = pParent.getFont();

    Button removeButton = new Button(pParent, SWT.PUSH);
    removeButton.setText("Remove All");
    removeButton.addSelectionListener(new SelectionAdapter()
      {
        public void widgetSelected(SelectionEvent pEvent)
        {
          handleValidationRemoveButtonPressed();
        }
      });

    removeButton.setEnabled(true);
    removeButton.setFont(font);
    setButtonLayoutData(removeButton);
  }

  private void handleValidationRemoveButtonPressed()
  {
    RemovePortletValidationMarkersOperation op = null;
    try
    {
      op = new RemovePortletValidationMarkersOperation();
      new ProgressMonitorDialog(mShell).run(true, true,
        new ExoRunnableAdapter(op));

      IStatus status = op.getStatus();
      if (!status.isOK())
      {
        ProblemDialog.open(mShell, PROBLEM_TITLE , null,
          status);
      }
    }
    catch (InvocationTargetException target)
    {
      Throwable cause = target.getTargetException();
      if (cause instanceof CoreException)
      {
        IStatus status = ((CoreException) cause).getStatus();
        ProblemDialog.open(mShell, PROBLEM_TITLE, null,
          status);      
        ExoUIPlugin.log(status);
      }
      else
      {
        String message =
          (cause == null) ? target.getLocalizedMessage() : cause.toString();
        MessageDialog.openError(mShell, PROBLEM_TITLE,
          "Encountered a problem while trying to remove validaton errors\n"
          + message);
        ExoUIPlugin.logError(cause);
      }
    }
    catch (InterruptedException e)
    {
      // no action is required if user requests cancelation
    }
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#doLoad()
   */
  protected void doLoad()
  {
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#doLoadDefault()
   */
  protected void doLoadDefault()
  {
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#doStore()
   */
  protected void doStore()
  {
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#getNumberOfControls()
   */
  public int getNumberOfControls()
  {
    return 1;
  }
}
