/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.preference.helper;


import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.viewers.ColumnLayoutData;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.exoplatform.eclipse.core.xmlvalidator.IPortletDescriptorValidator;
import org.exoplatform.eclipse.core.xmlvalidator.PortletDescriptorValidatorProvider;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * A field editor that actually does no editing at all. All it does is display
 * a table with information about the supported portlet versions for
 * validation.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ValidatorVersionTableFieldEditor extends FieldEditor
{
  public static final String CLASS_VERSION = "$Id: ValidatorVersionTableFieldEditor.java,v 1.1 2004/04/19 03:37:23 hatimk Exp $"; //$NON-NLS-1$
  private static final String TABLE_DESCRIPTION =
    "The eXo plugin-in portlet validator supports the following "
    + "versions of the portlet deployment descriptor. Next to each "
    + "portlet version is the supported version of the eXo extension.";
  private final String[] pColumnHeaders =
  { "JSR 168 Version", "eXo Extension Version" };
  private ColumnLayoutData[] pColumnLayouts =
  { new ColumnWeightData(200), new ColumnWeightData(200) };
  private Composite mMainComposite;

  /**
   * The constructor method.
   *
   * @param pParent
   */
  public ValidatorVersionTableFieldEditor(Composite pParent)
  {
    createControl(pParent);
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#adjustForNumColumns(int)
   */
  protected void adjustForNumColumns(int pNumColumns)
  {
    if (mMainComposite != null)
    {
      ((GridData) mMainComposite.getLayoutData()).horizontalSpan = pNumColumns;
    }
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#doFillIntoGrid(org.eclipse.swt.widgets.Composite,
   *      int)
   */
  protected void doFillIntoGrid(Composite pParent, int pNumColumns)
  {
    Group group =
      UICompositeUtil.createGroup(pParent, "Portlet validator information", 1,
        pNumColumns);
    UICompositeUtil.addLabelWithWrapping(group, TABLE_DESCRIPTION, 1);
    createValidatorTableComposite(group);

    mMainComposite = group;
  }

  private void createValidatorTableComposite(Composite pParent)
  {
    Table versionTable = createTable(pParent);
    createColumns(versionTable);

    TableViewer viewer = new TableViewer(versionTable);
    viewer.setContentProvider(new ValidatorVersionContentProvider());
    viewer.setLabelProvider(new ValidatorVersionLabelProvider());
    viewer.setInput(new PortletDescriptorValidatorProvider());
  }

  private Table createTable(Composite parent)
  {
    Table table =
      new Table(parent,
        SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.SINGLE
        | SWT.HIDE_SELECTION);
    table.setLinesVisible(true);

    GridData data = new GridData(GridData.FILL_BOTH);
    table.setLayoutData(data);

    return table;
  }

  private void createColumns(Table pTable)
  {
    TableLayout layout = new TableLayout();
    for (int i = 0; i < pColumnHeaders.length; i++)
    {
      layout.addColumnData(pColumnLayouts[i]);

      TableColumn tc = new TableColumn(pTable, SWT.NONE, i);
      tc.setResizable(pColumnLayouts[i].resizable);
      tc.setText(pColumnHeaders[i]);
    }
    pTable.setLayout(layout);
    pTable.setHeaderVisible(true);
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#doLoad()
   */
  protected void doLoad()
  {
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#doLoadDefault()
   */
  protected void doLoadDefault()
  {
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#doStore()
   */
  protected void doStore()
  {
  }

  /**   
   * @see org.eclipse.jface.preference.FieldEditor#getNumberOfControls()
   */
  public int getNumberOfControls()
  {
    return 1;
  }

  class ValidatorVersionLabelProvider extends LabelProvider
    implements ITableLabelProvider
  {
    /**   
     * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
     *      int)
     */
    public Image getColumnImage(Object pElement, int pColumnIndex)
    {
      return null;
    }

    /**   
     * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
     *      int)
     */
    public String getColumnText(Object pElement, int pColumnIndex)
    {
      String result = "";
      if ((pElement != null)
          && (pElement instanceof IPortletDescriptorValidator))
      {
        IPortletDescriptorValidator desc =
          (IPortletDescriptorValidator) pElement;
        if (pColumnIndex == 0)
        {
          result = desc.getSupportedSpecVersion();
        }
        else if (pColumnIndex == 1)
        {
          result = desc.getSupportedExoVersion();
        }
      }

      return result;
    }
  }

  class ValidatorVersionContentProvider implements IStructuredContentProvider
  {
    /**   
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object pInputElement)
    {
      if (pInputElement instanceof PortletDescriptorValidatorProvider)
      {
        return ((PortletDescriptorValidatorProvider) pInputElement)
        .getValidators();
      }
      else
      {
        return null;
      }
    }

    /**   
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {
    }

    /**   
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public void inputChanged(Viewer pViewer, Object pOldInput, Object pNewInput)
    {
    }
  }
}
