/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import org.eclipse.jface.wizard.Wizard;
import org.exoplatform.eclipse.core.exoservices.GlobalParam;
import org.exoplatform.eclipse.core.exoservices.ObjectParam;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.PropertiesParam;
import org.exoplatform.eclipse.core.exoservices.ValueParam;
import org.exoplatform.eclipse.core.exoservices.ValuesParam;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;

/**
 * @author Hatim Khan
 */
public class EditServiceParameterWizard extends Wizard
{
  public static final String ADD_SERVICE_PARAMETER_TITLE = "Add an eXo Platform service configuration parameter";
  public static final String ADD_GLOBAL_PARAMETER_TITLE = "Add an eXo Platform global parameter";
  public static final String EDIT_SERVICE_PARAMETER_TITLE = "Edit an eXo Platform service configuration parameter";
  public static final String EDIT_GLOBAL_PARAMETER_TITLE = "Edit an eXo Platform global parameter";
  public static final int ADD_MODE = EditServiceParameterPage.ADD_MODE;
  public static final int EDIT_MODE = EditServiceParameterPage.EDIT_MODE;

  private PortalSettings mPortalSettings;
  private Parameter mParameter;
  private Object mParameterParent;
  private int mMode = EDIT_MODE;
  private Class mType;
  private EditServiceParameterPage mParameterPage;

  /**
   * The constructor method.
   */
  public EditServiceParameterWizard(PortalSettings pPortalSettings, Parameter pParameter)
  {
    this(EDIT_MODE, null, pPortalSettings, pParameter.getParent(), pParameter);
  }

  /**
   * The constructor method.
   */
  public EditServiceParameterWizard(Class pType, PortalSettings pPortalSettings, Object pParameterParent)
  {
    this(ADD_MODE, pType, pPortalSettings, pParameterParent, null);
  }

  /**
   * The constructor method.
   */
  private EditServiceParameterWizard(int pMode, Class pType, PortalSettings pPortalSettings,
      Object pParameterParent, Parameter pParameter)
  {
    super();
    mPortalSettings = pPortalSettings;
    mParameter = pParameter;
    mParameterParent = pParameterParent;
    mMode = pMode;
    mType = pType;
    boolean globalParam = isGlobalParameter();
    setWindowTitle((globalParam) ? ((mMode == ADD_MODE) ? ADD_GLOBAL_PARAMETER_TITLE
        : EDIT_GLOBAL_PARAMETER_TITLE) : ((mMode == ADD_MODE) ? ADD_SERVICE_PARAMETER_TITLE
        : EDIT_SERVICE_PARAMETER_TITLE));
    setNeedsProgressMonitor(false);
    setForcePreviousAndNextButtons(false);
    setDefaultPageImageDescriptor((globalParam) ? ExoUIPluginImages.DESC_WIZBAN_EXO_GLOBAL_PARAM
        : ExoUIPluginImages.DESC_WIZBAN_EXO_SERVICES_PARAM);
  }

  private boolean isGlobalParameter()
  {
    Object parent = null;
    if (mMode == ADD_MODE) parent = mParameterParent;
    else parent = (mParameter != null) ? mParameter.getParent() : null;
    return (parent instanceof GlobalParam);
  }

  /**
   * @see org.eclipse.jface.wizard.Wizard#addPages()
   */
  public void addPages()
  {
    super.addPages();

    mParameterPage = (mMode == ADD_MODE) ? createAddPage() : createEditPage();
    if (mParameterPage != null) addPage(mParameterPage);
  }

  private EditServiceParameterPage createAddPage()
  {
    if (mType.equals(ValueParam.class))
    {
      return new EditServiceValueParamPage(EditServiceParameterPage.ADD_MODE, mPortalSettings,
          mParameterParent, null);
    }
    else if (mType.equals(ValuesParam.class))
    {
      return new EditServiceValuesParamPage(EditServiceParameterPage.ADD_MODE, mPortalSettings,
          mParameterParent, null);
    }
    else if (mType.equals(PropertiesParam.class))
    {
      return new EditServicePropertiesParamPage(EditServiceParameterPage.ADD_MODE, mPortalSettings,
          mParameterParent, null);
    }
    else if (mType.equals(ObjectParam.class))
    {
      return new EditServiceObjectParamPage(EditServiceParameterPage.ADD_MODE, mPortalSettings,
          mParameterParent, null);
    }

    return null;
  }

  private EditServiceParameterPage createEditPage()
  {
    Object parameterParent = (mParameter != null) ? mParameter.getParent() : null;
    if (mParameter instanceof ValueParam)
    {
      return new EditServiceValueParamPage(EditServiceParameterPage.EDIT_MODE, mPortalSettings,
          parameterParent, mParameter);
    }
    else if (mParameter instanceof ValuesParam)
    {
      return new EditServiceValuesParamPage(EditServiceParameterPage.EDIT_MODE, mPortalSettings,
          parameterParent, mParameter);
    }
    else if (mParameter instanceof PropertiesParam)
    {
      return new EditServicePropertiesParamPage(EditServiceParameterPage.EDIT_MODE, mPortalSettings,
          parameterParent, mParameter);
    }
    else if (mParameter instanceof ObjectParam)
    {
      return new EditServiceObjectParamPage(EditServiceParameterPage.EDIT_MODE, mPortalSettings,
          parameterParent, mParameter);
    }

    return null;
  }

  /**
   * @see org.eclipse.jface.wizard.IWizard#performFinish()
   */
  public boolean performFinish()
  {
    if (mParameterPage != null) mParameterPage.finish();
    return true;
  }

  public boolean isDirty()
  {
    return (mParameterPage != null) ? mParameterPage.isDirty() : false;
  }
}