/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.ValueParam;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.ui.common.DidYouKnowMessages;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * @author Hatim Khan
 */
public class EditServiceValueParamPage extends EditServiceParameterPage
{
  public static final String TITLE_EDIT = "Edit an eXo Platform service configuration ValueParam";
  public static final String TITLE_ADD = "Add an eXo Platform service configuration ValueParam";
  public static final String MESSAGE_EDIT = "You have the option of editing the name, description and value of this ValueParam parameter";
  public static final String MESSAGE_ADD = "Please provide the name, the description (optional) and the value of this new ValueParam parameter";
  public static final String PAGE_NAME = "EditServiceValueParamPage";
  public static final String DID_YOU_KNOW_ADD = DidYouKnowMessages.getString("EditServiceValueParamPage.add");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_EDIT = DidYouKnowMessages.getString("EditServiceValueParamPage.edit");//$NON-NLS-1$

  private String mOriginalValue;
  private Text mValueText;

  public EditServiceValueParamPage(int pMode, PortalSettings pPortalSettings, Object pParameterParent,
      Parameter pParameter)
  {
    super(PAGE_NAME, (pMode == ADD_MODE) ? DID_YOU_KNOW_ADD : DID_YOU_KNOW_EDIT, pMode, pPortalSettings,
        pParameterParent, pParameter);
    setTitle((pMode == ADD_MODE) ? TITLE_ADD : TITLE_EDIT);
    setMessage((pMode == ADD_MODE) ? MESSAGE_ADD : MESSAGE_EDIT);
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterPage#createNewParameter()
   */
  protected Parameter createNewParameter()
  {
    return new ValueParam();
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterPage#addValuesContent(org.eclipse.swt.widgets.Composite)
   */
  protected void addValuesContent(Composite pClient)
  {
    Label label = new Label(pClient, SWT.NONE);
    label.setText("Value : ");

    GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING | GridData.VERTICAL_ALIGN_BEGINNING);
    gridData.horizontalSpan = 1;
    label.setLayoutData(gridData);

    mValueText = new Text(pClient, SWT.MULTI | SWT.WRAP | SWT.LEFT | SWT.BORDER);
    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    mValueText.setLayoutData(gd);
    UICompositeUtil.setHeightHint(mValueText, 20);
    mValueText.setText("");
  }

  protected void updateFromModel()
  {
    super.updateFromModel();
    String value = "";
    if ((mParameter != null) && (mParameter instanceof ValueParam))
    {
      value = UICompositeUtil.emptyIfNull(((ValueParam) mParameter).getValue()).trim();
    }

    mOriginalValue = value;

    mValueText.setText(value);
  }

  protected void attachListeners()
  {
    super.attachListeners();
    mValueText.addListener(SWT.Modify, new Listener()
    {
      public void handleEvent(Event pEvent)
      {
        setPageComplete(validateInput());
      }
    });
  }

  public void finish()
  {
    // first lets detect if the user changed value
    String value = UICompositeUtil.emptyIfNull(mValueText.getText()).trim();

    if ((!value.equals(mOriginalValue)))
    {
      if ((mParameter != null) && (mParameter instanceof ValueParam))
      {
        ((ValueParam) mParameter).setValue(value);
        mDirty = true;
      }
    }
    super.finish();
  }

}