/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.exoplatform.eclipse.core.common.Utils;
import org.exoplatform.eclipse.core.exoservices.GlobalParam;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.ValuesParam;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;
import org.exoplatform.eclipse.ui.common.DidYouKnowMessages;
import org.exoplatform.eclipse.ui.common.PortalConfigurationContentProvider;
import org.exoplatform.eclipse.ui.common.PortalConfigurationLabelProvider;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;
import org.exoplatform.eclipse.ui.dialog.EditValueHolderDialog;

/**
 * @author Hatim Khan
 */
public class EditServiceValuesParamPage extends EditServiceParameterPage
{

  public static final String TITLE_EDIT = "Edit an eXo Platform service configuration ValuesParam";
  public static final String TITLE_ADD = "Add an eXo Platform service configuration ValuesParam";
  public static final String TITLE_GLOBAL_EDIT = "Edit an eXo Platform global parameter";
  public static final String TITLE_GLOBAL_ADD = "Add an eXo Platform global parameter";
  public static final String MESSAGE_EDIT = "You have the option of editing the name, description and values of this ValuesParam parameter";
  public static final String MESSAGE_ADD = "Please provide the name, the description (optional) and the values of this new ValuesParam parameter";
  public static final String MESSAGE_GLOBAL_EDIT = "Here is the information related to the global parameter that you wish to edit";
  public static final String MESSAGE_GLOBAL_ADD = "Please provide the name, the description (optional) and the values of this new global parameter";

  public static final String PAGE_NAME = "EditServiceValuesParamPage";
  public static final String DID_YOU_KNOW_ADD = DidYouKnowMessages.getString("EditServiceValuesParamPage.add");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_EDIT = DidYouKnowMessages.getString("EditServiceValuesParamPage.edit");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_GLOBAL_ADD = DidYouKnowMessages.getString("EditServiceValuesParamPage.global.add");//$NON-NLS-1$
  public static final String DID_YOU_KNOW_GLOBAL_EDIT = DidYouKnowMessages.getString("EditServiceValuesParamPage.global.edit");//$NON-NLS-1$

  public static final Class[] EDIT_ACCEPT_CLASSES = new Class[] { ValueHolder.class };

  private List mOriginalValues;
  private ValuesParam mCopyParameter;
  private StructuredViewer mViewer;
  private Button mAddButton;
  private Button mEditButton;
  private Button mRemoveButton;
  private Button mMoveUpButton;
  private Button mMoveDownButton;

  public EditServiceValuesParamPage(int pMode, PortalSettings pPortalSettings, Object pParameterParent,
      Parameter pParameter)
  {
    super(PAGE_NAME, (pMode == ADD_MODE) ? (isGlobalParameter(pParameter) ? DID_YOU_KNOW_GLOBAL_ADD
        : DID_YOU_KNOW_ADD) : (isGlobalParameter(pParameter) ? DID_YOU_KNOW_GLOBAL_EDIT : DID_YOU_KNOW_EDIT),
        pMode, pPortalSettings, pParameterParent, pParameter);
    setTitle((pMode == ADD_MODE) ? (isGlobalParameter(pParameter) ? TITLE_GLOBAL_ADD : TITLE_ADD)
        : (isGlobalParameter(pParameter) ? TITLE_GLOBAL_EDIT : TITLE_EDIT));
    setMessage((pMode == ADD_MODE) ? (isGlobalParameter(pParameter) ? MESSAGE_GLOBAL_ADD : MESSAGE_ADD)
        : (isGlobalParameter(pParameter) ? MESSAGE_GLOBAL_EDIT : MESSAGE_EDIT));
  }

  private static boolean isGlobalParameter(Parameter pParameter)
  {
    Object parent = (pParameter != null) ? pParameter.getParent() : null;
    return (parent instanceof GlobalParam);
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterPage#createNewParameter()
   */
  protected Parameter createNewParameter()
  {
    return new ValuesParam();
  }

  /**
   * @see org.exoplatform.eclipse.internal.ui.wizard.EditServiceParameterPage#addValuesContent(org.eclipse.swt.widgets.Composite)
   */
  protected void addValuesContent(Composite pClient)
  {
    Group group = UICompositeUtil.createGroup(pClient, "Values", 2, 2);
    mViewer = createTableViewer(group);
    createActionButtons(group);
  }

  protected void updateFromModel()
  {
    super.updateFromModel();

    if ((mParameter != null) && (mParameter instanceof ValuesParam))
    {
      mOriginalValues = ((ValuesParam) mParameter).getValues();
      mCopyParameter = (ValuesParam) mParameter.getCopy();
    }

    populateViewer(mViewer, mCopyParameter);
    updateButtonsState(null);
  }

  protected void attachListeners()
  {
    super.attachListeners();
    attacheListenersToButtons();
  }

  public void finish()
  {
    // first lets detect if the user changed any of the values
    if (!isSameStringList(mOriginalValues, (mCopyParameter == null) ? null : mCopyParameter.getValues()))
    {
      List newValues = new ArrayList();
      if (mCopyParameter != null) UICompositeUtil.addAllIfNotNull(newValues, mCopyParameter.getValues());
      if (mParameter instanceof ValuesParam)
      {
        ((ValuesParam) mParameter).setValues(newValues);
      }
      mDirty = true;
    }
    super.finish();
  }

  private StructuredViewer createTableViewer(Composite parent)
  {
    int style = SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
    TableViewer tableViewer = new TableViewer(parent, style);
    Control tree = tableViewer.getControl();
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = 1;
    tree.setLayoutData(gridData);
    UICompositeUtil.setWidthHeight(tree, -1, 150);

    return tableViewer;
  }

  private void createActionButtons(Composite pParent)
  {
    Composite container = UICompositeUtil.createFillBothComposite(pParent, 1, 1);
    Dialog.applyDialogFont(container);
    GridLayout layout = (GridLayout) container.getLayout();
    layout.makeColumnsEqualWidth = false;
    layout.marginHeight = 0;
    layout.marginWidth = 5;

    container.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));

    mAddButton = createButton("Add...", container);
    mEditButton = createButton("Edit...", container);
    mRemoveButton = createButton("Remove", container);
    mMoveUpButton = createButton("Up", container);
    mMoveDownButton = createButton("Down", container);
  }

  private Button createButton(String pLabel, Composite pContainer)
  {
    Button button = new Button(pContainer, SWT.PUSH);
    button.setText(pLabel);
    button.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING | GridData.FILL_HORIZONTAL));

    return button;
  }

  private void attacheListenersToButtons()
  {
    mAddButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        addButtonPressed();
      }
    });

    mEditButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mViewer.getSelection(), EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mAddButton.getShell(), "Error while trying to edit the selection", "Either the selection is empty or you are trying to edit a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          editButtonPressed(selection);
        }
      }
    });

    mRemoveButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mViewer.getSelection(), EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "Either the selection is empty or you are trying to remove a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          removeButtonPressed(selection);
        }
      }
    });

    mMoveUpButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mViewer.getSelection(), EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mMoveUpButton.getShell(), "Error while trying to move up the selection", "Either the selection is empty or you are trying to move a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          upDownButtonPressed(selection, -1);
        }
      }
    });

    mMoveDownButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mViewer.getSelection(), EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mMoveDownButton.getShell(), "Error while trying to move down the selection", "Either the selection is empty or you are trying to move a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          upDownButtonPressed(selection, 1);
        }
      }
    });

  }

  private void addButtonPressed()
  {
    EditValueHolderDialog dialog = new EditValueHolderDialog(mEditButton.getShell(), "Add a parameter value",
        mCopyParameter, null);
    dialog.open();
    if (dialog.isDirty()) mViewer.refresh();
  }

  private void editButtonPressed(Object pSelection)
  {
    if (pSelection instanceof ValueHolder)
    {
      ValueHolder valueHolder = (ValueHolder) pSelection;
      Object parent = valueHolder.getParent();
      EditValueHolderDialog dialog = new EditValueHolderDialog(mEditButton.getShell(),
          "Edit a parameter value", parent, valueHolder);
      dialog.open();
      if (dialog.isDirty()) mViewer.refresh(valueHolder);
    }
  }

  private void removeButtonPressed(Object pSelection)
  {
    if (pSelection instanceof INode)
    {
      INode node = (INode) pSelection;
      Object object = node.getParent();
      if ((object == null) || !(object instanceof INode))
      {
        MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "The selection you made does not appear to have a parent, removing it is not possible. Please make a different selection and try again.");
      }
      else
      {
        INode parent = (INode) object;
        boolean removed = parent.removeChild(node);
        if (removed)
        {
          mViewer.refresh();
        }
        else
        {
          MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "There seems to be inconsistency in the model, removing the selection is not possible. Please make a different selection and try again.");
        }
      }
    }
    else
    {
      MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "Removing the selection you made is not supported. Please make a different selection and try again.");
    }
  }

  private void upDownButtonPressed(Object pSelection, int pIncrement)
  {
    if (pSelection instanceof ValueHolder)
    {
      if (mCopyParameter == null) return;
      ValueHolder valueHolder = (ValueHolder) pSelection;
      int index = Utils.getIndexOf(valueHolder, mCopyParameter.getChildren());
      if (index > -1)
      {
        List newArray = new ArrayList();
        newArray.addAll(mCopyParameter.getValues());
        Object value = newArray.get(index);
        newArray.remove(value);
        index = index + pIncrement;
        if ((index > -1) && (index <= newArray.size()))
        {
          mCopyParameter.getValues().add(index, value);
          newArray.add(index, value);
          mCopyParameter.setValues(newArray);
          mViewer.refresh();
          mViewer.setSelection(new StructuredSelection(mCopyParameter.getChildren()[index]));
          mViewer.getControl().setFocus();
        }
      }
    }
  }

  private void populateViewer(StructuredViewer pViewer, Object pInput)
  {
    pViewer.setContentProvider(new PortalConfigurationContentProvider());
    pViewer.setLabelProvider(new PortalConfigurationLabelProvider());
    pViewer.setInput(pInput);
    pViewer.addSelectionChangedListener(new ISelectionChangedListener()
    {
      public void selectionChanged(SelectionChangedEvent pEvent)
      {
        updateButtonsState(pEvent.getSelection());
      }
    });
    pViewer.addDoubleClickListener(new IDoubleClickListener()
    {

      public void doubleClick(DoubleClickEvent pEvent)
      {
        updateButtonsState(pEvent.getSelection());
        editButtonPressed(UICompositeUtil.getSelection(pEvent.getSelection(), EDIT_ACCEPT_CLASSES));
      }
    });
  }

  private void updateButtonsState(ISelection pSelection)
  {
    Object selectedObject = UICompositeUtil.getSelection(pSelection, EDIT_ACCEPT_CLASSES);
    updateButtonsState(selectedObject);
  }

  private void updateButtonsState(Object pModelObject)
  {
    mAddButton.setEnabled(true);
    mEditButton.setEnabled(false);
    mRemoveButton.setEnabled(false);
    mMoveUpButton.setEnabled(false);
    mMoveDownButton.setEnabled(false);

    Object selection = UICompositeUtil.getSelection((mViewer != null) ? mViewer.getSelection() : null);
    if ((pModelObject == null) || (selection == null) || (pModelObject instanceof Parameter))
    {
      // allow add, but not edit or remove
      mAddButton.setEnabled(true);
      mEditButton.setEnabled(false);
      mRemoveButton.setEnabled(false);
      mMoveUpButton.setEnabled(false);
      mMoveDownButton.setEnabled(false);
      return;
    }

    if (pModelObject instanceof ValueHolder)
    {
      ValueHolder value = (ValueHolder) pModelObject;
      mAddButton.setEnabled(true);
      mEditButton.setEnabled(true);
      mRemoveButton.setEnabled(true);

      // check if we only have one element in the list if so then
      // we need to disable the 'up' and 'down' buttons
      if ((mCopyParameter.getValues() == null) || (mCopyParameter.getValues().size() < 2))
      {
        mMoveUpButton.setEnabled(false);
        mMoveDownButton.setEnabled(false);
      }
      else
      {
        Object[] children = (mCopyParameter != null) ? mCopyParameter.getChildren() : new Object[0];
        // now check if the selected value holder is the first in the list, if so, then
        // disable 'up' button and enable 'down' button
        int index = Utils.getIndexOf(value, children);
        if (index < 0)
        {
          mMoveUpButton.setEnabled(false);
          mMoveDownButton.setEnabled(false);
        }
        else if (index == 0)
        {
          mMoveUpButton.setEnabled(false);
          mMoveDownButton.setEnabled(true);
        }
        else if (index == (children.length - 1))
        {
          mMoveUpButton.setEnabled(true);
          mMoveDownButton.setEnabled(false);
        }
        else
        {
          mMoveUpButton.setEnabled(true);
          mMoveDownButton.setEnabled(true);
        }
      }
      return;
    }

  }

}