/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.wizard;


import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.help.WorkbenchHelp;
import org.exoplatform.eclipse.internal.ui.sample.IExoSampleConstant;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappContentProvider;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappInitializer;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappLabelProvider;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappProject;
import org.exoplatform.eclipse.ui.IExoHelpContextIds;

/**
 * This page allows the user to select zero or one sample portlets projects.
 * When the user selects a sample portlet project, the description of the
 * sample appears next to it. The sample portlets projects are coming from the
 * extension point org.exoplatform.eclipse.uiexoSampleWebappProviders
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class NewPortletsProjectSamplePage extends WizardPage
{
  public static final String CLASS_VERSION = "$Id: NewPortletsProjectSamplePage.java,v 1.1 2004/04/19 03:37:24 hatimk Exp $"; //$NON-NLS-1$
  private CheckboxTableViewer mSamplePortletViewer;
  private Text mSamplePortletDescription;
  private Text mComments;
  private SampleWebappProject mDefaultProject;

  /**
   * Creates a wizard page.
   *
   * @param pPageName the name of this page
   */
  public NewPortletsProjectSamplePage(String pPageName)
  {
    super(pPageName);
    setPageComplete(false);
  }

  /**   
   * @see org.eclipse.jface.dialogs.DialogPage#createControl(org.eclipse.swt.widgets.Composite)
   */
  public void createControl(Composite pParent)
  {
    Composite mainComposite = createFillBothComposite(pParent, 2);
    addLabel(mainComposite, "Sample Portlets Projects :", 1);
    addLabel(mainComposite, "Description :", 1);
    createSampleTableComposite(mainComposite, 1);
    createDescriptionTableComposite(mainComposite, 1);
    createCommentsGroup(mainComposite, 2);

    setPageComplete(true);

    // no error messages for this page
    setErrorMessage(null);
    setMessage(null);
    WorkbenchHelp.setHelp(mainComposite,
      IExoHelpContextIds.SELECT_SAMPLE_PORTLET_PROJECTS);
    setControl(mainComposite);
  }

  /**   
   * Returns all sample webapp projects that the user selected.
   *
   * @return all sample webapp projects that the user selected
   */
  public SampleWebappProject[] getSelectedSampleWebappProjects()
  {
    if (mSamplePortletViewer == null)
    {
      return new SampleWebappProject[0];
    }

    Object[] elements = mSamplePortletViewer.getCheckedElements();

    if ((elements == null) || (elements.length < 1))
    {
      return new SampleWebappProject[0];
    }

    SampleWebappProject[] result = new SampleWebappProject[elements.length];
    for (int i = 0, sizei = elements.length; i < sizei; ++i)
    {
      result[i] = (SampleWebappProject) elements[i];
    }

    return result;
  }

  /**   
   * Returns the default portlet project.
   *
   * @return the default portlet project
   */
  public SampleWebappProject getDefaultProject()
  {
    if (mDefaultProject == null)
    {
      SampleWebappProject[] webapps =
        SampleWebappInitializer.getSampleWebappProjects(IExoSampleConstant.KIND_PORTLET_DEFAULT);
      mDefaultProject = webapps[0];
    }

    return mDefaultProject;
  }

  /*
   * see @DialogPage.setVisible(boolean)
   */
  public void setVisible(boolean visible)
  {
    super.setVisible(visible);
    if ((visible) && (mComments != null) && (mSamplePortletViewer != null))
    {
      SampleWebappProject defaultProject = getDefaultProject();
      mComments.setText(
        "If you don't select any one of the above sample projects, "
        + "the wizard will create a default portlet project. "
        + defaultProject.getDescription());

      Control control = mSamplePortletViewer.getControl();
      if (control != null)
      {
        control.setFocus();
      }
    }
  }

  private void createCommentsGroup(Composite pParent, int pHorizontalSpan)
  {
    Composite group =
      createFillBothGroup(pParent, "Comments", 1, pHorizontalSpan);
    mComments =
      new Text(group,
        SWT.MULTI | SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL
        | SWT.H_SCROLL);

    GridData gridData = new GridData(GridData.FILL_BOTH);
    gridData.horizontalSpan = pHorizontalSpan;
    mComments.setLayoutData(gridData);
  }

  private void createDescriptionTableComposite(Composite pParent,
    int pHorizontalSpan)
  {
    mSamplePortletDescription =
      new Text(pParent,
        SWT.MULTI | SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL
        | SWT.H_SCROLL);

    GridData gridData = new GridData(GridData.FILL_BOTH);
    gridData.horizontalSpan = pHorizontalSpan;
    mSamplePortletDescription.setLayoutData(gridData);
  }

  private void createSampleTableComposite(Composite pParent, int pHorizontalSpan)
  {
    mSamplePortletViewer =
      CheckboxTableViewer.newCheckList(pParent,
        SWT.BORDER | SWT.SINGLE | SWT.HIDE_SELECTION | SWT.V_SCROLL
        | SWT.H_SCROLL);

    Control table = mSamplePortletViewer.getControl();
    GridData gridData = new GridData(GridData.FILL_BOTH);
    gridData.horizontalSpan = pHorizontalSpan;
    table.setLayoutData(gridData);
    table.setFont(pParent.getFont());
    mSamplePortletViewer.setContentProvider(new SampleWebappContentProvider(
        IExoSampleConstant.KIND_PORTLET_SAMPLE));
    mSamplePortletViewer.setLabelProvider(new SampleWebappLabelProvider());
    mSamplePortletViewer.setInput(new Object());
    mSamplePortletViewer.addCheckStateListener(new ICheckStateListener()
      {
        public void checkStateChanged(CheckStateChangedEvent pEvent)
        {
          Object selected = pEvent.getElement();
          boolean selectedstate = pEvent.getChecked();
          mSamplePortletViewer.setAllChecked(false);
          mSamplePortletViewer.setChecked(selected, selectedstate);

          ISelection selection;
          if (!selectedstate)
          {
            selection = new StructuredSelection();
          }
          else
          {
            selection = new StructuredSelection(selected);
          }
          mSamplePortletViewer.setSelection(selection, true);
          if ((selected instanceof SampleWebappProject) && (selectedstate))
          {
            mSamplePortletDescription.setText(((SampleWebappProject) selected)
              .getDescription());
          }
          else
          {
            mSamplePortletDescription.setText("");
          }
        }
      });
    mSamplePortletViewer.addSelectionChangedListener(new ISelectionChangedListener()
      {
        public void selectionChanged(SelectionChangedEvent pEvent)
        {
          if (pEvent.getSelection().isEmpty())
          {
            return;
          }

          if (pEvent.getSelection() instanceof IStructuredSelection)
          {
            IStructuredSelection selection =
              (IStructuredSelection) pEvent.getSelection();
            Object selected = selection.getFirstElement();
            if (selected instanceof SampleWebappProject)
            {
              mSamplePortletDescription.setText(((SampleWebappProject) selected)
                .getDescription());
            }
          }
        }
      });
  }

  private Composite createFillBothComposite(Composite pParent, int pNumColumns)
  {
    Composite composite = new Composite(pParent, SWT.NONE);
    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.makeColumnsEqualWidth = true;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;
    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    composite.setLayout(layout);
    composite.setLayoutData(data);
    composite.setFont(pParent.getFont());

    return composite;
  }

  private Group createFillBothGroup(Composite pParent, String pTitle,
    int pNumColumns, int pHorizontalSpan)
  {
    Group group = new Group(pParent, SWT.NONE);
    group.setText(pTitle);

    GridLayout layout = new GridLayout();
    layout.marginHeight = 5;
    layout.marginWidth = 5;
    layout.numColumns = pNumColumns;

    GridData data = new GridData();
    data.horizontalAlignment = GridData.FILL;
    data.verticalAlignment = GridData.FILL;
    data.grabExcessHorizontalSpace = true;
    data.grabExcessVerticalSpace = true;
    data.horizontalSpan = pHorizontalSpan;
    group.setLayout(layout);
    group.setLayoutData(data);
    group.setFont(pParent.getFont());

    return group;
  }

  private Text addText(Composite pComp, String pTextDefaultValue)
  {
    Text result = new Text(pComp, SWT.SINGLE | SWT.BORDER);
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    result.setLayoutData(gridData);
    result.setText(pTextDefaultValue);

    return result;
  }

  private void addLabel(Composite pComp, String pText, int pHorizontalSpan)
  {
    Label label = new Label(pComp, SWT.NONE);

    label.setText(pText);

    GridData gd = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
    gd.horizontalSpan = pHorizontalSpan;
    label.setLayoutData(gd);
  }

  /**   
   * Create some empty space.
   *
   * @param pComp the composite to add the space to
   * @param pColSpan the number of column to span
   */
  private void createVerticalSpacer(Composite pComp, int pColSpan)
  {
    Label label = new Label(pComp, SWT.NONE);
    GridData gd = new GridData();
    gd.horizontalSpan = pColSpan;
    label.setLayoutData(gd);
  }
}
