/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.internal.ui.wizard;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.internal.ui.actions.WorkbenchRunnableAdapter;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.OpenFileAction;
import org.eclipse.ui.part.ISetSelectionTarget;
import org.exoplatform.eclipse.core.ExoCorePlugin;
import org.exoplatform.eclipse.core.IPortletProject;
import org.exoplatform.eclipse.core.IPreferencesConstant;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.operation.PortletProjectCreateOperation;
import org.exoplatform.eclipse.core.operation.SimplePortletProjectStoreOperation;
import org.exoplatform.eclipse.core.operation.WebXMLModifyOperation;
import org.exoplatform.eclipse.core.operation.helper.IWebXMLModifyStrategy;
import org.exoplatform.eclipse.core.operation.helper.JavaProjectConfiguration;
import org.exoplatform.eclipse.core.runtime.IExoProjectRuntimeProperties;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappInitializer;
import org.exoplatform.eclipse.internal.ui.sample.SampleWebappProject;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.exoplatform.eclipse.ui.IExoUIPreferencesConstant;

import java.lang.reflect.InvocationTargetException;

/**
 * This wizard allows the user to create a portlet project.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class NewPortletsProjectWizard extends Wizard implements INewWizard
{
  public static final String CLASS_VERSION = "$Id: NewPortletsProjectWizard.java,v 1.1 2004/04/19 03:37:24 hatimk Exp $"; //$NON-NLS-1$
  protected boolean mFatalErrorOccurred = false;
  private NewPortletsProjectSettingPage mProjectSettingPage;
  private NewPortletsProjectSamplePage mProjectSamplePage;
  private NewPortletsProjectDeploymentPage mProjectDeploymentPage;
  private IWorkbench mWorkbench;
  private IProject mProject;

  /**
   * The constructor method.
   */
  public NewPortletsProjectWizard()
  {
    super();
    setWindowTitle("Create a Java Web Application Project that contains Portlet(s)");
    setNeedsProgressMonitor(true);
    setDefaultPageImageDescriptor(ExoUIPluginImages.DESC_WIZBAN_NEW_PORTLET_PROJECT);
  }

  /**
   * @see org.eclipse.jface.wizard.Wizard#addPages()
   */
  public void addPages()
  {
    super.addPages();
    mProjectSettingPage = new NewPortletsProjectSettingPage("NewPortletsProjectSettingPage"); //$NON-NLS-1$
    mProjectSettingPage.setTitle("Project Settings Page");
    mProjectSettingPage.setDescription("Supply the information necessary to create the project such as name, path, src and web.");

    mProjectSamplePage = new NewPortletsProjectSamplePage("NewPortletsProjectSamplePage"); //$NON-NLS-1$
    mProjectSamplePage.setTitle("Sample Portlets");
    mProjectSamplePage.setDescription("Select a sample portlets project for a quicker start.");

    mProjectDeploymentPage = new NewPortletsProjectDeploymentPage("NewPortletsProjectDeploymentPage"); //$NON-NLS-1$
    mProjectDeploymentPage.setTitle("Deployment Settings Page");
    mProjectDeploymentPage.setDescription("Specify the deployment settings such as deployment directory.");

    addPage(mProjectSettingPage);
    addPage(mProjectSamplePage);
    addPage(mProjectDeploymentPage);
  }

  /**
   * This method is run in the wizard container's context as a workspace runnable.
   * 
   * @param pMonitor to report progress
   * 
   * @throws CoreException something went wrong
   * @throws InvocationTargetException
   * @throws InterruptedException we got interrupted
   */
  protected void finishPage(IProgressMonitor pMonitor) throws CoreException, InvocationTargetException,
      InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("Creating a portlet project : ", 1550);

      // gather all the information from all the pages in preparation of
      // the project creation
      String projectName = getProjectName();
      IPath projectPath = getProjectLocationPath();
      String deploymentFolder = getDeploymentFolder();
      String deploymentOption = getDeploymentOption();
      IPath webFolder = getWebFolder();
      IPath contextRootPath = getContextRootPath();

      pMonitor.subTask("build the project configuration information");

      JavaProjectConfiguration javaConfig = getJavaProjectConfiguration();
      PortletProjectCreateOperation op = new PortletProjectCreateOperation(javaConfig, projectName,
          projectPath, new PromptOverwriteQuery());

      pMonitor.subTask("initialize and build the project");
      op.run(new SubProgressMonitor(pMonitor, 1000));

      pMonitor.subTask("adding project properties such as context root");

      // now add the context root and the web folder to the portlet project
      // xml store
      SimplePortletProjectStoreOperation storeOp = new SimplePortletProjectStoreOperation(op.getProject());
      storeOp.addPropertyToModify(IPreferencesConstant.PORTLET_PROJECT_STORE_CONTEXTROOT, contextRootPath.toString());
      storeOp.addPropertyToModify(IPreferencesConstant.PORTLET_PROJECT_STORE_WEBFOLDER, webFolder.toString());
      storeOp.run(new SubProgressMonitor(pMonitor, 50));

      // now add the deployment directory as a resource property for this project
      IResource projectResource = op.getJavaProject().getCorrespondingResource();
      projectResource.setPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_DIR, deploymentFolder);
      pMonitor.worked(50);

      // now add the deployment option as a resource property for this project
      projectResource.setPersistentProperty(IExoProjectRuntimeProperties.DEPLOYMENT_OPTION, deploymentOption);
      pMonitor.worked(50);

      // remember the deployment directory
      ExoUIPlugin.getDefault().getPreferenceStore().setValue(IExoUIPreferencesConstant.PREF_DEPLOYMENT_DIR, deploymentFolder);

      mProject = op.getProject();

      // compile the project
      pMonitor.subTask("Compiling source code");
      buildProject(mProject, new SubProgressMonitor(pMonitor, 300));

      // modify the <display-name> tag in web.xml if needed
      modifyWebXMLDisplayName(mProject, new SubProgressMonitor(pMonitor, 100));
    }
    finally
    {
      pMonitor.done();
    }
  }

  protected String getProjectName()
  {
    return mProjectSettingPage.getProjectName();
  }

  protected IPath getProjectLocationPath()
  {
    if (mProjectSettingPage.useDefaults())
    {
      // this is to cover the case where the user choses the default
      // workspace as the location of the project
      return null;
    }
    else
    {
      return mProjectSettingPage.getLocationPath();
    }
  }

  protected String getDeploymentFolder()
  {
    return mProjectDeploymentPage.getDeploymentFolder();
  }

  protected String getDeploymentOption()
  {
    return String.valueOf(mProjectDeploymentPage.getDeploymentOption());
  }

  protected IPath getSourceFolder()
  {
    return new Path(mProjectSettingPage.getSourceFolder());
  }

  protected IPath getWebFolder()
  {
    return new Path(mProjectSettingPage.getWebFolder());
  }

  protected IPath getContextRootPath()
  {
    return new Path(mProjectSettingPage.getContextRoot());
  }

  protected JavaProjectConfiguration getJavaProjectConfiguration()
  {
    SampleWebappProject webappProject;
    SampleWebappProject[] sampleWebapps = mProjectSamplePage.getSelectedSampleWebappProjects();
    if (sampleWebapps.length > 0)
    {
      // we don't support the creation of multiple projects in this release
      webappProject = sampleWebapps[0];
    }
    else
    {
      webappProject = mProjectSamplePage.getDefaultProject();
    }

    return SampleWebappInitializer.getJavaProjectConfiguration(webappProject, null, getSourceFolder(), getWebFolder());
  }

  /**
   * @see Wizard#performFinish
   */
  public boolean performFinish()
  {
    IWorkspaceRunnable op = new IWorkspaceRunnable()
    {
      public void run(IProgressMonitor monitor) throws CoreException, OperationCanceledException
      {
        try
        {
          finishPage(monitor);
        }
        catch (InvocationTargetException eTarget)
        {
          throw new CoreException(
              ExoUIPlugin.createErrorStatus(eTarget.getMessage(), eTarget.getTargetException()));
        }
        catch (InterruptedException e)
        {
          throw new OperationCanceledException(e.getMessage());
        }
      }
    };

    try
    {
      getContainer().run(false, true, new WorkbenchRunnableAdapter(op));
      mFatalErrorOccurred = false;
      if (mProject != null)
      {
        revealSelection(mProject);

        // you might want to use BasicNewResourceWizard.selectAndReveal(); instead
      }
    }
    catch (InvocationTargetException e)
    {
      handleException(e.getTargetException());
      mFatalErrorOccurred = true;

      return false;
    }
    catch (InterruptedException e)
    {
      mFatalErrorOccurred = true;

      return false;
    }

    return true;
  }

  protected void handleException(Throwable pTarget)
  {
    String title = "Error while creating the project";

    if (pTarget instanceof CoreException)
    {
      IStatus status = ((CoreException) pTarget).getStatus();
      ExoUIPlugin.log(status);
      ErrorDialog.openError(getShell(), title, status.getMessage(), status);
    }
    else
    {
      ExoUIPlugin.logError(pTarget);
      MessageDialog.openError(getShell(), title, title + " : " + pTarget);
    }
  }

  /**
   * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
   *      org.eclipse.jface.viewers.IStructuredSelection)
   */
  public void init(IWorkbench pWorkbench, IStructuredSelection selection)
  {
    mWorkbench = pWorkbench;
  }

  /**
   * @see org.eclipse.jface.wizard.IWizard#canFinish()
   */
  public boolean canFinish()
  {
    return super.canFinish() && !mFatalErrorOccurred;
  }

  protected void buildProject(IProject pProject, IProgressMonitor pMonitor) throws CoreException
  {
    pProject.build(IncrementalProjectBuilder.INCREMENTAL_BUILD, pMonitor);
  }

  protected void modifyWebXMLDisplayName(IProject pProject, IProgressMonitor pMonitor)
  {
    IWebXMLModifyStrategy strategy = new IWebXMLModifyStrategy()
    {
      public boolean detectedMACFormat()
      {
        // don't continue if it is a MAC format
        return false;
      }

      public boolean isOkToModify(IFile pWebXML)
      {
        // continue
        return true;
      }
    };

    try
    {
      IPortletProject project = ExoCorePlugin.createPortletProject(pProject);
      WebXMLModifyOperation op = new WebXMLModifyOperation(project, strategy);

      op.run(pMonitor);
    }
    catch (Exception e)
    {
      // simply don't involve the user with any exception because of this
      // information. Just put it in the log
      ExoUIPlugin.logError(e);
    }
  }

  protected void revealSelection(final Object pToSelect)
  {
    if (pToSelect == null)
    {
      return;
    }
    if (mWorkbench == null)
    {
      return;
    }

    IWorkbenchWindow ww = mWorkbench.getActiveWorkbenchWindow();
    if (ww == null)
    {
      return;
    }
    if (ww.getActivePage() == null)
    {
      return;
    }

    final IWorkbenchPart focusPart = ww.getActivePage().getActivePart();

    if (focusPart instanceof ISetSelectionTarget)
    {
      Display d = getShell().getDisplay();
      d.asyncExec(new Runnable()
      {
        public void run()
        {
          ISelection selection = new StructuredSelection(pToSelect);
          ((ISetSelectionTarget) focusPart).selectReveal(selection);

          OpenFileAction open = new OpenFileAction(mWorkbench.getActiveWorkbenchWindow().getActivePage());
          open.selectionChanged(new StructuredSelection(pToSelect));
          open.run();
        }
      });
    }
  }

  // overwrite dialog
  protected class PromptOverwriteQuery implements IOverwriteQuery
  {
    public String queryOverwrite(String file)
    {
      String[] returnCodes = { YES, NO, ALL, CANCEL };
      int returnVal = openDialog(file);

      return (returnVal < 0) ? CANCEL : returnCodes[returnVal];
    }

    private int openDialog(final String file)
    {
      final int[] result = { IDialogConstants.CANCEL_ID };
      getShell().getDisplay().syncExec(new Runnable()
      {
        public void run()
        {
          String title = "Overwrite";
          String msg = "Do you want to overwrite " + file + "?";
          String[] options = { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL,
              IDialogConstants.YES_TO_ALL_LABEL, IDialogConstants.CANCEL_LABEL };
          MessageDialog dialog = new MessageDialog(getShell(), title, null, msg, MessageDialog.QUESTION,
              options, 0);
          result[0] = dialog.open();
        }
      });

      return result[0];
    }
  }
}