/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jdt.internal.ui.actions.WorkbenchRunnableAdapter;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;
import org.exoplatform.eclipse.core.common.IOverwriteQuery;
import org.exoplatform.eclipse.core.exoservices.DefaultImportConfigurationURLResolver;
import org.exoplatform.eclipse.core.exoservices.IExoPlatformInstallationInfo;
import org.exoplatform.eclipse.core.exoservices.IImportConfigurationURLResolver;
import org.exoplatform.eclipse.core.operation.PortalProjectCreateOperation;
import org.exoplatform.eclipse.core.operation.helper.ProjectConfiguration;
import org.exoplatform.eclipse.core.portal.ExoTomcatInstallationInfo;
import org.exoplatform.eclipse.core.portal.PortalSettings;
import org.exoplatform.eclipse.core.portal.PortalSettingsBuilder;
import org.exoplatform.eclipse.ui.ExoUIPlugin;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;
import org.exoplatform.eclipse.ui.IExoUIPreferencesConstant;

/**
 * @author Hatim Khan
 */
public class NewTomcatPortalProjectWizard extends Wizard implements INewWizard
{
  private IWorkbench mWorkbench;
  private NewTomcatPortalProjectSettingPage mProjectSettingPage;
  private IProject mProject;
  private static final String PORTAL_SETTINGS_FILENAME = "portal-settings.exo";

  /**
   * The constructor method.
   */
  public NewTomcatPortalProjectWizard()
  {
    super();
    setWindowTitle("Create a custom eXo Platform Portal");
    setNeedsProgressMonitor(true);
    setDefaultPageImageDescriptor(ExoUIPluginImages.DESC_WIZBAN_NEW_PORTLET_PROJECT);
  }

  /**
   * @see org.eclipse.jface.wizard.Wizard#addPages()
   */
  public void addPages()
  {
    super.addPages();
    mProjectSettingPage = new NewTomcatPortalProjectSettingPage("NewTomcatPortalProjectSettingPage"); //$NON-NLS-1$
    mProjectSettingPage.setTitle("Project Settings Page");
    mProjectSettingPage.setDescription("Supply the information necessary to create the project such as name and path.");

    addPage(mProjectSettingPage);
  }

  /**
   * @see org.eclipse.jface.wizard.IWizard#performFinish()
   */
  public boolean performFinish()
  {
    IWorkspaceRunnable op = new IWorkspaceRunnable()
    {
      public void run(IProgressMonitor monitor) throws CoreException, OperationCanceledException
      {
        try
        {
          finishPage(monitor);
        }
        catch (InvocationTargetException eTarget)
        {
          throw new CoreException(
              ExoUIPlugin.createErrorStatus(eTarget.getMessage(), eTarget.getTargetException()));
        }
        catch (InterruptedException e)
        {
          throw new OperationCanceledException(e.getMessage());
        }
      }
    };

    try
    {
      getContainer().run(false, true, new WorkbenchRunnableAdapter(op));
      if (mProject != null)
      {
        IWorkbenchWindow window = mWorkbench.getActiveWorkbenchWindow();
        IFile file = mProject.getFile(PORTAL_SETTINGS_FILENAME);
        BasicNewResourceWizard.selectAndReveal(file, window);
        IWorkbenchPage page = window.getActivePage();
        if (page != null)
        {
          IEditorDescriptor desc = mWorkbench.getEditorRegistry().getDefaultEditor(file.getName());
          page.openEditor(new FileEditorInput(file), desc.getId());
        }
      }

    }
    catch (InvocationTargetException e)
    {
      handleException(e.getTargetException());
      return false;
    }
    catch (PartInitException e)
    {
      return true;
    }
    catch (InterruptedException e)
    {
      return false;
    }

    return true;
  }

  protected void handleException(Throwable pTarget)
  {
    String title = "Error while creating the project";

    if (pTarget instanceof CoreException)
    {
      IStatus status = ((CoreException) pTarget).getStatus();
      ExoUIPlugin.log(status);
      ErrorDialog.openError(getShell(), title, status.getMessage(), status);
    }
    else
    {
      ExoUIPlugin.logError(pTarget);
      MessageDialog.openError(getShell(), title, title + " : " + pTarget);
    }
  }

  /**
   * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
   *      org.eclipse.jface.viewers.IStructuredSelection)
   */
  public void init(IWorkbench pWorkbench, IStructuredSelection pSelection)
  {
    mWorkbench = pWorkbench;
  }

  /**
   * This method is run in the wizard container's context as a workspace runnable.
   * 
   * @param pMonitor to report progress
   * 
   * @throws CoreException something went wrong
   * @throws InvocationTargetException
   * @throws InterruptedException we got interrupted
   */
  protected void finishPage(IProgressMonitor pMonitor) throws CoreException, InvocationTargetException,
      InterruptedException
  {
    if (pMonitor == null)
    {
      pMonitor = new NullProgressMonitor();
    }

    try
    {
      pMonitor.beginTask("", 1000);

      // gather all the information from all the pages in preparation of
      // the project creation
      String projectName = getProjectName();
      IPath projectPath = getProjectLocationPath();
      String installationFolder = getInstallationFolder();
      IPath contextRootPath = getContextRootPath();

      ProjectConfiguration projectConfig = getProjectConfiguration();
      PortalProjectCreateOperation op = new PortalProjectCreateOperation(projectConfig, projectName,
          projectPath, new PromptOverwriteQuery());

      op.run(new SubProgressMonitor(pMonitor, 500));
      mProject = op.getProject();

      // remember the installatioin directory
      ExoUIPlugin.getDefault().getPreferenceStore().setValue(IExoUIPreferencesConstant.PREF_INSTALLATION_DIR, installationFolder);

      IFile file = mProject.getFile(PORTAL_SETTINGS_FILENAME);
      String jdkVersion = getJDKVersion();
      IExoPlatformInstallationInfo installationInfo = new ExoTomcatInstallationInfo(installationFolder,
          jdkVersion);
      IImportConfigurationURLResolver resolver = new DefaultImportConfigurationURLResolver();
      PortalSettings settings = PortalSettingsBuilder.scanPlatform(installationInfo, resolver, new SubProgressMonitor(
          pMonitor, 500));
      settings.setInstallationDirectory(installationFolder);
      settings.setJDKVersion(jdkVersion);
      settings.setContextRoot(getContextRootPath().toString());
      PortalSettingsBuilder.writeTo(settings, file);
    }
    finally
    {
      pMonitor.done();
    }

  }

  protected String getProjectName()
  {
    return mProjectSettingPage.getProjectName();
  }

  protected IPath getProjectLocationPath()
  {
    if (mProjectSettingPage.useDefaults())
    {
      // this is to cover the case where the user choses the default
      // workspace as the location of the project
      return null;
    }
    else
    {
      return mProjectSettingPage.getLocationPath();
    }
  }

  protected String getInstallationFolder()
  {
    return mProjectSettingPage.getInstallationFolder();
  }

  protected IPath getContextRootPath()
  {
    return new Path(mProjectSettingPage.getContextRoot());
  }

  protected String getJDKVersion()
  {
    return mProjectSettingPage.getJDKVersion();
  }

  protected ProjectConfiguration getProjectConfiguration()
  {
    return null;
  }

  // overwrite dialog
  protected class PromptOverwriteQuery implements IOverwriteQuery
  {
    public String queryOverwrite(String file)
    {
      String[] returnCodes = { YES, NO, ALL, CANCEL };
      int returnVal = openDialog(file);

      return (returnVal < 0) ? CANCEL : returnCodes[returnVal];
    }

    private int openDialog(final String file)
    {
      final int[] result = { IDialogConstants.CANCEL_ID };
      getShell().getDisplay().syncExec(new Runnable()
      {
        public void run()
        {
          String title = "Overwrite";
          String msg = "Do you want to overwrite " + file + "?";
          String[] options = { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL,
              IDialogConstants.YES_TO_ALL_LABEL, IDialogConstants.CANCEL_LABEL };
          MessageDialog dialog = new MessageDialog(getShell(), title, null, msg, MessageDialog.QUESTION,
              options, 0);
          result[0] = dialog.open();
        }
      });

      return result[0];
    }
  }

}