/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.internal.ui.wizard;

import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.Property;
import org.exoplatform.eclipse.core.portal.model.INode;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;
import org.exoplatform.eclipse.ui.common.PortalConfigurationContentProvider;
import org.exoplatform.eclipse.ui.common.PortalConfigurationLabelProvider;
import org.exoplatform.eclipse.ui.common.UICompositeUtil;

/**
 * This delegate handels the manipulation of the values of a given Parameter. The values
 * must be of type 'Property. The given Parameter can be either of type PropertiesParam or
 * ObjectParam.
 * 
 * @author Hatim Khan
 */
public class ParameterPropertyValueListPageDelegate
{
  public static final Class[] EDIT_ACCEPT_CLASSES = new Class[] { ValueHolder.class, Property.class };

  private StructuredViewer mViewer;
  private List mOriginalValues;
  private Parameter mParameter;
  private String mDidYouKnowAdd;
  private String mDidYouKnowEdit;

  private Shell mShell;
  private Button mAddButton;
  private Button mEditButton;
  private Button mRemoveButton;
  private Button mExpandAllButton;
  private Button mCollapseAllButton;

  public ParameterPropertyValueListPageDelegate(String pDidYouKnowAdd, String pDidYouKnowEdit)
  {
    mDidYouKnowAdd = pDidYouKnowAdd;
    mDidYouKnowEdit = pDidYouKnowEdit;
  }

  protected void addValuesContent(Composite pClient)
  {
    mShell = pClient.getShell();
    Group group = UICompositeUtil.createGroup(pClient, "Values", 2, 2);
    mViewer = createTreeViewer(group);
    createActionButtons(group);
  }

  private StructuredViewer createTreeViewer(Composite parent)
  {
    int style = SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER;
    TreeViewer treeViewer = new TreeViewer(parent, style);
    Control tree = treeViewer.getControl();
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = 1;
    tree.setLayoutData(gridData);
    UICompositeUtil.setWidthHeight(tree, -1, 150);

    return treeViewer;
  }

  private void createActionButtons(Composite pParent)
  {
    Composite container = UICompositeUtil.createFillBothComposite(pParent, 1, 1);
    Dialog.applyDialogFont(container);
    GridLayout layout = (GridLayout) container.getLayout();
    layout.makeColumnsEqualWidth = false;
    layout.marginHeight = 0;
    layout.marginWidth = 5;

    container.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));

    mAddButton = createButton("Add...", container);
    mEditButton = createButton("Edit...", container);
    mRemoveButton = createButton("Remove", container);
    mExpandAllButton = createButton("Expand All", container);
    mCollapseAllButton = createButton("Collapse All", container);
  }

  private Button createButton(String pLabel, Composite pContainer)
  {
    Button button = new Button(pContainer, SWT.PUSH);
    button.setText(pLabel);
    button.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING | GridData.FILL_HORIZONTAL));

    return button;
  }

  protected void updateFromModel(Parameter pParameter)
  {
    mParameter = pParameter;

    populateViewer(mViewer, mParameter);
    updateButtonsState(null);
  }

  private void populateViewer(StructuredViewer pViewer, Object pInput)
  {
    pViewer.setContentProvider(new PortalConfigurationContentProvider());
    pViewer.setLabelProvider(new PortalConfigurationLabelProvider());
    pViewer.setInput(pInput);
    pViewer.addSelectionChangedListener(new ISelectionChangedListener()
    {
      public void selectionChanged(SelectionChangedEvent pEvent)
      {
        updateButtonsState(pEvent.getSelection());
      }
    });
    pViewer.addDoubleClickListener(new IDoubleClickListener()
    {

      public void doubleClick(DoubleClickEvent pEvent)
      {
        updateButtonsState(pEvent.getSelection());
        editButtonPressed(UICompositeUtil.getSelection(pEvent.getSelection(), EDIT_ACCEPT_CLASSES));
      }
    });
  }

  private void updateButtonsState(ISelection pSelection)
  {
    Object selectedObject = UICompositeUtil.getSelection(pSelection, EDIT_ACCEPT_CLASSES);
    updateButtonsState(selectedObject);
  }

  private void updateButtonsState(Object pModelObject)
  {
    mAddButton.setEnabled(true);
    mEditButton.setEnabled(false);
    mRemoveButton.setEnabled(false);
    mExpandAllButton.setEnabled(false);
    mCollapseAllButton.setEnabled(false);

    // lets first determine if the expand all or collapse all need to be enabled
    if (mParameter != null)
    {
      Object[] values = mParameter.getChildren();
      if ((values != null) && (values.length > 0))
      {
        mExpandAllButton.setEnabled(true);
        mCollapseAllButton.setEnabled(true);
      }
    }
    Object selection = UICompositeUtil.getSelection((mViewer != null) ? mViewer.getSelection() : null);
    if ((pModelObject == null) || (selection == null) || (pModelObject instanceof Parameter))
    {
      // allow add, but not edit or remove
      mAddButton.setEnabled(true);
      mEditButton.setEnabled(false);
      mRemoveButton.setEnabled(false);
      return;
    }

    if (pModelObject instanceof ValueHolder)
    {
      ValueHolder value = (ValueHolder) pModelObject;
      mAddButton.setEnabled(true);
      mEditButton.setEnabled(true);
      mRemoveButton.setEnabled(false);

      return;
    }

    if (pModelObject instanceof Property)
    {
      Property value = (Property) pModelObject;
      mAddButton.setEnabled(true);
      mEditButton.setEnabled(true);
      mRemoveButton.setEnabled(true);

      return;
    }
  }

  protected void attachListeners()
  {
    attacheListenersToButtons();
  }

  private void attacheListenersToButtons()
  {
    mAddButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        addButtonPressed();
      }
    });

    mEditButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mViewer.getSelection(), EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mAddButton.getShell(), "Error while trying to edit the selection", "Either the selection is empty or you are trying to edit a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          editButtonPressed(selection);
        }
      }
    });

    mRemoveButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        Object selection = UICompositeUtil.getSelection(mViewer.getSelection(), EDIT_ACCEPT_CLASSES);
        if (selection == null)
        {
          MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "Either the selection is empty or you are trying to remove a selection that is not supported. Please make a different selection and try again.");
        }
        else
        {
          removeButtonPressed(selection);
        }
      }
    });

    mExpandAllButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        ((TreeViewer) mViewer).expandAll();
      }
    });

    mCollapseAllButton.addSelectionListener(new SelectionAdapter()
    {
      public void widgetSelected(SelectionEvent pEvent)
      {
        ((TreeViewer) mViewer).collapseAll();
      }
    });

  }

  private void addButtonPressed()
  {
    EditParameterPropertyValueWizard wizard = new EditParameterPropertyValueWizard(
        EditParameterPropertyValueWizard.ADD_MODE, mDidYouKnowAdd, mParameter, null);
    WizardDialog container = new WizardDialog(mShell, wizard);
    container.open();
    if (wizard.isDirty())
    {
      mViewer.refresh();
      Object[] children = mParameter.getChildren();
      if ((children != null) && (children.length > 0))
      {
        mViewer.setSelection(new StructuredSelection(children[children.length - 1]), true);
        mViewer.getControl().setFocus();
      }
    }
  }

  private void editButtonPressed(Object pSelection)
  {
    Property property = null;
    if (pSelection instanceof Property)
    {
      property = (Property) pSelection;
    }
    else if (pSelection instanceof ValueHolder)
    {
      property = (Property) ((ValueHolder) pSelection).getParent();
    }

    if (property != null)
    {
      Object parent = property.getParent();
      EditParameterPropertyValueWizard wizard = new EditParameterPropertyValueWizard(
          EditParameterPropertyValueWizard.EDIT_MODE,mDidYouKnowEdit, parent, property);
      WizardDialog container = new WizardDialog(mShell, wizard);
      container.open();
      if (wizard.isDirty())
      {
        mViewer.refresh(property);
      }
      if (pSelection instanceof ISelection)
      {
        mViewer.setSelection((ISelection) pSelection, true);
      }
      mViewer.getControl().setFocus();
    }
  }

  private void removeButtonPressed(Object pSelection)
  {
    if (pSelection instanceof INode)
    {
      INode node = (INode) pSelection;
      Object object = node.getParent();
      if ((object == null) || !(object instanceof INode))
      {
        MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "The selection you made does not appear to have a parent, removing it is not possible. Please make a different selection and try again.");
      }
      else
      {
        INode parent = (INode) object;
        boolean removed = parent.removeChild(node);
        if (removed)
        {
          mViewer.refresh();
        }
        else
        {
          MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "There seems to be inconsistency in the model, removing the selection is not possible. Please make a different selection and try again.");
        }
      }
    }
    else
    {
      MessageDialog.openError(mRemoveButton.getShell(), "Error while trying to remove the selection", "Removing the selection you made is not supported. Please make a different selection and try again.");
    }
  }

}