/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.osgi.framework.Bundle;

import java.net.MalformedURLException;
import java.net.URL;

/**
 * Provides a single access point to all the images used by this plugin. Most of the code
 * here is taken from JavaPluginImages class.
 * 
 * @author Hatim Khan
 * @version $Revision: 1.2 $
 */
public class ExoUIPluginImages
{
  public static final String CLASS_VERSION = "$Id: ExoUIPluginImages.java,v 1.2 2004/09/20 01:42:58 hatimk Exp $"; //$NON-NLS-1$
  private static URL mIconBaseURL = null;

  static
  {
    String pathSuffix = "icons/full/"; //$NON-NLS-1$

    Bundle bundle = Platform.getBundle(ExoUIPlugin.PLUGIN_ID);

    mIconBaseURL = bundle.getEntry(pathSuffix);
  }

  private static final String T_OBJ = "obj16"; //$NON-NLS-1$
  private static final String T_OVR = "ovr16"; //$NON-NLS-1$
  private static final String T_WIZBAN = "wizban"; //$NON-NLS-1$
  private static final String T_CLCL = "clcl16"; //$NON-NLS-1$
  private static final String T_DLCL = "dlcl16"; //$NON-NLS-1$
  private static final String T_CTOOL = "ctool16"; //$NON-NLS-1$
  private static final String T_CVIEW = "cview16"; //$NON-NLS-1$
  public static final ImageDescriptor DESC_WIZBAN_ADD_PORTLET_LIBRARY = create(T_WIZBAN, "add-portlet-library-wiz.gif"); //$NON-NLS-1$
  public static final ImageDescriptor DESC_WIZBAN_NEW_PORTLET_PROJECT = create(T_WIZBAN, "new-portlet-project-wiz.gif"); //$NON-NLS-1$
  public static final ImageDescriptor DESC_WIZBAN_UPGRADE_TO_PORTLET = create(T_WIZBAN, "upgrade-to-portlet-wiz.gif"); //$NON-NLS-1$
  public static final ImageDescriptor DESC_WIZBAN_CHANGE_WEB_FOLDER = create(T_WIZBAN, "change-web-folder-wiz.gif"); //$NON-NLS-1$    
  public static final ImageDescriptor DESC_WIZBAN_EXO_SERVICES = create(T_WIZBAN, "exo-platform-services-wiz.gif"); //$NON-NLS-1$      
  public static final ImageDescriptor DESC_WIZBAN_EXO_SERVICES_PARAM = create(T_WIZBAN, "exo-service-parameter-wiz.gif"); //$NON-NLS-1$        
  public static final ImageDescriptor DESC_WIZBAN_EXO_GLOBAL_PARAM = create(T_WIZBAN, "exo-global-parameter-wiz.gif"); //$NON-NLS-1$          
  public static final ImageDescriptor DESC_OBJ_SAMPLE_WEB_APP = create(T_OBJ, "sample-webapp-obj.gif"); //$NON-NLS-1$
  public static final ImageDescriptor DESC_OBJ_TOMCAT_HOME = create(T_OBJ, "home-tab-obj.gif"); //$NON-NLS-1$    
  public static final ImageDescriptor DESC_OBJ_JDK_HOME = create(T_OBJ, "jdk-tab-obj.gif"); //$NON-NLS-1$    		
  public static final ImageDescriptor DESC_OBJ_SOURCE_PORTLET_TAB = create(T_OBJ, "source-portlets-tab-obj.gif"); //$NON-NLS-1$    				
  public static final ImageDescriptor DESC_OBJ_SERVICES = create(T_OBJ, "services-obj.gif"); //$NON-NLS-1$    				  
  public static final ImageDescriptor DESC_OBJ_GLOBAL_PARAM = create(T_OBJ, "global-param-obj.gif"); //$NON-NLS-1$    				    
  public static final ImageDescriptor DESC_OBJ_PARAM_OBJ = create(T_OBJ, "parameter-object-obj.gif"); //$NON-NLS-1$    				      
  public static final ImageDescriptor DESC_OBJ_PARAM_PRO = create(T_OBJ, "parameter-properties-obj.gif"); //$NON-NLS-1$    				        
  public static final ImageDescriptor DESC_OBJ_PARAM_VALUE = create(T_OBJ, "parameter-value-obj.gif"); //$NON-NLS-1$    				        
  public static final ImageDescriptor DESC_OBJ_PARAM_KEY = create(T_OBJ, "key-obj.gif"); //$NON-NLS-1$    				          
  public static final ImageDescriptor DESC_OBJ_PARAM_VALUES = create(T_OBJ, "parameter-values-obj.gif"); //$NON-NLS-1$
  public static final ImageDescriptor DESC_OBJ_DELETE = create(T_OBJ, "delete-icon.gif"); //$NON-NLS-1$    				            
  public static final ImageDescriptor DESC_OBJ_BLUE_LINE = create(T_OBJ, "blue-line.gif"); //$NON-NLS-1$    				              
  public static final ImageDescriptor DESC_OVR_WEBFOLDER = create(T_OVR, "web-folder.gif"); //$NON-NLS-1$
  public static final ImageDescriptor DESC_OVR_PORTLET_V10 = create(T_OVR, "v1-0.gif"); //$NON-NLS-1$
  public static final ImageDescriptor DESC_OVR_PORTLET_V11 = create(T_OVR, "v1-1.gif"); //$NON-NLS-1$
  public static final ImageDescriptor DESC_OVR_PORTLET_V20 = create(T_OVR, "v2-0.gif"); //$NON-NLS-1$

  private static ImageDescriptor create(String pPrefix, String pName)
  {
    try
    {
      return ImageDescriptor.createFromURL(makeIconFileURL(pPrefix, pName));
    }
    catch (MalformedURLException e)
    {
      return ImageDescriptor.getMissingImageDescriptor();
    }
  }

  private static URL makeIconFileURL(String pPrefix, String pName) throws MalformedURLException
  {
    if (mIconBaseURL == null)
    {
      throw new MalformedURLException();
    }

    StringBuffer buffer = new StringBuffer(pPrefix);
    buffer.append('/');
    buffer.append(pName);

    return new URL(mIconBaseURL, buffer.toString());
  }
}