/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;


import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.exoplatform.eclipse.core.operation.IExoOperation;

import java.lang.reflect.InvocationTargetException;

/**
 * An <code>IRunnableWithProgress</code> that adapts and
 * <code>IExoOperation</code> so that is can be executed inside
 * <code>IRunnableContext</code>. <code>OperationCanceledException</code>
 * thrown by the adapted runnable are cought and rethrown as a
 * <code>InterruptedException</code>. (comments by WorkbenchRunnableAdapter).
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class ExoRunnableAdapter implements IRunnableWithProgress
{
  public static final String CLASS_VERSION = "$Id: ExoRunnableAdapter.java,v 1.1 2004/04/19 03:37:26 hatimk Exp $"; //$NON-NLS-1$
  private IExoOperation mOperation;

  /**
   * The constructor method.
   *
   * @param pOperation  
   */
  public ExoRunnableAdapter(IExoOperation pOperation)
  {
    mOperation = pOperation;
  }

  /*
   * @see IRunnableWithProgress#run(IProgressMonitor)
   */
  public void run(IProgressMonitor pMonitor)
    throws InvocationTargetException, InterruptedException
  {
    try
    {
      mOperation.run(pMonitor);
    }
    catch (OperationCanceledException e)
    {
      throw new InterruptedException(e.getMessage());
    }
    catch (CoreException e)
    {
      throw new InvocationTargetException(e);
    }
  }
}
