/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;

import java.util.Collection;

/**
 * Selection provider that always returns the same selection.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class SimpleSelectionProvider implements ISelectionProvider
{
  public static final String CLASS_VERSION = "$Id: SimpleSelectionProvider.java,v 1.1 2004/04/19 03:37:26 hatimk Exp $"; //$NON-NLS-1$
  private ISelection mSelection;

  /**
   * The constructor method.
   *
   * @param pCollection elements that represent the selection
   */
  public SimpleSelectionProvider(Collection pCollection)
  {
    this(pCollection.toArray());
  }

  /**
   * The constructor method.
   *
   * @param pElements elements that represent the selection
   */
  public SimpleSelectionProvider(Object[] pElements)
  {
    mSelection = new StructuredSelection(pElements);
  }

  /**
   * The constructor method.
   *
   * @param pElement element that represents the selection
   */
  public SimpleSelectionProvider(Object pElement)
  {
    mSelection = new StructuredSelection(pElement);
  }

  /**   
   * Faked the registering of listener. This method does nothing.
   *
   * @param pListener the listener
   */
  public void addSelectionChangedListener(ISelectionChangedListener pListener)
  {
  }

  /**   
   * Returns the same selection all the time.
   *
   * @return the same selection all the time
   */
  public ISelection getSelection()
  {
    return mSelection;
  }

  /**   
   * This method does nothing.
   *
   * @param pListener the listener
   */
  public void removeSelectionChangedListener(
    ISelectionChangedListener pListener)
  {
  }

  /**   
   * This method does nothing.
   *
   * @param selection the selection
   */
  public void setSelection(ISelection selection)
  {
  }
}
