/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.action;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.exoplatform.eclipse.internal.ui.wizard.UpgradeToPortletProjectWizard;

/**
 * The workbench will delegates the upgrade to the portlet project action to
 * this class which simply invokes the upgrade wizard.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class UpgradeToPortletProjectActionDelegate
  implements IObjectActionDelegate
{
  public static final String CLASS_VERSION = "$Id: UpgradeToPortletProjectActionDelegate.java,v 1.1 2004/04/19 03:37:26 hatimk Exp $"; //$NON-NLS-1$
  private IWorkbenchPart mActivePart;
  private ISelection mSelection;

  /**   
   * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
   *      org.eclipse.ui.IWorkbenchPart)
   */
  public void setActivePart(IAction pAction, IWorkbenchPart pTargetPart)
  {
    mActivePart = pTargetPart;
  }

  /**   
   * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
   */
  public void run(IAction pAction)
  {
    // for now it is assumed that the selection represents a valid java project
    // no checking is done to prove otherwise.
    UpgradeToPortletProjectWizard wizard = new UpgradeToPortletProjectWizard();
    wizard.init(getSelectedProject());
    WizardDialog dialog =
      new WizardDialog(mActivePart.getSite().getShell(), wizard);
    dialog.open();
  }

  /**   
   * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
   *      org.eclipse.jface.viewers.ISelection)
   */
  public void selectionChanged(IAction pAction, ISelection pSelection)
  {
    mSelection = pSelection;
  }

  /**   
   * Returns the current project that include the current selection.
   *
   * @return the project that contains the current selection
   */
  public IProject getSelectedProject()
  {
    IResource resource = null;

    if ((mSelection != null) && (!mSelection.isEmpty()))
    {
      if (mSelection instanceof IStructuredSelection)
      {
        Object selection =
          ((IStructuredSelection) mSelection).getFirstElement();
        if (selection instanceof IResource)
        {
          resource = (IResource) selection;
        }
        else if (selection instanceof IAdaptable)
        {
          resource =
            (IResource) ((IAdaptable) selection).getAdapter(IResource.class);
        }
      }
    }

    return (resource == null) ? null : resource.getProject();
  }
}
