/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.common;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Display;

/**
 * Wrappes a progress monitor implemenation so it can be accessed from none UI
 * threads. NOTE: don not use, does not work, because there is no way to tell
 * if the widget is disposed by just looking at IProgressMonitor
 * implemenation.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class NoneUIThreadProgressMonitorWrapper implements IProgressMonitor
{
  public static final String CLASS_VERSION = "$Id: NoneUIThreadProgressMonitorWrapper.java,v 1.1 2004/04/19 03:37:38 hatimk Exp $"; //$NON-NLS-1$
  private IProgressMonitor mMonitor = null;
  private Display mDisplay = null;
  private String mName = null;
  private String mTaskName = null;
  private String mSubTaskName = null;
  private int mWork;
  private int mDoneWork;
  private double mWorked;
  private boolean mCanceled = false;

  /**
   * The constructor method.
   *
   * @param pMonitor
   * @param pDisplay
   */
  public NoneUIThreadProgressMonitorWrapper(IProgressMonitor pMonitor,
    Display pDisplay)
  {
    mMonitor = pMonitor;
    mDisplay = pDisplay;
  }

  /**   
   * @see org.eclipse.core.runtime.IProgressMonitor#beginTask(java.lang.String,
   *      int)
   */
  public void beginTask(String pName, int pTotalWork)
  {
    if ((mMonitor == null) || (mDisplay == null))
    {
      return;
    }
    mName = pName;
    mWork = pTotalWork;
    mDisplay.asyncExec(new Runnable()
      {
        public void run()
        {
          mMonitor.beginTask(mName, mWork);
        }
      });
  }

  /**   
   * @see org.eclipse.core.runtime.IProgressMonitor#done()
   */
  public void done()
  {
    if ((mMonitor == null) || (mDisplay == null))
    {
      return;
    }
    mDisplay.asyncExec(new Runnable()
      {
        public void run()
        {
          mMonitor.done();
        }
      });
  }

  /**   
   * @see org.eclipse.core.runtime.IProgressMonitor#internalWorked(double)
   */
  public void internalWorked(double pWorked)
  {
    if ((mMonitor == null) || (mDisplay == null))
    {
      return;
    }
    mWorked = pWorked;
    mDisplay.asyncExec(new Runnable()
      {
        public void run()
        {
          mMonitor.internalWorked(mWorked);
        }
      });
  }

  /**   
   * @see org.eclipse.core.runtime.IProgressMonitor#isCanceled()
   */
  public boolean isCanceled()
  {
    if ((mMonitor == null) || (mDisplay == null))
    {
      return true;
    }
    mDisplay.asyncExec(new Runnable()
      {
        public void run()
        {
          mCanceled = mMonitor.isCanceled();
        }
      });

    return mCanceled;
  }

  /**   
   * @see org.eclipse.core.runtime.IProgressMonitor#setCanceled(boolean)
   */
  public void setCanceled(boolean pValue)
  {
    if ((mMonitor == null) || (mDisplay == null))
    {
      return;
    }
    mCanceled = pValue;
    mDisplay.asyncExec(new Runnable()
      {
        public void run()
        {
          mMonitor.setCanceled(mCanceled);
        }
      });
  }

  /**   
   * @see org.eclipse.core.runtime.IProgressMonitor#setTaskName(java.lang.String)
   */
  public void setTaskName(String pTaskName)
  {
    if ((mMonitor == null) || (mDisplay == null))
    {
      return;
    }
    mTaskName = pTaskName;
    mDisplay.asyncExec(new Runnable()
      {
        public void run()
        {
          mMonitor.setTaskName(mTaskName);
        }
      });
  }

  /**   
   * @see org.eclipse.core.runtime.IProgressMonitor#subTask(java.lang.String)
   */
  public void subTask(String pSubTaskName)
  {
    if ((mMonitor == null) || (mDisplay == null))
    {
      return;
    }
    mSubTaskName = pSubTaskName;
    mDisplay.asyncExec(new Runnable()
      {
        public void run()
        {
          mMonitor.subTask(mSubTaskName);
        }
      });
  }

  /**   
   * @see org.eclipse.core.runtime.IProgressMonitor#worked(int)
   */
  public void worked(int pDoneWork)
  {
    if ((mMonitor == null) || (mDisplay == null))
    {
      return;
    }
    mDoneWork = pDoneWork;
    mDisplay.asyncExec(new Runnable()
      {
        public void run()
        {
          mMonitor.worked(mDoneWork);
        }
      });
  }
}
