/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.common;


import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.exoplatform.eclipse.core.ExoCorePlugin;

/**
 * Will return true if the element given is part of a portlet project that is open.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class OpenedPortletProjectFilter extends ViewerFilter
{
  public static final String CLASS_VERSION = "$Id: OpenedPortletProjectFilter.java,v 1.1 2004/09/20 01:42:57 hatimk Exp $"; //$NON-NLS-1$

  /**   
   * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer,
   *      java.lang.Object, java.lang.Object)
   */
  public boolean select(Viewer pViewer, Object pParentElement, Object pElement)
  {
    IProject project = null;

    if (pElement instanceof IJavaElement)
    {
      project = ((IJavaElement) pElement).getJavaProject().getProject();
    }
    else if (pElement instanceof IResource)
    {
      project = ((IResource) pElement).getProject();
    }
    else
    {
      return false;
    }

    if (project == null)
    {
      return false;
    }

    try
    {
      if ((!project.isOpen()) || (!project.hasNature(ExoCorePlugin.PORTLET_NATURE_ID)))
      {
        // simply, we are not interested in none portlet projects or closed projects
        return false;
      }
      else
      {
        return true;
      }
    }
    catch (CoreException e)
    {
      // will just ignore this project
      return false;
    }
  }
}
