/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.ui.common;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.exoplatform.eclipse.core.exoservices.GlobalParam;
import org.exoplatform.eclipse.core.exoservices.ObjectParam;
import org.exoplatform.eclipse.core.exoservices.Parameter;
import org.exoplatform.eclipse.core.exoservices.PropertiesParam;
import org.exoplatform.eclipse.core.exoservices.Property;
import org.exoplatform.eclipse.core.exoservices.ServiceConfiguration;
import org.exoplatform.eclipse.core.exoservices.ValueParam;
import org.exoplatform.eclipse.core.exoservices.ValuesParam;
import org.exoplatform.eclipse.core.portal.model.ValueHolder;
import org.exoplatform.eclipse.ui.ExoUIPluginImages;

/**
 * @author Hatim Khan
 */
public class PortalConfigurationLabelProvider extends LabelProvider
{
  private Map mImages = new HashMap();

  /**
   * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
   */
  public Image getImage(Object pElement)
  {
    if (pElement instanceof ServiceConfiguration)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_SERVICES);
    }
    else if (pElement instanceof GlobalParam)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_GLOBAL_PARAM);
    }
    else if (pElement instanceof ObjectParam)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_PARAM_OBJ);
    }
    else if (pElement instanceof PropertiesParam)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_PARAM_PRO);
    }
    else if (pElement instanceof ValueParam)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_PARAM_VALUES);
    }
    else if (pElement instanceof ValuesParam)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_PARAM_VALUES);
    }
    else if (pElement instanceof Property)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_PARAM_KEY);
    }
    else if (pElement instanceof ValueHolder)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_PARAM_VALUE);
    }
    else if (pElement instanceof Property)
    {
      return getImage(ExoUIPluginImages.DESC_OBJ_PARAM_VALUE);
    }
    else return null;
  }

  private Image getImage(ImageDescriptor pDesc)
  {
    Image image = (Image) mImages.get(pDesc);
    if (image == null)
    {
      image = pDesc.createImage();
      mImages.put(pDesc, image);
    }
    return image;
  }

  /**
   * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
   */
  public String getText(Object pElement)
  {
    if (pElement instanceof GlobalParam) return "Global Parameters";
    else if (pElement instanceof ServiceConfiguration)
    {
      ServiceConfiguration sc = (ServiceConfiguration) pElement;
      String key = sc.getKey();
      if ((key != null) && (key.length() > 0)) return key;
      else return emptyIfNull(sc.getType());
    }
    else if (pElement instanceof ObjectParam)
    {
      ObjectParam parameter = (ObjectParam) pElement;
      String type = emptyIfNull(parameter.getType());
      String name = emptyIfNull(parameter.getName());

      return (name.length() > 0) ? name : type;
    }
    else if (pElement instanceof Parameter)
    {
      Parameter parameter = (Parameter) pElement;
      return emptyIfNull(parameter.getName());
    }
    else if (pElement instanceof ValueHolder)
    {
      ValueHolder value = (ValueHolder) pElement;
      return UICompositeUtil.escapeBackslash(emptyIfNull(value.getValue()));
    }
    else if (pElement instanceof Property)
    {
      Property parameter = (Property) pElement;
      return emptyIfNull(parameter.getName());
    }

    else return "";
  }

  /**
   * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
   */
  public void dispose()
  {
    super.dispose();
    Collection collection = mImages.values();
    Iterator iterator = collection.iterator();
    while (iterator.hasNext())
    {
      Image image = (Image) iterator.next();
      if (!image.isDisposed()) image.dispose();
    }
    mImages.clear();
  }

  private String emptyIfNull(String pString)
  {
    return (pString != null) ? pString : "";
  }

}