/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/

package org.exoplatform.eclipse.ui.common;

import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.preference.IPersistentPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;

import java.io.IOException;

/**
 * This is a simple wrapper around the non UI Preferences class to expose it as
 * IPreferenceStore. This will allow the eXo UI plugin to expose the
 * preferences stored for the exo Core plugin as a UI preferences store, this
 * way it could be used with FieldEditorPreferencesPage, etc. IMPORTANT NOTE:
 * this wrapper does not support any property change events.
 *
 * @author Hatim Khan
 * @version $Revision: 1.1 $
 */
public class PreferencesWrapper implements IPersistentPreferenceStore
{
  public static final String CLASS_VERSION = "$Id: PreferencesWrapper.java,v 1.1 2004/04/19 03:37:39 hatimk Exp $"; //$NON-NLS-1$

  /**
   * The underlying core runtime preference store
   */
  private Preferences mPreferences = null;
  private Plugin mCorePlugin = null;

  /**
   * The constructor method.
   *
   * @param pCorePlugin  
   */
  public PreferencesWrapper(Plugin pCorePlugin)
  {
    mCorePlugin = pCorePlugin;
    mPreferences = mCorePlugin.getPluginPreferences();
  }

  /**   
   * Returns the underlying preference store.
   *
   * @return the underlying preference store
   */
  private Preferences getPrefs()
  {
    return mPreferences;
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public boolean contains(String name)
  {
    return getPrefs().contains(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public boolean getBoolean(String name)
  {
    return getPrefs().getBoolean(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public boolean getDefaultBoolean(String name)
  {
    return getPrefs().getDefaultBoolean(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public double getDefaultDouble(String name)
  {
    return getPrefs().getDefaultDouble(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public float getDefaultFloat(String name)
  {
    return getPrefs().getDefaultFloat(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public int getDefaultInt(String name)
  {
    return getPrefs().getDefaultInt(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public long getDefaultLong(String name)
  {
    return getPrefs().getDefaultLong(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public String getDefaultString(String name)
  {
    return getPrefs().getDefaultString(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public double getDouble(String name)
  {
    return getPrefs().getDouble(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public float getFloat(String name)
  {
    return getPrefs().getFloat(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public int getInt(String name)
  {
    return getPrefs().getInt(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public long getLong(String name)
  {
    return getPrefs().getLong(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public String getString(String name)
  {
    return getPrefs().getString(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public boolean isDefault(String name)
  {
    return getPrefs().isDefault(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public boolean needsSaving()
  {
    return getPrefs().needsSaving();
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void putValue(String name, String value)
  {
    getPrefs().setValue(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setDefault(String name, double value)
  {
    getPrefs().setDefault(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setDefault(String name, float value)
  {
    getPrefs().setDefault(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setDefault(String name, int value)
  {
    getPrefs().setDefault(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setDefault(String name, long value)
  {
    getPrefs().setDefault(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setDefault(String name, String value)
  {
    getPrefs().setDefault(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setDefault(String name, boolean value)
  {
    getPrefs().setDefault(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setToDefault(String name)
  {
    getPrefs().setToDefault(name);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setValue(String name, double value)
  {
    getPrefs().setValue(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setValue(String name, float value)
  {
    getPrefs().setValue(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setValue(String name, int value)
  {
    getPrefs().setValue(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setValue(String name, long value)
  {
    getPrefs().setValue(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setValue(String name, String value)
  {
    getPrefs().setValue(name, value);
  }

  /* (non-javadoc)
   * Method declared on IPreferenceStore
   */
  public void setValue(String name, boolean value)
  {
    getPrefs().setValue(name, value);
  }

  /**   
   * @see org.eclipse.jface.preference.IPersistentPreferenceStore#save()
   */
  public void save() throws IOException
  {
    mCorePlugin.savePluginPreferences();
  }

  /**   
   * @see org.eclipse.jface.preference.IPreferenceStore#addPropertyChangeListener(org.eclipse.jface.util.IPropertyChangeListener)
   *      not supported
   */
  public void addPropertyChangeListener(IPropertyChangeListener listener)
  {
    // not supported
  }

  /**   
   * @see org.eclipse.jface.preference.IPreferenceStore#firePropertyChangeEvent(java.lang.String,
   *      java.lang.Object, java.lang.Object) not supported
   */
  public void firePropertyChangeEvent(String name, Object oldValue,
    Object newValue)
  {
    // not supported
  }

  /**   
   * @see org.eclipse.jface.preference.IPreferenceStore#removePropertyChangeListener(org.eclipse.jface.util.IPropertyChangeListener)
   *      not supported
   */
  public void removePropertyChangeListener(IPropertyChangeListener listener)
  {
    // not supported
  }
}
