/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please visit http://www.exoplatform.org for more license detail.        *
 **************************************************************************/
package org.exoplatform.eclipse.ui.dialog;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/**
 * A dialog that displays a 'did you know?' section at the end of the dialog. This dialog
 * is meant to be extended. Extenders should provide the content of the dialog by
 * overriding createActualContent() method. In this method, the extenders should set the
 * title by calling setTitle(), and a message by calling setMessage(), notice that
 * setTitle() is not the title of the window itself. The title of the window is given to
 * the constructor.
 * 
 * @author Hatim Khan
 */
public abstract class ExplainFormDialog extends TitleAreaDialog
{
  private String mWindowTitle;
  private String mDidYouKnowText;
  private ImageDescriptor mImageDescriptor;
  private Image mImage;

  public ExplainFormDialog(Shell pParentShell, String pWindowTitle, String pDidYouKnowText,
      ImageDescriptor pImageDescriptor)
  {
    super(pParentShell);
    setShellStyle(getShellStyle() | SWT.RESIZE);
    mWindowTitle = pWindowTitle;
    mDidYouKnowText = pDidYouKnowText;
    mImageDescriptor = pImageDescriptor;
  }

  /**
   * Creates and returns the contents of the upper part of this dialog (above the button
   * bar).
   * <p>
   * The <code>Dialog</code> implementation of this framework method creates and returns
   * a new <code>Composite</code> with no margins and spacing.
   * </p>
   * 
   * @param parent the parent composite to contain the dialog area
   * @return the dialog area control
   */
  protected Control createDialogArea(Composite parent)
  {
    Composite composite = (Composite) super.createDialogArea(parent);
    if (mImageDescriptor != null)
    {
      mImage = mImageDescriptor.createImage();
      setTitleImage(mImage);
    }
    //if (getTitleImageLabel() != null) getTitleImageLabel().setVisible(false);

    createContentBeforeDidYouKnow(composite);
    createDidYouKnowContent(composite);
    return composite;
  }

  private void createContentBeforeDidYouKnow(Composite pParent)
  {
    Composite container = new Composite(pParent, SWT.NONE);

    GridLayout layout = new GridLayout(1, false);

    layout.marginHeight = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
    layout.marginWidth = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
    layout.verticalSpacing = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
    layout.horizontalSpacing = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);

    container.setLayout(layout);
    container.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    container.setFont(pParent.getFont());
    createActualContent(container);
  }

  public abstract void createActualContent(Composite pParent);

  private void createDidYouKnowContent(Composite pParent)
  {

    Composite container = new Composite(pParent, SWT.NONE);
    container.setFont(pParent.getFont());

    GridLayout layout = new GridLayout(1, false);
    layout.marginHeight = 0;
    layout.marginWidth = 0;
    layout.verticalSpacing = 0;
    layout.horizontalSpacing = 0;

    container.setLayout(layout);
    container.setLayoutData(new GridData(GridData.FILL_BOTH));
    //UICompositeUtil.setHeightHint(container, 50);

    // Build the separator line
    Label titleBarSeparator = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
    titleBarSeparator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

    new DidYouKnowForm(container, mDidYouKnowText);

    // Build the separator line
    titleBarSeparator = new Label(container, SWT.HORIZONTAL | SWT.SEPARATOR);
    titleBarSeparator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
  }

  protected void configureShell(Shell pNewShell)
  {
    super.configureShell(pNewShell);
    pNewShell.setText(mWindowTitle);
  }

  /**
   * @see org.eclipse.jface.window.Window#close()
   */
  public boolean close()
  {
    boolean returnValue = super.close();
    if (returnValue)
    {
      if (mImage != null)
      {
        if (!mImage.isDisposed()) mImage.dispose();
      }
    }
    return returnValue;
  }

}