package org.exoplatform.eclipse.webunit.actions;

import groovy.lang.GroovyClassLoader;
import java.util.List ;
import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.*;
import org.eclipse.ui.internal.WorkbenchPage;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath; 
import org.eclipse.core.resources.IProject ;
import org.exoplatform.eclipse.webunit.ExceptionUtil;
import org.exoplatform.eclipse.webunit.views.ViewController;
import org.exoplatform.eclipse.webunit.views.WebunitView;
import org.exoplatform.test.web.TestSuites;
import org.exoplatform.test.web.WebUnitSuite;

public class OpenWebunitViewAction implements IObjectActionDelegate {
  private IWorkbenchPage page_ ;
  private IProject iproject_ ;
  private IPath resource_ ;

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
    IWorkbenchPartSite site = targetPart.getSite() ;
    page_ = site.getPage() ;
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
    ViewController controller = null ;
    try {
      WebunitView view = 
        (WebunitView) page_.showView("org.exoplatform.eclipse.webunit.views.WebunitView") ;
      if(view == null) {
        throw new Exception("Cannot find the view part") ;
      }
      IWorkbenchPartReference partRef = page_.getActivePartReference();
      if (partRef != null) {
        ((WorkbenchPage) page_).toggleZoom(partRef);
      }
      controller = view.getViewController() ;
      TestSuites suites = createTestSuites()  ;
      controller.getUnitTestView().setTestSuites(suites) ;
    } catch (Exception ex) {
      String trace =  ExceptionUtil.getStackTrace(ex, 100) ;
      if(controller != null) {
        controller.showError(ex, trace) ;
      } else {
        Shell shell = new Shell();
        MessageDialog.openInformation( shell, "eXo Webunit Plugin Wrror", trace);
      }
    }
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
    IStructuredSelection sel = (IStructuredSelection) selection ;
    IFile file =  (IFile)sel.getFirstElement() ;
    if(file == null) return ; 
    iproject_ = file.getProject() ;
    resource_ = file.getFullPath();
	}

  
	private TestSuites createTestSuites() throws Exception {
    IJavaProject iJavaProject = JavaCore.create(iproject_) ;
    String projectLocation = iproject_.getLocation().toString() + "/";
    String groovyPath = null ;
    String selectClass = resource_.toString() ;
    URL[] url = null ;
    if(iJavaProject == null) {
      url = new URL[1] ;
      selectClass = resource_.lastSegment() ;
      groovyPath = projectLocation + resource_.removeFirstSegments(1).removeLastSegments(1).toString()  + "/" ;
      url[0] = new URL("file:"  + groovyPath );
    } else {
      IClasspathEntry[] classPath = iJavaProject.getResolvedClasspath(true) ;
      ArrayList validPath = new ArrayList() ;
      validPath.add(projectLocation + iJavaProject.getOutputLocation().removeFirstSegments(1).toString() + "/") ;
      for(int i = 0; i < classPath.length; i++) {
        String path = classPath[i].getPath().toString()  ;
        if(path.indexOf("jre/lib") > 0  || path.indexOf("plugins/org.eclipse") > 0) {
        } else if(classPath[i].getEntryKind() == IClasspathEntry.CPE_PROJECT) {
        } else if(classPath[i].getEntryKind() == IClasspathEntry.CPE_SOURCE) {
          String srcPath =  classPath[i].getPath().removeFirstSegments(1).toString() ;
          int idx = selectClass.indexOf(srcPath) ;
          if( idx > 0) {
            selectClass = selectClass.substring(idx + srcPath.length() + 1, selectClass.length()) ;
          }
          groovyPath = projectLocation +  classPath[i].getPath().removeFirstSegments(1).toString() + "/" ;
          validPath.add(groovyPath) ;
          IPath ipath = classPath[i].getOutputLocation() ;
          if(ipath != null)  validPath.add(projectLocation + ipath.removeFirstSegments(1).toString() + "/") ;
        } else {
          validPath.add(path) ;
        }
      }
      url = new URL[validPath.size()] ;
      for (int i = 0 ; i < url.length; i++) {
        url[i] = new URL("file:" + (String)validPath.get(i)) ;
        System.out.println("path: " + url[i].toString()) ;
      }
    }
    //System.out.println("select class : " + selectClass) ;
    System.out.println("groovy path : " + groovyPath) ;
    ClassLoader parentLoader =  new URLClassLoader(url, TestSuites.class.getClassLoader()) ;
    GroovyClassLoader classloader =  new GroovyClassLoader(parentLoader) ;
    classloader.addClasspath(groovyPath) ;
    URL  classURL = classloader.getResource(selectClass) ;
    Class clazz = classloader.parseClass(classURL.openStream()) ;
    Object instance = clazz.newInstance() ;
    if(instance instanceof WebUnitSuite) {
      WebUnitSuite suite = (WebUnitSuite) instance ;
      List requiredSuites = suite.getRequiredSuites() ;
      TestSuites suites = new TestSuites() ;
      if(requiredSuites != null) {
        for(int i = 0; i < requiredSuites.size(); i++) {
          WebUnitSuite rsuite = (WebUnitSuite)requiredSuites.get(i) ;
          suites.addSuite(rsuite) ;
        }
      }
      suites.addSuite(suite) ;
      return suites ;
    } else if(instance instanceof TestSuites) {
      TestSuites suites = (TestSuites)clazz.newInstance() ;
      return suites ;

    }
    return null ;
  }
}
