package org.exoplatform.eclipse.webunit.views;

import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.CloseWindowListener;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.browser.StatusTextEvent;
import org.eclipse.swt.browser.StatusTextListener;
import org.eclipse.swt.browser.TitleEvent;
import org.eclipse.swt.browser.TitleListener;
import org.eclipse.swt.browser.VisibilityWindowListener;
import org.eclipse.swt.browser.WindowEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

public class BrowserView extends Composite {
  static ResourceBundle res_ = ResourceBundle.getBundle("org.exoplatform.eclipse.webunit.resource");
  
  private boolean busy_;
  private Image browserIcon_ ;
  private Text location_ ;
  final private ToolItem backButton_ ;
  final private ToolItem forwardButton_ ;
  final private ToolItem stopButton_ ;
  final private ToolItem refreshButton_ ;
  final private ToolItem goButton_ ;
  final private Label status_ ; 
  final private ProgressBar progressBar_ ; 
  private Browser browser_ ;
  
  public BrowserView(Composite parent)  {
    super(parent, SWT.NONE) ;
    initResources();
    FormLayout layout = new FormLayout();
    layout.spacing = 0 ;
    layout.marginHeight = 0 ;
    layout.marginWidth = 0 ;
    setLayout(layout);
    
    ToolBar toolbar = new ToolBar(this, SWT.NONE);
    backButton_ = new ToolItem(toolbar, SWT.PUSH);
    backButton_.setText(res_.getString("Back"));
    forwardButton_ = new ToolItem(toolbar, SWT.PUSH);
    forwardButton_.setText(res_.getString("Forward"));
    stopButton_ = new ToolItem(toolbar, SWT.PUSH);
    stopButton_.setText(res_.getString("Stop"));
    refreshButton_ = new ToolItem(toolbar, SWT.PUSH);
    refreshButton_.setText(res_.getString("Refresh"));
    goButton_ = new ToolItem(toolbar, SWT.PUSH);
    goButton_.setText(res_.getString("Go"));
    
    location_ = new Text(this, SWT.BORDER);
    
    final Canvas canvas = new Canvas(this, SWT.NO_BACKGROUND);
    final Rectangle rect = browserIcon_.getBounds() ;
    canvas.addListener(SWT.Paint, new Listener() {
      public void handleEvent(Event e) {
        Point pt = canvas.getSize();
        e.gc.drawImage(browserIcon_, 0, 0, rect.width, rect.height, 0, 0, pt.x, pt.y);     
      }
    });
    
    status_ = new Label(this, SWT.NONE);
    progressBar_ = new ProgressBar(this, SWT.NONE);
    
    FormData data = new FormData();
    data.top = new FormAttachment(0, 5);
    toolbar.setLayoutData(data);
    
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(canvas, 5, SWT.DEFAULT);
    data.bottom = new FormAttachment(status_, -5, SWT.DEFAULT);
    try {
      browser_ = new MyBrowser(this, SWT.NONE);
      browser_.setLayoutData(data);
    } catch (SWTError e) {
      /* Browser widget could not be instantiated */
      Label label = new Label(this, SWT.CENTER | SWT.WRAP);
      label.setText(res_.getString("BrowserNotCreated"));
      label.setLayoutData(data);
    }
    
    data = new FormData();
    data.width = 24;
    data.height = 24;
    data.top = new FormAttachment(0, 5);
    data.right = new FormAttachment(100, -5);
    canvas.setLayoutData(data);
    
    data = new FormData();
    data.top = new FormAttachment(toolbar, 0, SWT.TOP);
    data.left = new FormAttachment(toolbar, 5, SWT.RIGHT);
    data.right = new FormAttachment(canvas, -5, SWT.DEFAULT);
    location_.setLayoutData(data);
    
    data = new FormData();
    data.left = new FormAttachment(0, 5);
    data.right = new FormAttachment(progressBar_, 0, SWT.DEFAULT);
    data.bottom = new FormAttachment(100, -5);
    status_.setLayoutData(data);
    
    data = new FormData();
    data.right = new FormAttachment(100, -5);
    data.bottom = new FormAttachment(100, -5);
    progressBar_.setLayoutData(data);
    if (browser_ != null) {
      backButton_.setEnabled(browser_.isBackEnabled());
      forwardButton_.setEnabled(browser_.isForwardEnabled());
      browser_.addLocationListener(new BrowserLocationListener()) ;
      browser_.addProgressListener(new BrowserProgressListener()) ;
      browser_.addStatusTextListener(new BrowserStatusTextListener()) ; 
      backButton_.addListener(SWT.Selection, new BackButtonListener());
      forwardButton_.addListener(SWT.Selection, new ForwardButtonListener());
      stopButton_.addListener(SWT.Selection, new StopButtonListener());
      refreshButton_.addListener(SWT.Selection, new RefreshButtonListener());
      goButton_.addListener(SWT.Selection, new GoButtonListener());
      browser_.addVisibilityWindowListener(new BrowserVisibilityWindowListener()) ;
      browser_.addCloseWindowListener(new CloseBrowserWindowListener()) ;
      browser_.setUrl(res_.getString("Startup"));
    }
  }
  
  public void dispose() {  freeResources(); }
 
  private void initResources() {
    freeResources() ;
    Class clazz = getClass();
    try {
      ImageData source = new ImageData(clazz.getResourceAsStream("browser-icon.bmp"));
      ImageData mask = source.getTransparencyMask();
      browserIcon_ = new Image(null, source, mask);
      return;
    } catch (Throwable t) {
      throw new RuntimeException(t);
    }
  }
  
  void freeResources() {
    if (browserIcon_ != null)  {
      browserIcon_.dispose() ;
      browserIcon_ = null ;
    }
  }
  
  public Browser getBrowser() { return browser_ ; }
  
  class BackButtonListener implements  Listener {
    public void handleEvent(Event event) {  browser_.back(); }
  }
  
  class ForwardButtonListener implements  Listener {
    public void handleEvent(Event event) { browser_.forward() ; }
  }
  
  class StopButtonListener implements  Listener {
    public void handleEvent(Event event) { browser_.stop(); }
  }
  
  class RefreshButtonListener implements  Listener {
    public void handleEvent(Event event) {    browser_.refresh();  }
  }
  
  class GoButtonListener implements  Listener {
    public void handleEvent(Event event) {  browser_.setUrl(location_.getText()); }
  }
  
  class BrowserProgressListener implements ProgressListener {
    public void changed(ProgressEvent event) {
      if (event.total == 0) return;                            
      int ratio = event.current * 100 / event.total;
      progressBar_.setSelection(ratio);
      busy_ = event.current != event.total;
    }
    public void completed(ProgressEvent event) {
      backButton_.setEnabled(browser_.isBackEnabled());
      forwardButton_.setEnabled(browser_.isForwardEnabled());
      progressBar_.setSelection(0);
      busy_ = false;
    }
  }
  
  class BrowserStatusTextListener implements StatusTextListener {
    public void changed(StatusTextEvent event) {
      status_.setText(event.text); 
    }
  }
  
  class BrowserLocationListener implements LocationListener {
    public void changed(LocationEvent event) {
      busy_ = true;
      if (event.top) location_.setText(event.location);
    }
    public void changing(LocationEvent event) {
    }
  }
  
  class BrowserTitleListener implements  TitleListener {
    public void changed(TitleEvent event) {
      Shell shell = browser_.getShell();
      shell.setText(event.title+" - " +  res_.getString("window.title"));
    }
  }
  
  class BrowserVisibilityWindowListener implements VisibilityWindowListener {
    public void hide(WindowEvent event) {  }
    
    public void show(WindowEvent event) {
      Browser browser = (Browser)event.widget;
      Shell shell = browser.getShell();
      if (event.location != null) shell.setLocation(event.location);
      if (event.size != null) {
        Point size = event.size;
        shell.setSize(shell.computeSize(size.x, size.y));
      }
      shell.open();
    }
  }
  
  class CloseBrowserWindowListener implements CloseWindowListener {
    public void close(WindowEvent event) {
      Browser browser = (Browser)event.widget;
      Shell shell = browser.getShell();
      shell.close();
    }
  }
  
  class MyBrowser extends Browser {
    public MyBrowser(Composite parent , int style) {
      super(parent, style) ;
    }
    
    protected void checkWidget () {
      Display display = getDisplay();
      if (display == null) throw new RuntimeException("display is null");
      //if ((state & DISPOSED) != 0) error (SWT.ERROR_WIDGET_DISPOSED);
    }
    
    protected void checkSubclass() {
    }
  }
}