/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.eclipse.webunit.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.exoplatform.test.web.TestSuites;
import org.exoplatform.test.web.WebUnitMonitor;
import org.exoplatform.test.web.WebUnitSuite;
import org.exoplatform.test.web.unit.WebUnit;
import org.exoplatform.eclipse.webunit.widgets.ExoTree;
import org.exoplatform.eclipse.webunit.widgets.ExoTreeItem;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Oct 2, 2004
 * @version $Id: ExoWebUnitView.java,v 1.4 2004/10/12 03:30:42 tuan08 Exp $
 */
public class ExoWebUnitView extends Composite {
  private Tree unitTree_ ;
  private UnitTestControllerView unitTestControllerView_ ;
  
  public ExoWebUnitView(ViewController parent)  {
    super(parent, SWT.NONE);
    GridLayout gridLayout = new GridLayout();
    gridLayout.marginWidth = 0 ;
    gridLayout.marginHeight = 0 ;
    gridLayout.verticalSpacing = 0 ;
    gridLayout.horizontalSpacing = 0 ;
    setLayout(gridLayout);
    
    GridData gridData =
      new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
    unitTree_ =  new ExoTree(this ,SWT.SINGLE);
    unitTree_.setLayoutData(gridData) ;
    unitTree_.addSelectionListener(new TestItemSelecttionListener()) ;
    
    gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL );
    gridData.heightHint = 150 ;
    unitTestControllerView_  = new UnitTestControllerView(this) ;
    unitTestControllerView_.setLayoutData(gridData) ;
  }
  
  public UnitTestControllerView getUnitTestControlellerView() { return unitTestControllerView_ ; }
  
  public void setTestSuites(TestSuites testsuites) { 
    unitTestControllerView_.setTestSuites(testsuites) ;
    unitTree_.removeAll() ;
    java.util.List suites = testsuites.getSuites() ;
    for (int i = 0; i < suites.size(); i++) {
      WebUnitSuite suite = (WebUnitSuite) suites.get(i) ;
      final TreeItem suiteItem  =  new ExoTreeItem(unitTree_, SWT.NULL);
      suiteItem.setText(suite.getName()) ;
      suiteItem.setData(suite) ;
      //suiteItem.addListener(new SuiteTestSelecttionListener()) ;
      java.util.List units = suite.getWebUnits() ;
      for (int j = 0; j < units.size() ; j++) {
        WebUnit unit = (WebUnit) units.get(j) ;
        final TreeItem unitItem =  new ExoTreeItem(suiteItem,SWT.NULL);
        unitItem.setText(unit.getName());
        unitItem.setData(unit) ;
      }
    }
  }
  
  public void  updateUnitTreeLabelStatus() {
    TreeItem[] suiteItem = unitTree_.getItems() ;
    for(int i = 0 ; i < suiteItem.length; i++) {
      WebUnitSuite suite = (WebUnitSuite)suiteItem[i].getData() ;
      if(suite.getStatus() == WebUnitSuite.IGNORE_STATUS) {
        suiteItem[i].setText("[I] " + suite.getName()) ;
        continue;
      } else if(suite.getStatus() == WebUnitSuite.OK_STATUS) {
        suiteItem[i].setText("[V] " + suite.getName()) ;
      } else if(suite.getStatus() == WebUnitSuite.NOT_RUN_STATUS) {
        suiteItem[i].setText("[N] " + suite.getName()) ;
      } else {
        suiteItem[i].setText("[X] " + suite.getName()) ;
      }
      TreeItem[] unitItem = suiteItem[i].getItems() ;
      for(int j = 0 ; j < unitItem.length; j++) {
        WebUnit unit = (WebUnit) unitItem[j].getData() ;
        WebUnitMonitor  monitor = unit.getMonitor() ;
        if(suite.getStatus() == WebUnitSuite.IGNORE_STATUS) {
          unitItem[j].setText("[I]" +unit.getName()) ;
        } else {
          if(monitor.getCounter() <= 0) {
            unitItem[j].setText("[N] " + unit.getName()) ;
          } else if(monitor.getErrorCounter() > 0) {
            unitItem[j].setText("[X] " + unit.getName()) ;
          } else { 
            unitItem[j].setText("[V] " + unit.getName()) ;
          }
        }
      }
    }
  }
  
  public void  clearUnitTreeLabelStatus() {
    TreeItem[] suiteItem = unitTree_.getItems() ;
    for(int i = 0 ; i < suiteItem.length; i++) {
      WebUnitSuite suite = (WebUnitSuite)suiteItem[i].getData() ;
      if(suite.getStatus() == WebUnitSuite.IGNORE_STATUS) {
        suiteItem[i].setText("[I]" + suite.getName()) ;
      } else {
        suiteItem[i].setText("[N]" + suite.getName()) ;
      }
      TreeItem[] unitItem = suiteItem[i].getItems() ;
      for(int j = 0 ; j < unitItem.length; j++) {
        WebUnit unit = (WebUnit) unitItem[j].getData() ;
        if(suite.getStatus() == WebUnitSuite.IGNORE_STATUS) {
          unitItem[j].setText("[I]" +unit.getName()) ;
        } else {
          unitItem[j].setText("[N]" + unit.getName()) ;
        }
      }
    }
  }
  
  class TestItemSelecttionListener extends SelectionAdapter {
    public void widgetSelected(SelectionEvent e) {
      TreeItem item =   unitTree_.getSelection()[0];
      Object data = item.getData() ;
      ViewController controller = (ViewController) getParent() ;
      controller.getResultView().showBrowser()  ;
      if(data instanceof WebUnitSuite) {
        WebUnitSuite suite = (WebUnitSuite) data ;
        String html =  suite.getHtmlTextSummary()  ;
        controller.getResultView().getBrowserView().getBrowser().setText(html) ;
      } else {
        WebUnit unit = (WebUnit) data ;
        String html =  unit.getUnitSummaryInXHTML() ;
        controller.getResultView().getBrowserView().getBrowser().setText(html) ;
      }
    }
  }
}