/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.eclipse.webunit.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.*;
import org.exoplatform.test.web.ExoWebClient;
import org.exoplatform.test.web.TestSuites;
import org.exoplatform.test.web.WebLoadRunner;
import org.exoplatform.eclipse.webunit.widgets.ExoText;
/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Oct 7, 2004
 * @version $Id: UnitTestControllerView.java,v 1.2 2004/10/09 19:08:59 tuan08 Exp $
 */
public class UnitTestControllerView extends Composite {
  private TabFolder tabFolder_ ;
  
  private Text numberOfSessions_ ;
  private Text numberOfUsers_ ;
  private Text loadTestStatus_ ;
  
  private Text unitTestDescription_ ;
  
  private Text url_ ;
  private Text validateUnit_ ;
  private Text validateXHTML_ ;
  private Text interval_ ;
  
  private TestSuites suites_ ;
  private UnitTestPlayer unitTestPlayer_ ;
  private WebLoadRunner unitTestLoadRunner_ ;
  private Thread unitTestThread_ ;
  
  public UnitTestControllerView(Composite parent) {
    super(parent, SWT.NONE);
    GridLayout gridLayout = new GridLayout();
    gridLayout.marginHeight = 0 ;   gridLayout.marginWidth = 0  ;
    gridLayout.verticalSpacing = 0 ; gridLayout.horizontalSpacing = 0 ;
    setLayout(gridLayout);
    GridData gridData = 
      new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
    tabFolder_ = new TabFolder(this, SWT.NULL) ;
    tabFolder_.setLayoutData(gridData) ;
    
    //==================================================================
    TabItem testTab = new TabItem(tabFolder_, SWT.NULL );
    testTab.setText("Test") ;
    Composite testControl = new Composite(tabFolder_, SWT.NULL) ;
    gridLayout = new GridLayout();
    gridLayout.numColumns = 4;
    //gridLayout.marginHeight = 0 ;   gridLayout.marginWidth = 0  ;
    //gridLayout.verticalSpacing = 0 ;  gridLayout.horizontalSpacing = 0 ;
    testControl.setLayout(gridLayout);
    gridData = 
      new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
    testControl.setLayoutData(gridData) ;
    
    Button reset = new Button(testControl, SWT.PUSH);
    reset.setText("reset") ;  
    reset.addSelectionListener(new ResetSelectionListener()) ;
    
    Button playNext = new Button(testControl, SWT.PUSH);
    playNext.setText(">>") ;  
    playNext.addSelectionListener(new PlayNextSelectionListener()) ;
    
    Button playButton = new Button(testControl, SWT.PUSH);
    playButton.setText("play") ;  
    playButton.addSelectionListener(new PlaySelectionListener()) ;
    
    Button stopButton = new Button(testControl, SWT.PUSH);
    stopButton.setText("stop") ;  
    stopButton.addSelectionListener(new StopSelectionListener()) ; 
    
    unitTestDescription_ = new ExoText(testControl,  SWT.WRAP | SWT.MULTI | SWT.NONE);
    unitTestDescription_.setEditable(false) ;
    unitTestDescription_.setText("") ;
    gridData = 
      new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
    gridData.horizontalSpan = 4 ;
    unitTestDescription_.setLayoutData(gridData) ;
    
    testTab.setControl(testControl) ;
    //===================================================================
    TabItem loadTestTab = new TabItem(tabFolder_, SWT.NULL );
    loadTestTab.setText("Load Test") ;
    
    Composite loadTestControl = new Composite(tabFolder_, SWT.NULL) ;
    gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    loadTestControl.setLayout(gridLayout);
    gridData = 
      new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
    loadTestControl.setLayoutData(gridData) ;
    
    Label taskLabel = new Label(loadTestControl, SWT.NONE);
    taskLabel.setText("N# Of Sessions") ;  
    
    numberOfSessions_ = new Text(loadTestControl,  SWT.NONE | SWT.SINGLE);
    numberOfSessions_.setEditable(true) ;
    numberOfSessions_.setText("1") ;
    numberOfSessions_.setLayoutData(new GridData(GridData.FILL_HORIZONTAL)) ;
    
    Label userLabel = new Label(loadTestControl, SWT.NONE);
    userLabel.setText("N# Of Users") ;  
    
    numberOfUsers_ = new Text(loadTestControl,  SWT.NONE | SWT.SINGLE);
    numberOfUsers_.setEditable(true) ;
    numberOfUsers_.setText("1") ;
    numberOfUsers_.setLayoutData(new GridData(GridData.FILL_HORIZONTAL)) ;
    
    Composite buttonRow = new Composite(loadTestControl, SWT.NONE);
    gridData  = new GridData(GridData.HORIZONTAL_ALIGN_CENTER) ;
    gridData.horizontalSpan = 2 ;
    buttonRow.setLayoutData(gridData) ;
    buttonRow.setLayout(new RowLayout()) ;
    
    Button runButton = new Button(buttonRow, SWT.PUSH);
    runButton.setText("Run") ;  
    runButton.addSelectionListener(new RunLoadTestSelectionListener()) ;
    
    Button cancelButton = new Button(buttonRow, SWT.PUSH);
    cancelButton.setText("Stop") ;  
    cancelButton.addSelectionListener(new StopLoadTestSelectionListener()) ; 
    
    gridData  = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING) ;
    gridData.horizontalSpan = 2 ;
    loadTestStatus_ = new Text(loadTestControl, SWT.WRAP | SWT.MULTI | SWT.NONE);
    loadTestStatus_.setEditable(false) ;
    loadTestStatus_.setText("Done") ;
    gridData = 
      new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
    gridData.horizontalSpan = 2 ;
    loadTestStatus_.setLayoutData(gridData) ;
    
    loadTestTab.setControl(loadTestControl) ;
    //===================================================================
    TabItem configTab = new TabItem(tabFolder_, SWT.NULL );
    configTab.setText("Config") ;
    Composite configControl = new Composite(tabFolder_, SWT.NULL) ;
    gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    configControl.setLayout(gridLayout);
    gridData = 
      new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
    configControl.setLayoutData(gridData) ;
    
    Label urlLabel = new Label(configControl, SWT.NONE);
    urlLabel.setText("URL") ;  
    
    url_ = new Text(configControl,  SWT.NONE | SWT.SINGLE);
    url_.setEditable(true) ;
    url_.setText("http://localhost:8080/portal") ;
    url_.setLayoutData(new GridData(GridData.FILL_HORIZONTAL)) ;
    
    Label validateUnitLabel = new Label(configControl, SWT.NONE);
    validateUnitLabel.setText("Validate Unit") ;  
    
    validateUnit_ = new Text(configControl,  SWT.NONE | SWT.SINGLE);
    validateUnit_.setEditable(true) ;
    validateUnit_.setText("true") ;
    validateUnit_.setLayoutData(new GridData(GridData.FILL_HORIZONTAL)) ;
    
    Label validateXhtmlLabel = new Label(configControl, SWT.NONE);
    validateXhtmlLabel.setText("Validate XHTML") ;  
    
    validateXHTML_ = new Text(configControl,  SWT.NONE | SWT.SINGLE);
    validateXHTML_.setEditable(true) ;
    validateXHTML_.setText("true") ;
    validateXHTML_.setLayoutData(new GridData(GridData.FILL_HORIZONTAL)) ;
    
    Label intervalLabel = new Label(configControl, SWT.NONE);
    intervalLabel.setText("Interval") ;  
    
    interval_ = new Text(configControl,  SWT.NONE | SWT.SINGLE);
    interval_.setEditable(true) ;
    interval_.setText("1000") ;
    interval_.setLayoutData(new GridData(GridData.FILL_HORIZONTAL)) ;
    
    Button saveConfigButton = new Button(configControl, SWT.PUSH);
    saveConfigButton.setText("Save") ;  
    saveConfigButton.addSelectionListener(new SaveConfigSelectionListener()) ;
    gridData =    new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
    gridData.horizontalSpan = 2 ;
    saveConfigButton.setLayoutData(gridData) ;
    
    configTab.setControl(configControl) ;
  }
  
  public Text getUnitTestDescriptionWidget() { return unitTestDescription_ ; }
  
  public TestSuites getTestSuites() {  return suites_ ; }
  
  public void setTestSuites(TestSuites suites) {
    suites_ = suites ;
    ViewController controller = ViewController.findController(this) ;
    unitTestPlayer_ = new UnitTestPlayer(controller, suites) ;
  }
  
  private boolean canRunTest() {
    if(unitTestPlayer_ == null) {
      ViewController controller = ViewController.findController(getParent()) ;
      controller.showError(null, "The test suites is not initialized") ;
      return  false;
    }
    if(unitTestThread_ != null && unitTestThread_.isAlive())  {
      ViewController controller = ViewController.findController(getParent()) ;
      controller.showError(null, "Test is running ...................") ;
      return false  ;
    }
    if(unitTestLoadRunner_ != null && unitTestLoadRunner_.isRunning())  {
      ViewController controller = ViewController.findController(getParent()) ;
      controller.showError(null, "Load test is running ...................") ;
      return false ;
    }
    return true ;
  }
  
  class PlaySelectionListener extends SelectionAdapter{
    public void widgetSelected(SelectionEvent evt) {
      if(!canRunTest()) return ;
      ViewController controller = ViewController.findController(getParent()) ;
      controller.getUnitTestView().clearUnitTreeLabelStatus() ;
      unitTestThread_ = new Thread(unitTestPlayer_) ;
      unitTestThread_.start() ;
      //Display display = getDisplay() ;
      //display.asyncExec(unitTestPlayer_) ;
    }
  }
  
  class StopSelectionListener extends SelectionAdapter{
    public void widgetSelected(SelectionEvent evt) {
      if(unitTestPlayer_ == null) return ;
      if(unitTestThread_ == null || !unitTestThread_.isAlive()) return ;
      unitTestThread_.interrupt() ;
      unitTestThread_ = null ;
      ViewController controller = ViewController.findController(getParent()) ;
      controller.getUnitTestView().updateUnitTreeLabelStatus() ;
      unitTestPlayer_.reset() ;
    }
  }
  
  class ResetSelectionListener extends SelectionAdapter{
    public void widgetSelected(SelectionEvent evt) {
      if(unitTestPlayer_ == null) return ;
      if(unitTestThread_ != null && unitTestThread_.isAlive())  {
        unitTestThread_.interrupt() ;
        unitTestThread_ = null ;
      }
      unitTestPlayer_.reset() ;
      ViewController controller = ViewController.findController(getParent()) ;
      controller.getUnitTestView().clearUnitTreeLabelStatus() ;
      controller.getResultView().getHtmlEditorView().setText("") ;
      controller.getResultView().getErrorView().setText("") ;
    }
  }
  
  class PlayNextSelectionListener extends SelectionAdapter{
    public void widgetSelected(SelectionEvent evt) {
      if(!canRunTest()) return ;
      unitTestPlayer_.runNextUnit() ;
      unitTestDescription_.setText(unitTestPlayer_.getCurrentUnitTestDescription()) ;
      ViewController controller = ViewController.findController(getParent()) ;
      controller.getUnitTestView().updateUnitTreeLabelStatus() ;
      controller.getResultView().getHtmlEditorView().setText(unitTestPlayer_.getCurrentResponseText()) ;
    }
  }
  
  //===============================================================================
  class RunLoadTestSelectionListener extends SelectionAdapter{
    public void widgetSelected(SelectionEvent evt) {
      if(!canRunTest()) return ;
      unitTestPlayer_.reset() ;
      ExoWebClient webClient = unitTestPlayer_.getWebClient() ;
      int user = 1 ;
      try {
        user = Integer.parseInt(numberOfUsers_.getText()) ;
      } catch (Exception ex) { 
        numberOfUsers_.setText("1") ;
      }
      int task = 1 ;
      try {
        task = Integer.parseInt(numberOfSessions_.getText()) ;
      } catch (Exception ex) { 
        numberOfSessions_.setText("1") ;
      }
      unitTestLoadRunner_ = new WebLoadRunner(user, task) ;
      unitTestLoadRunner_.run(webClient) ;
      Display display = getDisplay() ;
      display.asyncExec(new UpdateLoadTestStatus(unitTestLoadRunner_)) ;
    }
  }
  
  class StopLoadTestSelectionListener extends SelectionAdapter{
    public void widgetSelected(SelectionEvent evt) {
      if(unitTestPlayer_ == null) return ;
      if(unitTestLoadRunner_ != null) {
        unitTestLoadRunner_.stop() ;
        ViewController controller = ViewController.findController(getParent()) ;
        controller.getUnitTestView().updateUnitTreeLabelStatus() ;
      }
    }
  }
  
  class UpdateLoadTestStatus  implements Runnable {
    private WebLoadRunner runner_ ;
    public UpdateLoadTestStatus(WebLoadRunner runner) {
      runner_ = runner ;
    }
    
    public void run() {
      try {
        while(runner_.isRunning()) {
          int taskLeft = runner_.getUnfinishedTaskCounter() ;
          loadTestStatus_.setText("There are " + taskLeft + " unfinished tasks") ;
          Thread.sleep(1500) ;
        }
      } catch (Exception ex) {
        ViewController controller = ViewController.findController(getParent()) ;
        controller.showError(ex , "Error in the monitor load test thread") ;
      }
      loadTestStatus_.setText("Done..............") ;
    }
  }
  
  //===============================================================================
  
  class SaveConfigSelectionListener extends SelectionAdapter{
    public void widgetSelected(SelectionEvent evt) {
      if(unitTestPlayer_ == null) return ;
      ExoWebClient webClient = unitTestPlayer_.getWebClient() ;
      webClient.setHomePageURL(url_.getText()) ;
      webClient.setValidateWebUnit("true".equals(validateUnit_.getText().toLowerCase())) ;
      webClient.setValidateWebUnit("true".equals(validateXHTML_.getText().toLowerCase())) ;
      long interval = 1000 ;
      try {
        interval = Long.parseLong(interval_.getText()) ;
      } catch (Exception ex) {
        interval_.setText("1000") ;
      }
      webClient.setInterval(interval) ;
      
    }
  }
}