/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.addrbook.component;


import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Cell;
import org.exoplatform.faces.core.component.model.ComponentCell;
import org.exoplatform.faces.core.component.model.FormButton;
import org.exoplatform.faces.core.component.model.HeaderRow;
import org.exoplatform.faces.core.component.model.LabelCell;
import org.exoplatform.faces.core.component.model.ListComponentCell;
import org.exoplatform.faces.core.component.model.Row;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.addrbook.AddrBookService;
import org.exoplatform.services.addrbook.model.AddrGroup;

/**
 * Jun 22, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIFileAddrBookEditor.java,v 1.3 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIAddrGroupForm extends UISimpleForm {
	static final public String SAVE_ACTION = "save";
	static final public String CANCEL_ACTION = "cancel";
	private AddrBookService service_;
	private UIStringInput name_;
	private UIStringInput label_;
	private UIStringInput description_;

	private AddrGroup addrGroup_;
	private AddrGroup parent_;

	public UIAddrGroupForm(AddrBookService service) throws Exception {
		super("addrGroupForm", "post", null);
		setId("UIAddrGroupForm");
		setClazz("UIAddrGroupForm");
		service_ = service;
		name_ = new UIStringInput("name", "");
		label_ = new UIStringInput("label", "");
		description_ = new UIStringInput("description", "");

		add(
			new HeaderRow().add(
				new Cell("#{UIAddrGroupForm.header.name}").addColspan("2")));
		add(
			new Row().add(new LabelCell("#{UIAddrGroupForm.label.name}")).add(
				new ComponentCell(this, name_)));
		add(
			new Row().add(new LabelCell("#{UIAddrGroupForm.label.label}")).add(
				new ComponentCell(this, label_)));
		add(
			new Row().add(
				new LabelCell("#{UIAddrGroupForm.label.description}")).add(
				new ComponentCell(this, description_)));
		add(
			new Row().add(
				new ListComponentCell()
					.add(
						new FormButton(
							"#{UIAddrGroupForm.button.save}",
							SAVE_ACTION))
					.add(
						new FormButton(
							"#{UIAddrGroupForm.button.cancel}",
							CANCEL_ACTION))
					.addColspan("2")
					.addAlign("center")));
		addActionListener(CancelActionListener.class, CANCEL_ACTION);
		addActionListener(SaveActionListener.class, SAVE_ACTION);

	}

	public void setNewFormData(AddrGroup parent) throws Exception {
		parent_ = parent;
		addrGroup_ = null;
		name_.setText("");
		label_.setText("");
		description_.setText("");
	}

	public void setEditFormData(AddrGroup addrGroup) throws Exception {
		addrGroup_ = addrGroup;
		parent_ = null;
		name_.setText(addrGroup.getName());
		label_.setText(addrGroup.getLabel());
		description_.setText(addrGroup.getDescription());

	}

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrGroupForm uiForm = (UIAddrGroupForm) event.getComponent();
                        uiForm.setRenderedSibling(UIAddrBookExplorer.class) ;
		}
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIAddrGroupForm uiForm = (UIAddrGroupForm) event.getComponent();
			UIAddrBookExplorer uiAddrBookExplorer =
				(UIAddrBookExplorer) uiForm.getSibling(UIAddrBookExplorer.class);
			AddrGroup newAddrGroup ;
			if (uiForm.addrGroup_ == null) {
				uiForm.addrGroup_ = uiForm.service_.createAddrGroupInstance();
				uiForm.addrGroup_.setName(uiForm.name_.getValue());
				uiForm.addrGroup_.setLabel(uiForm.label_.getValue());
				uiForm.addrGroup_.setDescription(uiForm.description_.getValue());
				if (uiForm.parent_ == null) {
					newAddrGroup = uiForm.service_.createAddrGroup(uiForm.addrGroup_);
				} else {
					newAddrGroup = uiForm.service_.addChild(uiForm.parent_, uiForm.addrGroup_);
				}

			} else {
				uiForm.addrGroup_.setLabel(uiForm.label_.getValue());
				uiForm.addrGroup_.setDescription(uiForm.description_.getValue());
				uiForm.service_.updateAddrGroup(uiForm.addrGroup_);
			}

			uiAddrBookExplorer.update();
			uiForm.setRenderedSibling(UIAddrBookExplorer.class) ;
			//uiAddrBookExplorer.setRendered(true);
		}
	}

}