/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.bookmark.component;


import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Cell;
import org.exoplatform.faces.core.component.model.ComponentCell;
import org.exoplatform.faces.core.component.model.FormButton;
import org.exoplatform.faces.core.component.model.HeaderRow;
import org.exoplatform.faces.core.component.model.LabelCell;
import org.exoplatform.faces.core.component.model.ListComponentCell;
import org.exoplatform.faces.core.component.model.Row;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.bookmark.BookmarkService;
import org.exoplatform.services.bookmark.model.BookmarkCategory;

/**
 * Jun 22, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIFileContentEditor.java,v 1.3 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIBookmarkCategoryForm extends UISimpleForm {
	static final public String SAVE_ACTION = "save";
	static final public String CANCEL_ACTION = "cancel";
	private BookmarkService service_;
	private UIStringInput name_;
	private UIStringInput label_;
	private UIStringInput description_;

	private BookmarkCategory category_;
	private BookmarkCategory parent_;

	public UIBookmarkCategoryForm(BookmarkService service) throws Exception {
		super("categoryForm", "post", null);
		setId("UIBookmarkCategoryForm");
		setClazz("UIBookmarkCategoryForm");
		service_ = service;
		name_ = new UIStringInput("name", "");
		label_ = new UIStringInput("label", "");
		description_ = new UIStringInput("description", "");

		add(
			new HeaderRow().add(
				new Cell("#{UIBookmarkCategoryForm.header.name}").addColspan("2")));
		add(
			new Row().add(new LabelCell("#{UIBookmarkCategoryForm.label.name}")).add(
				new ComponentCell(this, name_)));
		add(
			new Row().add(new LabelCell("#{UIBookmarkCategoryForm.label.label}")).add(
				new ComponentCell(this, label_)));
		add(
			new Row().add(
				new LabelCell("#{UIBookmarkCategoryForm.label.description}")).add(
				new ComponentCell(this, description_)));
		add(
			new Row().add(
				new ListComponentCell()
					.add(
						new FormButton(
							"#{UIBookmarkCategoryForm.button.save}",
							SAVE_ACTION))
					.add(
						new FormButton(
							"#{UIBookmarkCategoryForm.button.cancel}",
							CANCEL_ACTION))
					.addColspan("2")
					.addAlign("center")));
		addActionListener(CancelActionListener.class, CANCEL_ACTION);
		addActionListener(SaveActionListener.class, SAVE_ACTION);

	}

	public void setNewFormData(BookmarkCategory parent) throws Exception {
		parent_ = parent;
		category_ = null;
		name_.setText("");
		label_.setText("");
		description_.setText("");
	}

	public void setEditFormData(BookmarkCategory category) throws Exception {
		category_ = category;
		parent_ = null;
		name_.setText(category.getName());
		label_.setText(category.getLabel());
		description_.setText(category.getDescription());

	}

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarkCategoryForm uiForm = (UIBookmarkCategoryForm) event.getComponent();
			UIBookmarkPortlet bookmarkPortlet =
				(UIBookmarkPortlet) uiForm.getParent();

			bookmarkPortlet.setRenderedComponent(UIBookmarkExplorer.class);
		}
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIBookmarkCategoryForm uiForm = (UIBookmarkCategoryForm) event.getComponent();
			UIBookmarkExplorer uiBookmarkExplorer =
				(UIBookmarkExplorer) uiForm.getSibling(UIBookmarkExplorer.class);
			BookmarkCategory newBookmarkCategory ;
			if (uiForm.category_ == null) {
				uiForm.category_ = uiForm.service_.createBookmarkCategoryInstance();
				uiForm.category_.setName(uiForm.name_.getValue());
				uiForm.category_.setLabel(uiForm.label_.getValue());
				uiForm.category_.setDescription(uiForm.description_.getValue());
				if (uiForm.parent_ == null) {
					newBookmarkCategory = uiForm.service_.createBookmarkCategory(uiForm.category_);
				} else {
					newBookmarkCategory = uiForm.service_.addChild(uiForm.parent_, uiForm.category_);
				}

			} else {
				uiForm.category_.setLabel(uiForm.label_.getValue());
				uiForm.category_.setDescription(uiForm.description_.getValue());
				uiForm.service_.updateBookmarkCategory(uiForm.category_);
			}

			uiBookmarkExplorer.update();
			uiForm.setRenderedSibling(UIBookmarkExplorer.class) ;
			//uiContentExplorer.setRendered(true);
		}
	}

}