/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.publish.component;

import java.util.ArrayList;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIGrid;
import org.exoplatform.faces.core.component.model.*;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.publish.PublishService;
import org.exoplatform.services.organization.Query;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.publish.PublishService;
import org.exoplatform.services.publish.model.Content;
import org.exoplatform.services.publish.model.Section;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIListUser.java,v 1.18 2004/10/22 14:23:30 tuan08 Exp $
 */
public class UIContentsList extends UIGrid {
	public static final PageList EMPTY_LIST =
		new ObjectPageList(new ArrayList(), 20);
	private static Parameter[] ADD = { new Parameter(ACTION, "add")};
	private static Parameter[] EDIT = { new Parameter(ACTION, "edit")};
	private static Parameter[] DELETE = { new Parameter(ACTION, "delete")};

	public final static String CONTENT_ID = "contentId";
	public final static String SECTION_ID = "sectionId";
	private UIPageListIterator uiPageIterator_;

	private PublishService service_;
	private Section currentSection_;

	public UIContentsList(PublishService service) throws Exception {
		setId("UIContentsList");
		//setClazz("UIContentsList");
		service_ = service;

		uiPageIterator_ = new UIPageListIterator(new ContentDataHandler());

		add(
			new Rows(uiPageIterator_.getPageListDataHandler(), "even", "odd")
				.add(new Column("#{UIContentsList.header.title}", "title"))
				.add(
					new Column(
						"#{UIContentsList.header.createddate}",
						"createdDate"))
				.add(
					new Column(
						"#{UIContentsList.header.createdby}",
						"createdBy"))
				.add(
					new ActionColumn(
						"#{UIContentsList.header.action}",
						"contentId")
						.add(new Button("#{UIContentsList.button.edit}", EDIT))
						.add(
							new Button(
								"#{UIContentsList.button.delete}",
								DELETE))));
		add(
			new Row().add(
				new ComponentCell(this, uiPageIterator_).addColspan(
					"4").addStyle(
					"text-align: right")));
		add(
			new Row().add(
				new ListComponentCell()
					.add(new Button("#{UIContentsList.button.add}", ADD))
					.addColspan("4")
					.addAlign("center")));

		addActionListener(AddActionListener.class, "add");
		addActionListener(EditActionListener.class, "edit");
		addActionListener(DeleteActionListener.class, "delete");
		refresh();
	}
	public UIPageListIterator getUIPageIterator() {
		return uiPageIterator_;
	}

	public void setContentList(PageList list) throws Exception {
		uiPageIterator_.setPageList(list);
	}
	public void setSection(Section currentSection) {
		currentSection_ = currentSection;
	}

	public void refresh() throws Exception {
		if (currentSection_ == null) {
			uiPageIterator_.setPageList(service_.findContentsBySection("/"));
		} else {
			uiPageIterator_.setPageList(
				service_.findContentsBySection(currentSection_.getId()));
		}
	}

	static public class ContentDataHandler extends PageListDataHandler {
		private Content content_;

		public String getData(String fieldName) {
			if ("contentId".equals(fieldName))
				return content_.getContentId();
			if ("title".equals(fieldName))
				return content_.getTitle();
			if ("state".equals(fieldName))
				return content_.getState().toString();
			if ("createdDate".equals(fieldName))
				return content_.getCreatedDate().toGMTString();
			if ("createdBy".equals(fieldName))
				return content_.getCreatedBy();
			return null;
		}

		public void setCurrentObject(Object o) {
			content_ = (Content) o;
		}
	}

	static public class AddActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentsList uiContentsList =
				(UIContentsList) event.getComponent();
			//String sectionId = event.getParameter(OBJECTID);
			UIContentExplorer uiExplorer =
				(UIContentExplorer) uiContentsList.getParent();
			UIContentForm uiContentForm =
				(UIContentForm) uiExplorer.getSibling(UIContentForm.class);
			if(uiContentsList.currentSection_!=null){	
			
			uiContentForm.setNewContentInfo(
				uiContentsList.currentSection_.getId());
			}else {
				uiContentForm.setNewContentInfo("/");
			}
			uiExplorer.setRenderedSibling(UIContentForm.class);
		}
	}

	static public class EditActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentsList uiContentsList =
				(UIContentsList) event.getComponent();
			String contentId = event.getParameter(OBJECTID);
			UIContentExplorer uiExplorer =
				(UIContentExplorer) uiContentsList.getParent();
			UIContentForm uiContentForm =
				(UIContentForm) uiExplorer.getSibling(UIContentForm.class);
			uiContentForm.setEditContentInfo(contentId);
			uiExplorer.setRenderedSibling(UIContentForm.class);
		}
	}

	static public class DeleteActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UIContentsList uiContentsList =
				(UIContentsList) event.getComponent();
			String contentId = event.getParameter(OBJECTID);
			uiContentsList.service_.removeContent(contentId);
			uiContentsList.refresh();
		}
	}
}