/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.publish.component;


import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Cell;
import org.exoplatform.faces.core.component.model.ComponentCell;
import org.exoplatform.faces.core.component.model.FormButton;
import org.exoplatform.faces.core.component.model.HeaderRow;
import org.exoplatform.faces.core.component.model.LabelCell;
import org.exoplatform.faces.core.component.model.ListComponentCell;
import org.exoplatform.faces.core.component.model.Row;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.publish.PublishService;
import org.exoplatform.services.publish.model.Section;

/**
 * Jun 22, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIFileContentEditor.java,v 1.3 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UISectionForm extends UISimpleForm {
	static final public String SAVE_ACTION = "save";
	static final public String CANCEL_ACTION = "cancel";
	private PublishService service_;
	private UIStringInput name_;
	private UIStringInput label_;
	private UIStringInput description_;

	private Section section_;
	private Section parent_;

	public UISectionForm(PublishService service) throws Exception {
		super("sectionForm", "post", null);
		setId("UISectionForm");
		//setClazz("UISectionForm");
		service_ = service;
		name_ = new UIStringInput("name", "");
		label_ = new UIStringInput("label", "");
		description_ = new UIStringInput("description", "");

		add(
			new HeaderRow().add(
				new Cell("#{UISectionForm.header.name}").addColspan("2")));
		add(
			new Row().add(new LabelCell("#{UISectionForm.label.name}")).add(
				new ComponentCell(this, name_)));
		add(
			new Row().add(new LabelCell("#{UISectionForm.label.label}")).add(
				new ComponentCell(this, label_)));
		add(
			new Row().add(
				new LabelCell("#{UISectionForm.label.description}")).add(
				new ComponentCell(this, description_)));
		add(
			new Row().add(
				new ListComponentCell()
					.add(
						new FormButton(
							"#{UISectionForm.button.save}",
							SAVE_ACTION))
					.add(
						new FormButton(
							"#{UISectionForm.button.cancel}",
							CANCEL_ACTION))
					.addColspan("2")
					.addAlign("center")));
		addActionListener(CancelActionListener.class, CANCEL_ACTION);
		addActionListener(SaveActionListener.class, SAVE_ACTION);

	}

	public void setNewFormData(Section parent) throws Exception {
		parent_ = parent;
		section_ = null;
		name_.setText("");
		label_.setText("");
		description_.setText("");
	}

	public void setEditFormData(Section section) throws Exception {
		section_ = section;
		parent_ = null;
		name_.setText(section.getName());
		label_.setText(section.getLabel());
		description_.setText(section.getDescription());

	}

	static public class CancelActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UISectionForm uiForm = (UISectionForm) event.getComponent();
			UIPublishPortlet publishPortlet =
				(UIPublishPortlet) uiForm.getParent();

			publishPortlet.setRenderedComponent(UIContentExplorer.class);
		}
	}

	static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UISectionForm uiForm = (UISectionForm) event.getComponent();
			UIContentExplorer uiContentExplorer =
				(UIContentExplorer) uiForm.getSibling(UIContentExplorer.class);
			Section newSection ;
			if (uiForm.section_ == null) {
				uiForm.section_ = uiForm.service_.createSectionInstance();
				uiForm.section_.setName(uiForm.name_.getValue());
				uiForm.section_.setLabel(uiForm.label_.getValue());
				uiForm.section_.setDescription(uiForm.description_.getValue());
				if (uiForm.parent_ == null) {
					newSection = uiForm.service_.createSection(uiForm.section_);
				} else {
					newSection = uiForm.service_.addChild(uiForm.parent_, uiForm.section_);
				}

			} else {
				uiForm.section_.setLabel(uiForm.label_.getValue());
				uiForm.section_.setDescription(uiForm.description_.getValue());
				uiForm.service_.updateSection(uiForm.section_);
			}

			uiContentExplorer.update();
			uiForm.setRenderedSibling(UIContentExplorer.class) ;
			//uiContentExplorer.setRendered(true);
		}
	}

}