package org.exoplatform.ria.component.browser ;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import org.exoplatform.ria.ResourceManager;

public class BrowserAddressToolbar extends JPanel {
  
  JButton stopBt_ = new JButton("", ResourceManager.getIcon("browser.button.stop.icon"));
  JButton refreshBt_ = new JButton("", ResourceManager.getIcon("browser.button.reload.icon"));
  JButton forwardBt_ = new JButton("", ResourceManager.getIcon("browser.button.forward.icon"));
  JButton backBt_ = new JButton("", ResourceManager.getIcon("browser.button.back.icon"));
  JButton goBt_ = new JButton("Go", ResourceManager.getIcon("browser.button.go.icon"));
  JTextField addressField_ = new JTextField() ;
  
  public BrowserAddressToolbar(ExoWebBrowser browser) throws Exception {
    setLayout(new BorderLayout());
    JPanel jAddressPanel = new JPanel();
    JLabel jAddressLabel = new JLabel();
    jAddressLabel.setBorder(BorderFactory.createEmptyBorder(0, 4, 0, 4));
    jAddressLabel.setToolTipText("");
    jAddressLabel.setText(" URL: ");
    goBt_.setBorder(BorderFactory.createCompoundBorder(
                        new EmptyBorder(0, 2, 0, 2),
                        new EtchedBorder()));
    goBt_.setToolTipText("Load the given URL");
    goBt_.addActionListener(new BrowserGoActionAdapter(browser));
    
    jAddressPanel.setLayout(new BorderLayout());
    
    addressField_.addActionListener(new BrowserAddressTextFieldActionAdapter(browser));
    
    backBt_.setToolTipText("Go back one page");
    backBt_.setHorizontalTextPosition(SwingConstants.TRAILING);
    backBt_.setEnabled(false);
    backBt_.addActionListener(new BrowserBackActionAdapter(browser));
    
    forwardBt_.setToolTipText("Go forward one page");
    forwardBt_.setEnabled(false);
    forwardBt_.addActionListener(new BrowserForwardActionAdapter(browser));
    
    refreshBt_.setToolTipText("Reload current page");
    refreshBt_.setEnabled(true);
    refreshBt_.addActionListener(new BrowserRefreshActionAdapter(browser));
    
    stopBt_.setToolTipText("Stop loading this page");
    stopBt_.setVerifyInputWhenFocusTarget(true);
    stopBt_.setEnabled(true);
    stopBt_.addActionListener(new BrowserStopActionAdapter(browser));
    
    jAddressPanel.add(jAddressLabel, BorderLayout.WEST);
    jAddressPanel.add(addressField_, BorderLayout.CENTER);
    jAddressPanel.add(goBt_, BorderLayout.EAST);
    jAddressPanel.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createEtchedBorder(),
        BorderFactory.createEmptyBorder(2, 0, 2, 0)));
    
    JToolBar jBrowserToolBar = new JToolBar();
    jBrowserToolBar.setFloatable(false);
    jBrowserToolBar.add(backBt_, null);
    jBrowserToolBar.add(forwardBt_, null);
    jBrowserToolBar.addSeparator();
    jBrowserToolBar.add(refreshBt_, null);
    jBrowserToolBar.add(stopBt_, null);
    jBrowserToolBar.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createEtchedBorder(),
        BorderFactory.createEmptyBorder(2, 2, 2, 0)));
    add(jAddressPanel, BorderLayout.CENTER);
    add(jBrowserToolBar, BorderLayout.WEST);
    setBorder(BorderFactory.createEmptyBorder(0, 0, 2, 0));
  }
  
  static class BrowserAddressTextFieldActionAdapter implements ActionListener {
    ExoWebBrowser browser;
    BrowserAddressTextFieldActionAdapter(ExoWebBrowser br) {  browser = br; }
    
    public void actionPerformed(ActionEvent e) {  browser.loadURL(); }
  }

  static class BrowserBackActionAdapter implements ActionListener {
    ExoWebBrowser browser;
    
    BrowserBackActionAdapter(ExoWebBrowser br) { browser = br;  }
    
    public void actionPerformed(ActionEvent e) {  browser.webBrowser.back(); }
  }
  
  static class BrowserForwardActionAdapter implements java.awt.event.ActionListener {
    ExoWebBrowser browser;
    
    BrowserForwardActionAdapter(ExoWebBrowser adaptee) { browser = adaptee;  }
    
    public void actionPerformed(ActionEvent e) {  browser.webBrowser.forward(); }
  }
  
  class BrowserRefreshActionAdapter implements ActionListener {
    ExoWebBrowser browser;
    
    BrowserRefreshActionAdapter(ExoWebBrowser adaptee) { this.browser = adaptee; }
    
    public void actionPerformed(ActionEvent e) {  browser.webBrowser.refresh(); }
  }
  
  class BrowserStopActionAdapter implements ActionListener {
    ExoWebBrowser browser;
    
    BrowserStopActionAdapter(ExoWebBrowser adaptee) {  this.browser = adaptee;  }
    
    public void actionPerformed(ActionEvent e) {  browser.webBrowser.stop(); }
  }
  
  class BrowserGoActionAdapter implements ActionListener {
    ExoWebBrowser browser;
    
    BrowserGoActionAdapter(ExoWebBrowser adaptee) { browser = adaptee;}
    
    public void actionPerformed(ActionEvent e) { browser.loadURL(); }
  }
}