package org.exoplatform.ria.component.browser ;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;

import org.jdesktop.jdic.browser.WebBrowser;

public class ExoWebBrowser extends JPanel {
  BrowserAddressToolbar addrToolbar_ = new BrowserAddressToolbar(this) ;
  BrowserStatusBar statusBar = new BrowserStatusBar();
  WebBrowser webBrowser;
  
  public ExoWebBrowser() throws Exception {
    setLayout(new BorderLayout());
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setPreferredSize(new Dimension(screenSize.width * 9 / 10, screenSize.height * 8 / 10)); 
    ToolTipManager.sharedInstance().setLightWeightPopupEnabled(false);
    
    statusBar.setBorder(BorderFactory.createEmptyBorder(2, 0, 0, 0));
    statusBar.descLabel_.setText("JDIC API Demo - Browser") ;
    
    webBrowser = new WebBrowser(new URL("http://java.net"));
    webBrowser.addWebBrowserListener(new ExoWebBrowserListener(this)) ;
    
    JTextArea text = new JTextArea("test") ;
    JList list = new JList() ;
    String[] data = {"test" , "test1", "test 2" } ; 
    list.setListData( data ) ;
    add(list, BorderLayout.WEST) ;
    add(addrToolbar_, BorderLayout.NORTH);
    add(webBrowser, BorderLayout.CENTER);
    add(statusBar, BorderLayout.SOUTH);
  }
  
  void updateStatusInfo(String statusMessage) {
    statusBar.statusLabel_.setText(statusMessage);
  }
  
  public void loadURL(String url) {
    if (url == null) {
      JOptionPane.showMessageDialog(this, "The given URL is NULL:","Warning", JOptionPane.WARNING_MESSAGE);
      return ;
    }
    URL curUrl = null;
    try {
      if (url.startsWith("http://") || url.startsWith("https://") || 
          url.startsWith("ftp://")  || url.startsWith("file://")) {
        curUrl = new URL(url);
      } else {
        curUrl = new URL("http://" + url);
      }
      webBrowser.setURL(curUrl);
      updateStatusInfo("Loading " + curUrl.toString() + " ......");
    } catch (MalformedURLException mue) {
      JOptionPane.showMessageDialog(this, "The given URL is not valid:" + url, "Warning",
                                    JOptionPane.WARNING_MESSAGE);
    }
  }
  
  public void loadURL() {
    String inputValue = addrToolbar_.addressField_.getText();
    loadURL(inputValue) ;
  }
  
  public static void main(String[] args) throws Exception {
    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    JFrame frame = new JFrame("JDIC API Demo - Browser");
    Container contentPane = frame.getContentPane();
    contentPane.setLayout(new GridLayout(1, 1));
    contentPane.add(new ExoWebBrowser());
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });
    frame.pack();
    frame.setVisible(true);
  }
}