/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.bookmark.hibernate;

import org.exoplatform.commons.exception.UniqueObjectException;

import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.idgenerator.IDGeneratorService;
import org.exoplatform.services.bookmark.*;
import org.exoplatform.services.bookmark.model.Bookmark;
import org.exoplatform.services.bookmark.model.BookmarkCategory;
import org.exoplatform.services.bookmark.impl.*;

import java.util.*;

import org.exoplatform.commons.utils.*;
import org.hibernate.Hibernate;
import org.hibernate.Session;
/**
 * Created by The eXo Platform SARL
 * Author : Daniel Summer
 *          danny_xcz@users.sourceforge.net
 * Date: Nov 29, 2004
 * Time: 3:22:54 PM
 */
public class BookmarkServiceImpl implements BookmarkService {
	private static final String queryBookmarksByBookmarkCategory =
		"from bookmark in class org.exoplatform.services.bookmark.impl.BookmarkImpl "
			+ "where bookmark.bookmarkCategoryId = ?";

	/*
	 public static final String queryBookmarksByAuthor =
		"from bookmark in class org.exoplatform.services.publish.impl.BookmarkImpl " +
		"where bookmark.createdBy = ?";
		*/

	public static final String queryFindBookmarkById =
		"from bookmark in class org.exoplatform.services.bookmark.impl.BookmarkImpl "
			+ "where bookmark.bookmarkId = ?";

	private static final String queryFindBookmarkCategoryById =
		"from category in class org.exoplatform.services.bookmark.impl.BookmarkCategoryImpl "
			+ "where category.id = ? ";

	private static final String queryFindRootBookmarkCategorys =
		"from category in class org.exoplatform.services.bookmark.impl.BookmarkCategoryImpl "
			+ "where category.parentId is null";

	private static final String queryFindBookmarkCategoryByParent =
		"from category in class org.exoplatform.services.bookmark.impl.BookmarkCategoryImpl "
			+ "where category.parentId = ? ";

	private static final String queryFindBookmarksByBookmarkCategory =
		"from bookmark in class org.exoplatform.services.bookmark.impl.BookmarkImpl "
			+ "where bookmark.bookmarkCategoryId = ? ";

	private HibernateService hService_;
	private IDGeneratorService idService_;
	static private String[] MAPPING =
		{
			"org/exoplatform/services/bookmark/impl/BookmarkImpl.hbm.xml",
			"org/exoplatform/services/bookmark/impl/BookmarkCategoryImpl.hbm.xml" };

	public BookmarkServiceImpl(	HibernateService hibernateService, IDGeneratorService idService) {
//		hibernateService.addMappingFiles(MAPPING);
//		hService_ = hibernateService;
//		idService_ = idService;
	}

	//------------------------ Bookmark -------------------------//
	public Bookmark createBookmarkInstance() {
		return new BookmarkImpl();
	}

	public Bookmark createBookmark(BookmarkCategory category, Bookmark bookmark)
		throws Exception {
		Date now = new Date();
		BookmarkImpl impl = (BookmarkImpl) bookmark;
		impl.setBookmarkId(idService_.generateStringID(impl));
		if(category!=null){
		impl.setBookmarkCategoryId(category.getId());
		} else {
			impl.setBookmarkCategoryId("/");
		}
		impl.setCreatedDate(now);
		impl.setModifiedDate(now);
		Session session = hService_.openSession();
		session.save(impl);
		session.flush();
		return bookmark;
	}

	public Bookmark findBookmarkById(String bookmarkId) throws Exception {
		//return (bookmark) hservice_.findOne(BookmarkImpl.class, bookmarkId) ;
		Session session = hService_.openSession();
		Bookmark bookmark =
			(Bookmark) hService_.findOne(
				session,
				queryFindBookmarkById,
				bookmarkId);
		return bookmark;
	}

	public void updateBookmark(Bookmark bookmark) throws Exception {
		Date now = new Date();
		BookmarkImpl impl = (BookmarkImpl) bookmark;
		impl.setModifiedDate(now);
		Session session = hService_.openSession();
		session.update(impl);
		session.flush();

	}

	public Bookmark removeBookmark(String bookmarkId) throws Exception {
		Session session = hService_.openSession();
		Bookmark bookmark = (Bookmark) session.get(BookmarkImpl.class, bookmarkId);
		session.delete(bookmark);
		session.flush();

		return bookmark;
	}

	public PageList findBookmarksByBookmarkCategory(String categoryId) throws Exception {
		String queryFindBookmarksInBookmarkCategory =
			"from c in class org.exoplatform.services.bookmark.impl.BookmarkImpl "
				+ "where c.bookmarkCategoryId =  '"
				+ categoryId
				+ "'";

		String countBookmarksInBookmarkCategory =
			"select count(c) "
				+ "from c in class org.exoplatform.services.bookmark.impl.BookmarkImpl "
				+ "where c.bookmarkCategoryId =  '"
				+ categoryId
				+ "'";

		return new DBObjectPageList(
			hService_,
			20,
			queryFindBookmarksInBookmarkCategory,
			countBookmarksInBookmarkCategory);

	}

	public List findBookmarksByBookmarkCategoryAndNumber(String categoryId, int num)
		throws Exception {
//		Session session = hService_.openSession();
//
//		StringBuffer b = new StringBuffer() ;
//		b.append("from c in class org.exoplatform.services.bookmark.impl.BookmarkImpl  ").
//			append("where c.bookmarkCategoryId = :categoryId ").
//			append("order by c.createdDate desc") ;
//		net.sf.hibernate.Query query = session.createQuery(b.toString());
//		query.setString("categoryId", categoryId);
//		query.setFirstResult(0);
//		query.setMaxResults(num);
//		return query.list();
    return null;
	}


	//--------------------------- BookmarkCategory ------------------------------------------//

	public BookmarkCategory createBookmarkCategoryInstance() {
		return new BookmarkCategoryImpl();
	}

	public BookmarkCategory createBookmarkCategory(BookmarkCategory category) throws Exception {
		Session session = hService_.openSession();
		String categoryId = "/" + category.getName();
		Object o = session.get(BookmarkCategoryImpl.class, categoryId);
		if (o != null) {
			Object[] args = { category.getName()};
			throw new UniqueObjectException(
				"BookmarkService.unique-category-exception",
				args);
		}

		session.save(category, categoryId);
		session.flush();

		return category;
	}

	public BookmarkCategory findBookmarkCategoryById(String categoryId) throws Exception {
		Session session = hService_.openSession();
		BookmarkCategory category =
			(BookmarkCategory) hService_.findOne(
				session,
				queryFindBookmarkCategoryById,
				categoryId);
		return category;
	}

	public BookmarkCategory addChild(BookmarkCategory parent, BookmarkCategory child) throws Exception {
		Session session = hService_.openSession();
		BookmarkCategory parentBookmarkCategory =
			(BookmarkCategory) session.get(BookmarkCategoryImpl.class, parent.getId());
		String categoryId = parentBookmarkCategory.getId() + "/" + child.getName();
		Object o = session.get(BookmarkCategoryImpl.class, categoryId);
		if (o != null) {
			Object[] args = { child.getName()};
			throw new UniqueObjectException(
				"BookmarkService.unique-category-exception",
				args);
		}
		BookmarkCategoryImpl childImpl = (BookmarkCategoryImpl) child;

		childImpl.setParentId(parentBookmarkCategory.getId());
		session.save(childImpl, categoryId);
		session.flush();
		return parentBookmarkCategory;

	}

	public void updateBookmarkCategory(BookmarkCategory category) throws Exception {
		Session session = hService_.openSession();
		session.update(category);
		session.flush();
	}

	public BookmarkCategory removeBookmarkCategory(BookmarkCategory category) throws Exception {
//		Session session = hService_.openSession();
//		session.delete(category);
//
//		/*======= delete category from parent ========*/
//		List categorys =
//			session.find(
//				queryFindBookmarkCategoryByParent,
//				category.getId(),
//				Hibernate.STRING);
//		//session.delete(queryFindBookmarkCategoryByParent, category.getId(), Hibernate.STRING );
//		Iterator i = categorys.iterator();
//		while (i.hasNext()) {
//			BookmarkCategory s = (BookmarkCategory) i.next();
//			removeBookmarkCategory(s);
//		}
//		/*==========================================*/
//		removeBookmarksOfBookmarkCategory(category, session);
//		session.flush();
		return category;
	}
	/*
	public BookmarkCategory removeBookmarkCategory(String categoryId) throws Exception{
		Session session = hService_.openSession();
		BookmarkCategory category = (BookmarkCategory) session.get(BookmarkCategoryImpl.class, categoryId);
		session.delete(category);
		session.flush();
		return category;
		
	}
	*/

	public Collection findBookmarkCategorysByParent(BookmarkCategory parent) throws Exception {
//		Session session = hService_.openSession();
//		if (parent == null) {
//			return session.find(queryFindRootBookmarkCategorys);
//		}
//		return session.find(
//			queryFindBookmarkCategoryByParent,
//			parent.getId(),
//			Hibernate.STRING);
    return null;
	}

	/*
		public Collection findBookmarkCategorysByParent(String parentId) throws Exception {
			Session session = hService_.openSession();
			if (parentId == null) {
				return  session.find( queryFindRootBookmarkCategorys );
	  	    }
			
			return  session.find( queryFindBookmarkCategoryByParent, parentId, Hibernate.STRING );
		}
	*/

	void removeBookmarksOfBookmarkCategory(BookmarkCategory category, Session session)
		throws Exception {
//		session.delete(
//			queryFindBookmarksByBookmarkCategory,
//			category.getId(),
//			Hibernate.STRING);
	}
}
