/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.       *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.bookmark.hibernate;

import java.util.Collection;

import junit.framework.TestCase;
import org.exoplatform.services.bookmark.*;
import org.exoplatform.services.bookmark.model.Bookmark;
import org.exoplatform.services.bookmark.model.BookmarkCategory;
import org.exoplatform.services.bookmark.impl.*;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.database.HibernateService;
//import org.exoplatform.services.idgenerator.IDGeneratorService;
//import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.hibernate.Session;
import org.hibernate.Transaction;

//import net.sf.hibernate.Session;
/**
 * @author Bob
 *
 */
 
public class TestBookmarkServiceImpl extends TestCase {

	//protected BookmarkCategory category;
	//protected Bookmark bookmark;
	private static BookmarkService service;
	private static HibernateService hservice_;
	
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		if (service == null) {
			PortalContainer manager = PortalContainer.getInstance();
			hservice_ =
				(HibernateService) manager.getComponentInstanceOfType(
					HibernateService.class);
			service =
				(BookmarkService) manager.getComponentInstanceOfType(
						BookmarkService.class);
		}
		
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
	    hservice_.closeSession();
	}

	public void testBookmarkCategory() throws Exception  {
		Session session = hservice_.openSession();
		Transaction tx = session.beginTransaction();
		BookmarkCategory categoryParent = service.createBookmarkCategoryInstance();
		categoryParent.setName("BookmarkCategoryParent");
	        categoryParent.setLabel("BookmarkCategoryParent");
		categoryParent.setDescription("BookmarkCategory Parent");
		session.save(categoryParent,"/BookmarkCategoryParent");
		tx.commit();
		assertEquals(((BookmarkCategoryImpl) categoryParent).getId(),"/BookmarkCategoryParent");
		Collection c = service.findBookmarkCategorysByParent(null);
		assertEquals(1, c.size());

		BookmarkCategory categoryChild = service.createBookmarkCategoryInstance();
		categoryChild.setName("BookmarkCategoryChild");
		categoryChild.setLabel("fas");
		categoryChild.setDescription("Child Parent");

		service.addChild(categoryParent, categoryChild);

		categoryParent = service.findBookmarkCategoryById(categoryParent.getId());
		assertEquals(categoryParent.getName(), "BookmarkCategoryParent");

		categoryChild = service.findBookmarkCategoryById(categoryChild.getId());
		assertEquals(categoryChild.getParentId(), categoryParent.getId());

		categoryChild.setDescription("Changed Description");
		service.updateBookmarkCategory(categoryChild);
		/*
		assertEquals(
			service.findBookmarkCategoryById(categoryChild.getId()).getLabel(),
			"BookmarkCategoryRenamed");
		*/

		service.removeBookmarkCategory(categoryChild);
		assertNotNull(service.findBookmarkCategoryById(categoryParent.getId()));

		service.addChild(categoryParent, categoryChild);
		Collection categorys = service.findBookmarkCategorysByParent(categoryParent);
		assertEquals(1, categorys.size());

		service.removeBookmarkCategory(categoryParent);
		assertNull(service.findBookmarkCategoryById(categoryParent.getId()));
		assertNull(service.findBookmarkCategoryById(categoryChild.getId()));
		
	}
	
	public void testBookmark() throws Exception{
		Session session = hservice_.openSession();
		String BOOKMARK = "test";
		Transaction tx = session.beginTransaction();
		Bookmark bookmark = service.createBookmarkInstance();
		bookmark.setName(BOOKMARK);
		bookmark.setURL("www.163.com");
		bookmark.setCreatedBy("zhu");
		
		//service.createBookmark(null,bookmark);
		
		
		BookmarkCategory category = service.createBookmarkCategoryInstance();
		category.setName("test category");
		category.setDescription("this is test");
		
		service.createBookmarkCategory(category);
		service.createBookmark(category,bookmark);
		tx.commit();
		
		Bookmark u = service.findBookmarkById(bookmark.getBookmarkId());
		assertEquals(u.getName(), "test");
		
		//bookmarks = null;
		//bookmarks = service.findBookmarksByAuthor("bob");
		//assertTrue("findBookmarksByAuthor:Expect 1 bookmark found ", bookmarks.getAvailable() >= 1);
		
		PageList bookmarks = null;
		bookmarks = service.findBookmarksByBookmarkCategory(category.getId());
		assertTrue("findBookmarksByBookmarkCategory:Expect 1 bookmark found ", bookmarks.getAvailable() >= 1);

		service.removeBookmark(bookmark.getBookmarkId());
		service.removeBookmarkCategory(category);
	}

}
