/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.css.test;

import java.io.InputStream;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.commons.logging.Log;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.css.scanner.CssScanner;
import org.exoplatform.services.css.scanner.CssScannerService;
import org.exoplatform.services.css.scanner.event.ScannerEvent;
import org.exoplatform.services.css.scanner.event.ScannerListener;
import org.exoplatform.services.css.scanner.model.Document;
import org.exoplatform.services.css.scanner.model.Property;
import org.exoplatform.services.css.scanner.model.Selector;
import org.exoplatform.services.log.LogService;
import org.exoplatform.test.BasicTestCase;

/**
 * @author Ove Ranheim (oranheim@yahoo.no)
 * @since Jun 29, 2004 5:41:13 PM
 */
public class TestCssScannerService extends BasicTestCase {

  static protected CssScannerService _service;
  static protected Log _log;

  public TestCssScannerService(String name) {
    super(name);
  }

  public void setUp() throws Exception {
    if (_service == null) {
      PortalContainer manager = PortalContainer.getInstance();
      LogService _logservice = (LogService) manager.getComponentInstanceOfType(LogService.class);
      _service = (CssScannerService) manager.getComponentInstanceOfType(CssScannerService.class);
      //_service = new CssScannerServiceImpl();
      _log = _logservice.getLog("org.exoplatform.services.css.test");
      _logservice.setLogLevel("org.exoplatform.services.css.test", LogService.DEBUG, true);
    }
  }

  public void tearDown() throws Exception {
  }

  public void _testSplit() throws Exception {

    String s = "background: url(\"/portal/icons/metal-tab/left-tab-10x31.jpg\") no-repeat;\n" +
        "width: 8px; height: 31px;\n" +
        "padding: 0px 0px 0px 0px;\n" +
        "}\n";

    String s2 = "background: url(\"/portal/icons/metal-tab/left-tab-10x31.jpg\") no-repeat;\n";

    Pattern p = Pattern.compile(".;");
    String[] a = p.split(s);
    for (int i = 0; i < a.length; i++) {
      _log.debug(a[i].trim());

    }

    p = Pattern.compile("(.+):(.*);");
    Matcher m = p.matcher(s2);
    _log.debug(m.group(1).trim());
    _log.debug(m.group(2).trim());


  }

  public void testScanner() throws Exception {
    //if (true) return;

    URL uri = new URL("file:./src/test/org/exoplatform/services/css/test/test-portlet.css");
    InputStream stream = uri.openStream();

    System.out.println(_service);
    CssScanner scanner = _service.makeScanner();
    TestScannerListener listener = new TestScannerListener();
    scanner.addListener(listener);
    scanner.setSource(stream);
    Document document = scanner.parse();
    stream.close();

    _log.debug("\n" + listener.buf.toString());
    Selector selector = document.resolveSelector(".exo-content-portlet .UIContentHtmlEditorForm .htmlarea");
    assertEquals(selector.getName(), ".htmlarea");
  }


  // -----------------------------------------------------------------------------------------------------------------------

  public class TestScannerListener implements ScannerListener {

    public StringBuffer buf = new StringBuffer();

    /*
     * @see exo.services.portal.sac.event.ScannerListener#matchedRule(exo.services.portal.sac.event.ScannerEvent)
     */
    public void startRule(ScannerEvent event) {
      buf.append("matchedRule:: \n");
    }

    /*
     * @see org.exoplatform.services.css.scanner.event.ScannerListener#endRule(org.exoplatform.services.css.scanner.event.ScannerEvent)
     */
    public void endRule(ScannerEvent event) {
    }

    /*
     * @see exo.services.portal.sac.impl.event.ScannerListener#execute(exo.services.portal.sac.impl.event.ScannerEvent)
     */
    public void startSelector(ScannerEvent event) {
      Selector s = (Selector) event.getEventObject();
      buf.append(s.getName());
      if (!event.hasNext()) {
        buf.append("\n{\n");
      } else {
        buf.append(" ");
      }
    }

    /*
     * @see org.exoplatform.services.css.scanner.event.ScannerListener#endSelector(org.exoplatform.services.css.scanner.event.ScannerEvent)
     */
    public void endSelector(ScannerEvent event) {
      buf.append("}\n\n");
    }

    /*
     * @see exo.services.portal.sac.event.ScannerListener#matchedProperty(exo.services.portal.sac.event.ScannerEvent)
     */
    public void startProperty(ScannerEvent event) {
      Property property = (Property) event.getEventObject();
      if (!event.hasNext()) {
        buf.append("\t" + property.getName() + ": " + property.getValue() + ";\n");
      }
    }

    /*
     * @see org.exoplatform.services.css.scanner.event.ScannerListener#endProperty(org.exoplatform.services.css.scanner.event.ScannerEvent)
     */
    public void endProperty(ScannerEvent event) {
    }

    /*
     * @see exo.services.portal.sac.event.ScannerListener#matchedComment(exo.services.portal.sac.event.ScannerEvent)
     */
    public void startComment(ScannerEvent event) {
      buf.append("matchedComment:: \n");
    }

    /*
     * @see org.exoplatform.services.css.scanner.event.ScannerListener#endComment(org.exoplatform.services.css.scanner.event.ScannerEvent)
     */
    public void endComment(ScannerEvent event) {
    }

  }

  // -----------------------------------------------------------------------------------------------------------------------


  protected String getDescription() {
    return "Test CSS Scanner service";
  }
}
