/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.publish.hibernate;

import org.exoplatform.commons.exception.UniqueObjectException;

import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.idgenerator.IDGeneratorService;
import org.exoplatform.services.publish.*;
import org.exoplatform.services.publish.model.Content;
import org.exoplatform.services.publish.model.Section;
import org.exoplatform.services.publish.impl.*;

import java.util.*;

import org.exoplatform.commons.utils.*;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;
/**
 * Created by The eXo Platform SARL
 * Author : Daniel Summer
 *          danny_xcz@users.sourceforge.net
 * Date: Nov 29, 2004
 * Time: 3:22:54 PM
 */
public class PublishServiceImpl implements PublishService {
	private static final String queryContentsBySection =
		"from content in class org.exoplatform.services.publish.impl.ContentImpl "
			+ "where content.sectionId = ?";

	/*
	 public static final String queryContentsByAuthor =
		"from content in class org.exoplatform.services.publish.impl.ContentImpl " +
		"where content.createdBy = ?";
		*/

	public static final String queryFindContentById =
		"from content in class org.exoplatform.services.publish.impl.ContentImpl "
			+ "where content.contentId = ?";

	private static final String queryFindSectionById =
		"from section in class org.exoplatform.services.publish.impl.SectionImpl "
			+ "where section.id = ? ";

	private static final String queryFindRootSections =
		"from section in class org.exoplatform.services.publish.impl.SectionImpl "
			+ "where section.parentId is null";

	private static final String queryFindSectionByParent =
		"from section in class org.exoplatform.services.publish.impl.SectionImpl "
			+ "where section.parentId = ? ";

	private static final String queryFindContentsBySection =
		"from content in class org.exoplatform.services.publish.impl.ContentImpl "
			+ "where content.sectionId = ? ";

	private HibernateService hService_;
	private IDGeneratorService idService_;
	static private String[] MAPPING =
		{
			"org/exoplatform/services/publish/impl/ContentImpl.hbm.xml",
			"org/exoplatform/services/publish/impl/SectionImpl.hbm.xml" };

	public PublishServiceImpl(
		HibernateService hibernateService,
		IDGeneratorService idService) {
//		hibernateService.addMappingFiles(MAPPING);
//		hService_ = hibernateService;
//		idService_ = idService;
	}

	//------------------------ Content -------------------------//
	public Content createContentInstance() {
		return new ContentImpl();
	}

	public Content createContent(Section section, Content content)
		throws Exception {
		Date now = new Date();
		ContentImpl impl = (ContentImpl) content;
		impl.setContentId(idService_.generateStringID(impl));
		if(section!=null){
		impl.setSectionId(section.getId());
		} else {
			impl.setSectionId("/");
		}
		impl.setCreatedDate(now);
		impl.setModifiedDate(now);
		Session session = hService_.openSession();
		session.save(impl);
		session.flush();
		return content;
	}

	public Content findContentById(String contentId) throws Exception {
		//return (content) hservice_.findOne(ContentImpl.class, contentId) ;
		Session session = hService_.openSession();
		Content content =
			(Content) hService_.findOne(
				session,
				queryFindContentById,
				contentId);
		return content;
	}

	public void updateContent(Content content) throws Exception {
		Date now = new Date();
		ContentImpl impl = (ContentImpl) content;
		impl.setModifiedDate(now);
		Session session = hService_.openSession();
		session.update(impl);
		session.flush();

	}

	public Content removeContent(String contentId) throws Exception {
		Session session = hService_.openSession();
		Content content = (Content) session.get(ContentImpl.class, contentId);
		session.delete(content);
		session.flush();

		return content;
	}
	/*
		public PageList findContentsByAuthor(String userName) throws Exception {
			//Session session = hService_.openSession();
			// return  session.find( queryFindContentByAuthor, userName, Hibernate.STRING );
			String queryFindContentsByAuthor =
				"from c in class org.exoplatform.services.publish.impl.ContentImpl, " +
				"where c.createdBy =  '" + userName + "'" ;
			
			String countContentsByAuthor =
				"select count(c) " +
				"from c in class org.exoplatform.services.publish.impl.ContentImpl, " +
				"where c.createdBy =  '" + userName + "'" ;
			return new DBObjectPageList(hService_, 20,queryFindContentsByAuthor, countContentsByAuthor ) ;
		
		}
	*/
	public PageList findContentsBySection(String sectionId) throws Exception {
		String queryFindContentsInSection =
			"from c in class org.exoplatform.services.publish.impl.ContentImpl, "
				+ "where c.sectionId =  '"
				+ sectionId
				+ "'";

		String countContentsInSection =
			"select count(c) "
				+ "from c in class org.exoplatform.services.publish.impl.ContentImpl, "
				+ "where c.sectionId =  '"
				+ sectionId
				+ "'";

		return new DBObjectPageList(
			hService_,
			20,
			queryFindContentsInSection,
			countContentsInSection);

	}

	public List findContentsBySectionAndNumber(String sectionId, int num)
		throws Exception {
		String queryFindSectionByParentAndNumber =
			"from c in class org.exoplatform.services.publish.impl.ContentImpl, "
				+ "where c.sectionId =  '"
				+ sectionId
				+ "'";

		Session session = hService_.openSession();

		StringBuffer b = new StringBuffer() ;
		b.append("from c in class org.exoplatform.services.publish.impl.ContentImpl  ").
			append("where c.sectionId = :sectionId ").
			append("order by c.createdDate desc") ;
		Query query = session.createQuery(b.toString());
		query.setString("sectionId", sectionId);
		query.setFirstResult(0);
		query.setMaxResults(num);
		return query.list();
		//return session.find(
		//	queryFindSectionByParentAndNumber,
		//	sectionId,
		//	Hibernate.STRING);
	}

	public PageList findContents(ContentQuery q) throws Exception {
		Session session = hService_.openSession();
		String contentTitle = q.getContentTitle();
		if (contentTitle == null || contentTitle.length() == 0) {
			contentTitle = "%";
		}
		contentTitle = contentTitle.replace('*', '%');
		StringBuffer b = new StringBuffer();
		b
			.append("from c in class org.exoplatform.services.publish.impl.ContentImpl  ")
			.append("where c.title like :contentTitle");

		Query query = session.createQuery(b.toString());
		query.setString("contentTitle", contentTitle);
		query.setFirstResult(0);
		query.setMaxResults(q.getMaxSize());

		return new ObjectPageList(query.list(), 20);

	}

	//--------------------------- Section ------------------------------------------//

	public Section createSectionInstance() {
		return new SectionImpl();
	}

	public Section createSection(Section section) throws Exception {
		Session session = hService_.openSession();
		String sectionId = "/" + section.getName();
		Object o = session.get(SectionImpl.class, sectionId);
		if (o != null) {
			Object[] args = { section.getName()};
			throw new UniqueObjectException(
				"PublishService.unique-section-exception",
				args);
		}

		session.save(section, sectionId);
		session.flush();

		return section;
	}

	public Section findSectionById(String sectionId) throws Exception {
		Session session = hService_.openSession();
		Section section =
			(Section) hService_.findOne(
				session,
				queryFindSectionById,
				sectionId);
		return section;
	}

	public Section addChild(Section parent, Section child) throws Exception {
		Session session = hService_.openSession();
		Section parentSection =
			(Section) session.get(SectionImpl.class, parent.getId());
		String sectionId = parentSection.getId() + "/" + child.getName();
		Object o = session.get(SectionImpl.class, sectionId);
		if (o != null) {
			Object[] args = { child.getName()};
			throw new UniqueObjectException(
				"PublishService.unique-section-exception",
				args);
		}
		SectionImpl childImpl = (SectionImpl) child;

		childImpl.setParentId(parentSection.getId());
		session.save(childImpl, sectionId);
		session.flush();
		return parentSection;

	}

	public void updateSection(Section section) throws Exception {
		Session session = hService_.openSession();
		session.update(section);
		session.flush();
	}

	public Section removeSection(Section section) throws Exception {
		Session session = hService_.openSession();
		session.delete(section);

		/*======= delete section from parent ========*/
//		List sections =
//			session.find(
//				queryFindSectionByParent,
//				section.getId(),
//				Hibernate.STRING);
//		//session.delete(queryFindSectionByParent, section.getId(), Hibernate.STRING );
//		Iterator i = sections.iterator();
//		while (i.hasNext()) {
//			Section s = (Section) i.next();
//			removeSection(s);
//		}
//		/*==========================================*/
//		removeContentsOfSection(section, session);
//		session.flush();
//		return section;
    return null;
	}
	/*
	public Section removeSection(String sectionId) throws Exception{
		Session session = hService_.openSession();
		Section section = (Section) session.get(SectionImpl.class, sectionId);
		session.delete(section);
		session.flush();
		return section;
		
	}
	*/

	public Collection findSectionsByParent(Section parent) throws Exception {
//		Session session = hService_.openSession();
//		if (parent == null) {
//			return session.find(queryFindRootSections);
//		}
//		return session.find(
//			queryFindSectionByParent,
//			parent.getId(),
//			Hibernate.STRING);
    return null;
	}

	/*
		public Collection findSectionsByParent(String parentId) throws Exception {
			Session session = hService_.openSession();
			if (parentId == null) {
				return  session.find( queryFindRootSections );
	  	    }
			
			return  session.find( queryFindSectionByParent, parentId, Hibernate.STRING );
		}
	*/

	void removeContentsOfSection(Section section, Session session)
		throws Exception {
//		session.delete(
//			queryFindContentsBySection,
//			section.getId(),
//			Hibernate.STRING);
	}
}
