/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.rdf.query;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdql.Query;
import com.hp.hpl.jena.rdql.QueryExecution;


/**
 * Created by the Exo Development team.
 * @author Ove Ranheim (oranheim@users.sourceforge.net)
 * @since 08.apr.2004 11:30:57
 */
public interface RDFQueryService {

    /**
     * Create new Query with no initial RDQL statment.
     * 
     * @param model The source to perform query for
     * @return com.hp.hpl.jena.rdql.Query
     */
    public Query createQuery(Model model);
    
    /**
     * Create new Query with no initial RDQL statment.
     * 
     * @param sourceURL The source URL to perform query for
     * @return com.hp.hpl.jena.rdql.Query
     */
    public Query createQuery(String sourceURL);
    
    /**
     * Create new Query by RDQL statment.
     * 
     * @param model The source to perform query for
     * @param queryString The RDQL statment
     * @return com.hp.hpl.jena.rdql.Query
     */
    public Query createQuery(Model model, String queryString);
    
    /**
     * Create new Query by RDQL statment.
     * 
     * @param sourceURL The source URL to perform query for
     * @param queryString The RDQL statment
     * @return com.hp.hpl.jena.rdql.Query
     */
    public Query createQuery(String sourceURL, String queryString);
    
    /**
     * Prepare Query instance for execution.
     * 
     * @param query The Query object to prepare for execution
     * @return com.hp.hpl.jena.rdql.QueryExecution
     */
    public QueryExecution prepare(Query query);
}
