/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * (c) Copyright 2003, 2004, Hewlett-Packard Development Company, LP All rights reserved.
 * Please look at license.txt and license_hp in info directory for more license detail.
 **/

package org.exoplatform.services.rdf.jena.tutorial;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.VCARD;

/**
 * Created by the Exo Development team.
 * 
 * @author Ove Ranheim (oranheim@users.sourceforge.net)
 * @since 08.apr.2004 12:59:40
 */
public class TestTutorial03 extends JenaTestCase {

    public TestTutorial03(String name) {
        super(name);
    }
    
    public void testStatementAttributeAccessorMethods() {

        // some definitions
        String personURI = "http://somewhere/JohnSmith";
        String givenName = "John";
        String familyName = "Smith";
        String fullName = givenName + " " + familyName;
        // create an empty model
        Model model = _modelservice.createDefaultModel();

        // create the resource
        //   and add the properties cascading style
        Resource johnSmith = model.createResource(personURI).addProperty(VCARD.FN, fullName).addProperty(VCARD.N,
                model.createResource().addProperty(VCARD.Given, givenName).addProperty(VCARD.Family, familyName));

        // list the statements in the graph
        StmtIterator iter = model.listStatements();

        // print out the predicate, subject and object of each statement
        while (iter.hasNext()) {
            Statement stmt = iter.nextStatement(); // get next statement
            Resource subject = stmt.getSubject(); // get the subject
            Property predicate = stmt.getPredicate(); // get the predicate
            RDFNode object = stmt.getObject(); // get the object

            assertNotNull(subject);
            assertNotNull(predicate);
            assertNotNull(object	);
            
            System.out.print(subject.toString());
            System.out.print(" " + predicate.toString() + " ");
            if (object instanceof Resource) {
                System.out.print(object.toString());
            } else {
                // object is a literal
                System.out.print(" \"" + object.toString() + "\"");
            }
            System.out.println(" .");
            
            
        }
    }
}
