/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * (c) Copyright 2003, 2004, Hewlett-Packard Development Company, LP All rights reserved.
 * Please look at license.txt and license_hp in info directory for more license detail.
 **/


package org.exoplatform.services.rdf.jena.tutorial;

import java.io.InputStream;
import java.io.InputStreamReader;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ObjectF;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.VCARD;

/**
 * Created by the Exo Development team.
 * 
 * @author Ove Ranheim (oranheim@users.sourceforge.net)
 * @since 08.apr.2004 12:59:40
 */
public class TestTutorial06 extends JenaTestCase {

    static final String inputFileName = "vc-db-1.rdf";
    static final String johnSmithURI = "http://somewhere/JohnSmith/";

    public TestTutorial06(String name) {
        super(name);
    }
    
    public void testNavigatingAModel() {
        // create an empty model
        Model model = _modelservice.createDefaultModel();

        // use the class loader to find the input file
        InputStream in = getRDFResource(inputFileName);
        if (in == null) { throw new IllegalArgumentException("File: " + inputFileName + " not found"); }

        // read the RDF/XML file
        model.read(new InputStreamReader(in), "");

        // retrieve the Adam Smith vcard resource from the model
        Resource vcard = model.getResource(johnSmithURI);
        assertEquals(johnSmithURI, vcard.getURI().toString());

        // retrieve the value of the N property
        Resource name = (Resource) vcard.getRequiredProperty(VCARD.N).getObject();
        // retrieve the given name property
        String fullName = vcard.getRequiredProperty(VCARD.FN).getString();
        assertEquals("John Smith", fullName);
        
        // add two nick name properties to vcard
        vcard.addProperty(VCARD.NICKNAME, "Smithy").addProperty(VCARD.NICKNAME, "Adman");
        
        // set up the output
        System.out.println("The nicknames of \"" + fullName + "\" are:");
        // list the nicknames
        StmtIterator iter = vcard.listProperties(VCARD.NICKNAME);
        while (iter.hasNext()) {
            Statement stmt = iter.nextStatement();
            assertNotNull(stmt.getObject());
            System.out.println("    " + stmt.getObject().toString());
        }
    }
}
