/**
 * Copyright 2001-2004 The eXo platform SARL All rights reserved.
 * (c) Copyright 2003, 2004, Hewlett-Packard Development Company, LP All rights reserved.
 * Please look at license.txt and license_hp in info directory for more license detail.
 **/

package org.exoplatform.services.rdf.jena.tutorial;

import java.io.InputStream;
import java.io.InputStreamReader;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.VCARD;

/**
 * Created by the Exo Development team.
 * 
 * @author Ove Ranheim (oranheim@users.sourceforge.net)
 * @since 08.apr.2004 12:59:40
 */
public class TestTutorial08 extends JenaTestCase {

    static final String inputFileName = "vc-db-1.rdf";

    public TestTutorial08(String name) {
        super(name);
    }

    public void testDemonstrateSelectorMethods() {
        // create an empty model
        Model model = _modelservice.createDefaultModel();

        // use the class loader to find the input file
        InputStream in = getRDFResource(inputFileName);
        if (in == null) { throw new IllegalArgumentException("File: " + inputFileName + " not found"); }

        // read the RDF/XML file
        model.read(new InputStreamReader(in), "");

        // select all the resources with a VCARD.FN property
        // whose value ends with "Smith"
        StmtIterator iter = model.listStatements(new SimpleSelector(null, VCARD.FN, (RDFNode) null) {
            public boolean selects(Statement s) {
                return s.getString().endsWith("Smith");
            }
        });
        
        assertNotNull(iter);
        
        if (iter.hasNext()) {
            System.out.println("The database contains vcards for:");
            while (iter.hasNext()) {
                System.out.println("  " + iter.nextStatement().getString());
            }
        } else {
            System.out.println("No Smith's were found in the database");
        }
        
        assertTrue(true);
    }
}

