/***************************************************************************
 * Copyright 2004-2006 The VietSpider Studio All rights reserved.  * 
 **************************************************************************/
package org.vietspider.veis.impl.core.io.base;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.vietspider.veis.api.io.ProgressDialog;

/**
 * @by thuannd (nhudinhthuan@yahoo.com)
 * Jun 12, 2005
 */
public class ZipFolderImpl  {
  
  public void addToArchive( File input, File output, ProgressDialog progress) throws Exception {   
    String path = input.getAbsolutePath();            
    
    ZipOutputStream out = new ZipOutputStream(
        new BufferedOutputStream( new FileOutputStream( output)));
    
    final int BUFFER = 2048;
    byte data[] = new byte[BUFFER];
    
    BufferedInputStream origin = null;
    
    List<File> files = getFiles( input);  
    if( files == null || files.size() < 1){     
      progress.dispose();
      return;
    }
    progress.setMax( files.size());
    int index = 0;
    for (Iterator<File> iter = files.iterator(); iter.hasNext();) {
      File f = iter.next();
      index++;
      String filePath = f.getAbsolutePath();     
      progress.setValue( index);
      if (filePath.startsWith( path))
        filePath = filePath.substring( path.length());      
      // create "releative" path      
      FileInputStream fi = new FileInputStream(f);       
      origin = new BufferedInputStream(fi, BUFFER);
      
      ZipEntry entry = new ZipEntry(filePath);
      out.putNextEntry(entry); 
      
      int count;
      while ((count = origin.read(data, 0, BUFFER)) != -1)
        out.write(data, 0, count);
      origin.close();      
    }
    out.close();
    progress.dispose();
  }
  
  public void extractToFiles( File input, String output, ProgressDialog progress) throws Exception { 
    BufferedOutputStream dest = null;   
    FileInputStream fis = new  FileInputStream( input);
    ZipInputStream zis = new  ZipInputStream(new BufferedInputStream(fis));
    ZipEntry entry;
    progress.setMax( (int)input.length());    
    int total =  0;   
    UtilFile util = new UtilFile();    
    String dir = "" ;
    if( output != null) dir = util.getFolder( output);    
    while((entry = zis.getNextEntry()) != null) {             
      int count;
      final int BUFFER = 2048;
      byte data[] = new byte[BUFFER];     
      FileOutputStream fos = null;
      if( output != null)
         fos = new FileOutputStream( dir+entry.getName());
      else        
        fos = new FileOutputStream( this.createFolder(entry.getName()));         
      dest = new BufferedOutputStream(fos, BUFFER);
      while ((count = zis.read(data, 0, BUFFER)) != -1) {            
        dest.write(data, 0, count);
        total += count;
        progress.setValue( total);
      }
      dest.flush();
      dest.close();
    }
    zis.close();
    progress.dispose();
  }
  
  private String createFolder( String name) throws Exception {
    String [] split = name.split(File.separator+File.separator);
    if( split.length < 2) return name;  
    name = "data."+split[1];    
    int i= 2;   
    while( i<split.length-1){
      name += "."+split[i];
      i++;
    }    
    return (new UtilFile()).getFolder( name)+ File.separator+split[ split.length-1];
  }
  
  private List<File> getFiles(File dir) {
    final List <File> files = new LinkedList<File>();
    
    dir.listFiles(new FileFilter() {
      public boolean accept(File f) {
        if (f.isDirectory())
          return false;          
        files.add(f);
        return true;
      }
    });      
    dir.listFiles(new FileFilter() {
      public boolean accept(File f) {
        if (f.isFile())
          return false;
        
        files.addAll( getFiles(f));
        return false;
      }
    });
    return files;
  }
  
}
