/***************************************************************************
 * Copyright 2004-2006 The VietSpider Studio All rights reserved.  * 
 **************************************************************************/
package org.vietspider.veis.impl.core.io.net;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Properties;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthPolicy;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.methods.GetMethod;
import org.vietspider.veis.api.net.NetConnection;
import org.vietspider.veis.api.net.Proxy;
import org.vietspider.veis.impl.core.io.base.DataBuffer;
import org.vietspider.veis.impl.core.io.local.ProxyProperties;

/**
 * @by thuannd (nhudinhthuan@yahoo.com)
 * Jun 13, 2005
 */
public class HttpClientImpl implements NetConnection {
  
  private HttpClient http;
  //private UrlCreation creation;
  private DataBuffer buffer; 

  public HttpClientImpl( String protocol, String host, int port) throws Exception {
    http = new HttpClient( new MultiThreadedHttpConnectionManager());   
    http.getHttpConnectionManager().getParams().setConnectionTimeout(25000);
    if( port < 0) port = 80;
    http.getHostConfiguration().setHost( host, port, protocol); 
    //creation = new UrlCreation();
    
    ProxyProperties properties = new ProxyProperties();
    Properties proProxy = properties.get();
    if( proProxy.getProperty("ProxyType") == null) return;
    int type = Integer.parseInt( proProxy.getProperty("ProxyType"));
    
    if( type != Proxy.NONE){
      http.getState().setCredentials(
          new AuthScope( proProxy.getProperty("ProxyHost"), 80, null),
          new UsernamePasswordCredentials( 
              proProxy.getProperty("ProxyUser"), proProxy.getProperty("ProxyPass")));    
      ArrayList<String> authPrefs = new ArrayList<String>(3);
      authPrefs.add( AuthPolicy.BASIC);
      authPrefs.add( AuthPolicy.NTLM);
      authPrefs.add( AuthPolicy.DIGEST);
      http.getParams().setParameter(AuthPolicy.AUTH_SCHEME_PRIORITY, authPrefs);
    }
  }
  
  public byte[] loadContent(String link) throws Exception {
    GetMethod get = new GetMethod( link);   
    int status = http.executeMethod( get);  
    if( status == HttpStatus.SC_OK){      
      InputStream in = get.getResponseBodyAsStream();
      BufferedInputStream input = new BufferedInputStream(in);
      ByteArrayOutputStream output = new ByteArrayOutputStream();
      byte b[] = new byte[1024];
      int read = 0;
      while ((read = input.read(b)) > -1){
        output.write(b, 0, read);
      }
      in.close();
      input.close();
      get.releaseConnection();
      b  = output.toByteArray();
      output.flush();
      output.close();
      return b;
    }       
    return new byte[0];
  }
  
  public InputStream loadContentByInput(String link) throws Exception {
    GetMethod get = new GetMethod( link);   
    int status = http.executeMethod( get);    
    if( status == HttpStatus.SC_OK)      
      return get.getResponseBodyAsStream();           
    return null;
  }
  
  public String loadResource( String link, String path) throws Exception {
    GetMethod get = new GetMethod( link);          
    int status = http.executeMethod( get);   
    if( status != HttpStatus.SC_OK) return link;
    if( buffer == null)
       buffer = new DataBuffer();
    buffer.saveByte( path, get.getResponseBody());             
    get.releaseConnection();   
    return link;
  }

}
