/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.vietspider.veis.impl.ui.publisher;

import java.util.Vector;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.vietspider.veis.impl.core.io.base.UtilFile;
import org.vietspider.veis.impl.core.io.local.StoreFolder;
import org.vietspider.veis.impl.core.text.codec.UtilConvert;
import org.vietspider.veis.impl.ui.publisher.action.StoreEvent;
import org.vietspider.veis.impl.ui.publisher.action.StoreListener;
import org.vietspider.veis.impl.ui.widget.ApplicationFactory;

/**
 * @by thuannd (nhudinhthuan@yahoo.com)
 * @since Jul 4, 2005
 */
public class StoreDialogViewer {
  
  private Shell shell;
  
  private List list;
  private Text text;
  private Button ok;
  private Vector<StoreListener> listeners;
  
  public StoreDialogViewer( Shell parent){     
    listeners = new Vector<StoreListener>();    
    shell = new Shell( parent);  
    shell.setLocation( parent.getLocation().x+250, parent.getLocation().y+300);
    
    ApplicationFactory factory = new ApplicationFactory( shell, this.getClass().getName());
    
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 1;    
    shell.setLayout( gridLayout);
    
    list = factory.createList( new String[]{}, new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        try{
          store();
        }catch( Exception exp){
          exp.printStackTrace();
        }
      }
    });
    GridData gridData = new GridData(GridData.FILL_BOTH);   
    list.setLayoutData(gridData);
    
    text = factory.createText();
    gridData = new GridData(GridData.FILL_HORIZONTAL);
    text.setLayoutData(gridData);
    
    ok = factory.createButton("ok", new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        try{
          create();
        }catch( Exception exp){
          exp.printStackTrace();
        }
      }
    });
    gridData = new GridData(GridData.FILL_HORIZONTAL);     
    ok.setLayoutData(gridData);   
    
    shell.setSize( 300, 250);
    shell.open();
    try{
      this.load();
    }catch( Exception exp){
      exp.printStackTrace();
    }
  }
  public void load() throws Exception {
    StoreFolder store = new StoreFolder();  
    String[] files = store.getDomain();
    UtilConvert convert = new UtilConvert();
    for( int i=0; i<files.length; i++){
      files[i] = convert.deUnicode( files[i]);
    }
    this.list.setItems( files);
  }
  public void delete() throws Exception {
    if( this.list.getSelectionIndex() < 0) return;
    String name = this.list.getItem( this.list.getSelectionIndex());
    if( name == null ) return;
    UtilConvert convert = new UtilConvert();
    name = convert.enUnicode( name);
    
    StoreFolder store = new StoreFolder();  
    store.delDomain( name);
    this.load();
  }
  public void create() throws Exception {
    String name = this.text.getText();
    if( name.trim().equals( "")) return;
    UtilConvert convert = new UtilConvert();
    name = convert.enUnicode( name);
   
    StoreFolder store = new StoreFolder();
    store.setDomain( name);
    this.load();
  }
//event
  public void fireStoreEvent( StoreEvent e) {
    synchronized(listeners)  {
      for(int i = 0; i < listeners.size(); i++){
        try{
          StoreListener listener = listeners.get(i);         
          listener.store(e);       
          break;
        }catch(Exception exp){                  
        }       
      }
    }
  }
  public void addStoreListener( StoreListener listener) {      
    synchronized(listeners) {
      listeners.remove(listener);
      listeners.add(listener);
    }
  }
  public void removeStoreListener( StoreListener listener) {
    synchronized(listeners) {
      listeners.remove(listener);
    }
  }
  public void store( StoreEvent ev) {
    if(ev.getPath() == null || ev.getPath().equals("")) return;    
    fireStoreEvent(ev);    
  } 
  private void store(){    
    if( this.list.getSelectionIndex() < 0) return;
    String name = this.list.getItem( this.list.getSelectionIndex());
    if( name == null ) return;
    UtilConvert convert = new UtilConvert();
    name = convert.enUnicode( name);    
    try{
      UtilFile utils = new UtilFile();  
      String path = utils.getFolder("data.store."+name);
      StoreEvent event = new StoreEvent( path);    
      fireStoreEvent(event); 
    }catch( Exception exp){
      exp.printStackTrace();
    }
    shell.dispose();
  }
}
