/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.vietspider.veis.impl.ui.widget;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.forms.events.IHyperlinkListener;
import org.eclipse.ui.forms.widgets.ImageHyperlink;

/**
 * @by thuannd (nhudinhthuan@yahoo.com)
 * @since Jul 12, 2005
 */
public class WidgetFactory {
  
  public Button createButton( Composite com, int style){
    return new Button( com, style);
  }
  
  public Button createButton( Composite com, int style, String text){
    Button widget = this.createButton( com, style);
    widget.setFont( new Font(Display.getDefault(), "Tahoma", 9, SWT.NORMAL));
    widget.setText( text);
    return widget;
  }
  
  public Button createButton( Composite com, int style, String text, Font font){
    Button widget = this.createButton( com, style);
    widget.setFont( font);
    widget.setText( text);
    return widget;
  }
  
  public Button createButton( Composite com, int style, String text, SelectionListener listener){
    Button widget = this.createButton( com, style, text);
    widget.addSelectionListener( listener);
    return widget;
  }
  
  public Button createButton( Composite com, int style, String text, String tip, SelectionListener listener){
    Button widget = this.createButton( com, style, text);
    widget.addSelectionListener( listener);
    widget.setToolTipText( tip);
    return widget;
  }
  
  public Button createButton( Composite com, Image icon, String tip, SelectionListener listener){
    Button widget = this.createButton( com, SWT.PUSH, "");
    widget.setImage( icon);
    widget.setToolTipText( tip);
    widget.addSelectionListener( listener);
    return widget;
  }
  
  public ImageHyperlink createIcon( Composite com, int style,
      Image img, Image img1, String tip, IHyperlinkListener listener) {    
    ImageHyperlink button = new ImageHyperlink( com , SWT.PUSH);
    if( img != null){
      button.setImage( img);
      button.setActiveImage( img);
    }
    if( img1 != null) button.setHoverImage( img1);
    button.addHyperlinkListener( listener);
    button.setToolTipText( tip);
    return button;
    
  }
  
  
  public Label createLabel(Composite com, int style){
    return new Label( com, style);
  }
  
  public Label createLabel( Composite com, int style, String text){
    Label widget = this.createLabel( com, style);
    widget.setFont( new Font(Display.getDefault(), "Tahoma", 9, SWT.NORMAL));
    widget.setText( text);
    return widget;
  }
  
  public Label createLabel( Composite com, int style, String text, Font font){
    Label widget = this.createLabel( com, style);
    widget.setFont( font);
    widget.setText( text);
    return widget;
  }
  
  public Label createLabel( Composite com, int style, String text, Object layout){
    Label widget = this.createLabel( com, style, text);
    widget.setLayoutData( layout);
    return widget;
  }
  
  
  public Text createText(Composite com, int style){
    return new Text( com, style);
  }
  
  public Text createText( Composite com, int style, String txt){
    Text widget = this.createText( com, style);
    widget.setFont( new Font(Display.getDefault(), "Tahoma", 9, SWT.NORMAL));
    widget.setText( txt);
    return widget;
  }
  
  public Text createText( Composite com, int style, Font font){
    Text widget = this.createText( com, style);
    widget.setFont( font);    
    return widget;
  }
  
  public Text createText( Composite com, int style, String txt, Object layout){
    Text widget = this.createText( com, style, txt);
    widget.setLayoutData( layout);
    return widget;
  }  
  
  public Text createText( Composite com, int style, Object layout){
    Text widget = this.createText( com, style, "");
    widget.setLayoutData( layout);
    return widget;
  }  
  
  public Group createGroup( Composite com, int style){
    return new Group( com, style);
  }
  
  public Group createGroup( Composite com, int style, String txt){
    Group widget = this.createGroup( com, style);
    widget.setFont( new Font(Display.getDefault(), "Tahoma", 10, SWT.BOLD));  
    widget.setText( txt);
    return widget;
  }
  
  public Group createGroup( Composite com, int style, String txt, Object layoutData){
    Group widget = this.createGroup( com, style, txt);
    widget.setLayoutData( layoutData);
    return widget;
  }  
  
  public Group createGroup( Composite com, int style, String txt, Object layoutData, Layout layout){
    Group widget = this.createGroup( com, style, txt);
    widget.setLayoutData( layoutData);
    widget.setLayout( layout);
    return widget;
  }  
  
  
  public Combo createCombo(Composite com, int style){
    return new Combo( com, style);
  }
  
  public Combo createCombo( Composite com, int style, String[] txt){
    Combo widget = this.createCombo( com, style);
    widget.setFont( new Font(Display.getDefault(), "Tahoma", 9, SWT.NORMAL));
    widget.setItems( txt);
    return widget;
  }
  
  public Combo createCombo( Composite com, int style, String[] text, SelectionListener listener){
    Combo widget = this.createCombo( com, style, text);
    widget.addSelectionListener( listener);
    return widget;
  }
  
  
  public List createList(Composite com, int style){
    return new List( com, style);
  }
  
  public List createList( Composite com, int style, String[] txt){
    List widget = this.createList( com, style);
    widget.setFont( new Font(Display.getDefault(), "Tahoma", 9, SWT.NORMAL));
    widget.setItems( txt);
    return widget;
  }
  
  public List createList( Composite com, int style, String[] text, SelectionListener listener){
    List widget = this.createList( com, style, text);
    widget.addSelectionListener( listener);
    return widget;
  }
  
  public Table createTable( Composite com, 
      int style, String [] cols, int [] w, SelectionListener listener){
    Table table = new Table( com, style);
    table.setFont(new Font( com.getDisplay(), "Tahoma", 10, SWT.NORMAL));
    table.addSelectionListener( listener);
    table.setLinesVisible(true);
    table.setHeaderVisible(true);
    
    for( int i=0; i<cols.length; i++){
      TableColumn order = new TableColumn( table, SWT.LEFT);
      order.setText( cols[i]);
      order.setWidth( w[i]);
    }   
    return table;
  }
  
  public Menu createMenu( Menu menu, String text){
    MenuItem item = new MenuItem( menu, SWT.CASCADE);
    item.setText( text);     
    Menu mSub = new Menu(  menu.getShell(), SWT.DROP_DOWN);
    item.setMenu( mSub);
    return mSub;
  }
  
  public MenuItem createMenuItem( Menu menu, int style){
    return new MenuItem( menu, style);
  }
  
  public MenuItem createMenuItem( Menu menu, int style, String text){
    MenuItem item = this.createMenuItem( menu, style);
    item.setText( text);
    return item;
  }
  
  public MenuItem createMenuItem( Menu menu, int style, String text, SelectionListener listener){
    MenuItem item = this.createMenuItem( menu, style, text);    
    item.addSelectionListener( listener);
    return item;
  }
  
  public Menu createToolItem( final ToolBar bar, Image image ){
    final ToolItem tool = new ToolItem(bar, SWT.DROP_DOWN);     
    if( image != null) tool.setImage( image);    
    final Menu menu = new Menu( bar.getShell(), SWT.POP_UP);
    tool.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event) {
        if (event.detail == SWT.ARROW) {
          Rectangle rect = tool.getBounds();
          Point pt = new Point(rect.x, rect.y + rect.height);
          pt = bar.toDisplay(pt);
          menu.setLocation(pt.x, pt.y);
          menu.setVisible(true);
        }
      }
    });
    return menu;
  }
    
  public Shell createShell( Display display, final Class cl, String title, String icon , int w, int h){   
    final Shell shell = new Shell(display);
    new ShellGetter( cl, shell, w, h);
    shell.setText( title);   
    Image image = (new ImageLoader()).load( display, icon);  
    if( image != null)  shell.setImage( image);    
    shell.addShellListener( new ShellAdapter(){       
      public void shellClosed(ShellEvent e){
        new ShellSetter( cl, shell); 
      }      
    });
    return shell;
  }  
  
  
  public ProgressBar createProgress( Composite composite, GridData gridData){
    ProgressBar progressBar =  new ProgressBar( composite, SWT.NONE );   
    progressBar.setBackground( new Color( composite.getDisplay(), 255, 255, 255));
    progressBar.setForeground( new Color( composite.getDisplay(), 128, 128, 255));
    progressBar.setLayoutData(gridData);
    return progressBar;
  }
  
}
