/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.core.nodetype.mix;

import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeDefinition;

import javax.jcr.PropertyType;
import javax.jcr.Value;

import javax.jcr.version.OnParentVersionAction;

import org.exoplatform.services.jcr.impl.core.nodetype.NodeDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.value.StringValue;

import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.RepositoryException;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: VersionableType.java 3510 2005-12-29 17:32:57Z geaz $
 */

public class VersionableType extends NodeTypeImpl {

  public VersionableType(NodeTypeManager manager)
      throws NoSuchNodeTypeException, RepositoryException {
    super(manager);
    this.name = "mix:versionable";
    this.mixin = true;
    this.orderableChild = false;
    this.primaryItemName = null;
    this.declaredSupertypes = new NodeType[1];
    this.declaredSupertypes[0] = manager.getNodeType("mix:referenceable");
    this.declaredPropertyDefinitions = new PropertyDefinition[5];

    String[] historyConstraints = { "nt:versionHistory" };
    this.declaredPropertyDefinitions[0] = new PropertyDefinitionImpl(
        "jcr:versionHistory", // name
        this, // declaringNodeType
        PropertyType.REFERENCE, // requiredType
        historyConstraints, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        true, // mandatory
        OnParentVersionAction.COPY, // onVersion
        true, // readOnly
        false // multiple
    );

    String[] baseVersionConstraints = { "nt:version" };
    this.declaredPropertyDefinitions[1] = new PropertyDefinitionImpl(
        "jcr:baseVersion", // name
        this, // declaringNodeType
        PropertyType.REFERENCE, // requiredType
        baseVersionConstraints, // valueConstraints
        null, // defaultValues
        false, // autoCreate
        true, // mandatory
        OnParentVersionAction.IGNORE, // onVersion
        true, // readOnly
        false // multiple
    );

    Value[] checkedOutDefault = { new StringValue("true") };
    this.declaredPropertyDefinitions[2] = new PropertyDefinitionImpl(
        "jcr:isCheckedOut", // name
        this, // declaringNodeType
        PropertyType.BOOLEAN, // requiredType
        EMPTY_CONSTRAINTS, // valueConstraints
        checkedOutDefault, // defaultValues
        true, // autoCreate
        true, // mandatory
        OnParentVersionAction.IGNORE, // onVersion
        true, // readOnly
        false // multiple
    );

    String[] predesessorConstraints = { "nt:version" };
    this.declaredPropertyDefinitions[3] = new PropertyDefinitionImpl(
        "jcr:predecessors", // name
        this,                   // declaringNodeType
        PropertyType.REFERENCE, // requiredType
        predesessorConstraints, // valueConstraints
        null,                   // defaultValues
        false,                  // autoCreate
        true,                   // mandatory
        OnParentVersionAction.COPY, // onVersion
        true,                   // readOnly
        true                    // multiple
    );

    this.declaredPropertyDefinitions[4] = new PropertyDefinitionImpl(
        "jcr:mergeFailed",      // name
        this,                   // declaringNodeType
        PropertyType.REFERENCE, // requiredType
        EMPTY_CONSTRAINTS,      // valueConstraints
        null,                   // defaultValues
        false,                  // autoCreate
        false,                  // mandatory
        OnParentVersionAction.ABORT, // onVersion
        true,                   // readOnly
        true                    // multiple
    );

  }

}
