/*
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.jcr.impl.core.nodetype.nt;


import javax.jcr.PropertyType;
import javax.jcr.version.OnParentVersionAction;

import org.exoplatform.services.jcr.impl.core.nodetype.NodeTypeImpl;
import org.exoplatform.services.jcr.impl.core.nodetype.PropertyDefinitionImpl;
import org.exoplatform.services.jcr.impl.core.value.StringValue;

import javax.jcr.Value;
import javax.jcr.nodetype.*;
import javax.jcr.RepositoryException;

/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 31 juil. 2004
 */
public class ChildNodeDefType extends NodeTypeImpl {

  public ChildNodeDefType(NodeTypeManager manager) throws NoSuchNodeTypeException,RepositoryException {
    super(manager);
    this.name = "nt:childNodeDefinition";
    this.mixin = false;
    this.orderableChild = false;
    this.primaryItemName = null;
    this.declaredSupertypes = new NodeType[1];
    this.declaredSupertypes[0] = manager.getNodeType("nt:base");
    this.declaredPropertyDefinitions = new PropertyDefinition[8];

    this.declaredPropertyDefinitions[0] = new PropertyDefinitionImpl(
        "jcr:name",                    //name
        this,                          //declaringNodeType
        PropertyType.NAME,             //requiredType
        EMPTY_CONSTRAINTS,             //valueConstraints 
        null,                          //defaultValues
        false,                         //autoCreate
        false,                          //mandatory        
        OnParentVersionAction.COPY,    //onVersion        
        false,                         //readOnly         
        false                          //multiple         
        );

    Value[] defValues = {new StringValue("nt:base")};
    this.declaredPropertyDefinitions[1] = new PropertyDefinitionImpl(
        "jcr:requiredPrimaryTypes",    //name
        this,                         //declaringNodeType
        PropertyType.NAME,           //requiredType
        EMPTY_CONSTRAINTS,           //valueConstraints 
        defValues,                      //defaultValues
        false,                          //autoCreate
        true,                          //mandatory        
        OnParentVersionAction.COPY,    //onVersion        
        false,                         //readOnly         
        true                           //multiple         
        );

    this.declaredPropertyDefinitions[2] = new PropertyDefinitionImpl(
        "jcr:defaultPrimaryType",      //name
        this,                          //declaringNodeType
        PropertyType.NAME,              //requiredType
        EMPTY_CONSTRAINTS,              //valueConstraints 
        null,                           //defaultValues
        false,                          //autoCreate
        false,                          //mandatory
        OnParentVersionAction.COPY,    //onVersion        
        false,                         //readOnly         
        false                          //multiple         
        );

    this.declaredPropertyDefinitions[3] = new PropertyDefinitionImpl(
        "jcr:autoCreated",              //name
        this,                          //declaringNodeType
        PropertyType.BOOLEAN,          //requiredType     
        EMPTY_CONSTRAINTS,             //valueConstraints 
        null,                         //defaultValues
        false,                         //autoCreate
        true,                          //mandatory        
        OnParentVersionAction.COPY,    //onVersion        
        false,                         //readOnly         
        false                          //multiple         
        );

    this.declaredPropertyDefinitions[4] = new PropertyDefinitionImpl(
        "jcr:mandatory",               //name
        this,                          //declaringNodeType
        PropertyType.BOOLEAN,          //requiredType     
        EMPTY_CONSTRAINTS,            //valueConstraints 
        null,                         //defaultValues
        false,                         //autoCreate       
        true,                          //mandatory        
        OnParentVersionAction.COPY,    //onVersion        
        false,                         //readOnly         
        false                          //multiple         
        );

    this.declaredPropertyDefinitions[5] = new PropertyDefinitionImpl(
        "jcr:onParentVersion",         //name
        this,                          //declaringNodeType
        PropertyType.STRING,           //requiredType     
        new String[]{"COPY", "VERSION", 
        "INITIALIZE", "COMPUTE", 
		"IGNORE", "ABORT"},                 //valueConstraints 
        null,                           //defaultValues
        false,                         //autoCreate
        true,                          //mandatory        
        OnParentVersionAction.COPY,    //onVersion        
        false,                         //readOnly         
        false                          //multiple         
        );

    this.declaredPropertyDefinitions[6] = new PropertyDefinitionImpl(
        "jcr:protected",               //name
        this,                         //declaringNodeType
        PropertyType.BOOLEAN,         //requiredType     
        EMPTY_CONSTRAINTS,           //valueConstraints 
        null,                        //defaultValues
        false,                        //autoCreate
        true,                         //mandatory        
        OnParentVersionAction.COPY,   //onVersion        
        false,                        //readOnly         
        false                         //multiple         
        );

    this.declaredPropertyDefinitions[7] = new PropertyDefinitionImpl(
        //"jcr:sameNameSibs",           //name
        "jcr:sameNameSiblings",           //name        
        this,                         //declaringNodeType
        PropertyType.BOOLEAN,         //requiredType     
        EMPTY_CONSTRAINTS,            //valueConstraints 
        null,                         //defaultValues
        false,                         //autoCreate
        true,                         //mandatory        
        OnParentVersionAction.COPY,   //onVersion        
        false,                        //readOnly         
        false                         //multiple         
        );
  }
}
