/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

package org.exoplatform.services.jcr.impl.core;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.JcrImplBaseTest;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TestSessionDataManager.java 3510 2005-12-29 17:32:57Z geaz $
 */
public class TestSessionDataManager extends JcrImplBaseTest {
  
  private static String TEST_ROOT = "testRoot";

  private SessionDataManager modificationManager;
  
  private NodeImpl testRoot;

  public void setUp() throws Exception {
    super.setUp();
    modificationManager = session.getTransientNodesManager();
    NodeImpl root = (NodeImpl)session.getRootNode();
    
    // init some test items
    testRoot = (NodeImpl)root.createChildNode(TEST_ROOT, "nt:unstructured", true, true);
    log.debug("Test root >>>>>> "+testRoot);
    
    modificationManager.getWorkspaceDataManager().add(testRoot);
    
  }

  public void tearDown() throws Exception {
    testRoot.remove();
    testRoot.save();
  }
  
  public void testTheSameNodeInSequantalCall() throws Exception {
    Node n1 = (Node)session.getItem("/testRoot");
    Node n2 = (Node)session.getItem("/testRoot");
    assertEquals(n1.getPath(), n2.getPath());
    assertEquals(n1, n2);
    n2 = (Node)session.getRootNode().getNode("testRoot");
    assertEquals(n1, n2);
  }
  
  public void testIfAddedNodeIsTheSame() throws Exception {
    Node n1 = testRoot.addNode("newNode1");
    Node n2 = (Node)session.getItem("/testRoot/newNode1");
    assertEquals(n1.getPath(), n2.getPath());
    assertEquals(n1, n2);
    n2 = (Node)session.getRootNode().getNode("testRoot/newNode1");
    assertEquals(n1, n2);
    session.save();
    n2 = (Node)session.getRootNode().getNode("testRoot/newNode1");
    assertEquals(n1, n2);
  }
  
  public void testIfAddedPropertyIsTheSame() throws Exception {
    Property p1 = testRoot.setProperty("newProp1", "newProp");
    Property p2 = (Property)session.getItem("/testRoot/newProp1");
    assertEquals(p1.getPath(), p2.getPath());
    assertEquals(p1, p2);
    p2 = (Property)session.getRootNode().getProperty("testRoot/newProp1");
    assertEquals(p1, p2);
    session.save();
    p2 = (Property)session.getRootNode().getProperty("testRoot/newProp1");
    assertEquals(p1, p2);
  }

  public void testIfRemovedItemDeletedFromAll() throws Exception {
    
    testRoot.setProperty("newProp1", "newProp");
    testRoot.getProperty("newProp1");
    session.getItem("/testRoot/newProp1").remove();
    try {
      testRoot.getProperty("newProp1");
      fail("PathNotFoundException should have been thrown");
    } catch (PathNotFoundException e) {
    } 
  }

}