/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

package org.exoplatform.services.jcr.impl.core.nodetype;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PropertyType;
import javax.jcr.nodetype.NoSuchNodeTypeException;
import javax.jcr.nodetype.NodeType;
import javax.jcr.version.OnParentVersionAction;

import org.exoplatform.services.jcr.BasePortalTest;
import org.exoplatform.services.jcr.core.nodetype.NodeDefinitionValue;
import org.exoplatform.services.jcr.core.nodetype.NodeTypeValue;
import org.exoplatform.services.jcr.core.nodetype.NodeTypeValuesList;
import org.exoplatform.services.jcr.core.nodetype.PropertyDefinitionValue;
import org.jibx.runtime.BindingDirectory;
import org.jibx.runtime.IBindingFactory;
import org.jibx.runtime.IMarshallingContext;
import org.jibx.runtime.JiBXException;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: TestNodeTypeManagerImpl.java 3735 2006-01-11 18:20:47Z geaz $
 */

public class TestNodeTypeManagerImpl extends BasePortalTest {

  protected String getRepositoryName() {
    return "db1";
  }

  public void tearDown() throws Exception {
  }

  public void testDefaultNodeTypes() throws Exception {

    NodeTypeManagerImpl ntManager = (NodeTypeManagerImpl) session
        .getWorkspace().getNodeTypeManager();
    assertTrue(ntManager.getAllNodeTypes().hasNext());
    assertNotNull(ntManager.getNodeType("nt:base"));

  }

  public void testInitStorage() throws Exception {

    NodeTypeManagerImpl ntManager = (NodeTypeManagerImpl) session
        .getWorkspace().getNodeTypeManager();
    Node sys = (Node) repository.getSystemSession().getItem("/jcr:system");
    ntManager.initStorage(sys, true);
    Node ntRoot = (Node) repository.getSystemSession().getItem(
        NodeTypeManagerImpl.NT_ROOT);
    assertTrue(ntRoot.hasNodes());
    NodeIterator nodes = ntRoot.getNodes();
    while (nodes.hasNext())
      log.debug(">>. " + nodes.nextNode());
    assertTrue(ntRoot.hasNode("nt:base"));

  }

  public void testRegisterNodeTypeFromObject() throws Exception {
    NodeTypeManagerImpl ntManager = (NodeTypeManagerImpl) session
        .getWorkspace().getNodeTypeManager();
    // Node sys =
    // (Node)repository.getSystemSession().getItem("/jcr:system");
    // ntManager.initStorage(sys, true);

    try {
      ntManager.getNodeType("exo:exoType");
      fail("exo:testType should not be registered ");
    } catch (NoSuchNodeTypeException e) {
    }

    NodeType type = new ExoType(ntManager);

    log.debug("TEST Register");
    ntManager.registerNodeType(type, 0);

    assertNotNull(ntManager.getNodeType("exo:exoType"));
    //	
    // Node ntRoot =
    // (Node)repository.getSystemSession().getItem(NodeTypeManagerImpl.NT_ROOT);
    // assertTrue(ntRoot.hasNode("exo:exoType"));

    session.getRootNode().addNode("test", "exo:exoType");
    log.debug("TEST Register add node end");

    // ntManager.unregisterNodeType("exo:exoType");

  }

  public void testRegisterNodeTypeFromClass() throws Exception {
    NodeTypeManagerImpl ntManager = (NodeTypeManagerImpl) session
        .getWorkspace().getNodeTypeManager();
    // Node sys =
    // (Node)repository.getSystemSession().getItem("/jcr:system");
    // ntManager.initStorage(sys, true);

    // try {
    // ntManager.getNodeType("exo:exoType");
    // fail("exo:testType should not be registered ");
    // } catch (NoSuchNodeTypeException e) {
    // }

    ntManager.registerNodeType(
        org.exoplatform.services.jcr.impl.core.nodetype.ExoType1.class, 0);
    assertNotNull(ntManager.getNodeType("exo:exoType1"));

    // Node ntRoot =
    // (Node)repository.getSystemSession().getItem(NodeTypeManagerImpl.NT_ROOT);
    // assertTrue(ntRoot.hasNode("exo:exoType"));

    // session.getRootNode().addNode("test", "exo:exoType");

  }

  public void testRegisterNodeTypeFromValue() throws Exception {

    NodeTypeValue testNtValue = createTestNtValue();

    NodeTypeManagerImpl ntManager = (NodeTypeManagerImpl) session
        .getWorkspace().getNodeTypeManager();
    ntManager.registerNodeType(testNtValue, 0);

    assertNotNull(ntManager.getNodeType("exo:myNt"));

    Node ntRoot = (Node) repository.getSystemSession().getItem(
        NodeTypeManagerImpl.NT_ROOT);
    assertTrue(ntRoot.hasNode("exo:myNt"));

    session.getRootNode().addNode("test", "exo:myNt");

  }

  // Not used until we don't make true serialixation from here 
  public void notUsedTestRegisterNodeTypeFromXml() throws Exception {
    // This is test for make overridden test with collection of a
    // NodeTypeValue beans.

    log.info("testRegisterNodeTypeFromXml(): start");

    // Preparing xml stream...
    int ntCount = 15;
    String ntBaseName = "exo:myNt";
    ArrayList testList = new ArrayList();
    for (int i = 0; i < ntCount; i++) {
      String ntName = ntBaseName + i;
      NodeTypeValue testNtValue = createTestNtValue(ntName);
      testList.add(testNtValue);
    }

    NodeTypeValuesList testNtvList = new NodeTypeValuesList();
    testNtvList.setNodeTypeValuesList(testList);

    byte[] xmlData = makeXmlContent(testNtvList);

    // assertNotNull(xmlData); // Not actual for testRegisterNodeTypeFromXml()
    // testing

    //saveAsFile(xmlData, "xml-jibx.xml");

    ByteArrayInputStream xmlInput = new ByteArrayInputStream(xmlData);

    // So, we have a xml stream, let's test it...

    NodeTypeManagerImpl ntManager = (NodeTypeManagerImpl) session
        .getWorkspace().getNodeTypeManager();
    log.info("testRegisterNodeTypeFromXml(): registerNodeType()");
    ntManager.registerAllNodeTypes(xmlInput, 0);

    for (int i = 0; i < ntCount; i++) {
      String ntName = ntBaseName + i;
      log.info("testRegisterNodeTypeFromXml(): assertNotNull: " + ntName);
      assertNotNull(ntManager.getNodeType(ntName));
    }

    Node ntRoot = (Node) repository.getSystemSession().getItem(
        NodeTypeManagerImpl.NT_ROOT);

    for (int i = 0; i < ntCount; i++) {
      String ntName = ntBaseName + i;
      log.info("testRegisterNodeTypeFromXml(): assertTrue: " + ntName);
      assertTrue(ntRoot.hasNode(ntName));
    }

    for (int i = 0; i < ntCount; i++) {
      String ntName = ntBaseName + i;
      log.info("testRegisterNodeTypeFromXml(): session.getRootNode().addNode(): "
              + ntName);
      session.getRootNode().addNode("test", ntName);
    }

  }

  public void testRegisterNodeTypeFromXmlStorage() throws Exception {
    // This is test for make overridden test with collection of a
    // NodeTypeValue beans deserialized from xml file.

    log.info("testRegisterNodeTypeFromXmlNodeTypes(): start");

    // Preparing xml stream...
    byte[] xmlData = readXmlContent("nt.xml");

    // assertNotNull(xmlData); // Not actual for testRegisterNodeTypeFromXml()
    // testing

    //saveAsFile(xmlData, "nt-back.xml");

    ByteArrayInputStream xmlInput = new ByteArrayInputStream(xmlData);

    // So, we have a xml stream, let's test it...

    NodeTypeManagerImpl ntManager = (NodeTypeManagerImpl) session
        .getWorkspace().getNodeTypeManager();
    //log.info("testRegisterNodeTypeFromXmlNodeTypes(): registerNodeType()");
    ntManager.registerAllNodeTypes(xmlInput, 0);

    // test some nodes from nt set
    String ntName = "nt:base";
    //log.info("testRegisterNodeTypeFromXmlNodeTypes(): assertNotNull: " + ntName);
    assertNotNull(ntManager.getNodeType(ntName));

    Node ntRoot = (Node) repository.getSystemSession().getItem(
        NodeTypeManagerImpl.NT_ROOT);

    //log.info("testRegisterNodeTypeFromXmlNodeTypes(): assertTrue: " + ntName);
    assertTrue(ntRoot.hasNode(ntName));

    //log.info("testRegisterNodeTypeFromXmlNodeTypes(): session.getRootNode().addNode(): " + ntName);
    session.getRootNode().addNode("test", ntName);

    ntName = "nt:nodeType";
    assertNotNull(ntManager.getNodeType(ntName));
    ntRoot = (Node) repository.getSystemSession().getItem(NodeTypeManagerImpl.NT_ROOT);
    assertTrue(ntRoot.hasNode(ntName));
    session.getRootNode().addNode("test_nodeType", ntName);
    
    ntName = "nt:file";
    assertNotNull(ntManager.getNodeType(ntName));
    ntRoot = (Node) repository.getSystemSession().getItem(NodeTypeManagerImpl.NT_ROOT);
    assertTrue(ntRoot.hasNode(ntName));
    session.getRootNode().addNode("test_file", ntName);
    
    ntName = "nt:file";
    assertNotNull(ntManager.getNodeType(ntName));
    ntRoot = (Node) repository.getSystemSession().getItem(NodeTypeManagerImpl.NT_ROOT);
    assertTrue(ntRoot.hasNode(ntName));
    session.getRootNode().addNode("test_file", ntName);
}

  private byte[] makeXmlContent(NodeTypeValuesList testNtvList) {

    byte[] r = null;
    try {
      IBindingFactory factory = BindingDirectory
          .getFactory(NodeTypeValuesList.class);
      IMarshallingContext mctx = factory.createMarshallingContext();
      mctx.setIndent(4);

      ByteArrayOutputStream output = new ByteArrayOutputStream();
      mctx.marshalDocument(testNtvList, "UTF-8", null, output);

      r = output.toByteArray();
    } catch (JiBXException e) {
      e.printStackTrace();
      log.error("JiBX error in Xml content preparation: " + e);
    } catch (Exception e) {
      e.printStackTrace();
      log.error("Error in Xml content preparation: " + e);
    }
    return r;       
  }

  private byte[] readXmlContent(String fileName) {

    try {
      //FileInputStream fis = new FileInputStream(fileName);
      // aaa
      InputStream is = TestNodeTypeManagerImpl.class.getResourceAsStream("/"+fileName);
      ByteArrayOutputStream output = new ByteArrayOutputStream();

      int r = is.available();
      byte[] bs = new byte[r];
      while (r > 0) {
        r = is.read(bs);
        if (r > 0) {
          output.write(bs, 0, r);
        }
        r = is.available();
      }
      is.close();
      return output.toByteArray();
    } catch (Exception e) {
      log.error("Error read file '"+fileName+"' with NodeTypes. Error:" + e);
      return null;
    }
  }
  
  public void saveAsFile(byte[] data, String fileName) {
    try {
      FileOutputStream fos = new FileOutputStream(fileName);
      fos.write(data);
      fos.flush();
      fos.close();
    } catch (Exception e) {
      log.error("Error save as file " + fileName + ". Error:" + e);
    }
  }

  private NodeTypeValue createTestNtValue() {
    String ntBaseName = "exo:myNt";
    return createTestNtValue(ntBaseName);
  }

  private NodeTypeValue createTestNtValue(String ntName) {
    NodeTypeValue testNtValue = new NodeTypeValue();
    testNtValue.setName(ntName);
    testNtValue.setMixin(false);
    testNtValue.setOrderableChild(false);
    testNtValue.setPrimaryItemName(null);
    ArrayList supertypes = new ArrayList();
    supertypes.add("nt:base");
    testNtValue.setDeclaredSupertypeNames(supertypes);

    ArrayList props = new ArrayList();
    PropertyDefinitionValue prop1 = new PropertyDefinitionValue();
    prop1.setAutoCreate(false);
    ArrayList defVals = new ArrayList();
    defVals.add("test");
    prop1.setDefaultValueStrings(defVals);
    prop1.setMandatory(false);
    prop1.setMultiple(false);
    prop1.setName("myProperty");
    prop1.setOnVersion(OnParentVersionAction.IGNORE);
    prop1.setReadOnly(false);
    prop1.setRequiredType(PropertyType.STRING);
    ArrayList constraints = new ArrayList();
    constraints.add("test");
    constraints.add("TEST");
    prop1.setValueConstraints(constraints);
    testNtValue.setDeclaredPropertyDefinitionValues(props);

    ArrayList nodes = new ArrayList();
    NodeDefinitionValue node1 = new NodeDefinitionValue();
    node1.setAutoCreate(false);
    node1.setDefaultNodeTypeName("nt:unstructured");
    node1.setMandatory(false);
    node1.setSameNameSiblings(false);
    node1.setName("subNode");
    node1.setOnVersion(OnParentVersionAction.IGNORE);
    node1.setReadOnly(false);
    ArrayList nodeTypes = new ArrayList();
    nodeTypes.add("nt:unstructured");
    node1.setRequiredNodeTypeNames(nodeTypes);
    nodes.add(node1); // !!! Petro Nedonosko
    testNtValue.setDeclaredChildNodeDefinitionValues(nodes);

    return testNtValue;
  }

}
