/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
/**
 * Created by The eXo Platform SARL        .
 * Author : James Chamberlain
 *          james@echamberlains.com
 * Date: 11/2/2005
 * 
 */

package org.exoplatform.services.ldap.impl;


import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.directory.BasicAttributes;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.component.ComponentRequestLifecycle;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.ldap.CreateObjectCommand;
import org.exoplatform.services.ldap.DeleteObjectCommand;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.ldap.ObjectClassAttribute;

public class LDAPServiceImpl implements LDAPService, ComponentRequestLifecycle {
  private ThreadLocal<LdapContext> tlocal_ = new ThreadLocal<LdapContext >() ;
  private Map<String, String> env = null;
  
  public LDAPServiceImpl(InitParams params) throws Exception {
    LDAPConnectionConfig config = 
      (LDAPConnectionConfig)params.getObjectParam("ldap.config").getObject() ;   
    env = new HashMap<String, String>();
    env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
    env.put(Context.SECURITY_AUTHENTICATION, config.getAuthenticationType());
    env.put(Context.SECURITY_PRINCIPAL, config.getRootDN());
    env.put(Context.SECURITY_CREDENTIALS,  config.getPassword());
    env.put("com.sun.jndi.ldap.connect.timeout", "60000");
    env.put("com.sun.jndi.ldap.connect.pool", "true");
    env.put("java.naming.ldap.version", config.getVerion());
    env.put("java.naming.ldap.attributes.binary","tokenGroups");
    env.put(Context.PROVIDER_URL, config.getProviderURL());
    env.put(Context.REFERRAL, "follow");
  }
  
  
  public LdapContext getLdapContext() throws Exception {
    //new Exception("===================================").printStackTrace() ;
    LdapContext context = tlocal_.get() ;    
    if(context == null) {
      context = new InitialLdapContext(new Hashtable<String, String>(env), null) ;     
      tlocal_.set(context) ;
    } else {     
        context.setRequestControls(null);    
    }
    return context;
  }
  
  
  public InitialContext getInitialContext() throws Exception {
    Hashtable<String, String> props = new Hashtable<String, String>(env);
    props.put(Context.OBJECT_FACTORIES, "com.sun.jndi.ldap.obj.LdapGroupFactory");
    props.put(Context.STATE_FACTORIES, "com.sun.jndi.ldap.obj.LdapGroupFactory");
    return  new InitialLdapContext(props, null);
  }
  
  
  public boolean authenticate(String userDN, String password) throws Exception {
    Hashtable<String, String> props = new Hashtable<String, String>(env);
    props.put(Context.SECURITY_PRINCIPAL, userDN);
    props.put(Context.SECURITY_CREDENTIALS, password);
    props.remove("com.sun.jndi.ldap.connect.pool");
    props.put("com.sun.jndi.ldap.connect.pool", "false");
    new InitialLdapContext(props, null);
    return true;
  }
  
  public void createSubContext(String dn) throws Exception {
    Pattern pattern = Pattern.compile("\\b\\p{Space}*=\\p{Space}*", Pattern.CASE_INSENSITIVE);
    Matcher matcher = pattern.matcher(dn);
    dn = matcher.replaceAll("="); 
    LdapContext context = getLdapContext();
    Name name  = context.getNameParser("").parse(dn);    
    String [] explodeDN = explodeDN(name, false);
    if(explodeDN.length < 1) return;
    dn = explodeDN[explodeDN.length-1];      
    int i = explodeDN.length-2;
    for(; i > -1; i--){
      if(!explodeDN[i].startsWith("dc=")) break;
      dn = explodeDN[i]+","+dn;
    }   
    createDN(dn, context);      
    for(; i > -1; i--){
      dn = explodeDN[i]+","+dn;
      createDN(dn, context);      
    }    
  } 
  
  private void createDN(String dn, LdapContext context) throws Exception {
   
    try{
      Object obj = context.lookupLink(dn);      
      if(obj != null) return;
    }catch(Exception exp){}    
    String nameValue = dn.substring(dn.indexOf("=")+1, dn.indexOf(","));
    BasicAttributes attrs = new BasicAttributes();
    if(dn.startsWith("ou=")){
      attrs.put( new ObjectClassAttribute(new String[]{"top", "organizationalUnit"}));  
      attrs.put("ou", nameValue);   
    }else if(dn.startsWith("cn=")){
      attrs.put( new ObjectClassAttribute(new String[]{"top", "organizationalRole"}));  
      attrs.put("cn", nameValue);  
    }else if(dn.startsWith("c=")){      
      attrs.put( new ObjectClassAttribute(new String[]{"country"}));  
      attrs.put("c", nameValue);      
    }else if(dn.startsWith("o=")){      
      attrs.put( new ObjectClassAttribute(new String[]{"country"}));  
      attrs.put("c", nameValue);      
    }else if(dn.startsWith("dc=")){      
      attrs.put( new ObjectClassAttribute(new String[]{"top","dcObject","organization"}));  
      attrs.put("c", nameValue);      
    }
    attrs.put("description", nameValue);
    try{
      context.createSubcontext(dn, attrs);
    }catch(Exception exp){
      exp.printStackTrace();
    }
  } 
  
  public String[] explodeDN(Name dn, boolean removeTypes){
    String explodedDN[] = null;    
    Enumeration<String> enumeration = dn.getAll();
    List<String> list = new ArrayList<String>();
    while(enumeration.hasMoreElements()){
      String ldap = enumeration.nextElement();
      if (removeTypes){
        int position = ldap.indexOf("=");
        String value = ldap.substring(position + 1);
        list.add(0, value);
      } else {
        list.add(0, ldap);
      }
    }
    explodedDN = new String[list.size()];
    list.toArray(explodedDN);
    return explodedDN;
  } 
  
  public String escapeDN(String dn) {
    StringBuffer buf = new StringBuffer(dn.length());
    for (int i = 0; i < dn.length(); i++) {
      char c = dn.charAt(i);
      switch (c) {
      case '\\':
        buf.append("\\5c");
        break;
      case '*':
        buf.append("\\2a");
        break;
      case '(':
        buf.append("\\28");
        break;
      case ')':
        buf.append("\\29");
        break;
      case '\0':
        buf.append("\\00");
        break;
      default:
        buf.append(c);
      break;
      }
    }
    return buf.toString();
  }
  
  
  public void addDeleteObject(ComponentPlugin plugin) throws Exception {
    DeleteObjectCommand command = (DeleteObjectCommand) plugin ;
    LdapContext ctx = getLdapContext() ;
    command.deleteObjects(ctx) ;
  }
  
  public void addCreateObject(ComponentPlugin plugin) throws Exception {
    CreateObjectCommand command = (CreateObjectCommand) plugin ;
    LdapContext ctx = getLdapContext() ;
    command.addObjects(ctx) ;
  }


  public void startRequest(ExoContainer container) {   }


  public void endRequest(ExoContainer container) {
    LdapContext context = tlocal_.get() ;
    if(context != null) {
      try {
        context.close() ;
        tlocal_.set(null);
      } catch(Exception ex) {
        ex.printStackTrace() ;
      }
    }
  }
}
