/**
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

package org.exoplatform.services.organization.ldap;

import javax.naming.directory.Attributes;
import javax.naming.ldap.Control;
import javax.naming.ldap.LdapContext;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.User;

/**
 * Created by The eXo Platform SARL        .
 * Author : James Chamberlain
 *          james.chamberlain@gmail.com
*/

public class ActiveDirectoryUserHandlerImpl extends UserHandlerImpl {

	//	some useful constants from lmaccess.h
	private int UF_ACCOUNTDISABLE = 0x0002;
	private int UF_PASSWD_NOTREQD = 0x0020;
	private int UF_NORMAL_ACCOUNT = 0x0200;
	private int UF_PASSWORD_EXPIRED = 0x800000;
	
	public ActiveDirectoryUserHandlerImpl( LDAPAttributeMapping ldapAttrMapping, LDAPService ldapService) {
	  super(ldapAttrMapping, ldapService);
	}
	
	public void createUser(User user, boolean broadcast) throws Exception {   
	  String userDN = "cn="+user.getUserName()+","+ldapAttrMapping_.userURL;    
	  Attributes attrs = ldapAttrMapping_.userToAttributes( user);    
	  // create userAccountControl - create the account disabled so it doesn't need a password
	  attrs.put("userAccountControl",Integer.toString(
	      UF_NORMAL_ACCOUNT + UF_PASSWD_NOTREQD + UF_PASSWORD_EXPIRED+ UF_ACCOUNTDISABLE));
    if(broadcast) preSave(user, true) ;
	  // Create user in directory
	  LdapContext ctx = ldapService_.getLdapContext();
	  ctx.createSubcontext( userDN, attrs);		
    if(broadcast) postSave(user, true) ;    
	}   

  protected PageList findUsers(String searchBase, String filter, int pageSize) throws Exception {   
    return new LDAPUserPageList( 
        this, ldapAttrMapping_, ldapService_, searchBase, filter, pageSize, Control.NONCRITICAL);  
  }
 
//	private byte[] encryptPassword(String password) throws Exception {
//		byte passwordDigest[] = null;
//		String newQuotedPassword = "\"" + password + "\"";
//		passwordDigest = newQuotedPassword.getBytes("UTF-16LE");
//		return passwordDigest;
//	}
}
