/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.naming.Name;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.LdapContext;

import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.impl.GroupImpl;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Oct 14, 2005
 */
public class BaseHandler {
  
  protected LDAPAttributeMapping ldapAttrMapping_ ; 
  protected LDAPService ldapService_ ;
  
  public  BaseHandler(){    
  } 
  
  public  BaseHandler(LDAPAttributeMapping ldapAttrMapping, LDAPService ldapService) {
    ldapAttrMapping_ = ldapAttrMapping ;
    ldapService_ =  ldapService ;
  }
  
  protected String getGroupDNFromGroupId(String groupId) {   
    StringBuilder buffer = new StringBuilder();
    String groupParts[] = groupId.split("/");
    for (int x = (groupParts.length - 1); x > 0; x--) {
      buffer.append("ou=" + groupParts[x] + ", ");
    }   
    buffer.append( ldapAttrMapping_.groupsURL);
    
    return buffer.toString();
  }
  
  protected List<Object> getAttributes(Attributes attributes, String attribute) {
    List<Object> results = new ArrayList<Object>();
    try {
      if (attributes == null)  return results;
      Attribute attr = attributes.get(attribute);
      for (int x = 0; x < attr.size(); x++)  results.add(attr.get(x));  
    } catch (Exception e) {
    }
    return results;
  }
  
 
  
  protected Group getGroupFromMembershipDN(String membershipDN) throws Exception {
    LdapContext ctx = ldapService_.getLdapContext();
    NameParser parser = ctx.getNameParser("");
    String membershipParts[] = this.explodeDN(parser.parse(membershipDN), false);    
    StringBuffer buffer = new StringBuffer();
    for (int x = 1; x < membershipParts.length; x++) {
      if (x == membershipParts.length - 1) {
        buffer.append(membershipParts[x]);
      } else {
        buffer.append(membershipParts[x] + ",");
      }
    }
    Group group = this.getGroupByDN(buffer.toString());    
    return group;
  }
  
  protected Group getGroupByDN(String groupDN) throws Exception {
    LdapContext ctx = ldapService_.getLdapContext();    
    NameParser parser = ctx.getNameParser("");   
    StringBuffer idBuffer = new StringBuffer();
    String parentId = null;
    String baseParts[] = this.explodeDN(parser.parse( ldapAttrMapping_.groupsURL), true);    
    String membershipParts[] = explodeDN(parser.parse(groupDN), true);    
    for (int x = (membershipParts.length - baseParts.length - 1); x > -1; x--) {      
      idBuffer.append("/" + membershipParts[x]);
      if (x == 1) parentId = idBuffer.toString();      
    }    
    if (idBuffer == null) return null;
    Attributes attrs = ctx.getAttributes(groupDN);
    GroupImpl group = new GroupImpl();
    group.setGroupName( membershipParts[0]);
    group.setId( idBuffer.toString());
    group.setDescription( ldapAttrMapping_.getAttributeValueAsString(attrs, "description"));
    group.setLabel( ldapAttrMapping_.getAttributeValueAsString( attrs, "l"));  
    group.setParentId( parentId);
    return group;    
  }  
  
  protected String[] explodeDN(Name dn, boolean removeTypes){
    String explodedDN[] = null;    
    Enumeration<String> enumeration = dn.getAll();
    List<String> list = new ArrayList<String>();
    while(enumeration.hasMoreElements()){
      String ldap = enumeration.nextElement();
      if (removeTypes){
        int position = ldap.indexOf("=");
        String value = ldap.substring(position + 1);
        list.add(0, value);
      } else {
        list.add(0, ldap);
      }
    }
    explodedDN = new String[list.size()];
    list.toArray(explodedDN);
    return explodedDN;
  } 
  
  protected String getDNFromUsername( String username) throws Exception {
    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);   
    constraints.setReturningAttributes( new String[]{ "" });
    constraints.setDerefLinkFlag(true);
    String filter = ldapAttrMapping_.userUsernameAttr + "=" + username ;   
    LdapContext ctx = ldapService_.getLdapContext();  
    NamingEnumeration<SearchResult> answer = ctx.search(ldapAttrMapping_.baseURL, filter, constraints);    
    while ( answer.hasMoreElements()) {
      return answer.next().getNameInNamespace();     
    }
    return null;
  }
  
  protected void romoveAllSubtree( LdapContext context, String dn){
    try {        
      SearchControls constraints = new SearchControls();
      constraints.setSearchScope( SearchControls.ONELEVEL_SCOPE);      
      NamingEnumeration<SearchResult> results = 
        context.search(  dn, "(objectclass=*)",  constraints);
      while( results.hasMore()){
        SearchResult sr =  results.next();        
        romoveAllSubtree( context, sr.getNameInNamespace());
      }      
      context.destroySubcontext( dn);
    }catch( Exception exp){        
    }
  }
  
}
