/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import javax.naming.Name;
import javax.naming.NameNotFoundException;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.Control;
import javax.naming.ldap.LdapContext;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.Query;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserEventListener;
import org.exoplatform.services.organization.UserHandler;
import org.exoplatform.services.organization.impl.UserImpl;
/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Oct 14, 2005
 */
public class UserHandlerImpl extends BaseHandler implements  UserHandler {
  
  private List<UserEventListener> listeners_  = new ArrayList<UserEventListener>(5);
  
  public UserHandlerImpl(LDAPAttributeMapping ldapAttrMapping, LDAPService ldapService) {
    super(ldapAttrMapping, ldapService) ;
  }
  
  public void addUserEventListener(UserEventListener listener) {
    listeners_.add(listener) ;
  }
  
  public User createUserInstance() {  return new UserImpl() ;  }

  public void createUser(User user, boolean broadcast) throws Exception {   
    LdapContext ctx = ldapService_.getLdapContext();
    String userDN = "cn="+ user.getUserName()+ ","+  ldapAttrMapping_.userURL;    
    if(broadcast) preSave(user, true) ;    
    ctx.createSubcontext(userDN, ldapAttrMapping_.userToAttributes(user));    
    if(broadcast) postSave(user, true) ;
  }
  
  public void saveUser(User user, boolean broadcast) throws Exception {    
    LdapContext ctx = ldapService_.getLdapContext();    
    String userDN = this.getDNFromUsername(user.getUserName());
    if (userDN != null){
      User existingUser = this.findUserByDN(userDN, ctx);
      
      NameParser parser = ctx.getNameParser("");
      Name name = parser.parse(userDN);
  
      ArrayList<ModificationItem> modifications = new ArrayList<ModificationItem>();
      
      // update displayName & description
      if (!user.getFullName().equals(existingUser.getFullName())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
            new BasicAttribute( ldapAttrMapping_.userDisplayNameAttr, user.getFullName()));
        modifications.add(mod);
        mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
            new BasicAttribute( ldapAttrMapping_.ldapDescriptionAttr, user.getFullName()));
        modifications.add(mod);
      }      
      // update account name
      if (!user.getUserName().equals(existingUser.getUserName())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
            new BasicAttribute( ldapAttrMapping_.userUsernameAttr, user.getUserName()));
        modifications.add(mod);
      }      
      // update last name
      if (!user.getLastName().equals(existingUser.getLastName())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
            new BasicAttribute( ldapAttrMapping_.userLastNameAttr, user.getLastName()));
        modifications.add(mod);
      }      
      // update first name
      if (!user.getFirstName().equals(existingUser.getFirstName())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
            new BasicAttribute( ldapAttrMapping_.userFirstNameAttr, user.getFirstName()));
        modifications.add(mod);
      }      
      // update email
      if (!user.getEmail().equals(existingUser.getEmail())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
            new BasicAttribute( ldapAttrMapping_.userMailAttr, user.getEmail()));
        modifications.add(mod);
      }      
      if (!user.getPassword().equals("PASSWORD")){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
            new BasicAttribute( ldapAttrMapping_.userPassword, user.getPassword()));
        modifications.add(mod);
      }
      
      ModificationItem[] mods = new ModificationItem[ modifications.size()];      
      modifications.toArray(mods);
      if(broadcast) preSave(user, true);
      ctx.modifyAttributes(name, mods);
      if(broadcast) postSave(user, true);
    }
  } 

  public User removeUser(String userName, boolean broadcast) throws Exception {
    User user =  findUserByName(userName) ;
    if(user != null) {
      LdapContext ctx = ldapService_.getLdapContext();
      String userDN = "cn="+  userName + ","+  ldapAttrMapping_.userURL;      
      removeUserFromMembership(ctx, userName);
      if(broadcast) preDelete(user)  ;
      ctx.destroySubcontext( userDN);      
      if(broadcast) postDelete(user)  ;
    }
    return user ;
  }
  
  private void removeUserFromMembership(LdapContext ctx, String userName) throws Exception {     
    String userDN = this.getDNFromUsername( userName);    
    String filter =  ldapAttrMapping_.membershipTypeMemberValue + "=" +ldapService_.escapeDN(userDN);  

    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
    NamingEnumeration<SearchResult> results = ctx.search( ldapAttrMapping_.groupsURL, filter, constraints);
    
    while(results.hasMore()) {
      SearchResult sr = results.next();       
      try{
        Attributes attrs =  sr.getAttributes();   
        if( attrs.get( ldapAttrMapping_.membershipTypeMemberValue).size() > 1){          
          ModificationItem[] mods = new ModificationItem[1];
          mods[0] = new ModificationItem( DirContext.REMOVE_ATTRIBUTE,
              new BasicAttribute( ldapAttrMapping_.membershipTypeMemberValue, userDN));
          ctx.modifyAttributes( sr.getNameInNamespace(), mods); 
        }else
          ctx.destroySubcontext( sr.getNameInNamespace());        
      }catch( Exception exp){
        exp.printStackTrace();
      }
    }
  }

  public User findUserByName(String userName) throws Exception {
    User u  = this.findUserByName(userName, ldapAttrMapping_.userURL);
    if( u == null) u = this.findUserByNameInScope(userName, ldapAttrMapping_.baseURL);    
    return u;
  }
  
  public User findUserByName( String userName, String url) throws Exception{
    LdapContext ctx = ldapService_.getLdapContext();
    String userDN = "cn="+  userName + ","+  url;
    try{
      return ldapAttrMapping_.attributesToUser(this, ctx.getAttributes(userDN)) ;
    }catch( Exception exp){     
      return null;
    }
  }
  
  private User findUserByNameInScope( String userName, String url) throws Exception{    
    LdapContext ctx = ldapService_.getLdapContext();    
    String filter = "cn="+userName;
    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
    NamingEnumeration<SearchResult> results = ctx.search( url, filter, constraints);    
    if( results == null) return null;   
    while( results.hasMoreElements()){
      SearchResult sr = results.next();     
      Attributes attrs = sr.getAttributes();      
      if( attrs != null ) {            
        Attribute attr = attrs.get("objectClass");
        for( int i=0; i<attr.size(); i++){
          if( String.valueOf(attr.get(i)).equals("person"))
            return ldapAttrMapping_.attributesToUser(this, attrs);           
        } 
      }
    }   
    return null;
  }
  

  public PageList findUsersByGroup(String groupId) throws Exception {    
    ArrayList<User> users = new ArrayList<User>();
    TreeMap< String, User> map = new TreeMap< String, User>();
    
    LdapContext ctx = ldapService_.getLdapContext();
    String groupDN = this.getGroupDNFromGroupId(groupId);
    String searchBase = groupDN;
    String filter = ldapAttrMapping_.membershipObjectClassFilter;
    
    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    NamingEnumeration<SearchResult> results = ctx.search(searchBase, filter, constraints);
        
    while( results.hasMore()){
      SearchResult sr = results.next();
      Attributes attrs = sr.getAttributes();
      List<Object> members = this.getAttributes(attrs, ldapAttrMapping_.membershipTypeMemberValue);
      for (int x = 0; x < members.size(); x++){        
        User user = this.findUserByDN( (String)members.get(x), ctx);       
        if( user != null)  map.put(user.getUserName(), user);
      }
    }  
        
    for (Iterator<String> i = map.keySet().iterator(); i.hasNext();) users.add( map.get(i.next()));    
    return new ObjectPageList(users, 20);
  }
  
  protected User findUserByDN (String userDN, LdapContext ctx) throws Exception {   
    if (userDN == null) return null;
    try {
      NameParser parser = ctx.getNameParser("");
      Name dn = parser.parse( userDN);
      Attributes attrs = ctx.getAttributes(dn);
      if (attrs == null) return null;
      User user = this.ldapAttrMapping_.attributesToUser( this, attrs);         
      user.setFullName( user.getFirstName()+" "+user.getLastName());  
      return user;
    } catch (NameNotFoundException e){     
      return null;
    }
  }

  public PageList getUserPageList(int pageSize) throws Exception {      
    String searchBase = ldapAttrMapping_.userURL;
    String filter = ldapAttrMapping_.userObjectClassFilter;
    return findUsers( searchBase, filter, pageSize);
  }

  public PageList findUsers(Query q) throws Exception { 
    String filter = null;
    ArrayList<String> list = new ArrayList<String>();
    if (q.getUserName() != null && q.getUserName().length() > 0) {
      list.add("(" + ldapAttrMapping_.userUsernameAttr + "=" + q.getUserName() + ")");
    }
    if (q.getFirstName() != null && q.getFirstName().length() > 0) {
      list.add("(" + ldapAttrMapping_.userFirstNameAttr + "=" + q.getFirstName() + ")");
    }
    if (q.getLastName() != null && q.getLastName().length() > 0) {
      list.add("(" + ldapAttrMapping_.userLastNameAttr + "=" + q.getLastName() + ")");
    }
    if (q.getEmail() != null && q.getEmail().length() > 0) {
      list.add("(" + ldapAttrMapping_.userMailAttr + "=" + q.getEmail() + ")");
    }
    
    if (list.size() > 0){
      StringBuilder buffer = new StringBuilder();
      buffer.append("(&");
      if (list.size() > 1){
        for (int x = 0; x < list.size(); x++){
          if (x == (list.size() - 1)){
            buffer.append(list.get(x));
          } else {
            buffer.append(list.get(x)+" || ");
          }
        }
      } else {
        buffer.append(list.get(0));
      }
      buffer.append(" (" +ldapAttrMapping_.userObjectClassFilter + ") )");
      filter = buffer.toString();
    } else {
      filter = ldapAttrMapping_.userObjectClassFilter ;
    }
    String searchBase = ldapAttrMapping_.userURL ;
    return findUsers( searchBase, filter, 20);
  }
    
  protected PageList findUsers(String searchBase, String filter, int pageSize) throws Exception {    
    return new LDAPUserPageList( 
        this, ldapAttrMapping_, ldapService_, searchBase, filter, pageSize, Control.NONCRITICAL);  
  }
  
  public boolean authenticate(String username, String password) throws Exception {
    User user = findUserByName( username);   
    if(user == null)  return false ; 
    return  user.getPassword().equals(password); 
  }
  
  protected void preSave(User user , boolean isNew) throws Exception {
    for (UserEventListener listener : listeners_)   listener.preSave(user, isNew) ;
  }
  
  protected void postSave(User user , boolean isNew) throws Exception {
    for (UserEventListener listener : listeners_) listener.postSave(user, isNew) ;
  }
  
  protected void preDelete(User user) throws Exception {
    for (UserEventListener listener : listeners_)   listener.preDelete(user) ;
  }
  
  protected void postDelete(User user) throws Exception {
    for (UserEventListener listener : listeners_)    listener.postDelete(user)  ;
  }
}
