/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.core.component;

import java.util.*;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;
import org.exoplatform.faces.FacesConstants;
import org.exoplatform.faces.core.Util;
import org.exoplatform.portal.Information;

/**
 * Wed, Dec 22, 2003 @ 23:14
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UISimpleForm.java,v 1.23 2004/08/19 14:53:44 tuan08 Exp $
 */
public class UISimpleForm extends UIGrid {
  final public static String SIMPLE_FORM_RENDERER = "SimpleFormRenderer";
  final static public short ALL_MODE = 0;
  final static public short VIEW_MODE = 1;
  final static public short EDIT_MODE = 2;

  private List hiddenInputs_;
  private Map inputs_;
  protected boolean error_;
  private String formName_;
  private String method_;
  private String formId_;
  private String userAction_;
  protected List validators_;
  private String script_;
  private short mode_;

  public UISimpleForm(String name, String method, String formId) {
    formName_ = name;
    method_ = method;
    formId_ = formId;
    if (formId_ == null)
      formId_ = name;
    setRendererType(SIMPLE_FORM_RENDERER);
    setId(name);
    mode_ = EDIT_MODE;
    inputs_ = new HashMap();
  }

  public boolean hasError() { return error_; }

  public void setError(boolean b) {  error_ = b; }

  public String getFormName() { return formName_; }

  public String getMethod() {  return method_; }

  public String getFormId() {   return formId_; }

  public String getUserAction() {  return userAction_;}

  public void setUserAction(String userAction) {  userAction_ = userAction; }

  public List getHiddenInputs() {   return hiddenInputs_; }

  public String getScript() {
    if (script_ == null)  script_ = getNormalScript();
    return script_;
  }

  public String getSaveAction() {  return SAVE_ACTION; }

  public String getCancelAction() {  return CANCEL_ACTION; }

  public UISimpleForm add(UIHiddenInput input) {
    getChildren().add(input);
    if (hiddenInputs_ == null)
      hiddenInputs_ = new ArrayList(3);
    hiddenInputs_.add(input);
    return this;
  }

  public void add(UIInput input) {
    if (isMultiValue(input.getName())) {
      remove(input.getName());
    }
    getChildren().add(input);
    if (inputs_ == null)  inputs_ = new HashMap(10);
    inputs_.put(input.getName(), input);
  }

  public void addValue(UIInput input) {
    String name = input.getName();
    if (getUIInput(name) == null) {
      add(input);
      return;
    }
    int index = getNumberOfValues(name);
    String realName = name + "[" + index + "]";
    input.setName(realName);
    getChildren().add(input);
    if (inputs_ == null) inputs_ = new HashMap(10);
    inputs_.put(input.getName(), input);
  }

  public void removeValue(String valueName) {    
    int firstIndex = valueName.indexOf("[");
    String name = valueName.substring(0, firstIndex);
    int valueIndex = Integer.parseInt(valueName.substring(firstIndex + 1, firstIndex + 2));
    UIInput input = null;
    int i = 1;
    while (true) {
      String index = name + "[" + i + "]";
      input = (UIInput) inputs_.get(index);      
      if (input == null)
        break;
      if (i == valueIndex) {
        getChildren().remove(input);
        inputs_.remove(index);        
      } else if (i > valueIndex) {
        getChildren().remove(input);
        inputs_.remove(index);        
        String newName = name + "[" + (i - 1) + "]";
        input.setName(newName);
        getChildren().add(input);
        inputs_.put(newName, input);
      }
      i++;
    }
  }
  
  public void remove(String name) {
    if (isMultiValue(name)) {
      getChildren().removeAll(getMultipleValuesUIInput(name));
      inputs_.remove(name);
      int i = 1;
      while (true) {
        String index = name + "[" + i + "]";
        if (getUIInput(index) == null) break;
        inputs_.remove(index);
        i++;
      }
    } else {
      getChildren().remove(getUIInput(name));
      inputs_.remove(name);
    }
  }

  public UIInput getUIInput(String name) { return (UIInput) inputs_.get(name); }

  public void clearInputs() {
    inputs_ = new HashMap();
    hiddenInputs_ = new ArrayList();
  }

  public boolean isMultiValue(String name) {
    String firstIndex = name + "[1]";
    if (inputs_.get(firstIndex) != null) return true;
    return false;
  }

  public int getNumberOfValues(String name) {
    if (!isMultiValue(name)) return 1;
    return getMultipleValuesUIInput(name).size();
  }

  public List<UIInput> getMultipleValuesUIInput(String name) {
    List<UIInput> values = new ArrayList<UIInput>();
    UIInput input = (UIInput) inputs_.get(name);
    if (input == null)  return values;
    values.add(input);
    int i = 1;
    while (true) {
      String index = name + "[" + i + "]";
      input = (UIInput) inputs_.get(index);
      if (input == null) return values;
      values.add(input);
      i++;
    }
  }

  public UIStringInput getUIStringInput(String name) {
    return (UIStringInput) inputs_.get(name);
  }

  public UISelectBox getUISelectBox(String name) {
    return (UISelectBox) inputs_.get(name);
  }

  public UIRadioBox getUIRadioBox(String name) {
    return (UIRadioBox) inputs_.get(name);
  }

  public UICheckBox getUICheckBox(String name) {
    return (UICheckBox) inputs_.get(name);
  }

  public UIUploadInput getUIUploadInput(String name) {
    return (UIUploadInput) inputs_.get(name);
  }

  public UIDateInput getUIDateInput(String name) {
    return (UIDateInput) inputs_.get(name);
  }

  public UIDateTimeInput getUIDateTimeInput(String name) {
    return (UIDateTimeInput) inputs_.get(name);
  }
  
  public UIIntegerInput getUIIntegerInput(String name) {
    return (UIIntegerInput) inputs_.get(name);
  }

  public void setFieldValue(String name, String value) {
    UIInput input = (UIInput) inputs_.get(name);
    input.setValue(value);
  }

  public void setFieldValue(String name, Date date) {
    UIDateInput input = (UIDateInput) inputs_.get(name);
    input.addDate(date);
  }

  public Map getUIInputs() { return inputs_; }

  public void reset() {
    Map map = getUIInputs();
    Collection inputsToRemove = new ArrayList();
    Iterator i = map.values().iterator();
    while (i.hasNext()) {
      UIInput input = (UIInput) i.next();
      if(input.getName().contains("[")) {
        inputsToRemove.add(input);        
      } else {
        input.reset();
      }
    }
    for (Iterator iter = inputsToRemove.iterator(); iter.hasNext();) {
      UIInput input = (UIInput) iter.next();
      getChildren().remove(input);
      inputs_.remove(input.getName());
    }
  }

  public void addValidator(Validator validator) {
    if (validators_ == null) validators_ = new ArrayList(3);
    validators_.add(validator);
  }

  public void addValidator(Class clazz) {
    if (validators_ == null)  validators_ = new ArrayList(3);
    ComponentUtil.addValidator(validators_, clazz);
  }

  public short getMode() { return mode_; }

  public void setMode(short mode) {
    List children = getChildren();
    for (int i = 0; i < children.size(); i++) {
      Object child = children.get(i);
      if (child instanceof UIInput) {
        UIInput input = (UIInput) child;
        if (VIEW_MODE == mode) {
          input.setReadonly(true);
          mode_ = VIEW_MODE;
        } else {
          input.setReadonly(false);
          mode_ = EDIT_MODE;
        }
      }
    }
  }

  public void decode(FacesContext context) {
    getRenderer(context).decode(context, this);
  }

  public void processDecodes(FacesContext context) {
    error_ = false;
    Map paramMap = context.getExternalContext().getRequestParameterMap();
    String uicomponent = (String) paramMap.get(UICOMPONENT);
    if (!getId().equals(uicomponent))
      return;
    List children = getChildren();
    for (int i = 0; i < children.size(); i++) {
      UIComponent child = (UIComponent) children.get(i);
      child.processDecodes(context);
    }
    decode(context);
  }

  final public void processValidators(FacesContext context) {
    if (!isRendered())  return;
    List children = getChildren();
    for (int i = 0; i < children.size(); i++) {
      UIComponent uiChild = (UIComponent) children.get(i);
      if (uiChild.isRendered()) {
        uiChild.processValidators(context);
      }
    }
    if (context.getRenderResponse()) return ;
    error_ = false;
    if (validators_ != null) {
      Validator validator = null;
      try {
        for (int i = 0; i < validators_.size(); i++) {
          validator = (Validator) validators_.get(i);
          validator.validate(context, this, null);
        }
      } catch (ValidatorException ex) {
        error_ = true;
        FacesMessage message = ex.getFacesMessage();
        Information iprovider = Util.findInformationProvider(this) ;
        iprovider.addMessage(message);
        context.renderResponse();
      }
    }
  }

  private String getNormalScript() {
    String script = "<script type='text/javascript'>\n"
        + "  //ie bug  you cannot have more than one button tag\n"
        + "  function submit_" + formName_ + "(action) {\n" + "    document."
        + formName_ + ".elements['" + FacesConstants.ACTION
        + "'].value = action ;\n" + "    document." + formName_
        + ".submit();\n" + "  }\n" + "</script>\n";
    return script;
  }

  public void setNoScript() {
    script_ = "";
  }

  public void setEnhancedScript(String taId, String width, String height) {
    String script = "<script type='text/javascript'>\n"
        + "  _editor_url = '/htmlarea';\n"
        + "  _editor_lang = 'en'; \n"
        + "</script>"
        + "<script type='text/javascript' src='/htmlarea/htmlarea.js'></script>\n"
        + "<script type='text/javascript'>\n" + "  var editor = null;\n"
        + "  function initEditor() {\n" + "    editor = new HTMLArea('" + taId
        + "');\n"
        + "    var cfg = editor.config; // this is the default configuration\n"
        +
        // " cfg.height = '" + height + "';\n" +
        // " cfg.width = '" + width + "';\n" +
        "    editor.generate();\n" + "    editor._iframe.style.width = \""
        + width + "\";" + "    editor._textArea.style.width = \"" + width
        + "\";" + "    editor._iframe.style.height = \"" + height + "\";"
        + "    editor._textArea.style.height = \"" + height + "\";" + "  }\n" +

        "  //ie bug  you cannot have more than one button tag\n"
        + "  function submit_" + formName_ + "(action) {\n" + "    document."
        + formName_ + ".elements['" + ACTION + "'].value = action ;\n"
        + "    document." + formName_ + ".onsubmit();\n" + "    document."
        + formName_ + ".submit();\n" + "  }\n" + "  initEditor() ;\n"
        + "</script>\n";
    script_ = script;
  }
}