/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.remote.component;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.remote.group.CommunicationService;
import org.exoplatform.services.remote.group.CommunicationServiceMonitor;
import org.exoplatform.services.remote.group.MemberInfo;
import org.exoplatform.services.remote.group.PingResult;
/**
 * Wed, Dec 22, 2003 @ 23:14 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIMonitorPortlet.java,v 1.1 2004/08/02 12:04:26 tuan08 Exp $
 */
public class UIClusterInfo extends UIExoCommand {
  static String MEMBER_VIEW = "member" ;
  static String MONITOR_VIEW = "monitor" ;
  static String PING_VIEW = "ping" ;
  
  static String VIEW_MONITOR = "viewMonitor" ;
  static String VIEW_MEMBER = "viewMember" ;
  static String PING_MEMBER = "pingMember" ;
  static String PING_ALL = "pingAll" ;
 
  static Parameter[] VIEW_MONITOR_PARAMS = {new Parameter(ACTION, VIEW_MONITOR) } ;
  static Parameter[] VIEW_MEMBER_PARAMS = {new Parameter(ACTION, VIEW_MEMBER)} ;
  static Parameter[] PING_MEMBER_PARAMS = {new Parameter(ACTION, PING_MEMBER)} ;
  static Parameter[] PING_ALL_PARAMS = {new Parameter(ACTION, PING_ALL)} ;
  
  private List members_ ;
  private CommunicationServiceMonitor  monitor_ ;
  private List pingResults_ ;
  
  private String view_ = MEMBER_VIEW;
  CommunicationService service_ ;
  
  public UIClusterInfo(CommunicationService service) throws Exception {
    setRendererType("VelocityRenderer") ;
    setId("UIClusterInfo") ;
    service_ = service ;
    members_ = service.getMembersInfo() ;
    addActionListener(ViewMonitorActionListener.class, VIEW_MONITOR) ;
    addActionListener(ViewMembersActionListener.class, VIEW_MEMBER) ;
    addActionListener(PingMemberActionListener.class, PING_MEMBER) ;
    addActionListener(PingAllActionListener.class, PING_ALL) ;
	}
  
  public CommunicationServiceMonitor getMonitor() { return monitor_ ; }
  public List getClusterMember() { return members_ ;}
  public List getPingResults() { return pingResults_; }
  public String getView() { return view_ ; }
  
  private MemberInfo getMemberInfo(String objectid) throws Exception { 
    int hashcode = Integer.parseInt(objectid) ;
    for(int i = 0; i < members_.size(); i++) {
      MemberInfo info = (MemberInfo) members_.get(i) ;
      if(hashcode == info.hashCode())  {
        return info ;
      }
    }
    return null ;
  }
  
  
  static public Parameter[] getViewMonitorParams() { return VIEW_MONITOR_PARAMS ;}
  static public Parameter[] getViewMemberParams() { return VIEW_MEMBER_PARAMS ;}
  static public Parameter[] getPingMemberParams() { return PING_MEMBER_PARAMS ;}
  static public Parameter[] getPingAllParams() { return PING_ALL_PARAMS ;}
  
  static public class ViewMonitorActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIClusterInfo uicomp = (UIClusterInfo) event.getComponent() ;
      String objectid = event.getParameter(OBJECTID) ;
      MemberInfo info = uicomp.getMemberInfo(objectid) ;
      uicomp.monitor_ = uicomp.service_.getCommunicationServiceMonitor(info) ;
      uicomp.view_ = MONITOR_VIEW ;
    }
  }
  
  static public class PingMemberActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIClusterInfo uicomp = (UIClusterInfo) event.getComponent() ;
      String objectid = event.getParameter(OBJECTID) ;
      MemberInfo info = uicomp.getMemberInfo(objectid) ;
      PingResult result  = uicomp.service_.ping(info, "ping......") ;
      uicomp.pingResults_ = new ArrayList() ;
      uicomp.pingResults_.add(result) ;
      uicomp.view_ = PING_VIEW ;
    }
  }
  
  static public class PingAllActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIClusterInfo uicomp = (UIClusterInfo) event.getComponent() ;
      uicomp.pingResults_  = uicomp.service_.pingAll("ping all......") ;
      uicomp.view_ = PING_VIEW ;
    }
  }
  
  
  static public class ViewMembersActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIClusterInfo uicomp = (UIClusterInfo) event.getComponent() ;
      uicomp.view_ = MEMBER_VIEW ;
    }
  }
}