/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.admin;

import condition.* ;
import suites.share.CommonUnit ;
import suites.share.UserLoginSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.share.UserLogoutSuite;

/**
 * This suite does
 *     1) go to Admin Template Page
 *     2) Create a new template (add template)
 *     3) Fill view and dialog template and select any NodeType
 *     4) Edit the new Template for that NodeType
 *     5) Add a new dialog template with a custom permission
 *     6) Add a new view template with a custom permission
 *     7) Edit that new template and change the permission
 *     8) Remove the new dialog template
 *     9) Remove the new view template
 *     10) try to remove the default view and dialog (should print message)
 *     11) Go back to template list
 *     12) Delete the new template
 *     13) Go back to ECM admin home 
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class AdminTemplatesSuite extends WebUnitSuite {
  static String DESC = "Go to Admin The ECM page, click on Manage Template button and test all the basic actions" ;
  static String NAME = "AdminTemplatesSuite" ;
  String COMMENT = null;
  
  public AdminTemplatesSuite() {    
    super(NAME, DESC) ;
    //Go to Admin Template Page   
    goToPage("/Home/ecm") ;    
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", DESC).
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;          
    addWebUnit(
      new ClickURLLinkUnit("ManageTemplates", "Click on Manage Templates button").
      setURL("&uicomponent=UIECMAdminSummary&op=manageTemplates").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UITemplatesToolbar&op=addNodeType"))      
    ) ;
    //Create a new template (add template)    
    addWebUnit(
      new ClickURLLinkUnit("AddNewNodeType", "Click on add new Action button").
      setURL("&uicomponent=UITemplatesToolbar&op=addNodeType").        
      addValidator(new ExpectTextValidator("Add a new nodetype"))      
    ) ;    
    //Fill view and dialog template and select any NodeType, then Save
    addWebUnit(
      new SubmitFormUnit("SaveNodeTypeForm", "Fill all info and click on save button").
      setFormId("nodetypeForm").         
      setField("nodetype","exo:article").
      setField("dialog","exo:test-dialog").
      setField("view","exo:test-view").
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=viewNodeType&objectId=exo:article"))      
    ) ;    
    
    COMMENT = "NEED TO REVIEW ( YOU MUST LOGOUT, THEN LOGIN )";
    //Logout
    addWebSuite(new UserLogoutSuite());
    //Login        
    String[] rols = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite("admin", "exo", rols));
    //Go to Admin Template Page 
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", "Select Admin The ECM tab").
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;                
    addWebUnit(
      new ClickURLLinkUnit("ManageTemplates", "Click on Manage Templates button").
      setURL("&uicomponent=UIECMAdminSummary&op=manageTemplates").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UITemplatesToolbar&op=addNodeType"))      
    ) ;
    
    //Edit the new Template for that NodeType    
    addWebUnit(
      new ClickURLLinkUnit("EditCreatedTemplates", "Edit created node type").
      setURL("&uicomponent=UINodeTypesList&op=viewNodeType&objectId=exo:article").            
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=DialogList&op=editTemplate&objectId=dialog1"))           
    ) ;
    //-----Edit Dialog Template
    addWebUnit(
      new ClickURLLinkUnit("EditDialogTemplates", "Edit dialog created node type").
      setURL("&uicomponent=DialogList&op=editTemplate&objectId=dialog1").            
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("dialog1"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdatedDialogTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog(updated)").      
      setField("view","*:/admin").
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=dialogToolbar&op=addDialog"))      
    ) ;
    //-----Edit View Template
    addWebUnit(
      new ClickURLLinkUnit("EditViewTemplates", "Edit view created node type").
      setURL("&uicomponent=ViewList&op=editTemplate&objectId=view1").          
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("view1"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdateViewTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-view(updated)").      
      setField("view","*:/admin").
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=viewToolbar&op=addView"))         
    ) ;
    //-----Add a new dialog template with a custom permission
    addWebUnit(
      new ClickURLLinkUnit("AddNewDialogTemplates", "Add new a dialog template").
      setURL("&uicomponent=dialogToolbar&op=addDialog").           
      addValidator(new ExpectFormIdValidator("templateForm"))           
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchViewPermission", "Click on search button or search link").
      setFormId("templateForm").         
      setField("op","searchGroup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDialogPermission", "Select a group permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin").            
      addValidator(new ExpectFormIdValidator("templateForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewDialogTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog2").      
      setField("template","dialog2").      
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=dialogToolbar&op=addDialog"))      
    ) ;    
    //-----Add a new view template with a custom permission
    addWebUnit(
      new ClickURLLinkUnit("AddNewViewTemplates", "Add new a view template").
      setURL("&uicomponent=viewToolbar&op=addView").      
      addValidator(new ExpectFormIdValidator("templateForm"))           
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchViewPermission", "Click on search button or search link").
      setFormId("templateForm").         
      setField("op","searchGroup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectViewPermission", "Select a group permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin").            
      addValidator(new ExpectFormIdValidator("templateForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewViewTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-view2").   
      setField("template","view2").      
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=viewToolbar&op=addView"))      
    ) ;            
    //-----Edit that new dialog template and change the permission    
    addWebUnit(
      new ClickURLLinkUnit("EditDialog", "Edit and change dialog's permission").
      setURL("&uicomponent=DialogList&op=editTemplate&objectId=dialog2").            
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("dialog2"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchDialogPermission", "Click on search button or search link").
      setFormId("templateForm").         
      setField("op","searchGroup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDialogPermission", "Select a group permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user").            
      addValidator(new ExpectFormIdValidator("templateForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("UpdateNewDialogTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-dialog2(updated)").   
      setField("template","dialog2(updated)").      
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=dialogToolbar&op=addDialog"))           
    ) ;         
    //-----Edit that new view template and change the permission    
    addWebUnit(
      new ClickURLLinkUnit("EditView", "Edit and change view's permission").
      setURL("&uicomponent=ViewList&op=editTemplate&objectId=view2").            
      addValidator(new ExpectFormIdValidator("templateForm")).
      addValidator(new ExpectTextValidator("view2"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SearchViewPermission", "Click on search button or search link").
      setFormId("templateForm").         
      setField("op","searchGroup").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectViewPermission", "Select a group permission").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user").            
      addValidator(new ExpectFormIdValidator("templateForm"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewViewTemplate", "Fill all info and click on save button").
      setFormId("templateForm").         
      setField("content","exo:test-view2(updated)").   
      setField("template","view2(updated)").      
      setField("op","save").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=viewToolbar&op=addView"))      
    ) ;        
    //-----Remove the new dialog template
    addWebUnit(
      new ClickURLLinkUnit("RemoveCreatedDialog", "Remove created dialog").
      setURL("&uicomponent=DialogList&op=deleteTemplate&objectId=dialog2").      
      addValidator(new ExpectNoTextValidator("dialog2"))
    ) ;
    //-----Remove the new view template
    addWebUnit(
      new ClickURLLinkUnit("RemoveCreatedView", "Remove created dialog").
      setURL("&uicomponent=ViewList&op=deleteTemplate&objectId=view2").      
      addValidator(new ExpectNoTextValidator("view2"))
    ) ;
    //-----Try to remove the default view and dialog (should print message)
    addWebUnit(
      new ClickURLLinkUnit("RemoveDefaultDialog", "Remove default dialog").
      setURL("&uicomponent=DialogList&op=deleteTemplate&objectId=dialog1").           
      addValidator(new ExpectTextValidator("UITemplatesList.msg.undeletable"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("RemoveDefaultView", "Remove default view").
      setURL("&uicomponent=ViewList&op=deleteTemplate&objectId=view1").         
      addValidator(new ExpectTextValidator("UITemplatesList.msg.undeletable"))
    ) ;
    //Go back to template list
    addWebUnit(
      new ClickURLLinkUnit("BackToTemplateList", "click on back link").
      setURL("&uicomponent=UIListsManager&op=PhaseId[1].cancel").              
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UINodeTypesList&op=deleteNodeType&objectId=exo:article"))
    ) ;
    
    COMMENT = "IF YOU RUN THIS WEBUNIT, THEN YOU CAN'T CREATE A NEW TEMPLATE";
    /*
    //Delete the new template    
    addWebUnit(
      new ClickURLLinkUnit("DeleteCreatedTemplate", "delete created template").
      setURL("&uicomponent=UINodeTypesList&op=deleteNodeType&objectId=exo:article").              
      addValidator(new ExpectNoTextValidator("exo:article"))
    ) ;
    */
    
    //Go back to ECM admin home
    addWebUnit(
      new ClickURLLinkUnit("Back", "Click on back button").
      setURL("&uicomponent=UITemplatesToolbar&op=PhaseId[1].cancel").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageScripts"))      
    ) ;    
  }     
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user", "admin"] ;
    return [new UserLoginSuite("admin", "exo", roles)] ;
  }  
}